using System;
using System.Collections.Generic;
using System.Text;
using System.Data;


namespace ClubDev.LogBot
{
    class Program
    {
        private static Data data;

        private static DataTable currentUsers;
        private static DataTable currentSharedArtifacts;
        private static DataTable currentSessions;
        private static DataTable currentReplications;
        private static DataTable currentCollaborationRequests;



        static void Main(string[] args)
        {
            data = new Data();

            Console.WriteLine("Starting up logging bot...");

            Console.WriteLine("Getting inital tables...");

            currentUsers = data.GetUserTable();
            currentSharedArtifacts = data.GetSharedArtifactTable();
            currentSessions = data.GetSessionsTable();
            currentReplications = data.GetReplicationsTable();
            currentCollaborationRequests = data.GetCollaborationRequestsTable();

            Console.WriteLine("Starting main loop...");

            while (true)
            {
                CheckUsers();
                CheckSharedArtifacts();
                CheckSessions();
                CheckReplications();
                CheckCollaborationRequests();

                System.Threading.Thread.Sleep(2000);

            }

            
        }

        private static void CheckUsers()
        {
            DataTable dt = data.GetUserTable();

            DataTable additions = SQLOperations.Difference(dt, currentUsers);
            DataTable subtractions = SQLOperations.Difference(currentUsers, dt);

            foreach (DataRow row in additions.Rows)
            {
                Console.WriteLine("USER Addition: " + row["userID"].ToString() + "|" + row["userName"].ToString());

                data.InsertIntoShadowUserTable(DateTime.Now, (long)row["userID"], (long)row["userSessionID"], row["userName"].ToString(), row["userIM"].ToString(), row["userHost"].ToString(), (bool)row["userIsSharedDisplay"], (int)row["userResolutionX"], (int)row["userResolutionY"]);
            }


            foreach (DataRow row in subtractions.Rows)
            {
                Console.WriteLine("USER Removal: " + row["userID"].ToString() + "|" + row["userName"].ToString());

                data.UpdateShadowUserTable(DateTime.Now, (long)row["userID"], (long)row["userSessionID"]);
            }

            currentUsers = dt;
        }

        private static void CheckSharedArtifacts()
        {
            DataTable dt = data.GetSharedArtifactTable();

            DataTable additions = SQLOperations.Difference(dt, currentSharedArtifacts);
            DataTable subtractions = SQLOperations.Difference(currentSharedArtifacts, dt);

            foreach (DataRow row in additions.Rows)
            {
                Console.WriteLine("SHARED ARTIFACT Addition: " + row["artifactID"].ToString() + "|" + row["artifactName"].ToString());

                data.InsertIntoShaddowSharedArtifactsTable(DateTime.Now, (long)row["artifactID"], (int)row["artifactHWND"], (bool)row["artifactFullShare"], (int)row["artifactType"], row["artifactName"].ToString(), row["artifactGroup"].ToString(), row["artifactOwner"].ToString());
            }


            foreach (DataRow row in subtractions.Rows)
            {
                Console.WriteLine("SHARED ARTIFACT Removal: " + row["artifactID"].ToString() + "|" + row["artifactName"].ToString());

                data.UpdateShadowSharedArtifactsTable(DateTime.Now, (long)row["artifactID"]);
            }

            currentSharedArtifacts = dt;
        }

        private static void CheckSessions()
        {
            DataTable dt = data.GetSessionsTable();

            DataTable additions = SQLOperations.Difference(dt, currentSessions);
            DataTable subtractions = SQLOperations.Difference(currentSessions, dt);

            foreach (DataRow row in additions.Rows)
            {
                Console.WriteLine("SESSION Addition: " + row["sessionID"].ToString() + "|" + row["sessionName"].ToString());

                data.InsertIntoShadowSessionsTable(DateTime.Now, (long)row["sessionID"], row["sessionName"].ToString(), row["sessionLocation"].ToString(), row["sessionPassword"].ToString());
            }


            foreach (DataRow row in subtractions.Rows)
            {
                Console.WriteLine("SESSION Removal: " + row["sessionID"].ToString() + "|" + row["sessionName"].ToString());

                data.UpdateShadowSessionsTable(DateTime.Now, (long)row["sessionID"]);
            }

            currentSessions = dt;
        }

        private static void CheckReplications()
        {
            DataTable dt = data.GetReplicationsTable();

            DataTable additions = SQLOperations.Difference(dt, currentReplications);
            DataTable subtractions = SQLOperations.Difference(currentReplications, dt);

            foreach (DataRow row in additions.Rows)
            {
                Console.WriteLine("REPLICATION Addition: " + row["replicationID"].ToString() + "|" + row["replicationSourceHost"].ToString() + "->" + row["replicationClientHost"].ToString());

                int clientHWND = -1;
                if (!row.IsNull("replicationClientHWND"))
                    clientHWND = (int)row["replicationClientHWND"];

                int replicationLocationX = -1;
                if (!row.IsNull("replicationLocationX"))
                    replicationLocationX = (int)row["replicationLocationX"];

                int replicationLocationY = -1;
                if (!row.IsNull("replicationLocationY"))
                    replicationLocationY = (int)row["replicationLocationY"];

                data.InsertIntoShadowReplicationsTable(DateTime.Now, (long)row["replicationID"], row["replicationSourceHost"].ToString(), (int)row["replicationSourceHWND"], row["replicationClientHost"].ToString(), (bool)row["replicationIsSharedScreen"], clientHWND, replicationLocationX, replicationLocationY);
            }


            foreach (DataRow row in subtractions.Rows)
            {
                Console.WriteLine("REPLICATION Removal: " + row["replicationID"].ToString() + "|" + row["replicationSourceHost"].ToString() + "->" + row["replicationClientHost"].ToString());

                data.UpdateShadowReplicationsTable(DateTime.Now, (long)row["replicationID"]);
            }

            currentReplications = dt;
        }

        private static void CheckCollaborationRequests()
        {
            DataTable dt = data.GetCollaborationRequestsTable();

            DataTable additions = SQLOperations.Difference(dt, currentCollaborationRequests);
            DataTable subtractions = SQLOperations.Difference(currentCollaborationRequests, dt);

            foreach (DataRow row in additions.Rows)
            {
                Console.WriteLine("COLLABORATION REQUEST Addition: " + row["CollaborationRequestID"].ToString());

                data.InsertIntoShadowCollaborationRequestsTable(DateTime.Now, (long)row["CollaborationRequestID"], DateTime.Parse(row["requestTimeStamp"].ToString()), (long)row["sessionID"], (long)row["userID"], (long)row["artifactID"], row["comment"].ToString());
            }


            foreach (DataRow row in subtractions.Rows)
            {
                Console.WriteLine("COLLABORATION REQUEST Removal: " + row["CollaborationRequestID"].ToString());

                data.UpdateShadowCollaborationRequestsTable(DateTime.Now, (long)row["CollaborationRequestID"]);
            }

            currentCollaborationRequests = dt;
        }
    }
}
