using System;
using System.IO;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;

using ClubDev.Database;
using ClubDev.Database.ClubDevDataSetTableAdapters;

namespace ClubDev.CaptionButton
{
    public partial class CaptionButton : Form
    {
        private bool mouse_down;    // is the mouse down?
        public bool hidden;         // is the icon hidden?

        private int hwnd;

        
        private ClubDevDataSet clubDevDataSet;
        private SharedArtifactsTableAdapter sharedArtifactsTableAdapter;
        private bool isDatabaseInitialized;

        private string collabHost;
        private string collabName;

        //Don't call this one
        public CaptionButton()
        {
            InitializeComponent();
        }

        //Call this constructor in code
        public CaptionButton(int hwnd, Point location, string collabHost, string collabName)
        {
            InitializeComponent();
            this.hwnd = hwnd;
            this.Location = location;
            this.Show();
            this.MouseDown += new MouseEventHandler(Icon_MouseDown);
            this.MouseMove += new MouseEventHandler(Icon_MouseMove);
            this.MouseUp += new MouseEventHandler(Icon_MouseUp);
            this.hidden = false;

            this.collabHost = collabHost;
            this.collabName = collabName;

            this.collaborationState.ItemClicked += new ToolStripItemClickedEventHandler(contextMenuStrip1_ItemClicked);


        }

        void contextMenuStrip1_ItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {
            
            for (int i = 0; i < this.collaborationState.Items.Count; i++)
            {
                if (this.collaborationState.Items[i].Equals(e.ClickedItem))
                    ((ToolStripMenuItem)this.collaborationState.Items[i]).Checked = true;
                else
                    ((ToolStripMenuItem)this.collaborationState.Items[i]).Checked = false;
            }

            if (e.ClickedItem.Name.Equals("shareCollaboration"))
            {
                this.pictureBox1.Image = global::ClubDev.CaptionButton.CaptionButtonResources.captionButton_Share;
                this.UpdateSharedArtifact(true, true);
            }

            else if (e.ClickedItem.Name.Equals("showCollaboration"))
            {
                this.pictureBox1.Image = global::ClubDev.CaptionButton.CaptionButtonResources.captionButton_Show;
                this.UpdateSharedArtifact(true, false);
            }

            else if (e.ClickedItem.Name.Equals("noCollaboration"))
            {
                this.pictureBox1.Image = global::ClubDev.CaptionButton.CaptionButtonResources.captionButton_Off;
                this.UpdateSharedArtifact(false, false);
            }
        }


        // Handlers that allow the icon to be moved off of the title bar with the mouse
        void Icon_MouseUp(object sender, MouseEventArgs e)
        {
            mouse_down = false;


        }

        void Icon_MouseMove(object sender, MouseEventArgs e)
        {
            if (mouse_down)
            {
                this.Location = new Point(Cursor.Position.X-15, Cursor.Position.Y-15);
            }
        }

        void Icon_MouseDown(object sender, MouseEventArgs e)
        {
            mouse_down = true;
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

        }




        private void pictureBox1_Click(object sender, EventArgs e)
        {
            this.collaborationState.Show(pictureBox1, new Point(0, 20));
        }

        private void contextMenuStrip1_Opening(object sender, CancelEventArgs e)
        {

        }


        private void UpdateSharedArtifact(bool sharedAtAll, bool fullShare)
        {
            if (!isDatabaseInitialized)
            {
                clubDevDataSet = new ClubDevDataSet();
                sharedArtifactsTableAdapter = new SharedArtifactsTableAdapter();
                isDatabaseInitialized = true;
            }

            if (sharedAtAll)
            {

                sharedArtifactsTableAdapter.ClearBeforeFill = true;
                sharedArtifactsTableAdapter.FillMETAByHWNDandGroup(clubDevDataSet.SharedArtifacts, hwnd, this.collabHost);

                if (clubDevDataSet.SharedArtifacts.Count > 0)
                {
                    // do an update
                    Console.WriteLine("found " + hwnd.ToString());

                    sharedArtifactsTableAdapter.UpdateFullShareByID(fullShare, clubDevDataSet.SharedArtifacts[0].artifactID);
                    
                }

                else 
                {
                    // do an insert
                    Bitmap icon = ClubDev.Utilities.IconUtilities.GetWindowLargeIconAsBitmap(hwnd);

                    IntPtr hIconBitmap = icon.GetHbitmap();
                    System.Windows.Int32Rect iconRect = new System.Windows.Int32Rect(0, 0, icon.Width, icon.Height);
                    BitmapSource iconBitmapSource = System.Windows.Interop.Imaging.CreateBitmapSourceFromHBitmap(hIconBitmap, IntPtr.Zero, iconRect, BitmapSizeOptions.FromEmptyOptions());

                    PngBitmapEncoder iconBitmapEncoder = new PngBitmapEncoder();
                    iconBitmapEncoder.Frames.Add(BitmapFrame.Create(iconBitmapSource));
                    System.IO.MemoryStream iconStream = new MemoryStream();
                    iconBitmapEncoder.Save(iconStream);


                    StringBuilder title = new StringBuilder(256);
                    Utilities.Win32Api.GetWindowText(new IntPtr(hwnd), title, title.Capacity);


                    Utilities.Win32Api.Rect rc = new Utilities.Win32Api.Rect();
                    Utilities.Win32Api.GetWindowRect(new IntPtr(hwnd), ref rc);
                    

                    sharedArtifactsTableAdapter.InsertAllButImage(hwnd, fullShare, 1, title.ToString(), this.collabHost, this.collabName, iconStream.GetBuffer(), 
                        rc.left, rc.top, rc.Width, rc.Height);
                }

                ClubDev.Engines.ClientScreenShotEngine.SpawnCaptureScreenShotsWorker();
            }

            else
            {
                //delete record
                sharedArtifactsTableAdapter.DeleteByHWNDandGroup(hwnd, collabHost);
            }
            
        }
    }
}