using System;
using System.IO;
using System.Reflection;
using System.Data;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Shapes;
using System.Windows.Input;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using System.Windows.Media.Effects;
using System.Windows.Threading;
using System.Runtime.InteropServices;
using Microsoft.Win32;


namespace ClubDev
{
    class Collaborator : Canvas
    {
        private UIElement eventParent;
        private Canvas userImageCanvas;
        private TextBlock userLabelTextBlock;
        private TextBlock shareCountTextBlock;
        private TextBlock showCountTextBlock;
        private Image imStatusIconImage;
        private Image shareIconImage;
        private Image showIconImage;
        private Image collaboratorBubbleCreateImage;
        private Canvas summaryCanvas;
        private CollaboratorSummary collaboratorSummary;

        private BitmapSource messengerOnline = null;
        private BitmapSource messengerOffline = null;
        private BitmapSource messengerAway = null;
        private BitmapSource messengerBusy = null;

        private string collabHost;
        private string collabName;
        private string collabIM;
        private BitmapSource collabIcon;

        private System.Windows.Threading.DispatcherTimer fadeOutTimer;
        private System.Windows.Threading.DispatcherTimer driftGracePeriodTimer;
        private System.Windows.Threading.DispatcherTimer glowTimer;

        private bool expanded = false;
        private bool isDragInProcess = false;
        private bool isDropEventSet = false;

        private System.DateTime lastClickTime;
        private System.TimeSpan doubleClickInterval;

        //private static CollaboratorData collaboratorData;





        public Collaborator(Point location, string collabHost, string collabName, string collabIM, BitmapSource collabIcon)
        {
            NameScope.SetNameScope(this, new NameScope());

            Canvas.SetLeft(this, location.X);
            Canvas.SetTop(this, location.Y);

            this.Width = 1000;  //Should change to be automatically set based on item size
            this.Height = 220;
            
            this.collabHost = collabHost;
            this.collabName = collabName;
            this.collabIM = collabIM;
            this.collabIcon = collabIcon;

            collaboratorSummary = new CollaboratorSummary(new Point(location.X, location.Y), this.collabHost, this.collabName, this.collabIM);

            if (this.collabIcon == null)
            {
                try
                {
                    System.IO.StreamReader stream = new StreamReader(this.GetType().Assembly.GetManifestResourceStream("ClubDev.Icons.user.png"));
                    PngBitmapDecoder bitmapDecoder = new PngBitmapDecoder(stream.BaseStream, BitmapCreateOptions.PreservePixelFormat, BitmapCacheOption.Default);
                    collabIcon = bitmapDecoder.Frames[0];
                }
                catch {  }
            }

            Image userImage = new Image();
            userImage.Source = collabIcon;
            userImage.Width = 60;
            userImage.Height = collabIcon.Height * (userImage.Width / collabIcon.Width);

            Canvas.SetLeft(userImage, 3);
            Canvas.SetTop(userImage, 3);         


            Rectangle imageBorder = new Rectangle();
            imageBorder.Width = userImage.Width + 6;
            imageBorder.Height = userImage.Height + 6;
            imageBorder.Fill = new SolidColorBrush(Colors.WhiteSmoke);
            imageBorder.Stroke = new SolidColorBrush(Color.FromArgb(225, 80, 80, 80));
            imageBorder.StrokeThickness = 1.5;
            imageBorder.RadiusX = 1.5;
            imageBorder.RadiusY = 1.5;
            Canvas.SetLeft(imageBorder, 0);
            Canvas.SetTop(imageBorder, 0);

            userImageCanvas = new Canvas();
            userImageCanvas.Width = imageBorder.Width + 200;
            userImageCanvas.Height = imageBorder.Height;
            userImageCanvas.Children.Add(imageBorder);
            userImageCanvas.Children.Add(userImage);

            OuterGlowBitmapEffect glowEffect = new OuterGlowBitmapEffect();
            glowEffect.GlowSize = 20;
            glowEffect.GlowColor = Color.FromArgb(255, 255, 255, 0);
            glowEffect.Noise = 0;
            glowEffect.Opacity = 1;

            userImageCanvas.BitmapEffect = glowEffect;

            Canvas.SetLeft(userImageCanvas, location.X);
            Canvas.SetTop(userImageCanvas, (location.Y + (this.Height - userImage.Height) / 2) - 10);



            userLabelTextBlock = new TextBlock();
            userLabelTextBlock.Text = collabName;
            userLabelTextBlock.FontSize = 12;
            userLabelTextBlock.FontFamily = new FontFamily("Veranda");
            userLabelTextBlock.FontWeight = FontWeights.Bold;
            userLabelTextBlock.Foreground = new SolidColorBrush(Colors.WhiteSmoke);
            userLabelTextBlock.Name = "userLabel";
            this.RegisterName(userLabelTextBlock.Name, userLabelTextBlock);

            Canvas.SetTop(userLabelTextBlock, Canvas.GetTop(userImageCanvas) + userImageCanvas.Height + 5);

            if (Program.LiveMessenger != null && !(Program.CurrentUser.UserHost.Equals(this.collabHost)))
            {
                Canvas.SetLeft(userLabelTextBlock, 25);
                userLabelTextBlock.MouseLeftButtonDown +=new MouseButtonEventHandler(IMClickEventHandler);
            }
            else if (Program.CurrentUser.UserHost.Equals(this.collabHost))
            {
                Canvas.SetLeft(userLabelTextBlock, 25);
                userLabelTextBlock.MouseLeftButtonDown += new MouseButtonEventHandler(collaboratorBubbleCreateImageEventHandler);
            }
            else
                Canvas.SetLeft(userLabelTextBlock, 5);

            OuterGlowBitmapEffect labelGlowEffect = new OuterGlowBitmapEffect();
            labelGlowEffect.GlowSize = 10;
            labelGlowEffect.GlowColor = Color.FromArgb(255, 0, 0, 0);
            labelGlowEffect.Noise = 0;
            labelGlowEffect.Opacity = 1;

            userLabelTextBlock.BitmapEffect = labelGlowEffect;

            userImage.AllowDrop = true;




            shareCountTextBlock = new TextBlock();
            shareCountTextBlock.Text = "?";
            shareCountTextBlock.FontSize = 12;
            shareCountTextBlock.FontFamily = new FontFamily("Veranda");
            shareCountTextBlock.FontWeight = FontWeights.Normal;
            shareCountTextBlock.Foreground = new SolidColorBrush(Colors.WhiteSmoke);
            shareCountTextBlock.Name = "shareCount";
            shareCountTextBlock.BitmapEffect = labelGlowEffect;
            this.RegisterName(shareCountTextBlock.Name, shareCountTextBlock);

            Canvas.SetTop(shareCountTextBlock, Canvas.GetTop(userImageCanvas) + userImageCanvas.Height + 24);
            Canvas.SetLeft(shareCountTextBlock, 32);


            showCountTextBlock = new TextBlock();
            showCountTextBlock.Text = "?";
            showCountTextBlock.FontSize = 12;
            showCountTextBlock.FontFamily = new FontFamily("Veranda");
            showCountTextBlock.FontWeight = FontWeights.Normal;
            showCountTextBlock.Foreground = new SolidColorBrush(Colors.WhiteSmoke);
            showCountTextBlock.Name = "showCount";
            showCountTextBlock.BitmapEffect = labelGlowEffect;
            this.RegisterName(showCountTextBlock.Name, showCountTextBlock);

            Canvas.SetTop(showCountTextBlock, Canvas.GetTop(userImageCanvas) + userImageCanvas.Height + 41);
            Canvas.SetLeft(showCountTextBlock, 32);




            
            BitmapSource showIcon = null;
            BitmapSource shareIcon = null;
            try
            {
                Assembly assem = Assembly.GetExecutingAssembly();

                System.IO.StreamReader onlineStream = new StreamReader(assem.GetManifestResourceStream("ClubDev.Icons.collaboratorSummary-IM-Online.png"));
                PngBitmapDecoder onlineBitmapDecoder = new PngBitmapDecoder(onlineStream.BaseStream, BitmapCreateOptions.PreservePixelFormat, BitmapCacheOption.None);
                messengerOnline = onlineBitmapDecoder.Frames[0];

                System.IO.StreamReader offlineStream = new StreamReader(assem.GetManifestResourceStream("ClubDev.Icons.collaboratorSummary-IM-Offline.png"));
                PngBitmapDecoder offlineBitmapDecoder = new PngBitmapDecoder(offlineStream.BaseStream, BitmapCreateOptions.PreservePixelFormat, BitmapCacheOption.None);
                messengerOffline = offlineBitmapDecoder.Frames[0];

                System.IO.StreamReader awayStream = new StreamReader(assem.GetManifestResourceStream("ClubDev.Icons.collaboratorSummary-IM-Away.png"));
                PngBitmapDecoder awayBitmapDecoder = new PngBitmapDecoder(awayStream.BaseStream, BitmapCreateOptions.PreservePixelFormat, BitmapCacheOption.None);
                messengerAway = awayBitmapDecoder.Frames[0];

                System.IO.StreamReader busyStream = new StreamReader(assem.GetManifestResourceStream("ClubDev.Icons.collaboratorSummary-IM-Busy.png"));
                PngBitmapDecoder busyBitmapDecoder = new PngBitmapDecoder(busyStream.BaseStream, BitmapCreateOptions.PreservePixelFormat, BitmapCacheOption.None);
                messengerBusy = busyBitmapDecoder.Frames[0];

                System.IO.StreamReader showStream = new StreamReader(assem.GetManifestResourceStream("ClubDev.Icons.collaboratorSummary-Show.png"));
                PngBitmapDecoder showBitmapDecoder = new PngBitmapDecoder(showStream.BaseStream, BitmapCreateOptions.PreservePixelFormat, BitmapCacheOption.None);
                showIcon = showBitmapDecoder.Frames[0];

                System.IO.StreamReader shareStream = new StreamReader(assem.GetManifestResourceStream("ClubDev.Icons.collaboratorSummary-Share.png"));
                PngBitmapDecoder shareBitmapDecoder = new PngBitmapDecoder(shareStream.BaseStream, BitmapCreateOptions.PreservePixelFormat, BitmapCacheOption.None);
                shareIcon = shareBitmapDecoder.Frames[0];
            }
            catch
            {
                Console.WriteLine("Could not import embeded resources");
            }


            if (Program.LiveMessenger != null && !Program.CurrentUser.UserHost.Equals(this.collabHost))
            {
                
                imStatusIconImage = new Image();
                imStatusIconImage.Width = 20;
                imStatusIconImage.Height = 20;

                MessengerAPI.IMessengerContact contact = (MessengerAPI.IMessengerContact)Program.LiveMessenger.GetContact(collabIM, Program.LiveMessenger.MyServiceId);

                if (contact.Status == MessengerAPI.MISTATUS.MISTATUS_ONLINE)
                    imStatusIconImage.Source = messengerOnline;

                else if (contact.Status == MessengerAPI.MISTATUS.MISTATUS_IDLE ||
                        contact.Status == MessengerAPI.MISTATUS.MISTATUS_AWAY ||
                        contact.Status == MessengerAPI.MISTATUS.MISTATUS_BE_RIGHT_BACK ||
                        contact.Status == MessengerAPI.MISTATUS.MISTATUS_OUT_TO_LUNCH)
                    imStatusIconImage.Source = messengerAway;

                else if (contact.Status == MessengerAPI.MISTATUS.MISTATUS_BUSY ||
                        contact.Status == MessengerAPI.MISTATUS.MISTATUS_ON_THE_PHONE)
                    imStatusIconImage.Source = messengerBusy;

                else
                    imStatusIconImage.Source = messengerOffline;


                imStatusIconImage.BitmapEffect = labelGlowEffect;
                imStatusIconImage.Name = "imStatusIconImage";
                this.RegisterName(imStatusIconImage.Name, imStatusIconImage);
                Canvas.SetLeft(imStatusIconImage, 1);
                Canvas.SetTop(imStatusIconImage, Canvas.GetTop(userImageCanvas) + userImageCanvas.Height + 1);

                try
                {
                    RegistryKey mouseRegistryKey = Registry.CurrentUser.CreateSubKey("Control Panel\\Mouse");
                    doubleClickInterval = new TimeSpan(0, 0, 0, 0, int.Parse((string)mouseRegistryKey.GetValue("DoubleClickSpeed")));
                    mouseRegistryKey.Close();
                }
                catch
                {
                    doubleClickInterval = new TimeSpan(0, 0, 0, 0, 600);
                }
                
                
                imStatusIconImage.MouseLeftButtonDown +=new MouseButtonEventHandler(IMClickEventHandler);
                
            }

            else if (Program.CurrentUser.UserHost.Equals(this.collabHost))
            {
                collaboratorBubbleCreateImage = new Image();
                collaboratorBubbleCreateImage.Width = 20;
                collaboratorBubbleCreateImage.Height = 20;

                try
                {
                    Assembly assem = Assembly.GetExecutingAssembly();

                    System.IO.StreamReader megaphoneStream = new StreamReader(assem.GetManifestResourceStream("ClubDev.Icons.collaboratorBubble-Megaphone.png"));
                    PngBitmapDecoder megaphoneBitmapDecoder = new PngBitmapDecoder(megaphoneStream.BaseStream, BitmapCreateOptions.PreservePixelFormat, BitmapCacheOption.None);
                    collaboratorBubbleCreateImage.Source = megaphoneBitmapDecoder.Frames[0];
                }

                catch { }

                Canvas.SetLeft(collaboratorBubbleCreateImage, 1);
                Canvas.SetTop(collaboratorBubbleCreateImage, Canvas.GetTop(userImageCanvas) + userImageCanvas.Height + 1);
            
                try
                {
                    RegistryKey mouseRegistryKey = Registry.CurrentUser.CreateSubKey("Control Panel\\Mouse");
                    doubleClickInterval = new TimeSpan(0, 0, 0, 0, int.Parse((string)mouseRegistryKey.GetValue("DoubleClickSpeed")));
                    mouseRegistryKey.Close();
                }
                catch
                {
                    doubleClickInterval = new TimeSpan(0, 0, 0, 0, 600);
                }

                collaboratorBubbleCreateImage.MouseLeftButtonDown += new MouseButtonEventHandler(collaboratorBubbleCreateImageEventHandler);

            
            }

            showIconImage = new Image();
            showIconImage.Source = showIcon;
            showIconImage.BitmapEffect = labelGlowEffect;
            showIconImage.Name = "showIconImage";
            this.RegisterName(showIconImage.Name, showIconImage);
            Canvas.SetLeft(showIconImage, 6);
            Canvas.SetTop(showIconImage, Canvas.GetTop(userImageCanvas) + userImageCanvas.Height + 40);

            shareIconImage = new Image();
            shareIconImage.Source = shareIcon;
            shareIconImage.BitmapEffect = labelGlowEffect;
            shareIconImage.Name = "shareIconImage";
            this.RegisterName(shareIconImage.Name, shareIconImage);
            Canvas.SetLeft(shareIconImage, 3);
            Canvas.SetTop(shareIconImage, Canvas.GetTop(userImageCanvas) + userImageCanvas.Height + 23);

            summaryCanvas = new Canvas();
            summaryCanvas.Name = "summaryCanvas";
            this.RegisterName(summaryCanvas.Name, summaryCanvas);

            summaryCanvas.Children.Add(userLabelTextBlock);
            summaryCanvas.Children.Add(shareCountTextBlock);
            summaryCanvas.Children.Add(showCountTextBlock);
            summaryCanvas.Children.Add(shareIconImage);
            summaryCanvas.Children.Add(showIconImage);

            if (imStatusIconImage != null)
                summaryCanvas.Children.Add(imStatusIconImage);

            if (collaboratorBubbleCreateImage != null)
                summaryCanvas.Children.Add(collaboratorBubbleCreateImage);


            

            this.Children.Add(collaboratorSummary);
            this.Children.Add(userImageCanvas);
            this.Children.Add(summaryCanvas);
           


            SetupTimers();


            //collaboratorData = new CollaboratorData();
            //collaboratorData.OnNewSharedArtifactMessage += new CollaboratorData.NewMessage(OnNewSharedArtifactMessage);

            //OnNewSharedArtifactMessage();
         
        }



        public void InitializeEvents(UIElement eventParent)
        {
            this.eventParent = eventParent;

            userImageCanvas.MouseEnter += new System.Windows.Input.MouseEventHandler(UserImageMouseEnterEventHandler);
            this.MouseEnter += new System.Windows.Input.MouseEventHandler(MouseEnterEventHandler);
            this.MouseLeave += new System.Windows.Input.MouseEventHandler(MouseLeaveEventHandler);
            this.MouseDown += new MouseButtonEventHandler(MouseDownEventHandler);
            this.MouseUp += new MouseButtonEventHandler(MouseUpEventHandler);

            userImageCanvas.DragOver += new DragEventHandler(UserImageDragEventHandler);
            this.DragLeave += new DragEventHandler(DragLeaveEventHandler);
            this.DragEnter += new DragEventHandler(DragEnterEventHandler);

            collaboratorSummary.InitializeEvents(eventParent);
        }



        public void UpdateLocation(double X, double Y)
        {
            Canvas.SetLeft(this, X);
            Canvas.SetTop(this, Y);
        }


        public void UpdateShareShowCount(int shareCount, int showCount)
        {
            this.shareCountTextBlock.Text = shareCount.ToString();
            this.showCountTextBlock.Text = showCount.ToString();
        }


        private void OnDragPanel(object o, RoutedEventArgs args)
        {

            this.ExpandCollaborator();

        }



        void userImage_Drop(object sender, DragEventArgs e)
        {
            //throw new Exception("The method or operation is not implemented.");
        }



        private void SetupTimers()
        {
            fadeOutTimer = new System.Windows.Threading.DispatcherTimer();
            fadeOutTimer.Interval = TimeSpan.FromMilliseconds(250);
            fadeOutTimer.Tick += new EventHandler(FadeOutTimerEventHandler);

            driftGracePeriodTimer = new System.Windows.Threading.DispatcherTimer();
            driftGracePeriodTimer.Interval = TimeSpan.FromMilliseconds(500);
            driftGracePeriodTimer.Tick += new EventHandler(DriftGracePeriodTimerEventHandler);

            glowTimer = new System.Windows.Threading.DispatcherTimer();
            glowTimer.Interval = TimeSpan.FromMinutes(1);
            glowTimer.Tick += new EventHandler(GlowTimerEventHander);
            glowTimer.IsEnabled = true;
            glowTimer.Start();
        }


        void MouseUpEventHandler(object sender, MouseButtonEventArgs e)
        {
            isDragInProcess = false;
            Console.WriteLine("MOUSE UP EVENT");
        }

        void MouseDownEventHandler(object sender, MouseButtonEventArgs e)
        {
            isDragInProcess = true;
            Console.WriteLine("MOUSE Down EVENT");

        }

        void UserImageMouseEnterEventHandler(object sender, System.Windows.Input.MouseEventArgs e)
        {
            if(!expanded)
                this.ExpandCollaborator();

            //cleanup glow
            userImageCanvas.BitmapEffect = null;
        }

        void UserImageDragEventHandler(object sender, DragEventArgs e)
        {
            if(!expanded)
                this.ExpandCollaborator();
        }


        void MouseEnterEventHandler(object sender, System.Windows.Input.MouseEventArgs e)
        {
            

            if (driftGracePeriodTimer.IsEnabled)
            {
                driftGracePeriodTimer.Stop();
                driftGracePeriodTimer.IsEnabled = false;

                Console.WriteLine("TIMER DELAYED");
            }
        }



        void DragEnterEventHandler(object sender, DragEventArgs e)
        {
            if (driftGracePeriodTimer.IsEnabled)
            {
                driftGracePeriodTimer.Stop();
                driftGracePeriodTimer.IsEnabled = false;

                Console.WriteLine("TIMER DELAYED");
            }
        }

        void MouseLeaveEventHandler(object sender, System.Windows.Input.MouseEventArgs e)
        {
            if (expanded && !isDragInProcess)
            {
                if (!driftGracePeriodTimer.IsEnabled)
                {
                    driftGracePeriodTimer.Interval = TimeSpan.FromMilliseconds(500);

                    driftGracePeriodTimer.IsEnabled = true;
                    driftGracePeriodTimer.Start();

                    Console.WriteLine("MOUSE OUT");
                }
            }
            else
            {
                Console.WriteLine("DENY MOUSE OUT");
            }
        }


        void DragLeaveEventHandler(object sender, DragEventArgs e)
        {
            if (expanded && !driftGracePeriodTimer.IsEnabled)
            {
                if (isDragInProcess)
                {
                    if (!isDropEventSet)
                    {
                        eventParent.Drop += new DragEventHandler(DropEventHandler);
                        isDropEventSet = true;
                        Console.WriteLine("DRAG MOUSE EVENT SET");
                    }

                }
                else
                {
                    driftGracePeriodTimer.Interval = TimeSpan.FromMilliseconds(500);

                    driftGracePeriodTimer.IsEnabled = true;
                    driftGracePeriodTimer.Start();

                    Console.WriteLine("DRAG MOUSE OUT");
                }
                
            }
        }

        void IMClickEventHandler(object sender, System.Windows.RoutedEventArgs e)
        {
            System.DateTime currentTime = System.DateTime.Now;


            if ((lastClickTime != null) && (currentTime - lastClickTime < doubleClickInterval))
            {

                Console.WriteLine("double click");

                try
                {

                    MessengerAPI.IMessengerContact contact = (MessengerAPI.IMessengerContact)Program.LiveMessenger.GetContact(this.collabIM, Program.LiveMessenger.MyServiceId);

                    Program.LiveMessenger.InstantMessage(contact);
                }
                catch
                {
                    MessageBox.Show("Unable to initate IM with this person");
                }


            }
        
            lastClickTime = currentTime;

        }

        void collaboratorBubbleCreateImageEventHandler(object sender, MouseButtonEventArgs e)
        {
            System.DateTime currentTime = System.DateTime.Now;


            if ((lastClickTime != null) && (currentTime - lastClickTime < doubleClickInterval))
            {

                Console.WriteLine("double click");

                try
                {
                    CollaboratorBubbleCreate window = new CollaboratorBubbleCreate();
                    window.Show();
                   
                }
                catch
                {
                    MessageBox.Show("Unable to initate bubble create dialog");
                }


            }

            lastClickTime = currentTime;
        }


        void DropEventHandler(object sender, DragEventArgs e)
        {
            Rect currentCollaboratorBounds = VisualTreeHelper.GetDescendantBounds(this);
            Point currentMousePosition = Utilities.MouseUtilities.GetMousePosition(this);

            if (expanded && !currentCollaboratorBounds.Contains(currentMousePosition))
                this.CollapseCollaborator();

            eventParent.Drop -= DropEventHandler;
            isDropEventSet = false;
            isDragInProcess = false;
        }


        void FadeOutTimerEventHandler(object sender, EventArgs e)
        {
            fadeOutTimer.Stop();
            fadeOutTimer.IsEnabled = false;
            expanded = false;
        }

        private void DriftGracePeriodTimerEventHandler(object sender, EventArgs e)
        {
            Console.WriteLine("GRACE OVER");

            this.CollapseCollaborator();
        }

        void GlowTimerEventHander(object sender, EventArgs e)
        {
            glowTimer.Stop();
            userImageCanvas.BitmapEffect = null;
        }

        
        public void UpdateLiveMessengerStatus(string serviceName, MessengerAPI.MISTATUS status)
        {
            if (serviceName == this.collabIM)
            {
                Console.WriteLine("need to update " + this.collabIM + " to status " + status);

                if (Program.LiveMessenger != null)
                {


                    if (status == MessengerAPI.MISTATUS.MISTATUS_ONLINE)
                        imStatusIconImage.Source = messengerOnline;

                    else if (status == MessengerAPI.MISTATUS.MISTATUS_IDLE ||
                            status == MessengerAPI.MISTATUS.MISTATUS_AWAY ||
                            status == MessengerAPI.MISTATUS.MISTATUS_BE_RIGHT_BACK ||
                            status == MessengerAPI.MISTATUS.MISTATUS_OUT_TO_LUNCH)
                        imStatusIconImage.Source = messengerAway;

                    else if (status == MessengerAPI.MISTATUS.MISTATUS_BUSY ||
                       status == MessengerAPI.MISTATUS.MISTATUS_ON_THE_PHONE)
                        imStatusIconImage.Source = messengerBusy;

                    else
                        imStatusIconImage.Source = messengerOffline;


                    if (this.collaboratorSummary != null)
                        this.collaboratorSummary.UpdateLiveMessengerStatus(serviceName, status);
                }

            }

        }
        

        private void ExpandCollaborator()
        {

            if (!expanded)
            {
                expanded = true;

                DoubleAnimation transformAnimation = new DoubleAnimation();
                transformAnimation.From = 0;
                transformAnimation.To = 20;
                transformAnimation.Duration = new Duration(TimeSpan.FromMilliseconds(250));
                TranslateTransform animationTranslateTransform = new TranslateTransform();

                DoubleAnimation scaleTransformAnimation = new DoubleAnimation();
                scaleTransformAnimation.From = 1.0;
                scaleTransformAnimation.To = 1.25;
                scaleTransformAnimation.Duration = new Duration(TimeSpan.FromMilliseconds(250));
                ScaleTransform animationScaleTransform = new ScaleTransform();
                animationScaleTransform.CenterX = this.userImageCanvas.Width / 2;
                animationScaleTransform.CenterY = this.userImageCanvas.Height / 2;

                TransformGroup myTransformGroup = new TransformGroup();
                myTransformGroup.Children.Add(animationTranslateTransform);
                myTransformGroup.Children.Add(animationScaleTransform);
                userImageCanvas.RenderTransform = myTransformGroup;


                DoubleAnimation fadeAnimation = new DoubleAnimation();
                fadeAnimation.From = 1;
                fadeAnimation.To = 0;
                fadeAnimation.Duration = new Duration(TimeSpan.FromMilliseconds(250));


                Storyboard fadeStoryboard = new Storyboard();
                fadeStoryboard.Children.Add(fadeAnimation);
                Storyboard.SetTargetName(fadeAnimation, this.summaryCanvas.Name);
                Storyboard.SetTargetProperty(fadeAnimation, new PropertyPath(System.Windows.Shapes.Path.OpacityProperty));

                
                animationTranslateTransform.BeginAnimation(TranslateTransform.XProperty, transformAnimation);
                animationScaleTransform.BeginAnimation(ScaleTransform.ScaleXProperty, scaleTransformAnimation);
                animationScaleTransform.BeginAnimation(ScaleTransform.ScaleYProperty, scaleTransformAnimation);
                fadeStoryboard.Begin(this);

                collaboratorSummary.FadeIn();


                if (Program.ShadowLogging)
                {
                    SupportProviders.ShadowLoggingData shadowLoggingData = new SupportProviders.ShadowLoggingData();

                    shadowLoggingData.InsertIntoShadowEventCollaboratorBarRolloverTable(DateTime.Now, Program.CurrentUser.UserHost, this.collabHost);
                }

            }

        }

        public void CollapseCollaborator()
        {
            if (expanded)
            {
                driftGracePeriodTimer.Stop();
                driftGracePeriodTimer.IsEnabled = false;

                DoubleAnimation transformAnimation = new DoubleAnimation();
                transformAnimation.From = 20;
                transformAnimation.To = 0;
                transformAnimation.Duration = new Duration(TimeSpan.FromMilliseconds(1050));
                TranslateTransform animationTranslateTransform = new TranslateTransform();

                DoubleAnimation scaleTransformAnimation = new DoubleAnimation();
                scaleTransformAnimation.From = 1.25;
                scaleTransformAnimation.To = 1.0;
                scaleTransformAnimation.Duration = new Duration(TimeSpan.FromMilliseconds(1050));
                ScaleTransform animationScaleTransform = new ScaleTransform();
                animationScaleTransform.CenterX = this.userImageCanvas.Width / 2;
                animationScaleTransform.CenterY = this.userImageCanvas.Height / 2;

                TransformGroup myTransformGroup = new TransformGroup();
                myTransformGroup.Children.Add(animationScaleTransform);
                myTransformGroup.Children.Add(animationTranslateTransform);
                userImageCanvas.RenderTransform = myTransformGroup;

                DoubleAnimation fadeAnimation = new DoubleAnimation();
                fadeAnimation.From = 0;
                fadeAnimation.To = 1;
                fadeAnimation.Duration = new Duration(TimeSpan.FromMilliseconds(1050));
                fadeAnimation.BeginTime = TimeSpan.FromMilliseconds(500);

                Storyboard fadeStoryboard = new Storyboard();
                fadeStoryboard.Children.Add(fadeAnimation);
                Storyboard.SetTargetName(fadeAnimation, this.summaryCanvas.Name);
                Storyboard.SetTargetProperty(fadeAnimation, new PropertyPath(System.Windows.Shapes.Path.OpacityProperty));


                animationScaleTransform.BeginAnimation(ScaleTransform.ScaleXProperty, scaleTransformAnimation);
                animationScaleTransform.BeginAnimation(ScaleTransform.ScaleYProperty, scaleTransformAnimation);
                animationTranslateTransform.BeginAnimation(TranslateTransform.XProperty, transformAnimation);
                fadeStoryboard.Begin(this);

                collaboratorSummary.FadeOut();

                fadeOutTimer.IsEnabled = true;
                fadeOutTimer.Start();

                expanded = false;
                isDragInProcess = false;

            }
        }


    }
}
