using System;
using System.IO;
using System.Net;
using System.Data;
using System.Reflection;
using System.Collections;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Media;
using System.Windows.Shapes;
using System.Windows.Media.Imaging;
using System.Windows.Media.Animation;
using System.Windows.Navigation;
using System.Windows.Forms;
using System.Windows.Threading;
using System.Runtime.InteropServices;


namespace ClubDev
{
    class CollaboratorBar : Canvas
    {

        public Grid collaboratorGrid;

        private ScrollButton topScrollButton;
        private ScrollButton bottomScrollButton;

        private UIElement eventParent;

        private int currentGridOffset = 0;
        private int maxNumberInGrid = 0;
        private ArrayList connectedCollaborators;


        public CollaboratorBar(UIElement eventParent)
        {
            this.eventParent = eventParent;

            Controls.CollaboratorDock collaboratorDock = new Controls.CollaboratorDock(65);
            collaboratorDock.Show();


            GradientStopCollection collection = new GradientStopCollection();
            collection.Add(new GradientStop(Color.FromArgb(200, 0, 0, 0), 0.0));
            collection.Add(new GradientStop(Color.FromArgb(0, 0, 0, 0), 1.0));
            LinearGradientBrush fillBrush = new LinearGradientBrush(collection, 0);
            fillBrush.SpreadMethod = GradientSpreadMethod.Pad;

            Canvas collaboratorAnchorShadow = new Canvas();
            collaboratorAnchorShadow.Width = 75;
            collaboratorAnchorShadow.Height = System.Windows.Forms.Screen.PrimaryScreen.WorkingArea.Height;
            Canvas.SetTop(collaboratorAnchorShadow, 0);
            Canvas.SetLeft(collaboratorAnchorShadow, 0);
            collaboratorAnchorShadow.Background = fillBrush;


            this.Children.Add(collaboratorAnchorShadow);


            TextBlock titleTextBlock = new TextBlock();
            titleTextBlock.RenderTransform = new RotateTransform(-90);
            titleTextBlock.Text = "ClubDev";
            titleTextBlock.FontSize = 28;
            titleTextBlock.Foreground = new SolidColorBrush(Colors.WhiteSmoke);
            titleTextBlock.FontFamily = new FontFamily("Veranda");
            titleTextBlock.FontWeight = FontWeights.Bold;
            Canvas.SetLeft(titleTextBlock, 8);
            Canvas.SetTop(titleTextBlock, 115);

            this.Children.Add(titleTextBlock);




            topScrollButton = new ScrollButton(new Point(30, 125));
            topScrollButton.RenderTransform = new RotateTransform(90, topScrollButton.Width / 2, topScrollButton.Width / 2);
            topScrollButton.Enabled = false;
            topScrollButton.Visibility = Visibility.Hidden;
            topScrollButton.Click += new ScrollButton.ScrollButtonClickHandler(TopScrollButtonEventHandler);
            this.Children.Add(topScrollButton);

            bottomScrollButton = new ScrollButton(new Point(30, System.Windows.Forms.Screen.PrimaryScreen.WorkingArea.Height - 10));
            bottomScrollButton.RenderTransform = new RotateTransform(270, bottomScrollButton.Width / 2, bottomScrollButton.Width / 2);
            bottomScrollButton.Enabled = false;
            bottomScrollButton.Visibility = Visibility.Hidden;
            bottomScrollButton.Click += new ScrollButton.ScrollButtonClickHandler(BottomScrollButtonEventHandler);
            this.Children.Add(bottomScrollButton);





            maxNumberInGrid = (int)Math.Floor(((double)(System.Windows.Forms.Screen.PrimaryScreen.WorkingArea.Height - 155)) / ((double)220));
            
            collaboratorGrid = new Grid();
            collaboratorGrid.Width = 50;
            collaboratorGrid.Height = 0;
            collaboratorGrid.VerticalAlignment = VerticalAlignment.Top;
            collaboratorGrid.HorizontalAlignment = System.Windows.HorizontalAlignment.Left;
            collaboratorGrid.ShowGridLines = false;

            ColumnDefinition colDef1 = new ColumnDefinition();
            collaboratorGrid.ColumnDefinitions.Add(colDef1);
            Canvas.SetLeft(collaboratorGrid, 0);
            Canvas.SetTop(collaboratorGrid, 135);

            for (int i = 0; i < maxNumberInGrid; i++)
            {
                RowDefinition rowDef1 = new RowDefinition();
                collaboratorGrid.RowDefinitions.Add(rowDef1);
                collaboratorGrid.Height += 220;
            }

            this.Children.Add(collaboratorGrid);





            connectedCollaborators = new ArrayList();

        }



        public void AddCollaborator(Surogates.UserSurogate userSurogate)
        {

            Collaborator currentCollab = new Collaborator(new Point(0, 0), userSurogate.UserHost, userSurogate.UserName, userSurogate.UserIM, userSurogate.UserIcon);
            currentCollab.Name = "user" + userSurogate.UserID;
            currentCollab.InitializeEvents(this.eventParent);

            connectedCollaborators.Add(currentCollab);

            UpdateCollaboratorBarView();

            /*
             numCollaborators++;

                // check to see if enough collaborators slots
                while (numCollaborators > collaboratorGrid.RowDefinitions.Count)
                {
                    RowDefinition rowDef1 = new RowDefinition();
                    collaboratorGrid.RowDefinitions.Add(rowDef1);
                    collaboratorGrid.Height += 220;
                }

                Collaborator currentCollab = new Collaborator(new Point(0, 0), userSurogate.UserHost, userSurogate.UserName, userSurogate.UserIM, userSurogate.UserIcon);
                currentCollab.Name = "user" + userSurogate.UserID;

                Grid.SetRow(currentCollab, numCollaborators - 1);
                Grid.SetColumn(currentCollab, 0);
                collaboratorGrid.Children.Add(currentCollab);

                currentCollab.InitializeEvents(this.eventParent);
            */
                
        }


        public void RemoveCollaborator(string collabName)
        {
            for (int i = 0; i < connectedCollaborators.Count; i++)
                if (((Collaborator)connectedCollaborators[i]).Name.Equals(collabName))
                {
                    for(int j = 0; j < collaboratorGrid.Children.Count; j++)
                        if(collaboratorGrid.Children[j].Equals((Collaborator)connectedCollaborators[i]))
                            collaboratorGrid.Children.Remove((Collaborator)connectedCollaborators[i]);
                    
                    connectedCollaborators.RemoveAt(i);
                }

            UpdateCollaboratorBarView();
            /*
            int removedRow = -1;
            for (int i = 0; i < collaboratorGrid.Children.Count; i++)
            {
                if (((FrameworkElement)collaboratorGrid.Children[i]).Name == collabName)
                {

                    removedRow = Grid.GetRow(collaboratorGrid.Children[i]);
                    collaboratorGrid.Children.RemoveAt(i);

                    break;
                }
            }

            if (removedRow < 0)
                return;

            for (int i = 0; i < collaboratorGrid.Children.Count; i++)
            {
                if (Grid.GetRow(collaboratorGrid.Children[i]) > removedRow)
                {
                    int currentRow = Grid.GetRow(collaboratorGrid.Children[i]);
                    Grid.SetRow(collaboratorGrid.Children[i], currentRow - 1);
                }
            }

            numCollaborators--;
            */
            
        }


        private void UpdateCollaboratorBarView()
        {

            //for (int i = 0; i < collaboratorGrid.Children.Count; i++)
            //    collaboratorGrid.Children.RemoveAt(i);
            collaboratorGrid.Children.RemoveRange(0, collaboratorGrid.Children.Count);

            int endPosition = currentGridOffset + maxNumberInGrid;
            if (connectedCollaborators.Count < endPosition)
                endPosition = connectedCollaborators.Count;

            int gridPosition = 0;
            for (int i = currentGridOffset; i < endPosition; i++)
            {
                Grid.SetRow((Collaborator)connectedCollaborators[i], gridPosition);
                Grid.SetColumn((Collaborator)connectedCollaborators[i], 0);
                collaboratorGrid.Children.Add((Collaborator)connectedCollaborators[i]);


                gridPosition++;

            }

            if (connectedCollaborators.Count > maxNumberInGrid)
            {
                topScrollButton.Visibility = Visibility.Visible;
                bottomScrollButton.Visibility = Visibility.Visible;
            }

            if (currentGridOffset != 0)
                topScrollButton.Enabled = true;
            else
                topScrollButton.Enabled = false;

            if ((currentGridOffset + maxNumberInGrid) < connectedCollaborators.Count)
                bottomScrollButton.Enabled = true;
            else
                bottomScrollButton.Enabled = false;

        }


        private void TopScrollButtonEventHandler(object sender, ScrollButtonEventArgs e)
        {
            currentGridOffset--;

            if (currentGridOffset < 0)
                currentGridOffset = 0;

            UpdateCollaboratorBarView();
        }

        private void BottomScrollButtonEventHandler(object sender, ScrollButtonEventArgs e)
        {
            currentGridOffset++;

            if (currentGridOffset > (connectedCollaborators.Count - 1))
                currentGridOffset = (connectedCollaborators.Count - 1);

            UpdateCollaboratorBarView();
        }


        public void UpdateLiveMessengerStatus(string serviceName, MessengerAPI.MISTATUS status)
        {

            for(int i = 0; i < collaboratorGrid.Children.Count; i++)
            {
                if (collaboratorGrid.Children[i].GetType() == typeof(Collaborator))
                {
                    Collaborator currentCollaborator = (Collaborator)collaboratorGrid.Children[i];


                    currentCollaborator.UpdateLiveMessengerStatus(serviceName, status);
                }
            }
        }
        

    }
}
