using System;
using System.IO;
using System.Reflection;
using System.Data;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Shapes;
using System.Windows.Input;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using System.Windows.Media.Effects;
using System.Windows.Threading;
using System.Runtime.InteropServices;
using Microsoft.Win32;

using ClubDev.Database;
using ClubDev.Database.ClubDevDataSetTableAdapters;



namespace ClubDev
{
    class CollaboratorBubble : Canvas
    {
        private TextBlock messageTextBlock;
        private TextBlock closeX;
        private Rectangle background;
        //private Button acceptReplication;
        //private Button startIM;
        //private Button ignore;

        private Label acceptReplication;
        private Label ignore;

        private Image megaphoneImage;

        private System.Windows.Forms.Timer noAttentionTimer;
        private System.Windows.Forms.Timer afterFadeTimer;

        private Storyboard fadeInStoryboard;
        private Storyboard fadeOutStoryboard;

        private static ClubDevDataSet clubDevDataSet;
        private static UsersTableAdapter usersTableAdapter;
        private static SharedArtifactsTableAdapter sharedArtifactsTableAdapter;
        private static ReplicationsTableAdapter replicationsTableAdapter;

        private long sessionID;
        private long userID;
        private long artifactID;
        private string comment;




        public CollaboratorBubble(Point location, long sessionID, long userID, long artifactID, string comment)
        {
            this.sessionID = sessionID;
            this.userID = userID;
            this.artifactID = artifactID;
            this.comment = comment;

            Canvas.SetLeft(this, location.X );
            Canvas.SetTop(this, location.Y);


            NameScope.SetNameScope(this, new NameScope());
            this.Name = "CollaboratorBubble";
            this.RegisterName(this.Name, this);

            clubDevDataSet = new ClubDevDataSet();
            usersTableAdapter = new UsersTableAdapter();
            sharedArtifactsTableAdapter = new SharedArtifactsTableAdapter();
            replicationsTableAdapter = new ReplicationsTableAdapter();


            usersTableAdapter.ClearBeforeFill = true;
            usersTableAdapter.FillByUserID(clubDevDataSet.Users, userID);

            sharedArtifactsTableAdapter.ClearBeforeFill = true;
            sharedArtifactsTableAdapter.FillMETAByID(clubDevDataSet.SharedArtifacts, artifactID);


            noAttentionTimer = new System.Windows.Forms.Timer();
            noAttentionTimer.Tick += new EventHandler(noAttentionTimer_Tick);
            noAttentionTimer.Interval = 30000;
            noAttentionTimer.Start();


            background = new Rectangle();

            GradientStopCollection collection = new GradientStopCollection();
            collection.Add(new GradientStop(Color.FromArgb(255, 245, 245, 245), 0.0));
            collection.Add(new GradientStop(Color.FromArgb(255, 150, 150, 150), 1.0));
            LinearGradientBrush fillBrush = new LinearGradientBrush(collection, 85);
            fillBrush.SpreadMethod = GradientSpreadMethod.Pad;

            background.Fill = fillBrush;
            background.Stroke = new SolidColorBrush(Color.FromArgb(255, 110, 110, 110));
            background.StrokeThickness = 1.5;
            background.RadiusX = 8;
            background.RadiusY = 8;

            DropShadowBitmapEffect myDropShadowEffect = new DropShadowBitmapEffect();
            myDropShadowEffect.Color = Colors.Black;
            myDropShadowEffect.Softness = 1;
            myDropShadowEffect.ShadowDepth = 5;

            background.BitmapEffect = myDropShadowEffect;


            messageTextBlock = new TextBlock();
            messageTextBlock.MaxWidth = 350;
            messageTextBlock.Text = clubDevDataSet.Users[0].userName + " says: " + comment;
            messageTextBlock.FontSize = 13;
            messageTextBlock.FontStyle = FontStyles.Oblique;
            messageTextBlock.TextWrapping = TextWrapping.WrapWithOverflow;
            Canvas.SetLeft(messageTextBlock, 8 + 40);
            Canvas.SetTop(messageTextBlock, 16);




            try
            {

                Assembly assem = Assembly.GetExecutingAssembly();

                System.IO.StreamReader megaphoneStream = new StreamReader(assem.GetManifestResourceStream("ClubDev.Icons.collaboratorBubble-Megaphone.png"));
                PngBitmapDecoder megaphoneBitmapDecoder = new PngBitmapDecoder(megaphoneStream.BaseStream, BitmapCreateOptions.PreservePixelFormat, BitmapCacheOption.None);
                megaphoneImage = new Image();
                megaphoneImage.Source = megaphoneBitmapDecoder.Frames[0];

                megaphoneImage.Width = 32;
                megaphoneImage.Height = 32;

                Canvas.SetLeft(megaphoneImage, 5);
                Canvas.SetTop(megaphoneImage, 10);

            }

            catch { }

            closeX = new TextBlock();
            closeX.Text = " X ";
            closeX.FontSize = 14;
            closeX.FontWeight = FontWeights.UltraBold;
            closeX.Foreground = new SolidColorBrush(Color.FromRgb(100, 100, 100));
            closeX.Background = new SolidColorBrush(Color.FromArgb(0, 110, 110, 110));
            closeX.MouseEnter += new MouseEventHandler(closeX_MouseEnter);
            closeX.MouseLeave += new MouseEventHandler(closeX_MouseLeave);
            closeX.MouseUp += new MouseButtonEventHandler(closeX_MouseUp);


            /*
            acceptReplication = new Button();
            acceptReplication.Content = "Establish Replication";
            if (artifactID > 0)
                acceptReplication.Click += new RoutedEventHandler(acceptReplicationClickEventHandler);
            else
                acceptReplication.IsEnabled = false;
            */

            acceptReplication = new Label();
            acceptReplication.Content = "Establish Replication";
            acceptReplication.Background = new SolidColorBrush(Color.FromRgb(220, 220, 220));
            acceptReplication.BorderBrush = new SolidColorBrush(Colors.Black);
            acceptReplication.BorderThickness = new Thickness(1);
            if (artifactID > 0)
            {
                acceptReplication.MouseUp += new MouseButtonEventHandler(acceptReplicationClickEventHandler);
                acceptReplication.Foreground = new SolidColorBrush(Colors.Black);
            }
            else
                acceptReplication.Foreground = new SolidColorBrush(Color.FromRgb(160, 160, 160));


            /*
            startIM = new Button();
            startIM.Content = "Contact via IM";
            if (Program.LiveMessenger != null)
                Console.WriteLine("add IM handler support");
            else
                startIM.IsEnabled = false;
            */

            /*
            ignore = new Button();
            ignore.Content = "Ignore";
            ignore.Click += new RoutedEventHandler(ignoreClickEventHandler);
            */

            ignore = new Label();
            ignore.Content = "Ignore";
            ignore.Foreground = new SolidColorBrush(Colors.Black);
            ignore.Background = new SolidColorBrush(Color.FromRgb(220, 220, 220));
            ignore.BorderBrush = new SolidColorBrush(Colors.Black);
            ignore.BorderThickness = new Thickness(1);
            ignore.MouseUp +=new MouseButtonEventHandler(ignoreClickEventHandler);

           
            
            this.Children.Add(background);
            this.Children.Add(closeX);
            this.Children.Add(messageTextBlock);
            this.Children.Add(acceptReplication);
            //this.Children.Add(startIM);
            this.Children.Add(ignore);

            if (megaphoneImage != null)
                this.Children.Add(megaphoneImage);


            DoubleAnimation fadeInDoubleAnimation = new DoubleAnimation();
            fadeInDoubleAnimation.From = 0.0;
            fadeInDoubleAnimation.To = 1.0;
            fadeInDoubleAnimation.Duration = new Duration(TimeSpan.FromMilliseconds(500));
            fadeInDoubleAnimation.AutoReverse = false;

            fadeInStoryboard = new Storyboard();
            fadeInStoryboard.Children.Add(fadeInDoubleAnimation);
            Storyboard.SetTargetName(fadeInDoubleAnimation, this.Name);
            Storyboard.SetTargetProperty(fadeInDoubleAnimation, new PropertyPath(Rectangle.OpacityProperty));



            DoubleAnimation fadeOutDoubleAnimation = new DoubleAnimation();
            fadeOutDoubleAnimation.From = 1.0;
            fadeOutDoubleAnimation.To = 0.0;
            fadeOutDoubleAnimation.Duration = new Duration(TimeSpan.FromMilliseconds(2000));
            fadeOutDoubleAnimation.AutoReverse = false;

            fadeOutStoryboard = new Storyboard();
            fadeOutStoryboard.Children.Add(fadeOutDoubleAnimation);
            Storyboard.SetTargetName(fadeOutDoubleAnimation, this.Name);
            Storyboard.SetTargetProperty(fadeOutDoubleAnimation, new PropertyPath(Rectangle.OpacityProperty));




            // Use the Loaded event to start the Storyboard.
            this.Loaded += new RoutedEventHandler(myRectangleLoaded);

            this.LayoutUpdated += new EventHandler(CollaboratorBubble_LayoutUpdated);




            
        }





        public void Close()
        {
            this.Visibility = Visibility.Hidden;

            if (this.Parent != null)
                ((Canvas)(this.Parent)).Children.Remove(this);
        }




        void noAttentionTimer_Tick(object sender, EventArgs e)
        {
            fadeOutStoryboard.Begin(this);

            afterFadeTimer = new System.Windows.Forms.Timer();
            afterFadeTimer.Tick += new EventHandler(afterFadeTimer_Tick);
            afterFadeTimer.Interval = 2000;
            afterFadeTimer.Start();
        }

        void afterFadeTimer_Tick(object sender, EventArgs e)
        {
            this.Close();
        }



        void closeX_MouseEnter(object sender, MouseEventArgs e)
        {
            closeX.Foreground = new SolidColorBrush(Color.FromRgb(235, 170, 140));
        }

        void closeX_MouseLeave(object sender, MouseEventArgs e)
        {
            closeX.Foreground = new SolidColorBrush(Color.FromRgb(100, 100, 100));
        }

        void closeX_MouseUp(object sender, MouseButtonEventArgs e)
        {
            this.Close();
        }

        private void acceptReplicationClickEventHandler(object sender, RoutedEventArgs e)
        {
            if (clubDevDataSet.Users.Rows.Count == 1)
            {
                string sourceHost = clubDevDataSet.Users.Rows[0]["userHost"].ToString();

                if (clubDevDataSet.SharedArtifacts.Rows.Count == 1)
                {
                    int sourceHWND = int.Parse(clubDevDataSet.SharedArtifacts.Rows[0]["artifactHWND"].ToString());

                    replicationsTableAdapter.Insert(sourceHost, sourceHWND, Program.CurrentUser.UserHost, false, null, null, null);
                }
            }

            this.Close();

        }

        void ignoreClickEventHandler(object sender, RoutedEventArgs e)
        {
            this.Close();
        }


        void CollaboratorBubble_LayoutUpdated(object sender, EventArgs e)
        {
            if (messageTextBlock.ActualWidth < 250)
                background.Width = 268 + 50;
            else
                background.Width = messageTextBlock.ActualWidth + 18 + 50;

            if (messageTextBlock.ActualHeight < 50)
                background.Height = 76;
            else
                background.Height = messageTextBlock.ActualHeight + 22 + 30;

            Canvas.SetLeft(closeX, background.Width - 20);
            Canvas.SetTop(closeX, 2);

            Canvas.SetLeft(acceptReplication, 6);
            Canvas.SetTop(acceptReplication, background.Height - 30);

            Canvas.SetLeft(ignore, background.Width - ignore.ActualWidth - 6);
            Canvas.SetTop(ignore, background.Height - 30);


            //Canvas.SetLeft(startIM, 6 + acceptReplication.ActualWidth + ((((background.Width - ignore.ActualWidth - 6) - (10 + acceptReplication.ActualWidth)) - startIM.ActualWidth) / 2));
            //Canvas.SetTop(startIM, background.Height - 30);


        }


        public void myRectangleLoaded(object sender, RoutedEventArgs e)
        {
            fadeInStoryboard.Begin(this);

            this.UpdateLayout();

            
        }

    }
}
