using System;
using System.IO;
using System.Net;
using System.Windows;
using System.Windows.Input;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Media.Animation;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Media.Effects;

using ClubDev.SupportProviders;


namespace ClubDev
{
    class CollaboratorItemsGrid : Canvas
    {


        private CanvasDragDropManager<Canvas> dragMgr;

        private const int itemWidth = 90;
        private int positionOffset;

        private WaitStar waitStar;
        private ScrollButton leftScrollButton;
        private ScrollButton rightScrollButton;
        private Canvas sharedArtifactsCanvas;

        private SharedArtifactSurogate[] sharedArtifacts = null;

        private string collabHost;
        private bool isShare;
        private int numWide;




        public string CollabHost
        {
            get
            {
                return collabHost;
            }
        }





        public CollaboratorItemsGrid(Point location, string collabHost, int shareCount, int showCount, bool isShare)
        {
            this.collabHost = collabHost;
            this.isShare = isShare;

            numWide = Math.Min(Math.Max(shareCount, showCount), 5);
            this.Height = 100;
            this.Width = (itemWidth * numWide) + 60;



            Canvas.SetLeft(this, location.X);
            Canvas.SetTop(this, location.Y);


            waitStar = new WaitStar();
            this.Children.Add(waitStar);

            double scaleWidth = 40 / waitStar.Width;
            double scaleHeight = 40 / waitStar.Height;

            waitStar.RenderTransform = new ScaleTransform(scaleWidth, scaleHeight, waitStar.Width / 2, waitStar.Height / 2);

            Canvas.SetLeft(waitStar, ((this.Width - 20) / 2) - (waitStar.Width / 2));
            Canvas.SetTop(waitStar, ((this.Height - 22) / 2) - (scaleHeight * waitStar.Height) / 2);


            sharedArtifactsCanvas = new Canvas();
            this.Children.Add(sharedArtifactsCanvas);
            Canvas.SetLeft(sharedArtifactsCanvas, 20);
            Canvas.SetTop(sharedArtifactsCanvas, 0);
            sharedArtifactsCanvas.Width = this.Width - 40;
            sharedArtifactsCanvas.Height = this.Height;
            NameScope.SetNameScope(sharedArtifactsCanvas, new NameScope());


            this.Refresh();
            
        }

        public void Refresh()
        {
            sharedArtifactsCanvas.Children.Clear();

            if (this.Children.Contains(leftScrollButton))
                this.Children.Remove(leftScrollButton);
            
            if (this.Children.Contains(rightScrollButton))
                this.Children.Remove(rightScrollButton);

            waitStar.Visibility = Visibility.Visible;
            waitStar.BeginAnimation();

            if(isShare)
                ArtifactsGatherEngine.RetrieveArtifacts(this.collabHost, ArtifactsGatherEngine.RetreiveType.Share);
            else
                ArtifactsGatherEngine.RetrieveArtifacts(this.collabHost, ArtifactsGatherEngine.RetreiveType.Show);

            ArtifactsGatherEngine.Worker.RunWorkerCompleted += new System.ComponentModel.RunWorkerCompletedEventHandler(GatherWorkerCompleteEventHandler);
        
        }


        public void Unload()
        {

            // first write order preferences to the prefernece engine
            if (sharedArtifacts != null)
            {
                int[] currentSorting = new int[sharedArtifacts.Length];
                for (int i = 0; i < currentSorting.Length; i++)
                    currentSorting[i] = sharedArtifacts[i].ID;

                PreferencesEngine.SetPreferences(this.collabHost, currentSorting);
            }


            try
            {

                for (int i = 0; i < this.Children.Count; i++)
                {
                    Canvas sharedArtifact = this.Children[i] as Canvas;

                    for (int j = 0; j < sharedArtifact.Children.Count; j++)
                    {
                        UIElement sharedArtifactImage = sharedArtifact.Children[j] as UIElement;
                        sharedArtifactImage = null;
                    }

                    sharedArtifact = null;
                }
            }

            catch {
                Console.WriteLine("Could not destoy grid");
            }

            GC.Collect();
        }


        public void InitializeEvents(UIElement eventParent)
        {
            this.dragMgr = new CanvasDragDropManager<Canvas>(sharedArtifactsCanvas, eventParent);
            //this.dragMgr.InitializeEvents(eventParent);
        }



        public void RepositionItem(Canvas repositionItem, Point mouseLocation)
        {
            if (sharedArtifacts.Length <= 1)
                return;

            int newPosition = (int)Math.Round((double)mouseLocation.X / itemWidth);
            newPosition += positionOffset;

            // Check to make sure item is not being positioned outside the bounds of the containing array
            if (newPosition >= sharedArtifacts.Length)
                newPosition = sharedArtifacts.Length - 1;
            else if (newPosition < 0)
                newPosition = 0;

            SharedArtifactSurogate[] removedSharedArtifacts = new SharedArtifactSurogate[sharedArtifacts.Length - 1];
            int currentLocation = 0;

            SharedArtifactSurogate removedSharedArtifact = null;

            for (int i = 0; i < sharedArtifacts.Length; i++)
            {
                String artifactID = this.collabHost.Replace(".", "_").Replace("-", "_") + "__" + sharedArtifacts[i].ID.ToString();
                if (!artifactID.Equals(repositionItem.Name))
                {
                    removedSharedArtifacts[currentLocation] = sharedArtifacts[i];
                    currentLocation++;
                }
                else
                {
                    removedSharedArtifact = sharedArtifacts[i];
                }
            }

            if (removedSharedArtifact == null)
                sharedArtifacts = removedSharedArtifacts;

            else
            {
                SharedArtifactSurogate[] addedSharedArtifacts = new SharedArtifactSurogate[removedSharedArtifacts.Length + 1];

                for (int i = 0; i < newPosition; i++)
                    addedSharedArtifacts[i] = removedSharedArtifacts[i];

                addedSharedArtifacts[newPosition] = removedSharedArtifact;

                for (int i = newPosition + 1; i < addedSharedArtifacts.Length; i++)
                    addedSharedArtifacts[i] = removedSharedArtifacts[i - 1];

                sharedArtifacts = addedSharedArtifacts;

            }

            DrawSharedArtifactGrid(positionOffset, positionOffset + (numWide - 1));
        }



        private void GatherWorkerCompleteEventHandler(object sender, System.ComponentModel.RunWorkerCompletedEventArgs e)
        {
            if (waitStar != null)
            {
                waitStar.StopAnimation();
                this.Children.Remove(waitStar);
                waitStar.Visibility = Visibility.Hidden;
            }

            ArtifactsGatherEngine.AbortRetrieveArtifacts();
            SharedArtifactSurogate[] returnedSharedArtifacts = ArtifactsGatherEngine.SharedArtifactSurogate;

            if (returnedSharedArtifacts != null)
            {
                sharedArtifacts = new SharedArtifactSurogate[returnedSharedArtifacts.Length];

                for (int i = 0; i < returnedSharedArtifacts.Length; i++)
                {

                    double ratio = 1;

                    if (returnedSharedArtifacts[i].ScreenShotSource.Width > 75)
                        ratio = 75 / (double)returnedSharedArtifacts[i].ScreenShotSource.PixelWidth;

                    if((returnedSharedArtifacts[i].ScreenShotSource.Height * ratio) > 60)
                        ratio = 60 / (double)returnedSharedArtifacts[i].ScreenShotSource.PixelHeight;


                    BitmapSource screenShot = new TransformedBitmap(returnedSharedArtifacts[i].ScreenShotSource, new System.Windows.Media.ScaleTransform(ratio, ratio));

                    BitmapSource icon = new TransformedBitmap(returnedSharedArtifacts[i].IconSource, new System.Windows.Media.ScaleTransform(1, 1));

                    sharedArtifacts[i] = new SharedArtifactSurogate();
                    sharedArtifacts[i].UpdateArtifact(returnedSharedArtifacts[i].ID, returnedSharedArtifacts[i].Type, returnedSharedArtifacts[i].ArtifactName.ToString(), returnedSharedArtifacts[i].Group.ToString(), returnedSharedArtifacts[i].HWND, screenShot, icon, returnedSharedArtifacts[i].Size, returnedSharedArtifacts[i].Location, returnedSharedArtifacts[i].ZOrder);
                }
            }

            ArtifactsGatherEngine.CleanCache();

            leftScrollButton = new ScrollButton(new Point(10, this.Height / 2));
            rightScrollButton = new ScrollButton(new Point(this.Width - 30, this.Height / 2));
            rightScrollButton.RenderTransform = new RotateTransform(180, rightScrollButton.Width / 2, rightScrollButton.Width / 2);

            leftScrollButton.Click += new ScrollButton.ScrollButtonClickHandler(LeftScrollButtonClickEventHandler);
            rightScrollButton.Click += new ScrollButton.ScrollButtonClickHandler(RightScrollButtonClickEventHandler);

            leftScrollButton.Enabled = false;
            rightScrollButton.Enabled = false;


            this.Children.Add(rightScrollButton);
            this.Children.Add(leftScrollButton);

            if (sharedArtifacts != null)
                if (sharedArtifacts.Length > numWide)
                {
                    rightScrollButton.Enabled = true;
                    leftScrollButton.Visibility = Visibility.Visible;
                    rightScrollButton.Visibility = Visibility.Visible;
                }
                else
                {
                    leftScrollButton.Visibility = Visibility.Hidden;
                    rightScrollButton.Visibility = Visibility.Hidden;
                }


            DrawSharedArtifactGrid(0, (numWide - 1));


        }


        void LeftScrollButtonClickEventHandler(object sender, ScrollButtonEventArgs e)
        {
            if (positionOffset > 0)
            {
                positionOffset -= numWide;
                DrawSharedArtifactGrid(positionOffset, positionOffset + (numWide - 1));

                if (!(positionOffset > 0))
                {
                    leftScrollButton.Enabled = false;
                }

                if (sharedArtifacts.Length > numWide)
                    rightScrollButton.Enabled = true;
            }

        }

        void RightScrollButtonClickEventHandler(object sender, ScrollButtonEventArgs e)
        {
            if (sharedArtifacts.Length > (positionOffset + numWide))
            {
                positionOffset += numWide;
                DrawSharedArtifactGrid(positionOffset, positionOffset + (numWide - 1));

                if (!(sharedArtifacts.Length > (positionOffset + (numWide - 1))))
                {
                    rightScrollButton.Enabled = false;
                }
                leftScrollButton.Enabled = true;
            }
        }




        private void DrawSharedArtifactGrid(int from, int to)
        {
            ResetSharedArtifactGrid();

            if (to > (sharedArtifacts.Length - 1))
                to = (sharedArtifacts.Length - 1);

            if (from < 0)
                from = 0;

            int currentPosition = 0;

            for (int i = from; i <= to; i++)
            {
                Canvas currentArtifact = new Canvas();

                currentArtifact.Width = itemWidth;
                currentArtifact.Height = 100;


                Image currentArtifactThumbnail = new Image();
                currentArtifactThumbnail.Source = sharedArtifacts[i].ScreenShotSource;

                currentArtifactThumbnail.Width = currentArtifactThumbnail.Source.Width;
                currentArtifactThumbnail.Height = currentArtifactThumbnail.Source.Height;

                Canvas.SetLeft(currentArtifactThumbnail, (currentArtifact.Width - currentArtifactThumbnail.Width) / 2);
                Canvas.SetTop(currentArtifactThumbnail, ((currentArtifact.Height - currentArtifactThumbnail.Height) / 2) - 5);

                currentArtifact.Children.Add(currentArtifactThumbnail);


                Image currentArtifactIcon = new Image();
                currentArtifactIcon.Source = sharedArtifacts[i].IconSource;

                currentArtifactIcon.Width = sharedArtifacts[i].IconSource.Width;
                currentArtifactIcon.Height = sharedArtifacts[i].IconSource.Height;


                Canvas.SetLeft(currentArtifactIcon, Canvas.GetLeft(currentArtifactThumbnail) + (currentArtifactThumbnail.Width - currentArtifactIcon.Width));
                Canvas.SetTop(currentArtifactIcon, Canvas.GetTop(currentArtifactThumbnail) + (currentArtifactThumbnail.Height - currentArtifactIcon.Height));

                currentArtifact.Children.Add(currentArtifactIcon);


                Label currentArtifactLabel = new Label();
                currentArtifactLabel.Content = sharedArtifacts[i].ArtifactName;
                currentArtifactLabel.FontSize = 11;
                currentArtifactLabel.VerticalAlignment = VerticalAlignment.Top;
                currentArtifactLabel.Foreground = new SolidColorBrush(Colors.White);
               

                currentArtifact.Name = this.collabHost.Replace(".", "_").Replace("-", "_") + "__" + sharedArtifacts[i].ID.ToString();
                sharedArtifactsCanvas.RegisterName(currentArtifact.Name, currentArtifact);

                currentArtifact.Children.Add(currentArtifactLabel);

                ToolTip artifactNameTip = new ToolTip();
                artifactNameTip.Content = sharedArtifacts[i].ArtifactName;
                currentArtifact.ToolTip = artifactNameTip;

                currentArtifact.Cursor = Cursors.Hand;

                Canvas.SetLeft(currentArtifact, currentPosition * itemWidth);
                Canvas.SetTop(currentArtifact, 0);
                sharedArtifactsCanvas.Children.Add(currentArtifact);

                currentPosition++;

                currentArtifactLabel.UpdateLayout();

                FitLabel(currentArtifactLabel, currentArtifact.Width - 2);

                Canvas.SetLeft(currentArtifactLabel, (currentArtifact.Width - currentArtifactLabel.ActualWidth) / 2);
                Canvas.SetTop(currentArtifactLabel, (currentArtifact.Height - 25));

                currentArtifact.MouseEnter += new MouseEventHandler(currentArtifactMouseEnterEventHandler);
                currentArtifact.MouseLeave += new MouseEventHandler(currentArtifactMouseLeaveEventHandler);
                currentArtifact.MouseUp += new MouseButtonEventHandler(currentArtifactMouseUpEventHandler);

            }
        }


        private void ResetSharedArtifactGrid()
        {
            if (sharedArtifactsCanvas != null)
            {
                sharedArtifactsCanvas.Children.Clear();
                NameScope.SetNameScope(sharedArtifactsCanvas, new NameScope());
            }

        }

        private void FitLabel(Label label, double desiredWidth)
        {
            while (label.ActualWidth > desiredWidth)
            {
                if (label.Content.ToString().Equals("..."))
                    break;
                label.Content = label.Content.ToString().Substring(0, (label.Content.ToString().Length - 4)) + "...";
                label.UpdateLayout();
            }
        }


        void currentArtifactMouseEnterEventHandler(object sender, MouseEventArgs e)
        {
            if (sender.GetType() == typeof(Canvas))
            {
                OuterGlowBitmapEffect glowEffect = new OuterGlowBitmapEffect();
                glowEffect.GlowSize = 5;
                glowEffect.GlowColor = Color.FromArgb(255, 255, 255, 0);
                glowEffect.Noise = 0;
                glowEffect.Opacity = 1;

                ((Canvas)sender).BitmapEffect = glowEffect;
            }
        }

        void currentArtifactMouseLeaveEventHandler(object sender, MouseEventArgs e)
        {
            if (sender.GetType() == typeof(Canvas))
            {
                ((Canvas)sender).BitmapEffect = null;
            }

        }
        void currentArtifactMouseUpEventHandler(object sender, MouseButtonEventArgs e)
        {
            if (sender.GetType() == typeof(Canvas))
            {
                ((Canvas)sender).BitmapEffect = null;
            }
        }

        public void InitiateRelocationToLocalWorkspace(string artifactName)
        {
            int artifactID = int.Parse(artifactName.Substring(artifactName.LastIndexOf("__") + 2));
            Console.WriteLine(artifactID);

            for(int i = 0; i < sharedArtifacts.Length; i++)
                if (sharedArtifacts[i].ID == artifactID)
                {
                    ClubDev.Database.ClubDevDataSet clubDevDataSet = new ClubDev.Database.ClubDevDataSet();
                    ClubDev.Database.ClubDevDataSetTableAdapters.ReplicationsTableAdapter replicationsTableAdapter = new ClubDev.Database.ClubDevDataSetTableAdapters.ReplicationsTableAdapter();

                    replicationsTableAdapter.Insert(sharedArtifacts[i].Group, sharedArtifacts[i].HWND, Program.CurrentUser.UserHost, false, null, null, null);

                }

            //finish by collapsing the collaborator summary

            if (((FrameworkElement)(this.Parent)).Parent.GetType() == typeof(Collaborator))
                ((Collaborator)(((FrameworkElement)(this.Parent)).Parent)).CollapseCollaborator();
        }
        public void InitiateRelocation(UIElement sourceParent, string sourceItem)
        {
            if (sourceParent.GetType() == typeof(CollaboratorItemsGrid))
            {
                long id = long.Parse(sourceItem.Substring(sourceItem.LastIndexOf("__") + 2));

                //check to see if it's just the permissions changing
                if (((CollaboratorItemsGrid)sourceParent).Parent == this.Parent && this.collabHost == ClubDev.Program.CurrentUser.UserHost)
                {
                    ClubDev.Database.ClubDevDataSet clubDevDataSet = new ClubDev.Database.ClubDevDataSet();
                    ClubDev.Database.ClubDevDataSetTableAdapters.SharedArtifactsTableAdapter sharedArtifactsTableAdapter = new ClubDev.Database.ClubDevDataSetTableAdapters.SharedArtifactsTableAdapter();
                    sharedArtifactsTableAdapter.UpdateFullShareByID(this.isShare, id);

                    this.Refresh();
                    ((CollaboratorItemsGrid)sourceParent).Refresh();
                }

                /*

                CollaboratorItemsGrid parent = (CollaboratorItemsGrid)sourceParent;

                ClubDev.Database.ClubDevDataSet clubDevDataSet = new ClubDev.Database.ClubDevDataSet();
                ClubDev.Database.ClubDevDataSetTableAdapters.SharedArtifactsTableAdapter sharedArtifactsTableAdapter = new ClubDev.Database.ClubDevDataSetTableAdapters.SharedArtifactsTableAdapter();
                ClubDev.Database.ClubDevDataSetTableAdapters.ReplicationsTableAdapter replicationsTableAdapter = new ClubDev.Database.ClubDevDataSetTableAdapters.ReplicationsTableAdapter();

                sharedArtifactsTableAdapter.FillMETAByID(clubDevDataSet.SharedArtifacts, id);

                replicationsTableAdapter.Insert(parent.CollabHost, clubDevDataSet.SharedArtifacts[0].artifactHWND, this.collabHost, null, true);

                Console.WriteLine("MOVING " + parent.CollabHost + " " + clubDevDataSet.SharedArtifacts[0].artifactHWND + " TO " + this.collabHost);
                */
            }
        }
    }
}
