using System;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Shapes;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using System.Windows.Media.Effects;

using System.Windows.Forms;
using ClubDev.SupportProviders;


namespace ClubDev
{
    class CollaboratorItemsSummary : Canvas
    {
        private Canvas collageCanvas;
        private WaitStar waitStar;

        private string collabHost;
        private string collabName;
        private bool isShare;

        
        public CollaboratorItemsSummary(Point margin, string collabHost, string collabName, bool isShare)
        {
            this.Margin = new Thickness(margin.X, margin.Y, 0, 0);
            this.Width = 80;
            this.Height = 80;

            this.collabHost = collabHost;
            this.collabName = collabName;
            this.isShare = isShare;

            waitStar = new WaitStar();


            this.Children.Add(waitStar);

            double scaleWidth = 40 / waitStar.Width;
            double scaleHeight = 40 / waitStar.Height;

            waitStar.RenderTransform = new ScaleTransform(scaleWidth, scaleHeight, waitStar.Width / 2, waitStar.Height / 2);

            Canvas.SetLeft(waitStar, (this.Width - waitStar.Width) / 2);
            Canvas.SetTop(waitStar, (this.Height - waitStar.Height) / 2);



            waitStar.BeginAnimation();




            if(isShare)
                ArtifactsGatherEngine.RetrieveArtifacts(this.collabHost, ArtifactsGatherEngine.RetreiveType.Share);
            else
                ArtifactsGatherEngine.RetrieveArtifacts(this.collabHost, ArtifactsGatherEngine.RetreiveType.Show);

            ArtifactsGatherEngine.Worker.ProgressChanged += new System.ComponentModel.ProgressChangedEventHandler(GatherProgressChangedEventHandler);
            ArtifactsGatherEngine.Worker.RunWorkerCompleted += new System.ComponentModel.RunWorkerCompletedEventHandler(GatherWorkerCompleteEventHandler);
        }

        public void Unload()
        {
            try
            {

                for (int i = 0; i < collageCanvas.Children.Count; i++)
                {
                    Canvas collageArtifact = collageCanvas.Children[i] as Canvas;

                    if (collageArtifact != null)
                    {
                        for (int j = 0; j < collageArtifact.Children.Count; j++)
                        {
                            if (collageArtifact.Children[j] != null)
                            {
                                UIElement collageImage = collageArtifact.Children[j] as UIElement;
                                collageImage = null;
                            }
                        }

                        collageArtifact = null;
                    }
                }

                if(waitStar != null)
                    waitStar.StopAnimation();

                this.waitStar = null;
                this.collageCanvas = null;
            }

            catch {
                Console.WriteLine("could not unload collaboratory items summary");
            }

            GC.Collect();
        }

        void GatherProgressChangedEventHandler(object sender, System.ComponentModel.ProgressChangedEventArgs e)
        {
            //progbar.Value = e.ProgressPercentage;
        }

        void GatherWorkerCompleteEventHandler(object sender, System.ComponentModel.RunWorkerCompletedEventArgs e)
        {
            waitStar.StopAnimation();
            this.Children.Remove(waitStar);
            waitStar.Visibility = Visibility.Hidden;
            waitStar = null;

            ArtifactsGatherEngine.AbortRetrieveArtifacts();
            SharedArtifactSurogate[] artifacts = ArtifactsGatherEngine.SharedArtifactSurogate;

            if(artifacts != null)
                DrawSummaryCollage(artifacts);

            ArtifactsGatherEngine.CleanCache();

        }

        private void DrawSummaryCollage(SharedArtifactSurogate[] artifacts)
        {
            collageCanvas = new Canvas();

            double position = 0;
            double gutter = 0;

            if (artifacts.Length < 4)
                position += (4 - artifacts.Length) * 0.5;

            if (artifacts.Length > 4)
                gutter -= 1;

            collageCanvas.Width = 80;
            collageCanvas.Height = 80;

            //limit summary to only showing 8 thumbnails (after that, it's too hard to tell them apart)
            int maxLength = Math.Min(artifacts.Length, 8);

            for (int i = 0; i < maxLength; i++)
            {
                Canvas currentArtifact = new Canvas();

                double ratio = 1;
                if (artifacts[i].ScreenShotSource.Width > artifacts[i].ScreenShotSource.Height)
                    ratio = 50 / (double)artifacts[i].ScreenShotSource.PixelWidth;
                else
                    ratio = 50 / (double)artifacts[i].ScreenShotSource.PixelHeight;
                
                BitmapSource screenShot = new TransformedBitmap(artifacts[i].ScreenShotSource, new System.Windows.Media.ScaleTransform(ratio, ratio));

                Image currentApplicationThumbnail = new Image();
                currentApplicationThumbnail.Source = screenShot;

                currentApplicationThumbnail.Width = currentApplicationThumbnail.Source.Width;
                currentApplicationThumbnail.Height = currentApplicationThumbnail.Source.Height;


                currentArtifact.Width = currentApplicationThumbnail.Width;
                currentArtifact.Height = currentApplicationThumbnail.Height;

                currentArtifact.Children.Add(currentApplicationThumbnail);


                Image currentApplicationIcon = new Image();
                currentApplicationIcon.Source = artifacts[i].IconSource;

                currentApplicationIcon.Width = artifacts[i].IconSource.Width;
                currentApplicationIcon.Height = artifacts[i].IconSource.Height;

                Canvas.SetLeft(currentApplicationIcon, currentArtifact.Width - currentApplicationIcon.Width);
                Canvas.SetTop(currentApplicationIcon, currentArtifact.Height - currentApplicationIcon.Height);

                currentArtifact.Children.Add(currentApplicationIcon);

                Canvas.SetLeft(currentArtifact, (position * 10) - gutter);
                Canvas.SetTop(currentArtifact, position * 10);

                collageCanvas.Children.Add(currentArtifact);

                position += 1;

                if (position> 3)
                {
                    gutter += 20;
                    position = 0.5;
                }

            }


            TextBlock labelTextBlock = new TextBlock();
            labelTextBlock.FontSize = 10;
            if (artifacts.Length == 1)
                labelTextBlock.Text = "1 Item";
            else
                labelTextBlock.Text = artifacts.Length.ToString() + " Items";

            OuterGlowBitmapEffect labelGlowEffect = new OuterGlowBitmapEffect();
            labelGlowEffect.GlowSize = 5;
            labelGlowEffect.GlowColor = Color.FromArgb(255, 255, 255, 255);
            labelGlowEffect.Noise = 0;
            labelGlowEffect.Opacity = 1;

            labelTextBlock.BitmapEffect = labelGlowEffect;

            collageCanvas.Children.Add(labelTextBlock);

            Canvas.SetTop(labelTextBlock, 0);
            Canvas.SetLeft(labelTextBlock, 0);


            
            Canvas.SetTop(collageCanvas, 12);
            Canvas.SetLeft(collageCanvas, 5);


            this.Children.Add(collageCanvas);

            labelTextBlock.UpdateLayout();

            Canvas.SetTop(labelTextBlock, collageCanvas.Height - 10);
            Canvas.SetLeft(labelTextBlock, (collageCanvas.Width - labelTextBlock.ActualWidth) / 2);



        }



    }
}
