using System;
using System.Reflection;
using System.IO;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Shapes;
using System.Windows.Media.Animation;
using System.Windows.Media.Effects;
using System.Windows.Media.Imaging;
using Microsoft.Win32;




namespace ClubDev
{
    class CollaboratorSummary : Canvas
    {
        private bool isExpanded;

        private Rectangle backgroundRectangle;
        private Rectangle showTitleRectangle;
        private Rectangle shareTitleRectangle;
        private TextBlock showTitleTextBlock;
        private TextBlock shareTitleTextBlock;
        private TextBlock showZeroItemNumberTextBlock;
        private TextBlock shareZeroItemNumberTextBlock;
        private TextBlock showZeroItemTextTextBlock;
        private TextBlock shareZeroItemTextTextBlock;
        private Line dividerLine;
        private double expandWidth;
        private TextBlock collaboratorTextBlock;
        private Image imStatusIconImage;
        private Image collaboratorBubbleCreateImage;

        private UIElement eventParent;

        private CollaboratorItemsGrid showCollaboratorItemsGrid;
        private CollaboratorItemsGrid shareCollaboratorItemsGrid;


        private Storyboard fadeInStoryboard;
        private Storyboard fadeOutStoryboard;

        private Storyboard pullOutStoryboard;
        private Storyboard pullInStoryboard;

        private string collabHost;
        private string collabName;
        private string collabIM;

        private BitmapSource messengerOnline = null;
        private BitmapSource messengerOffline = null;
        private BitmapSource messengerAway = null;
        private BitmapSource messengerBusy = null;

        private Database.ClubDevDataSet clubDevDataSet;
        private Database.ClubDevDataSetTableAdapters.SharedArtifactsTableAdapter sharedArtifactsTableAdapter;

        private System.Windows.Forms.Timer UnhideGrid;
        private System.Windows.Forms.Timer UnhideSummary;

        private System.DateTime lastClickTime;
        private System.TimeSpan doubleClickInterval;


        
        public CollaboratorSummary(Point point, string collabHost, string collabName, string collabIM)
        {
            NameScope.SetNameScope(this, new NameScope());

            isExpanded = false;

            this.Width = 170;
            this.Height = 220;

            this.expandWidth = this.Width;

            this.Opacity = 0.0;
            this.Name = "collaboratorSummary";
            this.RegisterName(this.Name, this);

            Canvas.SetLeft(this, point.X);
            Canvas.SetTop(this, point.Y);


            this.collabHost = collabHost;
            this.collabName = collabName;
            this.collabIM = collabIM;

            this.clubDevDataSet = new ClubDev.Database.ClubDevDataSet();
            this.sharedArtifactsTableAdapter = new ClubDev.Database.ClubDevDataSetTableAdapters.SharedArtifactsTableAdapter();


            backgroundRectangle = new Rectangle();

            GradientStopCollection collection = new GradientStopCollection();
            collection.Add(new GradientStop(Color.FromArgb(240, 110, 110, 110), 0.0));
            collection.Add(new GradientStop(Color.FromArgb(240, 60, 60, 60), 0.5));
            collection.Add(new GradientStop(Color.FromArgb(240, 30, 30, 30), 1.0));
            LinearGradientBrush fillBrush = new LinearGradientBrush(collection, 90);
            fillBrush.SpreadMethod = GradientSpreadMethod.Pad;
            backgroundRectangle.Fill = fillBrush;


            SolidColorBrush strokeBrush = new SolidColorBrush(Color.FromArgb(255, 110, 110, 110));
            backgroundRectangle.Stroke = strokeBrush;
            backgroundRectangle.StrokeThickness = 1.5;
            backgroundRectangle.StrokeLineJoin = PenLineJoin.Round;

            backgroundRectangle.RadiusX = 20;
            backgroundRectangle.RadiusY = 20;

            backgroundRectangle.Width = this.Width + 20;
            backgroundRectangle.Height = this.Height;

            Canvas.SetLeft(backgroundRectangle, -20);
            Canvas.SetTop(backgroundRectangle, 0);


            DropShadowBitmapEffect myDropShadowEffect = new DropShadowBitmapEffect();

            myDropShadowEffect.Color = Colors.Black;
            myDropShadowEffect.Softness = 1;
            myDropShadowEffect.ShadowDepth = 10;

            backgroundRectangle.BitmapEffect = myDropShadowEffect;


            backgroundRectangle.Name = "backgroundRectangle";
            this.RegisterName(backgroundRectangle.Name, backgroundRectangle);



            showTitleRectangle = new Rectangle();

            GradientStopCollection showCollection = new GradientStopCollection();
            showCollection.Add(new GradientStop(Color.FromArgb(150, 170, 175, 200), 0.0));
            showCollection.Add(new GradientStop(Color.FromArgb(150, 100, 110, 170), 0.5));
            showCollection.Add(new GradientStop(Color.FromArgb(0  , 100, 110, 170), 1.0));

            LinearGradientBrush showBrush = new LinearGradientBrush(showCollection, 0);
            showBrush.SpreadMethod = GradientSpreadMethod.Pad;
            showTitleRectangle.Fill = showBrush;

            showTitleRectangle.Width = this.Width;
            showTitleRectangle.Height = 20;

            Canvas.SetTop(showTitleRectangle, this.Height - (showTitleRectangle.Height + 1.5));

            showTitleRectangle.Name = "showTitleRectangle";
            this.RegisterName(showTitleRectangle.Name, showTitleRectangle);



            shareTitleRectangle = new Rectangle();

            GradientStopCollection shareCollection = new GradientStopCollection();
            shareCollection.Add(new GradientStop(Color.FromArgb(150, 155, 175, 100), 0.0));
            shareCollection.Add(new GradientStop(Color.FromArgb(150, 205, 230, 170), 0.5));
            shareCollection.Add(new GradientStop(Color.FromArgb(0  , 205, 230, 170), 1.0));
            LinearGradientBrush shareBrush = new LinearGradientBrush(shareCollection, 0);
            shareBrush.SpreadMethod = GradientSpreadMethod.Pad;
            shareTitleRectangle.Fill = shareBrush;

            shareTitleRectangle.Width = this.Width;
            shareTitleRectangle.Height = 20;

            Canvas.SetTop(shareTitleRectangle, 1.5);

            shareTitleRectangle.Name = "shareTitleRectangle";
            this.RegisterName(shareTitleRectangle.Name, shareTitleRectangle);



            showTitleTextBlock = new TextBlock();
            showTitleTextBlock.Text = "SHOW";
            showTitleTextBlock.FontFamily = new FontFamily("Veranda");
            showTitleTextBlock.FontWeight = FontWeights.Bold;
            showTitleTextBlock.FontSize = 16.0;
            showTitleTextBlock.Foreground = new SolidColorBrush(Colors.WhiteSmoke);

            Canvas.SetLeft(showTitleTextBlock, 28);
            Canvas.SetTop(showTitleTextBlock, this.Height - (showTitleRectangle.Height - 2));


            shareTitleTextBlock = new TextBlock();
            shareTitleTextBlock.Text = "SHARE";
            shareTitleTextBlock.FontFamily = new FontFamily("Veranda");
            shareTitleTextBlock.FontWeight = FontWeights.Bold;
            shareTitleTextBlock.FontSize = 16.0;
            shareTitleTextBlock.Foreground = new SolidColorBrush(Colors.WhiteSmoke);

            Canvas.SetLeft(shareTitleTextBlock, 32);
            Canvas.SetTop(shareTitleTextBlock, 4);



            BitmapSource showIcon = null;
            BitmapSource shareIcon = null;
            try
            {
                Assembly assem = Assembly.GetExecutingAssembly();

                System.IO.StreamReader onlineStream = new StreamReader(assem.GetManifestResourceStream("ClubDev.Icons.collaboratorSummary-IM-Online.png"));
                PngBitmapDecoder onlineBitmapDecoder = new PngBitmapDecoder(onlineStream.BaseStream, BitmapCreateOptions.PreservePixelFormat, BitmapCacheOption.None);
                messengerOnline = onlineBitmapDecoder.Frames[0];

                System.IO.StreamReader offlineStream = new StreamReader(assem.GetManifestResourceStream("ClubDev.Icons.collaboratorSummary-IM-Offline.png"));
                PngBitmapDecoder offlineBitmapDecoder = new PngBitmapDecoder(offlineStream.BaseStream, BitmapCreateOptions.PreservePixelFormat, BitmapCacheOption.None);
                messengerOffline = offlineBitmapDecoder.Frames[0];

                System.IO.StreamReader awayStream = new StreamReader(assem.GetManifestResourceStream("ClubDev.Icons.collaboratorSummary-IM-Away.png"));
                PngBitmapDecoder awayBitmapDecoder = new PngBitmapDecoder(awayStream.BaseStream, BitmapCreateOptions.PreservePixelFormat, BitmapCacheOption.None);
                messengerAway = awayBitmapDecoder.Frames[0];

                System.IO.StreamReader busyStream = new StreamReader(assem.GetManifestResourceStream("ClubDev.Icons.collaboratorSummary-IM-Busy.png"));
                PngBitmapDecoder busyBitmapDecoder = new PngBitmapDecoder(busyStream.BaseStream, BitmapCreateOptions.PreservePixelFormat, BitmapCacheOption.None);
                messengerBusy = busyBitmapDecoder.Frames[0];

                System.IO.StreamReader showStream = new StreamReader(assem.GetManifestResourceStream("ClubDev.Icons.collaboratorSummary-Show.png"));
                PngBitmapDecoder showBitmapDecoder = new PngBitmapDecoder(showStream.BaseStream, BitmapCreateOptions.PreservePixelFormat, BitmapCacheOption.None);
                showIcon = showBitmapDecoder.Frames[0];

                System.IO.StreamReader shareStream = new StreamReader(assem.GetManifestResourceStream("ClubDev.Icons.collaboratorSummary-Share.png"));
                PngBitmapDecoder shareBitmapDecoder = new PngBitmapDecoder(shareStream.BaseStream, BitmapCreateOptions.PreservePixelFormat, BitmapCacheOption.None);
                shareIcon = shareBitmapDecoder.Frames[0];
            }
            catch
            {
                Console.WriteLine("Could not import embeded resources");
            }

            Image showIconImage = new Image();
            showIconImage.Source = showIcon;
            Canvas.SetLeft(showIconImage, 7);
            Canvas.SetTop(showIconImage, this.Height - (showIcon.Height + 2));

            Image shareIconImage = new Image();
            shareIconImage.Source = shareIcon;
            Canvas.SetLeft(shareIconImage, 3);
            Canvas.SetTop(shareIconImage, 4);

            
            if (Program.LiveMessenger != null && !(Program.CurrentUser.UserHost.Equals(this.collabHost)))
            {
                imStatusIconImage = new Image();
                imStatusIconImage.Width = 24;
                imStatusIconImage.Height = 24;

                MessengerAPI.IMessengerContact contact = (MessengerAPI.IMessengerContact)Program.LiveMessenger.GetContact(collabIM, Program.LiveMessenger.MyServiceId);

                if (contact.Status == MessengerAPI.MISTATUS.MISTATUS_ONLINE)
                    imStatusIconImage.Source = messengerOnline;

                else if (contact.Status == MessengerAPI.MISTATUS.MISTATUS_IDLE ||
                        contact.Status == MessengerAPI.MISTATUS.MISTATUS_AWAY ||
                        contact.Status == MessengerAPI.MISTATUS.MISTATUS_BE_RIGHT_BACK ||
                        contact.Status == MessengerAPI.MISTATUS.MISTATUS_OUT_TO_LUNCH)
                    imStatusIconImage.Source = messengerAway;

                else if (contact.Status == MessengerAPI.MISTATUS.MISTATUS_BUSY ||
                        contact.Status == MessengerAPI.MISTATUS.MISTATUS_ON_THE_PHONE)
                    imStatusIconImage.Source = messengerBusy;

                else
                    imStatusIconImage.Source = messengerOffline;


                imStatusIconImage.Name = "imStatusIconImage";
                this.RegisterName(imStatusIconImage.Name, imStatusIconImage);
                Canvas.SetLeft(imStatusIconImage, 1);
                Canvas.SetTop(imStatusIconImage, 145);

                try
                {
                    RegistryKey mouseRegistryKey = Registry.CurrentUser.CreateSubKey("Control Panel\\Mouse");
                    doubleClickInterval = new TimeSpan(0, 0, 0, 0, int.Parse((string)mouseRegistryKey.GetValue("DoubleClickSpeed")));
                    mouseRegistryKey.Close();
                }
                catch
                {
                    doubleClickInterval = new TimeSpan(0, 0, 0, 0, 600);
                }


                imStatusIconImage.MouseLeftButtonDown += new MouseButtonEventHandler(IMClickEventHandler);
            }

            else if (Program.CurrentUser.UserHost.Equals(this.collabHost))
            {
                collaboratorBubbleCreateImage = new Image();
                collaboratorBubbleCreateImage.Width = 20;
                collaboratorBubbleCreateImage.Height = 20;

                try
                {
                    Assembly assem = Assembly.GetExecutingAssembly();

                    System.IO.StreamReader megaphoneStream = new StreamReader(assem.GetManifestResourceStream("ClubDev.Icons.collaboratorBubble-Megaphone.png"));
                    PngBitmapDecoder megaphoneBitmapDecoder = new PngBitmapDecoder(megaphoneStream.BaseStream, BitmapCreateOptions.PreservePixelFormat, BitmapCacheOption.None);
                    collaboratorBubbleCreateImage.Source = megaphoneBitmapDecoder.Frames[0];
                }

                catch { }

                Canvas.SetLeft(collaboratorBubbleCreateImage, 2);
                Canvas.SetTop(collaboratorBubbleCreateImage, 148);


                collaboratorBubbleCreateImage.Name = "collaboratorBubbleCreateImage";
                this.RegisterName(collaboratorBubbleCreateImage.Name, collaboratorBubbleCreateImage);


                try
                {
                    RegistryKey mouseRegistryKey = Registry.CurrentUser.CreateSubKey("Control Panel\\Mouse");
                    doubleClickInterval = new TimeSpan(0, 0, 0, 0, int.Parse((string)mouseRegistryKey.GetValue("DoubleClickSpeed")));
                    mouseRegistryKey.Close();
                }
                catch
                {
                    doubleClickInterval = new TimeSpan(0, 0, 0, 0, 600);
                }

                collaboratorBubbleCreateImage.MouseLeftButtonDown += new MouseButtonEventHandler(collaboratorBubbleCreateImageEventHandler);


            }
            



            dividerLine = new Line();
            dividerLine.X1 = 90;
            dividerLine.Y1 = backgroundRectangle.Height / 2;

            dividerLine.X2 = backgroundRectangle.Width - 35;
            dividerLine.Y2 = backgroundRectangle.Height / 2;

            dividerLine.Fill = strokeBrush;
            dividerLine.Stroke = strokeBrush;

            dividerLine.Name = "dividerLine";
            this.RegisterName(dividerLine.Name, dividerLine);



            //expandButton = new ExpandButton(new Point(this.Width - 1, this.Height / 2), "collaborator");
            //expandButton.Name = "expandButton";
            //this.RegisterName(expandButton.Name, expandButton);

            this.collaboratorTextBlock = new TextBlock();
            this.collaboratorTextBlock.Text = this.collabName;
            this.collaboratorTextBlock.FontFamily = new FontFamily("Veranda");
            this.collaboratorTextBlock.FontWeight = FontWeights.Bold;
            this.collaboratorTextBlock.FontSize = 14;
            this.collaboratorTextBlock.Foreground = new SolidColorBrush(Colors.White);
            Canvas.SetTop(this.collaboratorTextBlock, 150);

            if (Program.LiveMessenger != null && !(Program.CurrentUser.UserHost.Equals(this.collabHost)))
            {
                Canvas.SetLeft(this.collaboratorTextBlock, 30);
                this.collaboratorTextBlock.MouseLeftButtonDown +=new MouseButtonEventHandler(IMClickEventHandler);
            }
            else if (Program.CurrentUser.UserHost.Equals(this.collabHost))
            {
                Canvas.SetLeft(this.collaboratorTextBlock, 26);
                this.collaboratorTextBlock.MouseLeftButtonDown += new MouseButtonEventHandler(collaboratorBubbleCreateImageEventHandler);

            }
            else
                Canvas.SetLeft(this.collaboratorTextBlock, 5);

            this.Children.Add(backgroundRectangle);
            this.Children.Add(showTitleRectangle);
            this.Children.Add(shareTitleRectangle);
            this.Children.Add(showTitleTextBlock);
            this.Children.Add(shareTitleTextBlock);
            this.Children.Add(showIconImage);
            this.Children.Add(shareIconImage);
            this.Children.Add(dividerLine);
            this.Children.Add(collaboratorTextBlock);
            
            if (imStatusIconImage != null)
                this.Children.Add(imStatusIconImage);

            if (collaboratorBubbleCreateImage != null)
                this.Children.Add(collaboratorBubbleCreateImage);
            
            this.InitalizeAnimations();

            this.AllowDrop = true;
            
        }

        public void InitializeEvents(UIElement eventParent)
        {
            this.eventParent = eventParent;
            //expandButton.Click += new ExpandButton.ExpandButtonClickHandler(ExpandButtonClickEventHandler);
        }


        private void InitalizeAnimations() 
        {

            // Fade In Animation
            DoubleAnimation fadeInAnimation = new DoubleAnimation();
            fadeInAnimation.From = 0.0;
            fadeInAnimation.To = 1.0;
            fadeInAnimation.Duration = new Duration(TimeSpan.FromMilliseconds(250));
            fadeInAnimation.AutoReverse = false;

            fadeInStoryboard = new Storyboard();
            fadeInStoryboard.Children.Add(fadeInAnimation);
            Storyboard.SetTargetName(fadeInAnimation, this.Name);

            Storyboard.SetTargetProperty(fadeInAnimation, new PropertyPath(System.Windows.Shapes.Path.OpacityProperty));


            // Fade Out Animation
            DoubleAnimation fadeOutAnimation = new DoubleAnimation();
            fadeOutAnimation.From = 1.0;
            fadeOutAnimation.To = 0.0;
            fadeOutAnimation.Duration = new Duration(TimeSpan.FromMilliseconds(500));
            fadeOutAnimation.AutoReverse = false;

            fadeOutStoryboard = new Storyboard();
            fadeOutStoryboard.Children.Add(fadeOutAnimation);
            Storyboard.SetTargetName(fadeOutAnimation, this.Name);

            Storyboard.SetTargetProperty(fadeOutAnimation, new PropertyPath(System.Windows.Shapes.Path.OpacityProperty));


            
            //Pull In/Out common items
            //double finalWidth = 1000; 

            /*
            RotateTransform pullRotateTransform = new RotateTransform();
            pullRotateTransform.CenterX = (expandButton.Width / 2);
            pullRotateTransform.CenterY = (expandButton.Height / 2);
            if (this.FindName("pullRotateTransform") != null)
                this.UnregisterName("pullRotateTransform");
            this.RegisterName("pullRotateTransform", pullRotateTransform);
            

            TranslateTransform pullTranslateTransform = new TranslateTransform();
            if (this.FindName("pullTranslateTransform") != null)
                this.UnregisterName("pullTranslateTransform");
            this.RegisterName("pullTranslateTransform", pullTranslateTransform);

            TransformGroup pullTransformGroup = new TransformGroup();
            pullTransformGroup.Children.Add(pullRotateTransform);
            pullTransformGroup.Children.Add(pullTranslateTransform);
            expandButton.RenderTransform = pullTransformGroup;
            */


            // Pull Out Animation
            DoubleAnimationUsingKeyFrames pullOutAnimation = new DoubleAnimationUsingKeyFrames();
            pullOutAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(backgroundRectangle.Width, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(0))));
            pullOutAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(this.expandWidth - 25, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(250))));
            pullOutAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(this.expandWidth, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(300))));
            pullOutAnimation.AutoReverse = false;

            DoubleAnimationUsingKeyFrames pullOutAnimationShowTitleRectangle = pullOutAnimation.Clone();
            DoubleAnimationUsingKeyFrames pullOutAnimationShareTitleRectangle = pullOutAnimation.Clone();

            DoubleAnimationUsingKeyFrames pullOutDividerFadeInAnimation = new DoubleAnimationUsingKeyFrames();
            pullOutDividerFadeInAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(0.01, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(300))));
            pullOutDividerFadeInAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(1, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(600))));
            pullOutDividerFadeInAnimation.AutoReverse = false;

            DoubleAnimationUsingKeyFrames pullOutDividerStretchAnimation = new DoubleAnimationUsingKeyFrames();
            pullOutDividerStretchAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(dividerLine.X2, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(200))));
            pullOutDividerStretchAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(this.expandWidth - 30, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(210))));
            pullOutDividerStretchAnimation.AutoReverse = false;

            //DoubleAnimationUsingKeyFrames pullOutExpandButtonFadeOutAnimation = new DoubleAnimationUsingKeyFrames();
            //pullOutExpandButtonFadeOutAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(1, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(0))));
            //pullOutExpandButtonFadeOutAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(0, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(100))));
            //pullOutExpandButtonFadeOutAnimation.AutoReverse = false;

            //DoubleAnimationUsingKeyFrames pullOutExpandButtonFadeInAnimation = new DoubleAnimationUsingKeyFrames();
            //pullOutExpandButtonFadeInAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(0, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(800))));
            //pullOutExpandButtonFadeInAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(1, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(1100))));
            //pullOutExpandButtonFadeInAnimation.AutoReverse = false;

            //DoubleAnimationUsingKeyFrames pullOutExpandButtonRotateAnimation = new DoubleAnimationUsingKeyFrames();
            //pullOutExpandButtonRotateAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(0, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(400))));
            //pullOutExpandButtonRotateAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(180, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(410))));
            //pullOutExpandButtonRotateAnimation.AutoReverse = false;

            //DoubleAnimationUsingKeyFrames pullOutExpandButtonTranslateAnimation = new DoubleAnimationUsingKeyFrames();
            //pullOutExpandButtonTranslateAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(0, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(400))));
            //pullOutExpandButtonTranslateAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(this.expandWidth - expandButton.Margin.Left - 30, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(410))));
            //pullOutExpandButtonTranslateAnimation.AutoReverse = false;

            pullOutStoryboard = new Storyboard();
            pullOutStoryboard.Children.Add(pullOutAnimation);
            pullOutStoryboard.Children.Add(pullOutAnimationShowTitleRectangle);
            pullOutStoryboard.Children.Add(pullOutAnimationShareTitleRectangle);
            //pullOutStoryboard.Children.Add(pullOutExpandButtonFadeOutAnimation);
            pullOutStoryboard.Children.Add(pullOutDividerStretchAnimation);
            //pullOutStoryboard.Children.Add(pullOutExpandButtonRotateAnimation);
            //pullOutStoryboard.Children.Add(pullOutExpandButtonTranslateAnimation);
            pullOutStoryboard.Children.Add(pullOutDividerFadeInAnimation);
            //pullOutStoryboard.Children.Add(pullOutExpandButtonFadeInAnimation);

            Storyboard.SetTargetName(pullOutAnimation, backgroundRectangle.Name);
            Storyboard.SetTargetName(pullOutAnimationShowTitleRectangle, showTitleRectangle.Name);
            Storyboard.SetTargetName(pullOutAnimationShareTitleRectangle, shareTitleRectangle.Name);
            Storyboard.SetTargetName(pullOutDividerFadeInAnimation, dividerLine.Name);
            Storyboard.SetTargetName(pullOutDividerStretchAnimation, dividerLine.Name);
            //Storyboard.SetTargetName(pullOutExpandButtonFadeOutAnimation, expandButton.Name);
            //Storyboard.SetTargetName(pullOutExpandButtonFadeInAnimation, expandButton.Name);
            //Storyboard.SetTargetName(pullOutExpandButtonRotateAnimation, "pullRotateTransform");
            //Storyboard.SetTargetName(pullOutExpandButtonTranslateAnimation, "pullTranslateTransform");

            Storyboard.SetTargetProperty(pullOutAnimation, new PropertyPath(System.Windows.Shapes.Rectangle.WidthProperty));
            Storyboard.SetTargetProperty(pullOutAnimationShowTitleRectangle, new PropertyPath(System.Windows.Shapes.Rectangle.WidthProperty));
            Storyboard.SetTargetProperty(pullOutAnimationShareTitleRectangle, new PropertyPath(System.Windows.Shapes.Rectangle.WidthProperty));
            Storyboard.SetTargetProperty(pullOutDividerFadeInAnimation, new PropertyPath(System.Windows.Shapes.Path.OpacityProperty));
            Storyboard.SetTargetProperty(pullOutDividerStretchAnimation, new PropertyPath(System.Windows.Shapes.Line.X2Property));
            //Storyboard.SetTargetProperty(pullOutExpandButtonFadeOutAnimation, new PropertyPath(System.Windows.Shapes.Path.OpacityProperty));
            //Storyboard.SetTargetProperty(pullOutExpandButtonFadeInAnimation, new PropertyPath(System.Windows.Shapes.Path.OpacityProperty));
            //Storyboard.SetTargetProperty(pullOutExpandButtonRotateAnimation, new PropertyPath(RotateTransform.AngleProperty));
            //Storyboard.SetTargetProperty(pullOutExpandButtonTranslateAnimation, new PropertyPath(TranslateTransform.XProperty)); 


            // Pull In Animation
            DoubleAnimationUsingKeyFrames pullInAnimation = new DoubleAnimationUsingKeyFrames();
            pullInAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(this.expandWidth, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(150))));
            pullInAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(this.expandWidth - 25, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(250))));
            pullInAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(backgroundRectangle.Width - 50, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(650))));
            pullInAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(backgroundRectangle.Width, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(750))));
            pullInAnimation.BeginTime = TimeSpan.FromMilliseconds(100);
            pullInAnimation.AutoReverse = false;

            DoubleAnimationUsingKeyFrames pullInAnimationShowTitleRectangle = pullInAnimation.Clone();
            DoubleAnimationUsingKeyFrames pullInAnimationShareTitleRectangle = pullInAnimation.Clone();

            DoubleAnimationUsingKeyFrames pullInDividerFadeInAnimation = new DoubleAnimationUsingKeyFrames();
            pullInDividerFadeInAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(0.01, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(800))));
            pullInDividerFadeInAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(1, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(1100))));
            pullInDividerFadeInAnimation.AutoReverse = false;

            DoubleAnimationUsingKeyFrames pullInDividerFadeOutAnimation = new DoubleAnimationUsingKeyFrames();
            pullInDividerFadeOutAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(1, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(0))));
            pullInDividerFadeOutAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(0.01, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(100))));
            pullInDividerFadeOutAnimation.AutoReverse = false;

            DoubleAnimationUsingKeyFrames pullInDividerStretchAnimation = new DoubleAnimationUsingKeyFrames();
            pullInDividerStretchAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(this.expandWidth - 30, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(400))));
            pullInDividerStretchAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(dividerLine.X2, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(410))));
            pullInDividerStretchAnimation.AutoReverse = false;

            //DoubleAnimation pullInExpandButtonFadeOutAnimation = new DoubleAnimation();
            //pullInExpandButtonFadeOutAnimation.From = 1.0;
            //pullInExpandButtonFadeOutAnimation.To = 0.0;
            //pullInExpandButtonFadeOutAnimation.Duration = new Duration(TimeSpan.FromMilliseconds(100));
            //pullInExpandButtonFadeOutAnimation.AutoReverse = false;

            //DoubleAnimationUsingKeyFrames pullInExpandButtonFadeInAnimation = new DoubleAnimationUsingKeyFrames();
            //pullInExpandButtonFadeInAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(0, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(800))));
            //pullInExpandButtonFadeInAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(1, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(1100))));
            //pullInExpandButtonFadeInAnimation.AutoReverse = false;

            //DoubleAnimationUsingKeyFrames pullInExpandButtonRotateAnimation = new DoubleAnimationUsingKeyFrames();
            //pullInExpandButtonRotateAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(180, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(400))));
            //pullInExpandButtonRotateAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(0, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(410))));
            //pullInExpandButtonRotateAnimation.AutoReverse = false;

            //DoubleAnimationUsingKeyFrames pullInExpandButtonTranslateAnimation = new DoubleAnimationUsingKeyFrames();
            //pullInExpandButtonTranslateAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(this.expandWidth - expandButton.Margin.Left - 30, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(400))));
            //pullInExpandButtonTranslateAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(0, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(410))));
            //pullInExpandButtonTranslateAnimation.AutoReverse = false;

            pullInStoryboard = new Storyboard();
            pullInStoryboard.Children.Add(pullInAnimation);
            pullInStoryboard.Children.Add(pullInAnimationShowTitleRectangle);
            pullInStoryboard.Children.Add(pullInAnimationShareTitleRectangle);
            pullInStoryboard.Children.Add(pullInDividerFadeOutAnimation);
            //pullInStoryboard.Children.Add(pullInExpandButtonFadeOutAnimation);
            pullInStoryboard.Children.Add(pullInDividerStretchAnimation);
            pullInStoryboard.Children.Add(pullInDividerFadeInAnimation);
            //pullInStoryboard.Children.Add(pullInExpandButtonRotateAnimation);
            //pullInStoryboard.Children.Add(pullInExpandButtonTranslateAnimation);
            //pullInStoryboard.Children.Add(pullInExpandButtonFadeInAnimation);

            Storyboard.SetTargetName(pullInAnimation, backgroundRectangle.Name);
            Storyboard.SetTargetName(pullInAnimationShowTitleRectangle, showTitleRectangle.Name);
            Storyboard.SetTargetName(pullInAnimationShareTitleRectangle, shareTitleRectangle.Name);
            Storyboard.SetTargetName(pullInDividerFadeInAnimation, dividerLine.Name);
            Storyboard.SetTargetName(pullInDividerFadeOutAnimation, dividerLine.Name);
            Storyboard.SetTargetName(pullInDividerStretchAnimation, dividerLine.Name);
            //Storyboard.SetTargetName(pullInExpandButtonFadeOutAnimation, expandButton.Name);
            //Storyboard.SetTargetName(pullInExpandButtonFadeInAnimation, expandButton.Name);
            //Storyboard.SetTargetName(pullInExpandButtonRotateAnimation, "pullRotateTransform");
            //Storyboard.SetTargetName(pullInExpandButtonTranslateAnimation, "pullTranslateTransform");

            Storyboard.SetTargetProperty(pullInAnimation, new PropertyPath(System.Windows.Shapes.Rectangle.WidthProperty));
            Storyboard.SetTargetProperty(pullInAnimationShowTitleRectangle, new PropertyPath(System.Windows.Shapes.Rectangle.WidthProperty));
            Storyboard.SetTargetProperty(pullInAnimationShareTitleRectangle, new PropertyPath(System.Windows.Shapes.Rectangle.WidthProperty));
            Storyboard.SetTargetProperty(pullInDividerFadeOutAnimation, new PropertyPath(System.Windows.Shapes.Path.OpacityProperty));
            Storyboard.SetTargetProperty(pullInDividerFadeInAnimation, new PropertyPath(System.Windows.Shapes.Path.OpacityProperty));
            Storyboard.SetTargetProperty(pullInDividerStretchAnimation, new PropertyPath(System.Windows.Shapes.Line.X2Property));
            //Storyboard.SetTargetProperty(pullInExpandButtonFadeOutAnimation, new PropertyPath(System.Windows.Shapes.Path.OpacityProperty));
            //Storyboard.SetTargetProperty(pullInExpandButtonFadeInAnimation, new PropertyPath(System.Windows.Shapes.Path.OpacityProperty));
            //Storyboard.SetTargetProperty(pullInExpandButtonRotateAnimation, new PropertyPath(RotateTransform.AngleProperty));
            //Storyboard.SetTargetProperty(pullInExpandButtonTranslateAnimation, new PropertyPath(TranslateTransform.XProperty)); 

            
        }

        public void FadeIn()
        {
            fadeInStoryboard.Begin(this);

            int shareCount = (int)this.sharedArtifactsTableAdapter.GetCountByGroupandShareLevel(this.collabHost, true);
            int showCount = (int)this.sharedArtifactsTableAdapter.GetCountByGroupandShareLevel(this.collabHost, false);


            this.expandWidth = 155 + (Math.Min(Math.Max(showCount, shareCount), 5) * 90);

            if (shareCount == 0)
            {
                shareZeroItemNumberTextBlock = new TextBlock();
                shareZeroItemNumberTextBlock.Text = "0";
                shareZeroItemNumberTextBlock.FontFamily = new FontFamily("Veranda");
                shareZeroItemNumberTextBlock.FontWeight = FontWeights.Bold;
                shareZeroItemNumberTextBlock.FontSize = 50.0;
                shareZeroItemNumberTextBlock.Foreground = new SolidColorBrush(Color.FromRgb(130, 130, 130));

                shareZeroItemTextTextBlock = new TextBlock();
                shareZeroItemTextTextBlock.Text = "Items Shared";
                shareZeroItemTextTextBlock.FontFamily = new FontFamily("Veranda");
                shareZeroItemTextTextBlock.FontWeight = FontWeights.Bold;
                shareZeroItemTextTextBlock.FontSize = 10.0;
                shareZeroItemTextTextBlock.Foreground = new SolidColorBrush(Color.FromRgb(130, 130, 130));
               
                Canvas.SetTop(shareZeroItemNumberTextBlock, 30);
                if(showCount == 0)
                    Canvas.SetLeft(shareZeroItemNumberTextBlock, 85 + 42.5 - 20);
                else
                    Canvas.SetLeft(shareZeroItemNumberTextBlock, 85 + ((this.expandWidth - 105) / 2) - 20);
                
                Canvas.SetTop(shareZeroItemTextTextBlock, 80);
                if (showCount == 0)
                    Canvas.SetLeft(shareZeroItemTextTextBlock, 85 + 42.5 - 35);
                else
                    Canvas.SetLeft(shareZeroItemTextTextBlock, 85 + ((this.expandWidth - 105) / 2) - 35);

                this.Children.Add(shareZeroItemNumberTextBlock);
                this.Children.Add(shareZeroItemTextTextBlock);

            }

            if (showCount == 0)
            {
                showZeroItemNumberTextBlock = new TextBlock();
                showZeroItemNumberTextBlock.Text = "0";
                showZeroItemNumberTextBlock.FontFamily = new FontFamily("Veranda");
                showZeroItemNumberTextBlock.FontWeight = FontWeights.Bold;
                showZeroItemNumberTextBlock.FontSize = 50.0;
                showZeroItemNumberTextBlock.Foreground = new SolidColorBrush(Color.FromRgb(110, 110, 110));

                showZeroItemTextTextBlock = new TextBlock();
                showZeroItemTextTextBlock.Text = "Items Shown";
                showZeroItemTextTextBlock.FontFamily = new FontFamily("Veranda");
                showZeroItemTextTextBlock.FontWeight = FontWeights.Bold;
                showZeroItemTextTextBlock.FontSize = 10.0;
                showZeroItemTextTextBlock.Foreground = new SolidColorBrush(Color.FromRgb(110, 110, 110));
                
                Canvas.SetTop(showZeroItemNumberTextBlock, 120);
                if (shareCount == 0)
                    Canvas.SetLeft(showZeroItemNumberTextBlock, 85 + 42.5 - 20);
                else
                    Canvas.SetLeft(showZeroItemNumberTextBlock, 85 + ((this.expandWidth - 105) / 2) - 20);

                Canvas.SetTop(showZeroItemTextTextBlock, 170);
                if (shareCount == 0)
                    Canvas.SetLeft(showZeroItemTextTextBlock, 85 + 42.5 - 35);
                else
                    Canvas.SetLeft(showZeroItemTextTextBlock, 85 + ((this.expandWidth - 105) / 2) - 35);

                this.Children.Add(showZeroItemNumberTextBlock);
                this.Children.Add(showZeroItemTextTextBlock);

            }


            if (showCount != 0 || shareCount != 0)
            {
                this.InitalizeAnimations();

                pullOutStoryboard.Begin(this);
                isExpanded = true;
            }

            if (shareCount != 0)
            {
                shareCollaboratorItemsGrid = new CollaboratorItemsGrid(new Point(85, 10), this.collabHost, shareCount, showCount, true);
                shareCollaboratorItemsGrid.Visibility = Visibility.Hidden;
                this.Children.Add(shareCollaboratorItemsGrid);

                if (eventParent != null)
                    shareCollaboratorItemsGrid.InitializeEvents(eventParent);
            }

            if (showCount != 0)
            {
                showCollaboratorItemsGrid = new CollaboratorItemsGrid(new Point(85, 100), this.collabHost, shareCount, showCount, false);
                showCollaboratorItemsGrid.Visibility = Visibility.Hidden;
                this.Children.Add(showCollaboratorItemsGrid);

                if (eventParent != null)
                    showCollaboratorItemsGrid.InitializeEvents(eventParent);
            }

            UnhideGrid = new System.Windows.Forms.Timer();
            UnhideGrid.Interval = 300;
            UnhideGrid.Tick += new EventHandler(UnhideGridEventHandler);
            UnhideGrid.Start();
        }

        public void FadeOut()
        {
            if (isExpanded)
            {
                pullInStoryboard.Begin(this);
                isExpanded = false;
            }
                
            fadeOutStoryboard.Begin(this);

            /*
            if (shareCollaboratorItemsSummary != null)
            {
                this.Children.Remove(shareCollaboratorItemsSummary);
                shareCollaboratorItemsSummary.Unload();
                shareCollaboratorItemsSummary = null;
            }

            if (showCollaboratorItemsSummary != null)
            {
                this.Children.Remove(showCollaboratorItemsSummary);
                showCollaboratorItemsSummary.Unload();
                showCollaboratorItemsSummary = null;
            }
            */
            
            if (shareCollaboratorItemsGrid != null)
            {
                this.Children.Remove(shareCollaboratorItemsGrid);
                shareCollaboratorItemsGrid.Unload();
                shareCollaboratorItemsGrid = null;
            }

            if (showCollaboratorItemsGrid != null)
            {
                this.Children.Remove(showCollaboratorItemsGrid);
                showCollaboratorItemsGrid.Unload();
                showCollaboratorItemsGrid = null;
            }

            if (showZeroItemNumberTextBlock != null)
            {
                this.Children.Remove(showZeroItemNumberTextBlock);
                showZeroItemNumberTextBlock = null;
            }

            if (shareZeroItemNumberTextBlock != null)
            {
                this.Children.Remove(shareZeroItemNumberTextBlock);
                shareZeroItemNumberTextBlock = null;
            }

            if (showZeroItemTextTextBlock != null)
            {
                this.Children.Remove(showZeroItemTextTextBlock);
                showZeroItemTextTextBlock = null;
            }

            if (shareZeroItemTextTextBlock != null)
            {
                this.Children.Remove(shareZeroItemTextTextBlock);
                shareZeroItemTextTextBlock = null;
            }
            

        }

        void ExpandButtonClickEventHandler(object sender, ExpandButtonEventArgs e)
        {
            if (isExpanded)
            {
                pullInStoryboard.Begin(this);
                isExpanded = false;

                
                if (shareCollaboratorItemsGrid != null)
                {
                    this.Children.Remove(shareCollaboratorItemsGrid);
                    shareCollaboratorItemsGrid.Unload();
                    shareCollaboratorItemsGrid = null;
                }

                if (showCollaboratorItemsGrid != null)
                {
                    this.Children.Remove(showCollaboratorItemsGrid);
                    showCollaboratorItemsGrid.Unload();
                    showCollaboratorItemsGrid = null;
                }
                
                /*
                shareCollaboratorItemsSummary = new CollaboratorItemsSummary(new Point(85, 10), this.collabHost, this.collabName, true);
                shareCollaboratorItemsSummary.Visibility = Visibility.Hidden;
                this.Children.Add(shareCollaboratorItemsSummary);

                showCollaboratorItemsSummary = new CollaboratorItemsSummary(new Point(85, 100), this.collabHost, this.collabName, false);
                showCollaboratorItemsSummary.Visibility = Visibility.Hidden;
                this.Children.Add(showCollaboratorItemsSummary);

                UnhideSummary = new System.Windows.Forms.Timer();
                UnhideSummary.Interval = 850;
                UnhideSummary.Tick += new EventHandler(UnhideSummaryEventHandler);
                UnhideSummary.Start();
                */
            }
            else
            {
                int shareCount = (int)this.sharedArtifactsTableAdapter.GetCountByGroupandShareLevel(this.collabHost, true);
                int showCount = (int)this.sharedArtifactsTableAdapter.GetCountByGroupandShareLevel(this.collabHost, false);

                if (showCount == 0 && shareCount == 0)
                    return;

                this.expandWidth = 155 + (Math.Min(Math.Max(showCount, shareCount), 5) * 90);
                this.InitalizeAnimations();

                pullOutStoryboard.Begin(this);
                isExpanded = true;

                /*
                if (shareCollaboratorItemsSummary != null)
                {
                    this.Children.Remove(shareCollaboratorItemsSummary);
                    shareCollaboratorItemsSummary.Unload();
                    shareCollaboratorItemsSummary = null;
                }

                if (showCollaboratorItemsSummary != null)
                {
                    this.Children.Remove(showCollaboratorItemsSummary);
                    showCollaboratorItemsSummary.Unload();
                    showCollaboratorItemsSummary = null;
                }
                */
                
                shareCollaboratorItemsGrid = new CollaboratorItemsGrid(new Point(85, 10), this.collabHost, shareCount, showCount, true);
                shareCollaboratorItemsGrid.Visibility = Visibility.Hidden;
                this.Children.Add(shareCollaboratorItemsGrid);

                showCollaboratorItemsGrid = new CollaboratorItemsGrid(new Point(85, 100), this.collabHost, shareCount, showCount, false);
                showCollaboratorItemsGrid.Visibility = Visibility.Hidden;
                this.Children.Add(showCollaboratorItemsGrid);

                if (eventParent != null)
                {
                    shareCollaboratorItemsGrid.InitializeEvents(eventParent);
                    showCollaboratorItemsGrid.InitializeEvents(eventParent);
                }
                UnhideGrid = new System.Windows.Forms.Timer();
                UnhideGrid.Interval = 800;
                UnhideGrid.Tick += new EventHandler(UnhideGridEventHandler);
                UnhideGrid.Start();
                

            }
        }

        void UnhideSummaryEventHandler(object sender, EventArgs e)
        {
            UnhideSummary.Stop();
            UnhideSummary.Dispose();

            /*
            if(shareCollaboratorItemsSummary != null)
                shareCollaboratorItemsSummary.Visibility = Visibility.Visible;

            if (showCollaboratorItemsSummary != null)
                showCollaboratorItemsSummary.Visibility = Visibility.Visible;
            */
        }

        
        void UnhideGridEventHandler(object sender, EventArgs e)
        {
            UnhideGrid.Stop();
            UnhideGrid.Dispose();

            if(shareCollaboratorItemsGrid != null)
                shareCollaboratorItemsGrid.Visibility = Visibility.Visible;

            if (showCollaboratorItemsGrid != null)
                showCollaboratorItemsGrid.Visibility = Visibility.Visible;
        }



        


        void IMClickEventHandler(object sender, System.Windows.RoutedEventArgs e)
        {
            System.DateTime currentTime = System.DateTime.Now;


            if ((lastClickTime != null) && (currentTime - lastClickTime < doubleClickInterval))
            {

                Console.WriteLine("double click");

                try
                {

                    MessengerAPI.IMessengerContact contact = (MessengerAPI.IMessengerContact)Program.LiveMessenger.GetContact(this.collabIM, Program.LiveMessenger.MyServiceId);

                    Program.LiveMessenger.InstantMessage(contact);
                }
                catch
                {
                    MessageBox.Show("Unable to initate IM with this person");
                }


            }

            lastClickTime = currentTime;

        }

        void collaboratorBubbleCreateImageEventHandler(object sender, MouseButtonEventArgs e)
        {
            System.DateTime currentTime = System.DateTime.Now;


            if ((lastClickTime != null) && (currentTime - lastClickTime < doubleClickInterval))
            {

                Console.WriteLine("double click");

                try
                {
                    CollaboratorBubbleCreate window = new CollaboratorBubbleCreate();
                    window.Show();

                }
                catch
                {
                    MessageBox.Show("Unable to initate bubble create dialog");
                }


            }

            lastClickTime = currentTime;
        }
        
        public void UpdateLiveMessengerStatus(string serviceName, MessengerAPI.MISTATUS status)
        {
            if (serviceName == this.collabIM)
            {
                if (Program.LiveMessenger != null)
                {


                    if (status == MessengerAPI.MISTATUS.MISTATUS_ONLINE)
                        imStatusIconImage.Source = messengerOnline;

                    else if (status == MessengerAPI.MISTATUS.MISTATUS_IDLE ||
                            status == MessengerAPI.MISTATUS.MISTATUS_AWAY ||
                            status == MessengerAPI.MISTATUS.MISTATUS_BE_RIGHT_BACK ||
                            status == MessengerAPI.MISTATUS.MISTATUS_OUT_TO_LUNCH)
                        imStatusIconImage.Source = messengerAway;

                    else if (status == MessengerAPI.MISTATUS.MISTATUS_BUSY ||
                       status == MessengerAPI.MISTATUS.MISTATUS_ON_THE_PHONE)
                        imStatusIconImage.Source = messengerBusy;

                    else
                        imStatusIconImage.Source = messengerOffline;


                }

            }
        }
        



    }
}
