using System;
using System.IO;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using System.Windows.Media.Imaging;
using Microsoft.Win32;

using ClubDev.Database;
using ClubDev.Database.ClubDevDataSetTableAdapters;


namespace ClubDev.ConfigurationDialogs
{
    public delegate void DelegateThreadFinished();

    public partial class CheckingDatabaseWindow : Form
    {

        Thread workerThread;

        // events used to stop worker thread
        ManualResetEvent eventStopThread;
        ManualResetEvent eventThreadStopped;

        // Delegate instance
        public DelegateThreadFinished delegateThreadFinished;

        private ClubDevDataSet clubDevDataSet;
        private SessionsTableAdapter sessionsTableAdapter;


        public CheckingDatabaseWindow()
        {
            InitializeComponent();

            // initialize delegates
            delegateThreadFinished = new DelegateThreadFinished(this.ThreadFinished);

            // initialize events
            eventStopThread = new ManualResetEvent(false);
            eventThreadStopped = new ManualResetEvent(false);

            clubDevDataSet = new ClubDevDataSet();
            sessionsTableAdapter = new SessionsTableAdapter();

        }


        private void CheckingDatabaseWindow_Load(object sender, EventArgs e)
        {
            eventStopThread.Reset();
            eventThreadStopped.Reset();

            // create worker thread instance
            workerThread = new Thread(new ThreadStart(this.WorkerThreadFunction));

            workerThread.Name = "Check and Initalize Database Thread";

            workerThread.Start();
        }

        private void WorkerThreadFunction()
        {

            try
            {

                sessionsTableAdapter.ClearBeforeFill = true;
                sessionsTableAdapter.Fill(clubDevDataSet.Sessions);

                Surogates.CollaborationSessionSurogate[] existingSessions = new ClubDev.Surogates.CollaborationSessionSurogate[clubDevDataSet.Sessions.Count];

                for (int i = 0; i < clubDevDataSet.Sessions.Count; i++)
                {
                    existingSessions[i] = new ClubDev.Surogates.CollaborationSessionSurogate(clubDevDataSet.Sessions[i].sessionID,
                        clubDevDataSet.Sessions[i].sessionName, clubDevDataSet.Sessions[i].sessionLocation, clubDevDataSet.Sessions[i].sessionPassword);
                }

                ClubDev.Program.AvalialableSessions = existingSessions;
            }

            catch
            {
                DialogResult results = MessageBox.Show("Could not connect to database.  Please insure that you are connected to the network and that you have the appropraite ports open in your firewall settings.  \n\n Would you like to update your server settings?", "Database Connection Error", MessageBoxButtons.YesNo, MessageBoxIcon.Error);

                if (results == DialogResult.Yes)
                    ClubDev.Program.AllowRentryOfServerInfo = true;

            }

            

            // check if thread is cancelled
            if (eventStopThread.WaitOne(0, true))
            {

                // inform main thread that this thread stopped
                eventThreadStopped.Set();

                return;
            }

            // Make synchronous call to main form
            // to inform it that thread finished
            this.Invoke(this.delegateThreadFinished, null);
        }

        // Stop worker thread if it is running.
        // Called when user presses Stop button or form is closed.
        private void StopThread()
        {
            if (workerThread != null && workerThread.IsAlive)  // thread is active
            {
                // set event "Stop"
                eventStopThread.Set();

                // wait when thread  will stop or finish
                while (workerThread.IsAlive)
                {
                    // We cannot use here infinite wait because our thread
                    // makes syncronous calls to main form, this will cause deadlock.
                    // Instead of this we wait for event some appropriate time
                    // (and by the way give time to worker thread) and
                    // process events. These events may contain Invoke calls.
                    if (WaitHandle.WaitAll(
                        (new ManualResetEvent[] { eventThreadStopped }),
                        100,
                        true))
                    {
                        break;
                    }

                    Application.DoEvents();
                }
            }
        }


        // Set initial state of controls.
        // Called from worker thread using delegate and Control.Invoke
        private void ThreadFinished()
        {
            this.Close();
        }
    }

}