using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using Microsoft.Win32;

namespace ClubDev.ConfigurationDialogs
{
    public partial class ServerSettingsWindow : Form
    {
        string password;
        string userImageFileName;

        public ServerSettingsWindow()
        {
            InitializeComponent();
        }

        private void ServerSettingsWindow_Load(object sender, EventArgs e)
        {
            RegistryKey clubDevRegistryKey = Registry.CurrentUser.CreateSubKey("SOFTWARE\\UIUC\\ORCHID\\ClubDev");

            string collaboratorType = (string)clubDevRegistryKey.GetValue("collaboratorType");
            if (collaboratorType == "sharedScreen")
            {
                this.SharedScreenRadioButton.Checked = true;
                this.collabotatorNameLabel.Text = "Name of Shared Screen:";
            }
            else if (collaboratorType == "collaborator")
            {
                this.CollaboratorRadioButton.Checked = true;
                this.collabotatorNameLabel.Text = "Collaborator Name (this is the name that is used to identify you):";
            }


            this.databaseServerHostName.Text = (string)clubDevRegistryKey.GetValue("databaseServerHostName");
            this.databaseTableName.Text = (string)clubDevRegistryKey.GetValue("databaseTableName");
            this.databaseUsername.Text = (string)clubDevRegistryKey.GetValue("databaseUsername");
            this.collaboratorName.Text = (string)clubDevRegistryKey.GetValue("collaboratorName");
            this.collabotatorIM.Text = (string)clubDevRegistryKey.GetValue("collaboratorIM");
            this.userImageFileName = (string)clubDevRegistryKey.GetValue("collaboratorImage");

            password = (string)clubDevRegistryKey.GetValue("databasePassword");
            if(password != null)
                this.databasePassword.Text = "                ";
            
            clubDevRegistryKey.Close();

            if (collaboratorType == "collaborator")
            {
                try
                {
                    Bitmap bitmap = new Bitmap(this.userImageFileName);

                    userIconPictureBox.Image = bitmap;
                    userIconPictureBox.SizeMode = PictureBoxSizeMode.StretchImage;

                    //userImageFileName = userIconFileDialog.FileName;
                }
                catch
                { }
            }
            else
            {
                userIconPictureBox.Visible = false;
                userIconButton.Visible = false;
            }
        }

        void databasePassword_GotFocus(object sender, System.EventArgs e)
        {
            this.databasePassword.Text = null;
        }

        private void saveButton_Click(object sender, EventArgs e)
        {
            RegistryKey clubDevRegistryKey = Registry.CurrentUser.CreateSubKey("SOFTWARE\\UIUC\\ORCHID\\ClubDev");

            if(this.SharedScreenRadioButton.Checked)
                clubDevRegistryKey.SetValue("collaboratorType", "sharedScreen");
            else
                clubDevRegistryKey.SetValue("collaboratorType", "collaborator");

            clubDevRegistryKey.SetValue("databaseServerHostName", this.databaseServerHostName.Text);
            clubDevRegistryKey.SetValue("databaseTableName", this.databaseTableName.Text);
            clubDevRegistryKey.SetValue("databaseUsername", this.databaseUsername.Text);
            clubDevRegistryKey.SetValue("collaboratorName", this.collaboratorName.Text);
            clubDevRegistryKey.SetValue("collaboratorIM", this.collabotatorIM.Text);

            if (!this.SharedScreenRadioButton.Checked)
                clubDevRegistryKey.SetValue("collaboratorImage", this.userImageFileName);

            if(this.databasePassword.Text.Equals("                "))
                clubDevRegistryKey.SetValue("databasePassword", password);
            else
                clubDevRegistryKey.SetValue("databasePassword", this.databasePassword.Text);


            clubDevRegistryKey.Close();

            Program.RentryAfterSuccess = false;

            this.Close();
        }

        private void cancelButton_Click(object sender, EventArgs e)
        {
            ClubDev.Program.AllowRentryOfServerInfo = false;
            this.Close();
        }

        private void CollaboratorRadioButton_CheckedChanged(object sender, EventArgs e)
        {
            this.collabotatorNameLabel.Text = "Collaborator Name (this is the name that is used to identify you):";

            this.userIconPictureBox.Visible = true;
            this.userIconButton.Visible = true;

            try
            {
                Bitmap bitmap = new Bitmap(this.userImageFileName);

                userIconPictureBox.Image = bitmap;
                userIconPictureBox.SizeMode = PictureBoxSizeMode.StretchImage;

                userImageFileName = userIconFileDialog.FileName;
            }
            catch
            { }
            
        }

        private void SharedScreenRadioButton_CheckedChanged(object sender, EventArgs e)
        {
            this.collabotatorNameLabel.Text = "Name of Shared Screen:";
            this.userIconPictureBox.Visible = false;
            this.userIconButton.Visible = false;
        }

        private void userIconButton_Click(object sender, EventArgs e)
        {
            userIconFileDialog.AddExtension = true;
            userIconFileDialog.Multiselect = false;
            userIconFileDialog.CheckFileExists = true;
            userIconFileDialog.CheckPathExists = true;
            userIconFileDialog.Filter = "Portable Network Graphic (*.png)|*.png";

            userIconFileDialog.FileName = this.userImageFileName;

            DialogResult result = userIconFileDialog.ShowDialog();

            if (result == DialogResult.OK)
            {
                try
                {
                    Bitmap bitmap = new Bitmap(userIconFileDialog.FileName);

                    userIconPictureBox.Image = bitmap;
                    userIconPictureBox.SizeMode = PictureBoxSizeMode.StretchImage;

                    userImageFileName = userIconFileDialog.FileName;
                }
                catch
                {
                    MessageBox.Show("Error! Could not open user icon file!", "Error!", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }


        }


    }
}
