using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Microsoft.Win32;

using ClubDev.Database;
using ClubDev.Database.ClubDevDataSetTableAdapters;

namespace ClubDev.ConfigurationDialogs
{
    public partial class SessionSelectionWindow : Form
    {
        private string databaseServerHostName;
        private string databaseTableName;
        private string databaseUsername;
        private string collaboratorName;
        private string databasePassword;

        private ClubDevDataSet clubDevDataSet;
        private SessionsTableAdapter sessionsTableAdapter;

        private Surogates.CollaborationSessionSurogate chosenSession;


        public SessionSelectionWindow()
        {
            InitializeComponent();

            clubDevDataSet = new ClubDevDataSet();
            sessionsTableAdapter = new SessionsTableAdapter();
        }

        private void SessionSelectionWindow_Load(object sender, EventArgs e)
        {
            this.GetEstablishedSessions();
        }

        private void cancelButton_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void settingButton_Click(object sender, EventArgs e)
        {
            Program.AllowRentryOfServerInfo = true;
            Program.RentryAfterSuccess = true;
            this.Close();

        }

        private void retrieveServerSettingsFromRegistry()
        {
            RegistryKey clubDevRegistryKey = Registry.CurrentUser.CreateSubKey("SOFTWARE\\UIUC\\ORCHID\\ClubDev");

            databaseServerHostName = (string)clubDevRegistryKey.GetValue("databaseServerHostName");
            databaseTableName = (string)clubDevRegistryKey.GetValue("databaseTableName");
            databaseUsername = (string)clubDevRegistryKey.GetValue("databaseUsername");
            collaboratorName = (string)clubDevRegistryKey.GetValue("collaboratorName");
            databasePassword = (string)clubDevRegistryKey.GetValue("databasePassword");

            clubDevRegistryKey.Close();

        }

        private void establishSession_Click(object sender, EventArgs e)
        {
            if (!this.establishSessionPassword.Text.Equals(this.establishSessionPasswordConfirm.Text))
            {
                MessageBox.Show("Password and Password Confirm values do not match. Please retype passwords and try again.", "Password Error!", MessageBoxButtons.OK, MessageBoxIcon.Error);
                this.establishSessionPassword.Text = null;
                this.establishSessionPasswordConfirm.Text = null;
                return;
            }

            if (this.establishSessionName.Text == "" || this.establishSessionName.Text == null)
            {
                MessageBox.Show("You must specify a session name.", "Session Name Required!", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (this.establishSessionLocation.Text == "" || this.establishSessionLocation.Text == null)
            {
                MessageBox.Show("You must specify a location name.", "Session Location Required!", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            object returnIDObject = sessionsTableAdapter.InsertandReturnIdentity(this.establishSessionName.Text, this.establishSessionLocation.Text, this.establishSessionPassword.Text);
            long returnID = (long)Convert.ChangeType(returnIDObject, typeof(long));

            Surogates.CollaborationSessionSurogate createdSession = new Surogates.CollaborationSessionSurogate(returnID, this.establishSessionName.Text, this.establishSessionLocation.Text, this.establishSessionPassword.Text);

            ClubDev.Program.ChosenSession = createdSession;
            
            this.Close();
        }

        private void joinRefreshButton_Click(object sender, EventArgs e)
        {
            this.GetEstablishedSessions();
        }

        private void joinSessionNameandLocation_SelectedIndexChanged(object sender, EventArgs e)
        {
            Surogates.CollaborationSessionSurogate chosenSession = new ClubDev.Surogates.CollaborationSessionSurogate(
                clubDevDataSet.Sessions[this.joinSessionNameandLocation.SelectedIndex].sessionID,
                clubDevDataSet.Sessions[this.joinSessionNameandLocation.SelectedIndex].sessionName,
                clubDevDataSet.Sessions[this.joinSessionNameandLocation.SelectedIndex].sessionLocation,
                clubDevDataSet.Sessions[this.joinSessionNameandLocation.SelectedIndex].sessionPassword);

            this.chosenSession = chosenSession;
        }

        private void joinSession_Click(object sender, EventArgs e)
        {

            if (this.chosenSession == null)
                return;

            if (this.chosenSession.SessionPassword != this.joinSessionPassword.Text)
            {
                MessageBox.Show("Incorrect password! Please type in password again.", "Incorrect Password", MessageBoxButtons.OK, MessageBoxIcon.Error);
                this.joinSessionPassword.Text = null;
            }
            else
            {
                ClubDev.Program.ChosenSession = this.chosenSession;
                this.Close();
            }
        }

        private void GetEstablishedSessions()
        {
            joinSessionNameandLocation.Items.Clear();

            sessionsTableAdapter.Fill(clubDevDataSet.Sessions);

            int maxLength = 0;
            for (int i = 0; i < clubDevDataSet.Sessions.Count; i++)
            {
                int currentLength = clubDevDataSet.Sessions[i].sessionName.Length;
                if (currentLength > maxLength)
                    maxLength = currentLength;
            }

            maxLength += 3;

            for (int i = 0; i < clubDevDataSet.Sessions.Count; i++)
            {
                string space = "";
                for (int j = 0; j < (maxLength - clubDevDataSet.Sessions[i].sessionName.Length); j++)
                    space = space + " ";

                joinSessionNameandLocation.Items.Add(clubDevDataSet.Sessions[i].sessionName + space + clubDevDataSet.Sessions[i].sessionLocation);
            }
        }

    }
}