using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Shapes;
using System.Windows.Media.Animation;

namespace ClubDev
{
    class ExpandButton : Canvas
    {
        public delegate void ExpandButtonClickHandler(object sender, ExpandButtonEventArgs e);
        public event ExpandButtonClickHandler Click;

        private Ellipse ellipseBackground;
        private Line topOfArrow;
        private Line bottomOfArrow;

        private String parentType;

        public ExpandButton(Point margin, String parentType)
        {
            this.Width = 20;
            this.Height = 20;
            this.parentType = parentType;

            ellipseBackground = new Ellipse();
            ellipseBackground.Width = this.Width;
            ellipseBackground.Height = this.Height;
            ellipseBackground.StrokeThickness = 2;

            topOfArrow = new Line();
            topOfArrow.X1 = 6;
            topOfArrow.Y1 = 6;
            topOfArrow.X2 = 14;
            topOfArrow.Y2 = 10;

            bottomOfArrow = new Line();
            bottomOfArrow.X1 = 6;
            bottomOfArrow.Y1 = 14;
            bottomOfArrow.X2 = 14;
            bottomOfArrow.Y2 = 10;

            topOfArrow.StrokeThickness = 2.5;
            bottomOfArrow.StrokeThickness = 2.5;
            topOfArrow.StrokeLineJoin = PenLineJoin.Round;
            bottomOfArrow.StrokeLineJoin = PenLineJoin.Round;

            if (parentType.Equals("collaborator"))
            {
                ellipseBackground.Stroke = new SolidColorBrush(Color.FromRgb(80, 80, 80));
                ellipseBackground.Fill = new SolidColorBrush(Color.FromRgb(130, 130, 130));

                topOfArrow.Stroke = new SolidColorBrush(Color.FromRgb(220, 220, 220));
                bottomOfArrow.Stroke = new SolidColorBrush(Color.FromRgb(220, 220, 220));
            }
            else
            {
                ellipseBackground.Stroke = new SolidColorBrush(Color.FromRgb(0, 100, 120));
                ellipseBackground.Fill = new SolidColorBrush(Color.FromRgb(0, 70, 110));

                topOfArrow.Stroke = new SolidColorBrush(Color.FromRgb(220, 220, 220));
                bottomOfArrow.Stroke = new SolidColorBrush(Color.FromRgb(220, 220, 220));
            }






            this.Children.Add(ellipseBackground);
            this.Children.Add(topOfArrow);
            this.Children.Add(bottomOfArrow);

            this.Margin = new Thickness(margin.X - (this.Width / 2), margin.Y - (this.Height / 2), 0, 0);

            this.MouseEnter += new System.Windows.Input.MouseEventHandler(MouseEnterEventHandler);
            this.MouseLeave += new System.Windows.Input.MouseEventHandler(MouseLeaveEventHandler);
            this.MouseDown += new System.Windows.Input.MouseButtonEventHandler(MouseDownEventHandler);
            this.MouseUp += new System.Windows.Input.MouseButtonEventHandler(MouseUpEventHandler);


        }

    
        void MouseEnterEventHandler(object sender, System.Windows.Input.MouseEventArgs e)
        {
            if (parentType.Equals("collaborator"))
            {
                ellipseBackground.Fill = new SolidColorBrush(Color.FromRgb(180, 180, 180));
                topOfArrow.Stroke = new SolidColorBrush(Color.FromRgb(240, 240, 240));
                bottomOfArrow.Stroke = new SolidColorBrush(Color.FromRgb(240, 240, 240));
            }
            else
            {
                ellipseBackground.Fill = new SolidColorBrush(Color.FromRgb(0, 200, 235));
                topOfArrow.Stroke = new SolidColorBrush(Color.FromRgb(240, 240, 240));
                bottomOfArrow.Stroke = new SolidColorBrush(Color.FromRgb(240, 240, 240));
            }
        }

        void MouseLeaveEventHandler(object sender, System.Windows.Input.MouseEventArgs e)
        {
            if (parentType.Equals("collaborator"))
            {
                ellipseBackground.Fill = new SolidColorBrush(Color.FromRgb(130, 130, 130));
                topOfArrow.Stroke = new SolidColorBrush(Color.FromRgb(220, 220, 220));
                bottomOfArrow.Stroke = new SolidColorBrush(Color.FromRgb(220, 220, 220));
            }
            else
            {
                ellipseBackground.Fill = new SolidColorBrush(Color.FromRgb(0, 70, 110));
                topOfArrow.Stroke = new SolidColorBrush(Color.FromRgb(220, 220, 220));
                bottomOfArrow.Stroke = new SolidColorBrush(Color.FromRgb(220, 220, 220));
            }
        }

        void MouseDownEventHandler(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            if (parentType.Equals("collaborator"))
            {
                ellipseBackground.Fill = new SolidColorBrush(Color.FromRgb(240, 240, 240));
                topOfArrow.Stroke = new SolidColorBrush(Color.FromRgb(130, 130, 130));
                bottomOfArrow.Stroke = new SolidColorBrush(Color.FromRgb(130, 130, 130));
            }
            else
            {
                ellipseBackground.Fill = new SolidColorBrush(Color.FromRgb(240, 240, 240));
                topOfArrow.Stroke = new SolidColorBrush(Color.FromRgb(0, 100, 120));
                bottomOfArrow.Stroke = new SolidColorBrush(Color.FromRgb(0, 100, 120));
            }
        }

        void MouseUpEventHandler(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            if (parentType.Equals("collaborator"))
            {
                ellipseBackground.Fill = new SolidColorBrush(Color.FromRgb(130, 130, 130));
                topOfArrow.Stroke = new SolidColorBrush(Color.FromRgb(220, 220, 220));
                bottomOfArrow.Stroke = new SolidColorBrush(Color.FromRgb(220, 220, 220));
            }
            else
            {
                ellipseBackground.Fill = new SolidColorBrush(Color.FromRgb(0, 70, 120));
                topOfArrow.Stroke = new SolidColorBrush(Color.FromRgb(220, 220, 220));
                bottomOfArrow.Stroke = new SolidColorBrush(Color.FromRgb(220, 220, 220));
            }


            ExpandButtonEventArgs eventArgs = new ExpandButtonEventArgs(this);

            if (this.Click != null)
                Click(this, eventArgs);

        }


    }


    class ExpandButtonEventArgs : EventArgs
    {
        private Object sender = null;

        public ExpandButtonEventArgs(Object sender)
        {
            this.sender = sender;
        }

        public Object Sender
        {
            get { return this.sender; }
        }
    }
}
