using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Shapes;
using System.Windows.Media.Animation;

namespace ClubDev
{
    class ScrollButton : Canvas
    {
        public delegate void ScrollButtonClickHandler(object sender, ScrollButtonEventArgs e);
        public event ScrollButtonClickHandler Click;

        private Ellipse ellipseBackground;
        private Line topOfArrowLeft;
        private Line bottomOfArrowLeft;
        private Line topOfArrowRight;
        private Line bottomOfArrowRight;

        private String parentType;
        private bool enabled;

        public bool Enabled
        {
            get
            {
                return enabled;
            }
            set
            {
                enabled = value;

                if (enabled)
                {
                    topOfArrowLeft.Stroke = new SolidColorBrush(Color.FromRgb(220, 220, 220));
                    bottomOfArrowLeft.Stroke = new SolidColorBrush(Color.FromRgb(220, 220, 220));
                    topOfArrowRight.Stroke = new SolidColorBrush(Color.FromRgb(220, 220, 220));
                    bottomOfArrowRight.Stroke = new SolidColorBrush(Color.FromRgb(220, 220, 220));

                }
                else
                {
                    ellipseBackground.Visibility = Visibility.Hidden;
                    topOfArrowLeft.Stroke = new SolidColorBrush(Color.FromRgb(150, 150, 150));
                    bottomOfArrowLeft.Stroke = new SolidColorBrush(Color.FromRgb(150, 150, 150));
                    topOfArrowRight.Stroke = new SolidColorBrush(Color.FromRgb(150, 150, 150));
                    bottomOfArrowRight.Stroke = new SolidColorBrush(Color.FromRgb(150, 150, 150));
                }

            }
        }

        public ScrollButton(Point point)
        {
            this.Width = 20;
            this.Height = 20;
            this.parentType = "collaborator";

            ellipseBackground = new Ellipse();
            ellipseBackground.Width = 20;
            ellipseBackground.Height = this.Height + 8;
            ellipseBackground.StrokeThickness = 0;
            Canvas.SetLeft(ellipseBackground, 0);
            Canvas.SetTop(ellipseBackground, -4);

            GradientBrush highlightBrush = new RadialGradientBrush(Color.FromArgb(180, 255, 255, 255), Color.FromArgb(0, 255, 255, 255));
            ellipseBackground.Fill = highlightBrush;
            ellipseBackground.Visibility = Visibility.Hidden;

            topOfArrowLeft = new Line();
            topOfArrowLeft.X1 = 10;
            topOfArrowLeft.Y1 = 2;
            topOfArrowLeft.X2 = 6;
            topOfArrowLeft.Y2 = 10;

            bottomOfArrowLeft = new Line();
            bottomOfArrowLeft.X1 = 6;
            bottomOfArrowLeft.Y1 = 10;
            bottomOfArrowLeft.X2 = 10;
            bottomOfArrowLeft.Y2 = 18;

            topOfArrowRight = new Line();
            topOfArrowRight.X1 = 14;
            topOfArrowRight.Y1 = 2;
            topOfArrowRight.X2 = 10;
            topOfArrowRight.Y2 = 10;

            bottomOfArrowRight = new Line();
            bottomOfArrowRight.X1 = 10;
            bottomOfArrowRight.Y1 = 10;
            bottomOfArrowRight.X2 = 14;
            bottomOfArrowRight.Y2 = 18;

            topOfArrowLeft.StrokeThickness = 2;
            bottomOfArrowLeft.StrokeThickness = 2;
            topOfArrowLeft.StrokeLineJoin = PenLineJoin.Round;
            bottomOfArrowLeft.StrokeLineJoin = PenLineJoin.Round;

            topOfArrowRight.StrokeThickness = 2;
            bottomOfArrowRight.StrokeThickness = 2;
            topOfArrowRight.StrokeLineJoin = PenLineJoin.Round;
            bottomOfArrowRight.StrokeLineJoin = PenLineJoin.Round;

            topOfArrowLeft.Stroke = new SolidColorBrush(Color.FromRgb(220, 220, 220));
            bottomOfArrowLeft.Stroke = new SolidColorBrush(Color.FromRgb(220, 220, 220));

            topOfArrowRight.Stroke = new SolidColorBrush(Color.FromRgb(220, 220, 220));
            bottomOfArrowRight.Stroke = new SolidColorBrush(Color.FromRgb(220, 220, 220));

            

            






            this.Children.Add(ellipseBackground);
            this.Children.Add(topOfArrowLeft);
            this.Children.Add(bottomOfArrowLeft);
            this.Children.Add(topOfArrowRight);
            this.Children.Add(bottomOfArrowRight);

            Canvas.SetLeft(this, point.X - (this.Width / 2));
            Canvas.SetTop(this, point.Y - (this.Height / 2));

            this.MouseEnter += new System.Windows.Input.MouseEventHandler(MouseEnterEventHandler);
            this.MouseLeave += new System.Windows.Input.MouseEventHandler(MouseLeaveEventHandler);
            this.MouseDown += new System.Windows.Input.MouseButtonEventHandler(MouseDownEventHandler);
            this.MouseUp += new System.Windows.Input.MouseButtonEventHandler(MouseUpEventHandler);


        }

    
        void MouseEnterEventHandler(object sender, System.Windows.Input.MouseEventArgs e)
        {
            if (enabled)
            {
                ellipseBackground.Visibility = Visibility.Visible;
                topOfArrowLeft.Stroke = new SolidColorBrush(Color.FromRgb(240, 240, 240));
                bottomOfArrowLeft.Stroke = new SolidColorBrush(Color.FromRgb(240, 240, 240));
                topOfArrowRight.Stroke = new SolidColorBrush(Color.FromRgb(240, 240, 240));
                bottomOfArrowRight.Stroke = new SolidColorBrush(Color.FromRgb(240, 240, 240));
            }
        }

        void MouseLeaveEventHandler(object sender, System.Windows.Input.MouseEventArgs e)
        {
            if (enabled)
            {
                ellipseBackground.Visibility = Visibility.Hidden;
                topOfArrowLeft.Stroke = new SolidColorBrush(Color.FromRgb(220, 220, 220));
                bottomOfArrowLeft.Stroke = new SolidColorBrush(Color.FromRgb(220, 220, 220));
                topOfArrowRight.Stroke = new SolidColorBrush(Color.FromRgb(220, 220, 220));
                bottomOfArrowRight.Stroke = new SolidColorBrush(Color.FromRgb(220, 220, 220));
            }
        }

        void MouseDownEventHandler(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            if (enabled)
            {
                topOfArrowLeft.Stroke = new SolidColorBrush(Color.FromRgb(130, 130, 130));
                bottomOfArrowLeft.Stroke = new SolidColorBrush(Color.FromRgb(130, 130, 130));
                topOfArrowRight.Stroke = new SolidColorBrush(Color.FromRgb(130, 130, 130));
                bottomOfArrowRight.Stroke = new SolidColorBrush(Color.FromRgb(130, 130, 130));
            }
        }

        void MouseUpEventHandler(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            if (enabled)
            {
                topOfArrowLeft.Stroke = new SolidColorBrush(Color.FromRgb(220, 220, 220));
                bottomOfArrowLeft.Stroke = new SolidColorBrush(Color.FromRgb(220, 220, 220));
                topOfArrowRight.Stroke = new SolidColorBrush(Color.FromRgb(220, 220, 220));
                bottomOfArrowRight.Stroke = new SolidColorBrush(Color.FromRgb(220, 220, 220));
            }


            ScrollButtonEventArgs eventArgs = new ScrollButtonEventArgs(this);

            if (this.Click != null && this.enabled)
                Click(this, eventArgs);

        }


    }


    class ScrollButtonEventArgs : EventArgs
    {
        private Object sender = null;

        public ScrollButtonEventArgs(Object sender)
        {
            this.sender = sender;
        }

        public Object Sender
        {
            get { return this.sender; }
        }
    }
}
