using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Data;

using UIUC.DCS.ORCHID.CuRTClientLibrary;

namespace ClubDev.CuRT
{
    public static class CuRT
    {
        private static CuRTClient.CursorOnEdgeListener onEdgeListener;
        private static CursorColors[] colorScheme;
        private static EdgeDetectionParams edgeDetectionParams;
        private static CuRTClient client = null;

        private static string currentServerHost;
        private static string currentUserName;

        private static System.Windows.Forms.Timer statusChangeTimer;

        private static CuRTData cuRTData;

        private static System.Threading.Thread cuRTServiceThread;


        public static void InitializeCuRTSupport()
        {

            onEdgeListener = new CuRTClient.CursorOnEdgeListener();
            CursorModule.CursorOnEdge += new CursorOnEdgeHandler(onEdgeListener.AnnounceCursorOnEdge);

            edgeDetectionParams = new EdgeDetectionParams(2, 30, 5, 100);


            colorScheme = new CursorColors[10];
            colorScheme[0] = CursorColors.Green;
            colorScheme[1] = CursorColors.Blue;
            colorScheme[2] = CursorColors.Red;
            colorScheme[3] = CursorColors.Yellow;
            colorScheme[4] = CursorColors.Magenta;
            colorScheme[5] = CursorColors.Gray;
            colorScheme[6] = CursorColors.Purple;
            colorScheme[7] = CursorColors.Aqua;
            colorScheme[8] = CursorColors.Brown;
            colorScheme[9] = CursorColors.White;

            cuRTData = new CuRTData();
            cuRTData.ClearExistingServerEntry();
            //cuRTData.OnMessage += new CuRTData.NewMessage(OnMessage);
            //OnMessage();

            statusChangeTimer = new Timer();
            statusChangeTimer.Tick += new EventHandler(statusChangeTimerEventHandler);
            statusChangeTimer.Interval = 10000;
            statusChangeTimer.Start();

        }


        public static void KillCuRTSupport()
        {
            //uRTData.OnMessage -= OnMessage;
            cuRTData = null;

            DisconnectFromServer();
        }

        private static void ConnectToServer(string userName, string serverHost)
        {

                SysTrayIcon.ShowBalloonTip(60000, "Cursor Redirection Enabled", "Cursor redirection services are being hosted by " + serverHost + ".", System.Windows.Forms.ToolTipIcon.Info);

                currentUserName = userName;
                currentServerHost = serverHost;

                client = new CuRTClient(userName, serverHost, KeyModifiers.Control, Keys.Home, colorScheme, 3, edgeDetectionParams);


        }

        static void startCuRTServiceThread()
        {
            new CuRTClient();
        }

        private static void DisconnectFromServer()
        {
            SysTrayIcon.ShowBalloonTip(60000, "Cursor Redirection Disabled", "There are no shared displays connected and cursor redirection has been disabled.", System.Windows.Forms.ToolTipIcon.Info);

            currentServerHost = null;

                if (client != null)
                {
                    client.Disconnect();
                    client = null;
                }

        }


        static void statusChangeTimerEventHandler(object sender, EventArgs e)
        {
            UpdateServerAssignment();
        }


        /*
        private static void OnMessage()
        {

            if (System.Windows.Threading.Dispatcher.CurrentDispatcher.CheckAccess())
                UpdateServerAssignment();
            else
                System.Windows.Threading.Dispatcher.CurrentDispatcher.BeginInvoke(System.Windows.Threading.DispatcherPriority.Normal, (System.Windows.Threading.DispatcherOperationCallback)delegate(object arg)
                {
                    // HANDLED ON UI THREAD
                    UpdateServerAssignment();
                    return null;
                }, null);
        }
        */

        private static void UpdateServerAssignment()
        {
            if (cuRTData == null)
                return;

            DataTable dt = cuRTData.GetCurrentCuRTServer(ClubDev.Program.ChosenSession.SessionID);

            if (dt.Rows.Count == 0 && currentServerHost != null)
                DisconnectFromServer();

            else if (dt.Rows.Count == 1)
            {

                if (dt.Rows[0]["CuRTServer"].ToString().Equals(currentServerHost))
                    return;

                else
                {
                    //DisconnectFromServer();
                    ConnectToServer(ClubDev.Program.CurrentUser.UserName, dt.Rows[0]["CuRTServer"].ToString());
                }
            }

            else if (dt.Rows.Count > 1)
                MessageBox.Show("ERROR!!!  >1 CuRT SERVERS FOUND!!");

        }



        public static void SendInputToHost(string sourceUsername, string destinationUsername)
        {
            if (client == null)
                MessageBox.Show("Could not connect to redirect server.  Perhaps ClubDev has not fully started up...", "CuRT Framework Error", MessageBoxButtons.OK, MessageBoxIcon.Warning);

            else
            {
                if (Program.CurrentUser.UserName.Equals(sourceUsername))
                    client.SendLocalCursorToRemoteMachine(destinationUsername, new System.Drawing.Point(200, 200));

                else if (Program.CurrentUser.UserName.Equals(destinationUsername))
                    client.SendRemoteCursorToOwnMachine(destinationUsername, new System.Drawing.Point(200, 200));

                else
                    client.SendRemoteCursorToRemoteMachine(sourceUsername, destinationUsername, new System.Drawing.Point(200, 200));
            }
        }
    }
}
