using System;
using System.IO;
using System.Threading;
using System.ComponentModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Shapes;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using System.Reflection;

using System.Data.SqlClient;
using System.Data;

using ClubDev.Database;
using ClubDev.Database.ClubDevDataSetTableAdapters;

namespace ClubDev
{
    class ArtifactsGatherEngine
    {
        private static BackgroundWorker worker;

        private static SharedArtifactSurogate[] sharedArtifactSurogate;
        private static BitmapSource defaultScreenShot;
        private static BitmapSource defaultIcon;

        private static ClubDevDataSet clubDevDataSet;

        private static SharedArtifactsTableAdapter sharedArtifactsTableAdapter;
        private static UsersTableAdapter usersTableAdapter;
        

        private static bool isInitialized = false;

        public enum RetreiveType
        {
            Share, Show, Both
        };


        public static BackgroundWorker Worker
        {
            get
            {
                return worker;
            }
        }

        public static SharedArtifactSurogate[] SharedArtifactSurogate
        {
            get
            {
                return sharedArtifactSurogate;
            }
        }


        public static void Initialize()
        {
            try
            {
                Assembly assem = Assembly.GetExecutingAssembly();

                System.IO.StreamReader stream = new StreamReader(assem.GetManifestResourceStream("ClubDev.Icons.defaultScreenShot.jpg"));
                JpegBitmapDecoder bitmapDecoder = new JpegBitmapDecoder(stream.BaseStream, BitmapCreateOptions.PreservePixelFormat, BitmapCacheOption.None);
                defaultScreenShot = bitmapDecoder.Frames[0];
            }
            catch
            {
                Console.WriteLine("Could not import embeded resource");
            }

            try
            {
                Assembly assem = Assembly.GetExecutingAssembly();

                System.IO.StreamReader stream = new StreamReader(assem.GetManifestResourceStream("ClubDev.Icons.defaultIcon.png"));
                PngBitmapDecoder bitmapDecoder = new PngBitmapDecoder(stream.BaseStream, BitmapCreateOptions.PreservePixelFormat, BitmapCacheOption.None);
                defaultIcon = bitmapDecoder.Frames[0];
            }
            catch
            {
                Console.WriteLine("Could not import embeded resource");
            }

            clubDevDataSet = new ClubDevDataSet();
            sharedArtifactsTableAdapter = new SharedArtifactsTableAdapter();
            usersTableAdapter = new UsersTableAdapter();

            isInitialized = true;
        }

        public static void CleanCache()
        {
            if (sharedArtifactSurogate != null)
            {

                try
                {
                    for (int i = 0; i < sharedArtifactSurogate.Length; i++)
                    {
                        sharedArtifactSurogate[i].IconSource = null;
                        sharedArtifactSurogate[i].ScreenShotSource = null;
                        sharedArtifactSurogate[i] = null;
                    }

                    sharedArtifactSurogate = null;

                    Console.WriteLine("CACHE -- CLEANED");

                }
                catch
                {
                    Console.WriteLine("COULD NOT CLEAN CACHE");
                }

                GC.Collect();
            }
        }




        public static void RetrieveArtifacts(string host, RetreiveType retreiveType)
        {
            if (!isInitialized)
                Initialize();

            Console.WriteLine("INSIDE RETRIEVE ARTIFACTS");

            worker = new BackgroundWorker();
            worker.WorkerReportsProgress = true;
            worker.WorkerSupportsCancellation = true;

            worker.DoWork += delegate(object s, DoWorkEventArgs args)
                {
                    BackgroundWorker delagateWorker = s as BackgroundWorker;

                    if (delagateWorker.CancellationPending)
                    {
                        Console.WriteLine("CANCEL PENDING");

                        args.Cancel = true;
                        return;
                    }

                    Console.WriteLine("CALLING DELEGATE WORKER");
                    RetreiveThumbnailsFromServer(delagateWorker, host, retreiveType);

                };

            worker.RunWorkerAsync();

            
        }

        public static void AbortRetrieveArtifacts()
        {
            worker.CancelAsync();
        }

        public static void RetreiveThumbnailsFromServer(BackgroundWorker delagateWorker, String host, RetreiveType retreiveType)
        {
            while (sharedArtifactSurogate != null)
            {
                Console.WriteLine("waiting...");
                System.Threading.Thread.Sleep(50);
            }

            // lock out other threads from access
            sharedArtifactSurogate = new SharedArtifactSurogate[0];


            try
            {
                sharedArtifactsTableAdapter.ClearBeforeFill = true;
                usersTableAdapter.ClearBeforeFill = true;

                usersTableAdapter.FillByUserHost(clubDevDataSet.Users, host);

                if (clubDevDataSet.Users[0].userIsSharedDisplay)
                    sharedArtifactsTableAdapter.FillByGroupWithReplicationsIncluded(clubDevDataSet.SharedArtifacts, host);
                else if (retreiveType == RetreiveType.Both)
                    sharedArtifactsTableAdapter.FillByGroup(clubDevDataSet.SharedArtifacts, host);
                else if (retreiveType == RetreiveType.Share)
                    sharedArtifactsTableAdapter.FillByGroupandShareLevel(clubDevDataSet.SharedArtifacts, host, true);
                else
                    sharedArtifactsTableAdapter.FillByGroupandShareLevel(clubDevDataSet.SharedArtifacts, host, false);

                Console.WriteLine("found " + clubDevDataSet.SharedArtifacts.Count + " records");

                sharedArtifactSurogate = new SharedArtifactSurogate[clubDevDataSet.SharedArtifacts.Count];

                for (int i = 0; i < sharedArtifactSurogate.Length; i++)
                {


                    sharedArtifactSurogate[i] = new SharedArtifactSurogate();





                    BitmapSource screenShotBitmapSource = null;


                    if (clubDevDataSet.SharedArtifacts[i].IsartifactImageNull())
                        screenShotBitmapSource = defaultScreenShot;

                    else
                    {
                        MemoryStream screenShotStream = new MemoryStream(clubDevDataSet.SharedArtifacts[i].artifactImage);
                        JpegBitmapDecoder screenShotJpegDecoder = new JpegBitmapDecoder(screenShotStream, BitmapCreateOptions.PreservePixelFormat, BitmapCacheOption.None);
                        screenShotBitmapSource = screenShotJpegDecoder.Frames[0];
                    }


                    BitmapSource iconBitmapSource = null;


                    if (clubDevDataSet.SharedArtifacts[i].IsartifactIconNull())
                        iconBitmapSource = defaultIcon;

                    else
                    {
                        MemoryStream iconStream = new MemoryStream(clubDevDataSet.SharedArtifacts[i].artifactIcon);
                        PngBitmapDecoder iconPngDecoder = new PngBitmapDecoder(iconStream, BitmapCreateOptions.PreservePixelFormat, BitmapCacheOption.None);
                        iconBitmapSource = iconPngDecoder.Frames[0];
                    }



                    Size size;
                    if (clubDevDataSet.SharedArtifacts[i].IsartifactWidthNull() || clubDevDataSet.SharedArtifacts[i].IsartifactHeightNull())
                        size = new Size(150, 150);
                    else
                        size = new Size(clubDevDataSet.SharedArtifacts[i].artifactWidth, clubDevDataSet.SharedArtifacts[i].artifactHeight);

                    Point location;
                    if (clubDevDataSet.SharedArtifacts[i].IsartifactLocationXNull() || clubDevDataSet.SharedArtifacts[i].IsartifactLocationYNull())
                        location = new Point(5 * i, 5 * i);
                    else
                        location = new Point(clubDevDataSet.SharedArtifacts[i].artifactLocationX, clubDevDataSet.SharedArtifacts[i].artifactLocationY);


                    int zOrder;
                    if (clubDevDataSet.SharedArtifacts[i].IsartifactZOrderNull())
                        zOrder = i;
                    else
                        zOrder = clubDevDataSet.SharedArtifacts[i].artifactZOrder;



                    sharedArtifactSurogate[i].UpdateArtifact((int)clubDevDataSet.SharedArtifacts[i].artifactID,
                        clubDevDataSet.SharedArtifacts[i].artifactType,
                        clubDevDataSet.SharedArtifacts[i].artifactName,
                        clubDevDataSet.SharedArtifacts[i].artifactGroup,
                        clubDevDataSet.SharedArtifacts[i].artifactHWND,
                        screenShotBitmapSource, iconBitmapSource,
                        size, location, zOrder);


                    delagateWorker.ReportProgress((int)((((double)(i + 1)) / sharedArtifactSurogate.Length) * 100));

                }

                int[] previousSorting = PreferencesEngine.GetPreferences(host);

                if (previousSorting == null)
                    return;

                SharedArtifactSurogate[] sortedSharedArtifactSurogate = new SharedArtifactSurogate[sharedArtifactSurogate.Length];
                int sortedCount = 0;

                for (int i = 0; i < previousSorting.Length; i++)
                {
                    for (int j = 0; j < sharedArtifactSurogate.Length; j++)
                    {
                        if (sharedArtifactSurogate[j] != null)
                        {
                            if (previousSorting[i].Equals(sharedArtifactSurogate[j].ID))
                            {
                                sortedSharedArtifactSurogate[sortedCount] = sharedArtifactSurogate[j];
                                sharedArtifactSurogate[j] = null;
                                sortedCount++;
                            }
                        }

                    }
                }

                for (int j = 0; j < sharedArtifactSurogate.Length; j++)
                {
                    if (sharedArtifactSurogate[j] != null)
                    {
                        sortedSharedArtifactSurogate[sortedCount] = sharedArtifactSurogate[j];
                        sortedCount++;
                    }
                }

                sharedArtifactSurogate = sortedSharedArtifactSurogate;
            }
            catch { }

        }

    }
}
