using System;
using System.IO;
using System.Threading;
using System.ComponentModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Shapes;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using System.Reflection;

using System.Security.Cryptography;
using System.Text;

using System.Data.SqlClient;
using System.Data;

using ClubDev.Utilities;
using ClubDev.Database;
using ClubDev.Database.ClubDevDataSetTableAdapters;


using System.Collections.Generic;
using System.Drawing;
using System.Runtime.InteropServices;
using System.Collections;


namespace ClubDev.Engines
{
    class ClientScreenShotEngine
    {
        private static BackgroundWorker worker;
        private static System.Windows.Forms.Timer timer;
        private static Hashtable screenShotsHash;
        private static Hashtable zOrderHash;
        private static Hashtable locationHash;

        private static ClubDevDataSet clubDevDataSet;
        private static SharedArtifactsTableAdapter sharedArtifactsTableAdapter;

        private static bool isInitialized = false;

        // Options for capturing screenshots
        //
        // WINDOW = Grab and display the whole window, including titlebar (if applicable)
        // CLIENT = Grab only the client area of the window
        // WINDOWSHOWCLIENT = Grab the whole window, but offset it to hide the titlebar
        public enum ScreenCaptureOptions { WINDOW, CLIENT, WINDOWSHOWCLIENT };




        public static BackgroundWorker Worker
        {
            get
            {
                return worker;
            }
        }



        private static void Initialize()
        {
            screenShotsHash = new Hashtable();
            zOrderHash = new Hashtable();
            locationHash = new Hashtable();

            clubDevDataSet = new ClubDevDataSet();

            sharedArtifactsTableAdapter = new SharedArtifactsTableAdapter();


            isInitialized = true;

            timer = new System.Windows.Forms.Timer();
            timer.Interval = 10000;
            timer.Tick += new EventHandler(timerTick);

        }

        private static void timerTick(object sender, EventArgs e)
        {
            SpawnCaptureScreenShotsWorker();
        }



        public static void StartEngine()
        {
            if (!isInitialized)
                Initialize();

            timer.Enabled = true;
            timer.Start();
        }

        public static void AbortStartEngine()
        {
            worker.CancelAsync();
            timer.Enabled = false;
            timer.Stop();
        }

        public static void SpawnCaptureScreenShotsWorker()
        {
            timer.Stop();
            timer.Enabled = false;

            worker = new BackgroundWorker();
            worker.WorkerReportsProgress = true;
            worker.WorkerSupportsCancellation = true;

            worker.DoWork += delegate(object s, DoWorkEventArgs args)
                {
                    BackgroundWorker delagateWorker = s as BackgroundWorker;

                    if (delagateWorker.CancellationPending)
                    {
                        args.Cancel = true;
                        return;
                    }

                    CaptureScreenShots(delagateWorker);

                };

            worker.RunWorkerAsync();

            timer.Enabled = true;
            timer.Start();
        }

        private static void CaptureScreenShots(BackgroundWorker delagateWorker)
        {
            while (sharedArtifactsTableAdapter.Connection.State == System.Data.ConnectionState.Connecting)
                Thread.Sleep(100);

            ClubDevDataSet.SharedArtifactsDataTable sharedArtifactsDataTable = new ClubDevDataSet.SharedArtifactsDataTable();
            sharedArtifactsTableAdapter.FillMETAByGroup(sharedArtifactsDataTable, ClubDev.Program.CurrentUser.UserHost);

            for (int i = 0; i < sharedArtifactsDataTable.Count; i++)
            {
                if (sharedArtifactsDataTable[i].artifactType != 1 || sharedArtifactsDataTable[i].IsartifactHWNDNull())
                    continue;

                try
                {

                    int hwnd = sharedArtifactsDataTable[i].artifactHWND;
                    System.IntPtr hwnd_target = new System.IntPtr(hwnd);

                    Win32Api.CursorInfo cinfo;
                    cinfo = new Win32Api.CursorInfo();
                    cinfo.cbSize = (uint)Marshal.SizeOf(cinfo);
                    Win32Api.GetCursorInfo(out cinfo);
                    IntPtr cursor_handle = new IntPtr(cinfo.hCursor);
                    IntPtr copy_cursor = Win32Api.CopyIcon(cursor_handle);
                    IntPtr foreground_app = Win32Api.GetForegroundWindow();
                    bool copied_cursor = false;

                    // This call will prevent flicker from the PrintWindow call
                    if (hwnd_target == foreground_app)
                    {
                        Win32Api.SetCursor(copy_cursor);
                        copied_cursor = true;
                    }

                    Win32Api.SendMessage(hwnd_target, Win32Api.WM_SETREDRAW, IntPtr.Zero, IntPtr.Zero);

                    Win32Api.Rect rc = new Win32Api.Rect();

                    if (!Win32Api.GetWindowRect(hwnd_target, ref rc))
                        continue;

                    Bitmap screenshot = new Bitmap(rc.Width, rc.Height);
                    Graphics gfxBitmap = Graphics.FromImage(screenshot);
                    IntPtr hdcBitmap = gfxBitmap.GetHdc();
                    Win32Api.PrintWindow(hwnd_target, hdcBitmap, 0);
                    gfxBitmap.ReleaseHdc(hdcBitmap);
                    gfxBitmap.Dispose();


                    int True = 1;
                    IntPtr TRUE = new IntPtr(True);
                    Win32Api.SendMessage(hwnd_target, Win32Api.WM_SETREDRAW, TRUE, IntPtr.Zero);
                    ChildWindowArray warray = new ChildWindowArray(hwnd_target);
                    foreach (int win in warray)
                    {
                        Win32Api.InvalidateRect(new IntPtr(win), IntPtr.Zero, true);
                    }
                    if (copied_cursor)
                        Win32Api.SetCursor(cursor_handle);

                    // The title may have changed
                    StringBuilder title = new StringBuilder(256);
                    Win32Api.GetWindowText(hwnd_target, title, title.Capacity);


                    System.IO.MemoryStream memoryStream = new System.IO.MemoryStream();
                    screenshot.Save(memoryStream, System.Drawing.Imaging.ImageFormat.Bmp);
                    memoryStream.Position = 0;

                    byte[] bitmapBytes = memoryStream.GetBuffer();

                    byte[] hash = new MD5CryptoServiceProvider().ComputeHash(bitmapBytes);

                    String hashString = ByteArrayToString(hash);

                    memoryStream.Dispose();
                    bitmapBytes = null;



                    ///
                    // THIS IS STUFF MOSTLY FOR SHARED SCREEN ITEMS -- NOT REALLY NEEDED ON THE LOCAL CLIENTS
                    ///

                    bool needsLocationZOrderUpdate = false;

                    int zOrder = -1;
                    IntPtr currentWindowPointer = Win32Api.GetTopWindow(IntPtr.Zero);
                    while (currentWindowPointer.ToInt32() != hwnd && currentWindowPointer.ToInt32() > 0)
                    {
                        zOrder++;
                        currentWindowPointer = Win32Api.GetWindow(currentWindowPointer, Win32Api.GW_HWNDNEXT);
                    }


                    // check to see if we've seen this application before
                    if (zOrderHash.ContainsKey(hwnd))
                        if (!((int)zOrderHash[hwnd]).Equals(zOrder))
                        {
                            zOrderHash[hwnd] = zOrder;
                            needsLocationZOrderUpdate = true;
                        }
                        else
                        {
                            // Update will happen with orginal include, but we note the zOrder so we can track later on
                            zOrderHash.Add(hwnd, zOrder);
                        }


                    System.Drawing.Rectangle bounds = new System.Drawing.Rectangle(rc.left, rc.top, rc.Width, rc.Height);

                    // check to see if we've seen this application before
                    if (locationHash.ContainsKey(hwnd))
                        if (!((System.Drawing.Rectangle)locationHash[hwnd]).Equals(bounds))
                        {
                            locationHash[hwnd] = bounds;
                            needsLocationZOrderUpdate = true;
                        }
                        else
                        {
                            // Update will happen with orginal include, but we note the zOrder so we can track later on
                            locationHash.Add(hwnd, bounds);
                        }


                    ///
                    // END SHARED SCREEN SPECIFIC STUFF
                    ///



                    // check to see if we've seen this application before
                    if (screenShotsHash.ContainsKey(hwnd))
                    {
                        //check to see if we should update to database
                        if (((String)screenShotsHash[hwnd]).Equals(hashString))
                        {
                            Console.WriteLine("no changes, do not update DB");
                            sharedArtifactsTableAdapter.UpdateLocationandZOrder(rc.left, rc.top, rc.Width, rc.Height, zOrder, ClubDev.Program.CurrentUser.UserHost, hwnd);
                            continue;
                        }
                        else
                        {
                            Console.WriteLine("CHANGED, need to update");
                            screenShotsHash[hwnd] = hashString;
                        }
                    }
                    else
                    {
                        Console.WriteLine("new app seen");
                        screenShotsHash.Add(hwnd, hashString);
                    }







                    // Convert the System.Drawing.Bitmap to a WPF BitmapSource that we can use
                    IntPtr hScreenshotBitmap = screenshot.GetHbitmap();
                    System.Windows.Int32Rect screenshotRect = new Int32Rect(0, 0, screenshot.Width, screenshot.Height);
                    BitmapSource screenshotBitmapSource = System.Windows.Interop.Imaging.CreateBitmapSourceFromHBitmap(hScreenshotBitmap, IntPtr.Zero, screenshotRect, BitmapSizeOptions.FromEmptyOptions());


                    double ratio = 1;
                    if (screenshotBitmapSource.Width > screenshotBitmapSource.Height)
                        ratio = 300 / (double)screenshotBitmapSource.PixelWidth;
                    else
                        ratio = 300 / (double)screenshotBitmapSource.PixelHeight;

                    BitmapSource scaledScreenshotBitmapSource = new TransformedBitmap(screenshotBitmapSource, new System.Windows.Media.ScaleTransform(ratio, ratio));

                    JpegBitmapEncoder screenShotBitmapEncoder = new JpegBitmapEncoder();
                    screenShotBitmapEncoder.Frames.Add(BitmapFrame.Create(scaledScreenshotBitmapSource));
                    System.IO.MemoryStream screenShotStream = new MemoryStream();
                    screenShotBitmapEncoder.Save(screenShotStream);

                    sharedArtifactsTableAdapter.UpdateTitleImageLocationandZOrder(title.ToString(), screenShotStream.GetBuffer(), rc.left, rc.top, rc.Width, rc.Height, zOrder, hwnd, ClubDev.Program.CurrentUser.UserHost);

                    Win32Api.DeleteObject(hScreenshotBitmap);
                    screenshot.Dispose();
                    screenshotBitmapSource = null;
                    scaledScreenshotBitmapSource = null;
                    screenShotStream.Dispose();

                    Console.WriteLine("z order: " + zOrder.ToString());
                }
                catch { }
            }


        }

        private static string ByteArrayToString(byte[] arrInput)
        {
            int i;
            StringBuilder sOutput = new StringBuilder(arrInput.Length);
            for (i = 0; i < arrInput.Length; i++)
            {
                sOutput.Append(arrInput[i].ToString("X2"));
            }
            return sOutput.ToString();
        }
    }


    // Helper Class
    public class ChildWindowArray : ArrayList
    {
        private static bool windowCallBack(int hwnd, IntPtr param)
        {
            GCHandle gch = (GCHandle)param;
            ChildWindowArray warray = (ChildWindowArray)gch.Target;
            warray.Add(hwnd);
            return true;
        }

        public ChildWindowArray(IntPtr parentWnd)
        {
            GCHandle gch = GCHandle.Alloc(this);
            Win32Api.EnumChildCallBack ewcb = new Win32Api.EnumChildCallBack(windowCallBack);
            Win32Api.EnumChildWindows(parentWnd, ewcb, (IntPtr)gch);
            gch.Free();
        }


    }
}