using System;
using System.IO;
using System.Net;
using System.Data;
using System.Text;
using System.Reflection;
using System.Collections;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Media;
using System.Windows.Shapes;
using System.Windows.Media.Imaging;
using System.Windows.Media.Animation;
using System.Windows.Navigation;
using System.Windows.Forms;
using System.Windows.Threading;
using System.Runtime.InteropServices;



namespace ClubDev
{
	public partial class MainWindow
	{
        private Canvas root; 
        private bool neverRendered = true;

        private Database.ClubDevDataSet clubDevDataSet;
        private Database.ClubDevDataSetTableAdapters.SharedArtifactsTableAdapter sharedArtifactsTableAdapter;
        private Database.ClubDevDataSetTableAdapters.UsersTableAdapter userTableAdapter;
        private Database.ClubDevDataSetTableAdapters.CuRTSessionsTableAdapter CuRTSessionsTableAdapter;

        private CaptionButton.CaptionButtonManager captionIconsManager;

        private static System.Windows.Forms.Timer databaseUserPingTimer;
        private static System.Windows.Forms.Timer forcedPeriodicUpdateTimer;

        private static MainWindowData mainWindowData;
        private static CollaboratorBubbleData collaboratorBubbleData;

        private DataTable currentUsersTable;
        private DataTable currentCollaboratorBubbles;
        private DataTable currentWindowLocations;

        private int numCollaborators;
        private int numSharedScreens;

        private CollaboratorBar collaboratorBar;

        private double offSet = 115.0;

        private bool isCuRTServer = false;

        private bool isExiting = false;


        private MessengerAPI.DMessengerEvents_OnContactStatusChangeEventHandler liveMessengerStatusChangeEvent;
        private MessengerAPI.DMessengerEvents_OnMyStatusChangeEventHandler liveMeesengerMyStatusChangeEvent;
        


		public MainWindow()
		{

            System.Threading.Thread.CurrentThread.SetApartmentState(System.Threading.ApartmentState.STA);
            
			this.InitializeComponent();

            this.Closing += new System.ComponentModel.CancelEventHandler(MainWindowClosingEventHander);
			

            this.Width = System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width;
            this.Height = System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height;
            this.Left = 0;
            this.Top = 0;


            this.Topmost = true;

            Canvas.SetTop(this, 0);
            Canvas.SetLeft(this, 0);


            clubDevDataSet = new ClubDev.Database.ClubDevDataSet();
            sharedArtifactsTableAdapter = new Database.ClubDevDataSetTableAdapters.SharedArtifactsTableAdapter();
            userTableAdapter = new Database.ClubDevDataSetTableAdapters.UsersTableAdapter();

            try
            {
                sharedArtifactsTableAdapter.DeleteByGroup(ClubDev.Program.CurrentUser.UserHost);
            }
            catch
            {
                System.Windows.Forms.MessageBox.Show("Cannot contact database");
                return;
            }


            root = this.Content as Canvas;

            Canvas.SetLeft(root, 0);
            Canvas.SetTop(root, 0);

            numCollaborators = 0;
            numSharedScreens = 0;


            currentUsersTable = new DataTable();
            currentUsersTable.Columns.Add("userID", typeof(long));
            currentUsersTable.Columns.Add("userSessionID", typeof(long));
            currentUsersTable.Columns.Add("userName", typeof(string));
            currentUsersTable.Columns.Add("userIM", typeof(string));
            currentUsersTable.Columns.Add("userHost", typeof(string));
            currentUsersTable.Columns.Add("userIsSharedDisplay", typeof(bool));
            currentUsersTable.Columns.Add("userResolutionX", typeof(int));
            currentUsersTable.Columns.Add("userResolutionY", typeof(int));

            currentCollaboratorBubbles = new DataTable();
            currentCollaboratorBubbles.Columns.Add("requestTimeStamp", typeof(DateTime));
            currentCollaboratorBubbles.Columns.Add("sessionID", typeof(long));
            currentCollaboratorBubbles.Columns.Add("userID", typeof(long));
            currentCollaboratorBubbles.Columns.Add("artifactID", typeof(long));
            currentCollaboratorBubbles.Columns.Add("comment", typeof(string));



            this.AllowDrop = true;

            

            databaseUserPingTimer = new Timer();
            databaseUserPingTimer.Tick += new EventHandler(DatabaseUserPingTimerEventHandler);
            databaseUserPingTimer.Interval = 120000;
            databaseUserPingTimer.Start();

            forcedPeriodicUpdateTimer = new Timer();
            forcedPeriodicUpdateTimer.Tick += new EventHandler(ForcedPeriodicUpdateTimerEventHandlerForUser);
            forcedPeriodicUpdateTimer.Interval = 10000;
            forcedPeriodicUpdateTimer.Start();

            captionIconsManager = new CaptionButton.CaptionButtonManager(ClubDev.Program.CurrentUser.UserHost, ClubDev.Program.CurrentUser.UserName);

            Engines.ClientScreenShotEngine.StartEngine();


            collaboratorBar = new CollaboratorBar(root);
            root.Children.Add(collaboratorBar);



            mainWindowData = new MainWindowData();
            //mainWindowData.OnNewUserMessage += new MainWindowData.NewMessage(OnNewUserMessage);
            //mainWindowData.OnNewSharedArtifactMessage +=new MainWindowData.NewMessage(OnNewSharedArtifactMessage);

            collaboratorBubbleData = new CollaboratorBubbleData();
            //collaboratorBubbleData.OnMessage += new CollaboratorBubbleData.NewMessage(OnNewCollaboratorBubbleMessage);

            if (Program.LiveMessenger != null)
            {
                liveMessengerStatusChangeEvent = new MessengerAPI.DMessengerEvents_OnContactStatusChangeEventHandler(OnLiveMessengerStatusChange);
                Program.LiveMessenger.OnContactStatusChange += liveMessengerStatusChangeEvent;

                liveMeesengerMyStatusChangeEvent = new MessengerAPI.DMessengerEvents_OnMyStatusChangeEventHandler(OnLiveMessengerMyStatusChange);
                Program.LiveMessenger.OnMyStatusChange += liveMeesengerMyStatusChangeEvent;
            }
            
            startSysTrayIconThread();

            CuRT.CuRT.InitializeCuRTSupport();

            UpdateConnectedUsers();

            //OnNewUserMessage();

            //OnNewSharedArtifactMessage();

            //OnNewCollaboratorBubbleMessage();


		}



        //Really, we just need this for shared screen collaborators
        public MainWindow(string collaborator_type)
        {

            System.Threading.Thread.CurrentThread.SetApartmentState(System.Threading.ApartmentState.STA);

            this.InitializeComponent();

            this.Closing += new System.ComponentModel.CancelEventHandler(MainWindowClosingEventHander);


            this.Width = System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width;
            this.Height = System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height;
            this.Left = 0;
            this.Top = 0;


            this.Topmost = true;

            Canvas.SetTop(this, 0);
            Canvas.SetLeft(this, 0);


            clubDevDataSet = new ClubDev.Database.ClubDevDataSet();
            sharedArtifactsTableAdapter = new Database.ClubDevDataSetTableAdapters.SharedArtifactsTableAdapter();
            userTableAdapter = new Database.ClubDevDataSetTableAdapters.UsersTableAdapter();
            CuRTSessionsTableAdapter = new Database.ClubDevDataSetTableAdapters.CuRTSessionsTableAdapter();
            
            try
            {
                sharedArtifactsTableAdapter.DeleteByGroup(ClubDev.Program.CurrentUser.UserHost);
            }
            catch
            {
                System.Windows.Forms.MessageBox.Show("Cannot contact database");
                return;
            }


            root = this.Content as Canvas;

            Canvas.SetLeft(root, 0);
            Canvas.SetTop(root, 0);

            numCollaborators = 0;
            numSharedScreens = 0;

            currentWindowLocations = new DataTable();
            currentWindowLocations.Columns.Add("replicationClientHWND", typeof(int));
            currentWindowLocations.Columns.Add("replicationLocationX", typeof(int));
            currentWindowLocations.Columns.Add("replicationLocationY", typeof(int));

            this.AllowDrop = true;

            databaseUserPingTimer = new Timer();
            databaseUserPingTimer.Tick += new EventHandler(DatabaseUserPingTimerEventHandler);
            databaseUserPingTimer.Interval = 120000;
            databaseUserPingTimer.Start();

            forcedPeriodicUpdateTimer = new Timer();
            forcedPeriodicUpdateTimer.Tick += new EventHandler(ForcedPeriodicUpdateTimerEventHandlerForSharedScreen);
            forcedPeriodicUpdateTimer.Interval = 1000;
            forcedPeriodicUpdateTimer.Start();


            mainWindowData = new MainWindowData();
            //mainWindowData.OnAppMoveMessage += new MainWindowData.NewMessage(OnAppMoveMessage);
            //mainWindowData.OnNewCuRTSessionMessage += new MainWindowData.NewMessage(OnCuRTSessionMessage);
            //OnAppMoveMessage();
            //OnCuRTSessionMessage();

            startSysTrayIconThread();


            CuRT.CuRT.InitializeCuRTSupport();
       


        }


        void startSysTrayIconThread()
        {
            System.Threading.ParameterizedThreadStart threadDelegate = new System.Threading.ParameterizedThreadStart(SysTrayIcon.startSysTrayIcon);
            System.Threading.Thread newthread = new System.Threading.Thread(threadDelegate);
            SysTrayIcon.CallBack cb = new SysTrayIcon.CallBack(messageAppFromSysTray);

            newthread.Start(cb);
        }


        void messageAppFromSysTray(SysTrayIcon.TrayMessages message)
        {

            if (this.CheckAccess())
                messageAppFromSysTrayWorker(message);
            else
                this.Dispatcher.BeginInvoke(System.Windows.Threading.DispatcherPriority.Normal, (DispatcherOperationCallback)delegate(object arg)
                {
                    // HANDLED ON UI THREAD
                    messageAppFromSysTrayWorker(message);
                    return null;
                }, null);
        }

        void messageAppFromSysTrayWorker(SysTrayIcon.TrayMessages message)
        {
            if (message == SysTrayIcon.TrayMessages.Exit)
            {
                this.Close();
                ClubDev.Program.killDispatchServer();
                ClubDev.Program.KillHook();
                ClubDev.Program.KillCuRTServer();
            }
            else if (message == SysTrayIcon.TrayMessages.LeaveSession)
            {
                this.Close();
                ClubDev.Program.killDispatchServer();
                ClubDev.Program.KillHook();
                ClubDev.Program.KillCuRTServer();
                System.Diagnostics.Process.Start(@"ClubDev.exe");
            }
        }

        /*
        void OnAppMoveMessage()
        {

            if (this.CheckAccess())
                MoveWindows();
            else
                this.Dispatcher.BeginInvoke(System.Windows.Threading.DispatcherPriority.Normal, (DispatcherOperationCallback)delegate(object arg)
                {
                    // HANDLED ON UI THREAD
                    MoveWindows();
                    return null;
                }, null);
        }
        
        void OnNewUserMessage()
        {

            if (this.CheckAccess())
                LoadMessages();
            else
                this.Dispatcher.BeginInvoke(System.Windows.Threading.DispatcherPriority.Normal, (DispatcherOperationCallback)delegate(object arg)
                {
                    // HANDLED ON UI THREAD
                    LoadMessages();
                    return null;
                }, null);


        }

        void OnCuRTSessionMessage()
        {
            if (this.CheckAccess())
                UpdateCuRTServer();
            else
                this.Dispatcher.BeginInvoke(System.Windows.Threading.DispatcherPriority.Normal, (DispatcherOperationCallback)delegate(object arg)
                {
                    // HANDLED ON UI THREAD
                    UpdateCuRTServer();
                    return null;
                }, null);
        }

        void OnNewSharedArtifactMessage()
        {

            if (this.CheckAccess())
                UpdateSharedArtifactsCount();
            else
                this.Dispatcher.BeginInvoke(System.Windows.Threading.DispatcherPriority.Normal, (DispatcherOperationCallback)delegate(object arg)
                {
                    // HANDLED ON UI THREAD
                    UpdateSharedArtifactsCount();
                    return null;
                }, null);
        }

       

        void OnNewCollaboratorBubbleMessage()
        {
            if (this.CheckAccess())
                UpdateCollaboratorBubble();
            else
                this.Dispatcher.BeginInvoke(System.Windows.Threading.DispatcherPriority.Normal, (DispatcherOperationCallback)delegate(object arg)
                {
                    // HANDLED ON UI THREAD
                    UpdateCollaboratorBubble();
                    return null;
                }, null);
        }
        */

        
        void OnLiveMessengerStatusChange(object contact, MessengerAPI.MISTATUS status)
        {
            MessengerAPI.MISTATUS actualStatus = ((MessengerAPI.IMessengerContact)contact).Status;
            string serviceName = ((MessengerAPI.IMessengerContact)contact).SigninName;

            if (this.CheckAccess())
                this.collaboratorBar.UpdateLiveMessengerStatus(serviceName, actualStatus);
            else
                this.Dispatcher.BeginInvoke(System.Windows.Threading.DispatcherPriority.Normal, (DispatcherOperationCallback)delegate(object arg)
                {
                    // HANDLED ON UI THREAD
                    this.collaboratorBar.UpdateLiveMessengerStatus(serviceName, actualStatus);
                    return null;
                }, null);

            
        }

        void OnLiveMessengerMyStatusChange(int value, MessengerAPI.MISTATUS status)
        {
            string serviceName = Program.LiveMessenger.MySigninName;

            if (this.CheckAccess())
                this.collaboratorBar.UpdateLiveMessengerStatus(serviceName, status);
            else
                this.Dispatcher.BeginInvoke(System.Windows.Threading.DispatcherPriority.Normal, (DispatcherOperationCallback)delegate(object arg)
                {
                    // HANDLED ON UI THREAD
                    this.collaboratorBar.UpdateLiveMessengerStatus(serviceName, status);
                    return null;
                }, null);


        }
        

        private void MoveWindows()
        {
            DataTable dt = mainWindowData.GetWindowLocations(System.Net.Dns.GetHostEntry(System.Net.Dns.GetHostName()).HostName);

            DataTable additions = SupportProviders.SQLOperations.Difference(dt, currentWindowLocations);
            int hwnd;
            int locationX;
            int locationY;

            foreach (DataRow row in additions.Rows)
            {
                hwnd = (int)row["replicationClientHWND"];
                locationX = (int)row["replicationLocationX"];
                locationY = (int)row["replicationLocationY"];
                Utilities.Win32Api.SetWindowPos(hwnd, 0, locationX, locationY, 0, 0,
                    Utilities.Win32Api.SWP_NOACTIVATE | Utilities.Win32Api.SWP_NOSIZE | Utilities.Win32Api.SWP_NOZORDER);
                Utilities.Win32Api.SetForegroundWindow(hwnd);
            }
            currentWindowLocations = dt;
        }
        

        private void UpdateConnectedUsers()
        {
            // Get the messages
            DataTable dt = mainWindowData.GetUsers(ClubDev.Program.ChosenSession.SessionID);

            DataTable additions = SupportProviders.SQLOperations.Difference(dt, currentUsersTable);
            DataTable subtractions = SupportProviders.SQLOperations.Difference(currentUsersTable, dt);

            foreach (DataRow row in additions.Rows)
            {
                BitmapSource userIcon = mainWindowData.GetUserIcon((long)row["userID"]);

                Surogates.UserSurogate userSurogate = new ClubDev.Surogates.UserSurogate(
                    (long)row["userID"], (long)row["userSessionID"], row["userHost"].ToString(), 
                    row["userName"].ToString(), row["userIM"].ToString(),
                    (bool)row["userIsSharedDisplay"], (int)row["userResolutionX"], 
                    (int)row["userResolutionY"], userIcon);
                
                AddUser(userSurogate);
            }

            foreach (DataRow row in subtractions.Rows)
            {
                RemoveUser("user" + row["userID"].ToString(), (bool)row["userIsSharedDisplay"]);
            }




            currentUsersTable = dt;

        }



        private void UpdateSharedArtifactsCount()
        {
            // clean up left over shared artifacts first
            if (sharedArtifactsTableAdapter != null)
            {
                sharedArtifactsTableAdapter.FillByGroup(clubDevDataSet.SharedArtifacts, Program.CurrentUser.UserHost);

                for (int i = 0; i < clubDevDataSet.SharedArtifacts.Count; i++)
                {
                    StringBuilder title = new StringBuilder(256);
                    int result = Utilities.Win32Api.GetWindowText(new IntPtr(clubDevDataSet.SharedArtifacts[i].artifactHWND), title, title.Capacity);

                    if (result <= 0)
                    {
                        sharedArtifactsTableAdapter.DeleteByHWNDandGroup(clubDevDataSet.SharedArtifacts[i].artifactHWND, Program.CurrentUser.UserHost);
                        Console.WriteLine("Automated cleanup of: " + clubDevDataSet.SharedArtifacts[i].artifactHWND.ToString());
                    }
                }
            }

            DataTable dt = mainWindowData.GetSharedArtifacts();

            IDictionaryEnumerator connectedHostsEnumerator = mainWindowData.ConnectedHosts.GetEnumerator();
            while (connectedHostsEnumerator.MoveNext())
            {
                DataRow[] shareCount = dt.Select("artifactGroup = '" + connectedHostsEnumerator.Value.ToString() + "' AND artifactFullShare = true");
                DataRow[] showCount = dt.Select("artifactGroup = '" + connectedHostsEnumerator.Value.ToString() + "' AND artifactFullShare = false");

                for (int i = 0; i < collaboratorBar.collaboratorGrid.Children.Count; i++)
                    if (((FrameworkElement)collaboratorBar.collaboratorGrid.Children[i]).Name == "user" + connectedHostsEnumerator.Key.ToString())
                        if (collaboratorBar.collaboratorGrid.Children[i].GetType() == typeof(Collaborator))
                        {
                            Collaborator currentCollaborator = ((Collaborator)collaboratorBar.collaboratorGrid.Children[i]);
                            currentCollaborator.UpdateShareShowCount(shareCount.Length, showCount.Length);
                        }
            }
        }




        private void UpdateCuRTServer()
        {
            DataTable sessions = mainWindowData.GetCuRTSessions(Program.CurrentUser.UserSessionID);

            if (sessions.Rows.Count == 0 && !this.isExiting)
            {
                if (Program.CurrentUser.UserIsSharedDisplay)
                {
                    DataTable dt = mainWindowData.GetUsersWithoutCallBack(Program.CurrentUser.UserSessionID);

                    long highestID = -1;
                    long localID = -1;

                    foreach (DataRow dr in dt.Rows)
                    {
                        if ((bool.Parse(dr["userIsSharedDisplay"].ToString())))
                        {
                            if (highestID < long.Parse(dr["userID"].ToString()))
                                highestID = long.Parse(dr["userID"].ToString());

                            if(dr["userHost"].ToString().Equals(Program.CurrentUser.UserHost))
                                localID = long.Parse(dr["userID"].ToString());
                        }
                    }

                    if ((localID != -1) && (highestID != -1) && (highestID == localID))
                    {
                        Console.WriteLine("startup server");

                        this.isCuRTServer = true;

                        try
                        {
                            CuRTSessionsTableAdapter.Insert(Program.CurrentUser.UserSessionID, Program.CurrentUser.UserHost);
                        }
                        catch
                        {
                            System.Windows.Forms.MessageBox.Show("Cannot update database with CuRT toolkit support");
                            return;
                        }

                        Program.StartCuRTServer();
                    }

                }
            }
            
            
        }

        private void UpdateCollaboratorBubble()
        {
            if(this.isExiting)
                return;

            DateTime oneMinuteAgo = DateTime.Now;
            oneMinuteAgo = oneMinuteAgo.AddSeconds(-60);


            DataTable dt = collaboratorBubbleData.GetCollaborationRequests(Program.CurrentUser.UserSessionID);

            DataTable additions = SupportProviders.SQLOperations.Difference(dt, currentCollaboratorBubbles);


            foreach (DataRow row in additions.Rows)
            {
                DateTime timeStamp = (DateTime)row["requestTimeStamp"];

                if (timeStamp > oneMinuteAgo)
                {
                    CollaboratorBubble bubble = new CollaboratorBubble(new Point(50, 50), (long)row["sessionID"], (long)row["userID"], (long)row["artifactID"], row["comment"].ToString());
                    root.Children.Add(bubble);
                }
            }

            currentCollaboratorBubbles = dt;

        }


        private void AddUser(Surogates.UserSurogate userSurogate)
        {
            if (userSurogate.UserIsSharedDisplay)
            { // handle creation of shared screen
                SharedScreen currentSharedScreen = new SharedScreen(new Point(300, 0), userSurogate.UserHost, userSurogate.UserName, new Size(userSurogate.UserResolutionX, userSurogate.UserResolutionY));
                currentSharedScreen.Name = "user" + userSurogate.UserID;

                root.Children.Add(currentSharedScreen);

                currentSharedScreen.InitializeEvents(root);

                numSharedScreens++;
            }

            else
            { //handle creation of collaborator

                collaboratorBar.AddCollaborator(userSurogate);

                //add to the list of items that we should check for shared item updated for
                mainWindowData.ConnectedHosts.Add(userSurogate.UserID, userSurogate.UserHost);

                

            }



           

        }

        private void RemoveUser(String collabName, bool isSharedDisplay)
        {
            if (isSharedDisplay)
            {
                numSharedScreens--;

                for (int i = 0; i < root.Children.Count; i++)
                {
                    if (((FrameworkElement)root.Children[i]).Name == collabName)
                    {
                        root.Children.RemoveAt(i);
                        break;
                    }
                }
            }

            else //isCollaborator
            {

                collaboratorBar.RemoveCollaborator(collabName);

                //remove from the list of collaborators to update shared artifact count
                int index = collabName.LastIndexOf("user");
                string number = collabName.Substring(index + 5);
                mainWindowData.ConnectedHosts.Remove(long.Parse(number));

            }                
    
        }


        void DatabaseUserPingTimerEventHandler(object sender, EventArgs e)
        {
            if(this.userTableAdapter != null)
                this.userTableAdapter.UserPing(ClubDev.Program.CurrentUser.UserID);

        }

        void ForcedPeriodicUpdateTimerEventHandlerForUser(object sender, EventArgs e)
        {
            UpdateConnectedUsers();
            UpdateSharedArtifactsCount();
            UpdateCollaboratorBubble();
            
        }

        void ForcedPeriodicUpdateTimerEventHandlerForSharedScreen(object sender, EventArgs e)
        {
            MoveWindows();
            UpdateCuRTServer();

        }

        void MainWindowClosingEventHander(object sender, System.ComponentModel.CancelEventArgs e)
        {
            this.isExiting = true;

            databaseUserPingTimer.Stop();
            databaseUserPingTimer.Dispose();

            forcedPeriodicUpdateTimer.Stop();
            forcedPeriodicUpdateTimer.Dispose();

            if(sharedArtifactsTableAdapter != null)
                sharedArtifactsTableAdapter.DeleteByGroup(ClubDev.Program.CurrentUser.UserHost);

            if (userTableAdapter != null)
                userTableAdapter.DeleteByUserID(ClubDev.Program.CurrentUser.UserID);

            if (Program.LiveMessenger != null)
            {
                Program.LiveMessenger.OnContactStatusChange -= liveMessengerStatusChangeEvent;
                Program.LiveMessenger.OnMyStatusChange -= liveMeesengerMyStatusChangeEvent;
            }

            CuRT.CuRT.KillCuRTSupport();

            if (this.isCuRTServer)
            {
                if (CuRTSessionsTableAdapter != null)
                    CuRTSessionsTableAdapter.Delete1(Program.CurrentUser.UserSessionID);
                try
                {
                    Program.KillCuRTServer();
                }
                catch (Exception) { }
            }

            Program.KillHook();


        }


    

		protected override void OnContentRendered(EventArgs e)
		{
			if (this.neverRendered)
			{
				// The window takes the size of its content because SizeToContent
				// is set to WidthAndHeight in the markup. We then allow
				// it to be set by the user, and have the content take the size
				// of the window.
				this.SizeToContent = SizeToContent.Manual;

				FrameworkElement root = this.Content as FrameworkElement;
				if (root != null)
				{
					root.Width = double.NaN;
					root.Height = double.NaN;
				}

				this.neverRendered = false;
			}

			base.OnContentRendered(e);
		}

        
	}
}
