using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Data.SqlClient;
using System.Data;

namespace ClubDev
{

    public class MainWindowData
    {
        private static string m_ConnectionString = ClubDev.Properties.Settings.Default.ClubDevConnectionString;
        private SqlConnection m_sqlConn = null;

        public delegate void NewMessage();
        //public event NewMessage OnNewUserMessage;
        //public event NewMessage OnAppMoveMessage;
        //public event NewMessage OnNewSharedArtifactMessage;
        //public event NewMessage OnNewCuRTSessionMessage;

        public Hashtable ConnectedHosts; 

        /// <summary>
        /// Constructor
        /// </summary>
        public MainWindowData()
        {
            // Stop an existing services on this connection string
            // just be sure
            SqlDependency.Stop(m_ConnectionString);

            // Start the dependency
            // User must have SUBSCRIBE QUERY NOTIFICATIONS permission
            // Database must also have SSB enabled
            // ALTER DATABASE Chatter SET ENABLE_BROKER
            SqlDependency.Start(m_ConnectionString);

            // Create the connection
            m_sqlConn = new SqlConnection(m_ConnectionString);

            ConnectedHosts = new Hashtable();
        }

        /// <summary>
        /// Destructor
        /// </summary>
        ~MainWindowData()
        {
            // Stop the dependency before exiting
            SqlDependency.Stop(m_ConnectionString);
        }

        public DataTable GetWindowLocations(string hostname)
        {
            DataTable dt = new DataTable();

            try
            {
                // Create command
                // Command must use two part names for tables
                // SELECT <field> FROM dbo.Table rather than 
                // SELECT <field> FROM Table
                // Query also can not use *, fields must be designated
                SqlCommand cmd = new SqlCommand("SELECT replicationClientHWND, replicationLocationX, replicationLocationY FROM dbo.Replications WHERE  replicationClientHost = @hostname AND replicationClientHWND IS NOT NULL AND replicationLocationX IS NOT NULL AND replicationLocationY IS NOT NULL", m_sqlConn);
                cmd.CommandType = CommandType.Text;
                cmd.Parameters.AddWithValue("@hostname", hostname);

                // Clear any existing notifications
                cmd.Notification = null;

                // Create the dependency for this command
                //SqlDependency dependency = new SqlDependency(cmd);

                // Add the event handler
                //dependency.OnChange += new OnChangeEventHandler(OnWindowLocationChange);

                // Open the connection if necessary
                if (m_sqlConn.State == ConnectionState.Closed)
                    m_sqlConn.Open();

                // Get the messages
                dt.Load(cmd.ExecuteReader(CommandBehavior.CloseConnection));
            }
            catch (Exception ex)
            {
                throw ex;
            }

            return dt;
        }

        /// <summary>
        /// Retreive messages from database
        /// </summary>
        /// <returns></returns>
        public DataTable GetUsers(long sessionID)
        {
            DataTable dt = new DataTable();

            try
            {
                // Create command
                // Command must use two part names for tables
                // SELECT <field> FROM dbo.Table rather than 
                // SELECT <field> FROM Table
                // Query also can not use *, fields must be designated
                SqlCommand cmd = new SqlCommand("SELECT userID, userSessionID, userName, userIM, userHost, userIsSharedDisplay, userResolutionX, userResolutionY FROM dbo.Users WHERE userSessionID = @userSessionID", m_sqlConn);
                cmd.CommandType = CommandType.Text;
                cmd.Parameters.AddWithValue("@userSessionID", sessionID);

                // Clear any existing notifications
                cmd.Notification = null;

                // Create the dependency for this command
                //SqlDependency dependency = new SqlDependency(cmd);

                // Add the event handler
                //dependency.OnChange += new OnChangeEventHandler(OnUserChange);

                // Open the connection if necessary
                if (m_sqlConn.State == ConnectionState.Closed)
                    m_sqlConn.Open();

                // Get the messages
                dt.Load(cmd.ExecuteReader(CommandBehavior.CloseConnection));
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message.ToString());
            }

            return dt;
        }


        public DataTable GetSharedArtifacts()
        {
            DataTable dt = new DataTable();


            IDictionaryEnumerator connectedHostsEnumerator = ConnectedHosts.GetEnumerator();
            string sqlFragment = "";
            bool first = true;

            while (connectedHostsEnumerator.MoveNext())
            {
                if (!first)
                    sqlFragment += " OR ";

                sqlFragment += "artifactGroup = '" + connectedHostsEnumerator.Value.ToString() + "'";

                first = false;
            }


            try
            {
                // Create command
                // Command must use two part names for tables
                // SELECT <field> FROM dbo.Table rather than 
                // SELECT <field> FROM Table
                // Query also can not use *, fields must be designated
                SqlCommand cmd = new SqlCommand("SELECT artifactID, artifactGroup, artifactFullShare FROM dbo.SharedArtifacts WHERE " + sqlFragment, m_sqlConn);
                cmd.CommandType = CommandType.Text;

                // Clear any existing notifications
                cmd.Notification = null;

                // Create the dependency for this command
                //SqlDependency dependency = new SqlDependency(cmd);

                // Add the event handler
                //dependency.OnChange += new OnChangeEventHandler(OnSharedArtifactChange);

                // Open the connection if necessary
                if (m_sqlConn.State == ConnectionState.Closed)
                    m_sqlConn.Open();

                // Get the messages
                dt.Load(cmd.ExecuteReader(CommandBehavior.CloseConnection));
            }
            catch (Exception ex)
            {
                throw ex;
            }

            return dt;
        }


        public DataTable GetCuRTSessions(long sessionID)
        {
            DataTable dt = new DataTable();

            try
            {
                // Create command
                // Command must use two part names for tables
                // SELECT <field> FROM dbo.Table rather than 
                // SELECT <field> FROM Table
                // Query also can not use *, fields must be designated
                SqlCommand cmd = new SqlCommand("SELECT sessionID, CuRTServer FROM dbo.CuRTSessions WHERE sessionID = @sessionID", m_sqlConn);
                cmd.CommandType = CommandType.Text;
                cmd.Parameters.AddWithValue("@sessionID", sessionID);

                // Clear any existing notifications
                cmd.Notification = null;

                // Create the dependency for this command
                //SqlDependency dependency = new SqlDependency(cmd);

                // Add the event handler
                //dependency.OnChange += new OnChangeEventHandler(OnCuRTSessionChange);

                // Open the connection if necessary
                if (m_sqlConn.State == ConnectionState.Closed)
                    m_sqlConn.Open();

                // Get the messages
                dt.Load(cmd.ExecuteReader(CommandBehavior.CloseConnection));
            }
            catch (Exception ex)
            {
                throw ex;
            }

            return dt;
        }



        public DataTable GetUsersWithoutCallBack(long sessionID)
        {
            DataTable dt = new DataTable();

            try
            {
                // Create command
                // Command must use two part names for tables
                // SELECT <field> FROM dbo.Table rather than 
                // SELECT <field> FROM Table
                // Query also can not use *, fields must be designated
                SqlCommand cmd = new SqlCommand("SELECT userID, userSessionID, userName, userIM, userHost, userIsSharedDisplay, userResolutionX, userResolutionY FROM dbo.Users WHERE userSessionID = @userSessionID", m_sqlConn);
                cmd.CommandType = CommandType.Text;
                cmd.Parameters.AddWithValue("@userSessionID", sessionID);

                // Clear any existing notifications
                cmd.Notification = null;

                
                // Open the connection if necessary
                if (m_sqlConn.State == ConnectionState.Closed)
                    m_sqlConn.Open();

                // Get the messages
                dt.Load(cmd.ExecuteReader(CommandBehavior.CloseConnection));
            }
            catch (Exception ex)
            {
                throw ex;
            }

            return dt;
        }





        /*
        void OnSharedArtifactChange(object sender, SqlNotificationEventArgs e)
        {
            SqlDependency dependency = sender as SqlDependency;

            // Notices are only a one shot deal
            // so remove the existing one so a new 
            // one can be added
            dependency.OnChange -= OnSharedArtifactChange;

            // Fire the event
            if (OnNewSharedArtifactMessage != null)
            {
                OnNewSharedArtifactMessage();
            }
        }
        

        void OnWindowLocationChange(object sender, SqlNotificationEventArgs e)
        {
            SqlDependency dependency = sender as SqlDependency;

            // Notices are only a one shot deal
            // so remove the existing one so a new 
            // one can be added
            dependency.OnChange -= OnWindowLocationChange;

            // Fire the event
            if (OnAppMoveMessage != null)
            {
                OnAppMoveMessage();
            }
        }

        /// <summary>
        /// Handler for the SqlDependency OnChange event
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void OnUserChange(object sender, SqlNotificationEventArgs e)
        {
            SqlDependency dependency = sender as SqlDependency;

            // Notices are only a one shot deal
            // so remove the existing one so a new 
            // one can be added
            dependency.OnChange -= OnUserChange;

            // Fire the event
            if (OnNewUserMessage != null)
            {
                OnNewUserMessage();
            }
        }


        void OnCuRTSessionChange(object sender, SqlNotificationEventArgs e)
        {
            SqlDependency dependency = sender as SqlDependency;

            // Notices are only a one shot deal
            // so remove the existing one so a new 
            // one can be added
            dependency.OnChange -= OnCuRTSessionChange;

            // Fire the event
            if (OnNewCuRTSessionMessage != null)
            {
                OnNewCuRTSessionMessage();
            }
        }

        */


        public System.Windows.Media.Imaging.BitmapSource GetUserIcon(long userID)
        {
            DataTable dt = new DataTable();

            try
            {
                // Create command
                // Command must use two part names for tables
                // SELECT <field> FROM dbo.Table rather than 
                // SELECT <field> FROM Table
                // Query also can not use *, fields must be designated
                SqlCommand cmd = new SqlCommand("SELECT userIcon FROM dbo.Users WHERE userID = @userID", m_sqlConn);
                cmd.CommandType = CommandType.Text;
                cmd.Parameters.AddWithValue("@userID", userID);

                // Open the connection if necessary
                if (m_sqlConn.State == ConnectionState.Closed)
                    m_sqlConn.Open();

                // Get the messages
                dt.Load(cmd.ExecuteReader(CommandBehavior.CloseConnection));
            }
            catch (Exception ex)
            {
                throw ex;
            }

            if (dt.Rows.Count == 1 && dt.Rows[0]["userIcon"] != DBNull.Value)
            {
                byte[] imageArray = (byte[])dt.Rows[0]["userIcon"];

                System.IO.MemoryStream stream = new System.IO.MemoryStream(imageArray);
                System.Windows.Media.Imaging.PngBitmapDecoder pngBitmapDecoder = new System.Windows.Media.Imaging.PngBitmapDecoder(stream, System.Windows.Media.Imaging.BitmapCreateOptions.PreservePixelFormat, System.Windows.Media.Imaging.BitmapCacheOption.None);
                return pngBitmapDecoder.Frames[0];

            }

            else
            {
                return null;
            }
        }
    }


}