using System;
using System.IO;
using System.Windows;
using System.Collections.Generic;
using System.Windows.Forms;
using System.Windows.Media.Imaging;
using Microsoft.Win32;

namespace ClubDev
{
    static class Program
    {
        private static string databaseServerHostName;
        private static string databaseTableName;
        private static string databaseUsername;
        private static string collaboratorName;
        private static string collaboratorIM;
        private static string databasePassword;
        private static string collaboratorType;
        private static BitmapSource collaboratorIcon;

        private static bool allowRentryOfServerInfo = true;
        private static bool rentryAfterSuccess = false;

        private static Surogates.CollaborationSessionSurogate[] avaliableSessions;
        private static Surogates.CollaborationSessionSurogate chosenSession;
        private static Surogates.UserSurogate currentUser;
        //private static System.Diagnostics.Process dispatch_server;
        private static System.Diagnostics.Process hook_catcher;
        private static System.Diagnostics.Process CuRTServer;
        private static bool shadowLogging = true;


        private static MessengerAPI.Messenger liveMessenger;

        #region Properties

        public static bool AllowRentryOfServerInfo
        {
            get
            {
                return allowRentryOfServerInfo;
            }
            set
            {
                allowRentryOfServerInfo = value;
            }
        }

        public static bool RentryAfterSuccess
        {
            get
            {
                return rentryAfterSuccess;
            }
            set
            {
                rentryAfterSuccess = value;
            }
        }

        public static Surogates.CollaborationSessionSurogate[] AvalialableSessions
        {
            get
            {
                return avaliableSessions;
            }
            set
            {
                avaliableSessions = value;
            }
        }

        public static Surogates.CollaborationSessionSurogate ChosenSession
        {
            get
            {
                return chosenSession;
            }
            set
            {
                chosenSession = value;
            }
        }

        public static Surogates.UserSurogate CurrentUser
        {
            get
            {
                return currentUser;
            }
            set
            {
                value = currentUser;
            }
        }

        public static MessengerAPI.Messenger LiveMessenger
        {
            get
            {
                return liveMessenger;
            }
        }

        public static bool ShadowLogging
        {
            get
            {
                return shadowLogging;
            }
        }



        #endregion


        [STAThread]
        static void Main()
        {
            System.Windows.Forms.Application.EnableVisualStyles();
            System.Windows.Forms.Application.SetCompatibleTextRenderingDefault(false);

            while (LaunchSequence())
            {
                if (!allowRentryOfServerInfo)
                    break;

                System.Windows.Forms.Application.Run(new ConfigurationDialogs.ServerSettingsWindow());

                if (!allowRentryOfServerInfo && !rentryAfterSuccess)
                    break;

            }
            
        }

        private static bool LaunchSequence()
        {
            allowRentryOfServerInfo = false;
            avaliableSessions = null;
            chosenSession = null;

            CheckForValidServerSettings();

            System.Windows.Forms.Application.Run(new ConfigurationDialogs.CheckingDatabaseWindow());

            if (avaliableSessions == null)
                return true;

            System.Windows.Forms.Application.Run(new ConfigurationDialogs.SessionSelectionWindow());

            if (allowRentryOfServerInfo)
                return true;

            if (chosenSession == null)
                return true;

            JoinUserToSession();

            
            /*killRunningDispatchServers();

            dispatch_server = null;
            try
            {
                dispatch_server = System.Diagnostics.Process.Start(@"..\..\..\ApplicationReplicator\ApplicationReplicator\DispatchServer\bin\Debug\DispatchServer.exe");
            }
            catch
            {
                System.Windows.Forms.MessageBox.Show("Unable to start dispatch server");
            }*/
            

            KillHook();

            hook_catcher = null;
            try
            {
                //hook_catcher = System.Diagnostics.Process.Start(@"MinimizeHooks\hook.exe");
            }
            catch
            {
                System.Windows.Forms.MessageBox.Show("Unable to start global hooks");
            }
            


            System.Windows.Application application = new System.Windows.Application();
            

            RegistryKey clubDevRegistryKey = Registry.CurrentUser.CreateSubKey("SOFTWARE\\UIUC\\ORCHID\\ClubDev");
            string collaboratorType = (string)clubDevRegistryKey.GetValue("collaboratorType");

            if (collaboratorType == "sharedScreen")
            {
                application.Run(new MainWindow(collaboratorType));
            }
            else if (collaboratorType == "collaborator")
            {
                //try
                //{
                    //liveMessenger = new MessengerAPI.Messenger();
                //}
                //catch(Exception e)
                //{
                //    SysTrayIcon.ShowBalloonTip(60000, "ERROR: Could not start Live Messenger support", e.Message.ToString(), System.Windows.Forms.ToolTipIcon.Error);
                //}


                application.Run(new MainWindow());

            }
            return false;
        }

        private static void killRunningHooks()
        {
            System.Diagnostics.Process[] running_hooks = System.Diagnostics.Process.GetProcessesByName("hook");
            foreach (System.Diagnostics.Process p in running_hooks)
                p.Kill();
        }

        private static void killRunningDispatchServers()
        {
            //System.Diagnostics.Process[] running_dispatch_servers = System.Diagnostics.Process.GetProcessesByName("DispatchServer");
            //foreach (System.Diagnostics.Process p in running_dispatch_servers)
            //    p.CloseMainWindow();
        }

        public static void KillHook()
        {
            if (hook_catcher != null)
            {
                try
                {
                    hook_catcher.Kill();
                }
                catch (Exception) { }
            }
        }

        public static void killDispatchServer()
        {
            //if(dispatch_server!=null)
            //    dispatch_server.CloseMainWindow();
        }

        public static void StartCuRTServer()
        {
            CuRTServer = null;
            try
            {
                CuRTServer = System.Diagnostics.Process.Start(@"CuRT\CuRTServer.exe");
            }
            catch
            {
                System.Windows.Forms.MessageBox.Show("Unable to start CuRTServer");
            }
        }

        public static void KillCuRTServer()
        {
            if(CuRTServer != null)
                try
                {
                    CuRTServer.Kill();
                }

                catch (Exception) { }
        }

        public static void CheckForValidServerSettings()
        {
            RegistryKey clubDevRegistryKey = Registry.CurrentUser.CreateSubKey("SOFTWARE\\UIUC\\ORCHID\\ClubDev");

            databaseServerHostName = (string)clubDevRegistryKey.GetValue("databaseServerHostName");
            databaseTableName = (string)clubDevRegistryKey.GetValue("databaseTableName");
            databaseUsername = (string)clubDevRegistryKey.GetValue("databaseUsername");
            collaboratorName = (string)clubDevRegistryKey.GetValue("collaboratorName");
            collaboratorIM = (string)clubDevRegistryKey.GetValue("collaboratorIM");
            databasePassword = (string)clubDevRegistryKey.GetValue("databasePassword");
            collaboratorType = (string)clubDevRegistryKey.GetValue("collaboratorType");

            string collabotoratorImageFileName = (string)clubDevRegistryKey.GetValue("collaboratorImage");

            clubDevRegistryKey.Close();

            if (collaboratorType != null && collaboratorType == "collaborator")
            {
                try
                {
                    System.IO.StreamReader stream = new StreamReader(collabotoratorImageFileName);
                    PngBitmapDecoder bitmapDecoder = new PngBitmapDecoder(stream.BaseStream, BitmapCreateOptions.PreservePixelFormat, BitmapCacheOption.Default);
                    collaboratorIcon = bitmapDecoder.Frames[0];
                }

                catch
                {
                    DialogResult results = System.Windows.Forms.MessageBox.Show("Could not load user image.  Please insure that you have selected the correct file and location.  \n\n Would you like to update your settings?", "User Image Error", MessageBoxButtons.YesNo, MessageBoxIcon.Error);
                }
            }


            if (databaseServerHostName == null || databaseServerHostName == "" ||
                databaseTableName == null || databaseTableName == "" ||
                databaseUsername == null || databaseUsername == "" ||
                databasePassword == null || databasePassword == "" ||
                collaboratorName == null || collaboratorName == "" ||
                collaboratorType == null || collaboratorType == "")
            {
                System.Windows.Forms.Application.Run(new ConfigurationDialogs.ServerSettingsWindow());
                CheckForValidServerSettings();
            }
            else if (collaboratorType == "collaborator" && collaboratorIcon == null)
            {
                System.Windows.Forms.Application.Run(new ConfigurationDialogs.ServerSettingsWindow());
                CheckForValidServerSettings();
            }
            else {
                Properties.Settings.Default.ClubDevConnectionString = "Data Source=" + databaseServerHostName + ";Initial Catalog=" + databaseTableName + ";Persist Security Info=True;User ID=" + databaseUsername + ";Password=" + databasePassword + ";Network Library=dbmssocn";
                
            }
        }


        private static void JoinUserToSession()
        {
            string hostname = System.Net.Dns.GetHostEntry(System.Net.Dns.GetHostName()).HostName;

            ClubDev.Database.ClubDevDataSet clubDevDataSet = new ClubDev.Database.ClubDevDataSet();
            ClubDev.Database.ClubDevDataSetTableAdapters.UsersTableAdapter userTableAdapter = new ClubDev.Database.ClubDevDataSetTableAdapters.UsersTableAdapter();

            Console.WriteLine("Adding collaborator to session ID: " + chosenSession.SessionID.ToString() + "  collaborator: " + collaboratorName + "  hostname: " + hostname);

            //Check to insure no other user records are present
            userTableAdapter.DeleteBySessionandHost(chosenSession.SessionID, hostname);

            bool isSharedDisplay = false;
            if (collaboratorType == "sharedScreen")
                isSharedDisplay = true;


            byte[] bitmapBytes = null;

            if (!isSharedDisplay)
            {
                PngBitmapEncoder bitmapEncoder = new PngBitmapEncoder();
                bitmapEncoder.Frames.Add(BitmapFrame.Create(collaboratorIcon));

                System.IO.MemoryStream memoryStream = new System.IO.MemoryStream();
                bitmapEncoder.Save(memoryStream);
                memoryStream.Position = 0;

                bitmapBytes = memoryStream.GetBuffer();
            }

            object returnIDObject = userTableAdapter.InsertandReturnIdentity(chosenSession.SessionID, collaboratorName, collaboratorIM, hostname, isSharedDisplay, System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width, System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height, bitmapBytes);
            long returnID = (long)Convert.ChangeType(returnIDObject, typeof(long));

            userTableAdapter.UserPing(returnID);

            currentUser = new ClubDev.Surogates.UserSurogate(returnID, chosenSession.SessionID, hostname, collaboratorName, collaboratorIM, isSharedDisplay, System.Windows.Forms.Screen.PrimaryScreen.Bounds.Width, System.Windows.Forms.Screen.PrimaryScreen.Bounds.Height, collaboratorIcon);
        }
    }
}