using System;
using System.IO;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Shapes;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
namespace ClubDev
{
    class SharedScreen : Canvas
    {
        private UIElement eventParent;

        private bool isMouseDown;
        private bool isDragInProgress;
        private bool isExpanded;
        private bool isSummaryShown;
        private bool isDropEventSet;
        private Point ptMouseDown;

        private Size sharedScreenResolution;
        private String host;
        private String screenName;
        private Rectangle backgroundRectangle;
        private ExpandButton expandButton;
        private Button redirectInputButton;
        private Label redirectInputButton2;

        private SharedScreenItemsSummary sharedScreenItemsSummary;
        private SharedScreenItemsFullView sharedScreenItemsFullView;

        private Storyboard showStoryboard;
        private Storyboard hideStoryboard;
        private Storyboard expandStoryboard;
        private Storyboard collapseStoryboard;

        private System.Windows.Threading.DispatcherTimer hideTimer;
        private System.Windows.Threading.DispatcherTimer driftGracePeriodTimer;
        private System.Windows.Threading.DispatcherTimer unhideFullViewTimer;
        private System.Windows.Threading.DispatcherTimer unhideSummaryTimer;


        public SharedScreen(Point location, String host, String screenName, Size sharedScreenResolution)
        {
            NameScope.SetNameScope(this, new NameScope());

            this.Width = 450;
            this.Height = 15;

            this.sharedScreenResolution = sharedScreenResolution;

            this.host = host;
            this.screenName = screenName;
            this.Name = "SharedScreen_" + host.Replace('.', '_').Replace('-', '_');
            this.RegisterName(this.Name, this);

            backgroundRectangle = new Rectangle();

            RadialGradientBrush showFillBrush = new RadialGradientBrush(Color.FromArgb(235, 128, 233, 255), Color.FromArgb(235, 0, 156, 190));
            backgroundRectangle.Fill = showFillBrush;

            backgroundRectangle.Stroke = new SolidColorBrush(Color.FromRgb(0, 100, 120));
            backgroundRectangle.StrokeThickness = 2;

            backgroundRectangle.Width = this.Width;
            backgroundRectangle.Height = this.Height;

            backgroundRectangle.Opacity = 1.0;

            backgroundRectangle.Name = "backgroundRectangle";
            this.RegisterName(backgroundRectangle.Name, backgroundRectangle);


            Canvas.SetLeft(this, location.X);
            Canvas.SetTop(this, -5);

            this.Children.Add(backgroundRectangle);

            expandButton = new ExpandButton(new Point(225, 100), "sharedScreen");
            expandButton.Name = "expandButton";
            expandButton.RenderTransform = new RotateTransform(90, expandButton.Width / 2, expandButton.Height / 2);
            this.RegisterName(expandButton.Name, expandButton);
            this.Children.Add(expandButton);

            expandButton.Opacity = 0.0;


            redirectInputButton = new Button();
            redirectInputButton.Name = "redirectButton";
            redirectInputButton.Content = "Redirect Input";
            //redirectInputButton.Background = new SolidColorBrush(Color.FromRgb(0, 70, 110));
            //redirectInputButton.Foreground = new SolidColorBrush(Colors.White);
            //redirectInputButton.MouseEnter += new System.Windows.Input.MouseEventHandler(redirectInputButton_MouseEnter);
            //redirectInputButton.MouseLeave += new System.Windows.Input.MouseEventHandler(redirectInputButton_MouseLeave);
            redirectInputButton.Click += new RoutedEventHandler(redirectInputButton_Click);
            this.RegisterName(redirectInputButton.Name, redirectInputButton);
            Canvas.SetLeft(redirectInputButton, 350);
            Canvas.SetTop(redirectInputButton, 75);
            //this.Children.Add(redirectInputButton);

            redirectInputButton.Opacity = 0.0;


            redirectInputButton2 = new Label();
            redirectInputButton2.Name = "redirectButton2";
            redirectInputButton2.Content = "Redirect Input";
            this.RegisterName(redirectInputButton2.Name, redirectInputButton2);
            redirectInputButton2.Width = 84;
            redirectInputButton2.Height = 25;
            redirectInputButton2.Background = new SolidColorBrush(Color.FromRgb(0,70,110));
            redirectInputButton2.Foreground = new SolidColorBrush(Colors.White);
            redirectInputButton2.BorderThickness = new Thickness(1);
            redirectInputButton2.BorderBrush = new SolidColorBrush(Color.FromRgb(0, 100, 120));
            redirectInputButton2.MouseEnter += new System.Windows.Input.MouseEventHandler(redirectInputButton2_MouseEnter);
            redirectInputButton2.MouseLeave += new System.Windows.Input.MouseEventHandler(redirectInputButton2_MouseLeave);
            redirectInputButton2.MouseUp += new System.Windows.Input.MouseButtonEventHandler(redirectInputButton2_MouseUp);
            Canvas.SetLeft(redirectInputButton2, 360);
            Canvas.SetTop(redirectInputButton2, 70);
            this.Children.Add(redirectInputButton2);

            redirectInputButton2.Opacity = 0.0;



            this.MouseLeave += new System.Windows.Input.MouseEventHandler(MouseLeaveEventHandler);
            this.MouseEnter += new System.Windows.Input.MouseEventHandler(MouseEnterEventHandler);

            this.MouseDown +=new System.Windows.Input.MouseButtonEventHandler(MouseDownEventHandler);
            this.MouseUp += new System.Windows.Input.MouseButtonEventHandler(MouseUpEventHandler);


            backgroundRectangle.MouseDown += new System.Windows.Input.MouseButtonEventHandler(BackgroundRectangleMouseDownEventHandler);
            backgroundRectangle.MouseMove += new System.Windows.Input.MouseEventHandler(BackgroundRectangleMouseMoveEventHandler);
            backgroundRectangle.MouseUp += new System.Windows.Input.MouseButtonEventHandler(BackgroundRectangleMouseUpEventHandler);

            expandButton.Click += new ExpandButton.ExpandButtonClickHandler(expandButtonEventHandler);

            this.SetupTimers();
            this.InitalizeAnimations();

            this.AllowDrop = true;
            this.DragEnter += new DragEventHandler(DragEnterEventHandler);
            this.DragLeave += new DragEventHandler(DragLeaveEventHandler);
            this.Drop +=new DragEventHandler(DropEventHandler);
        }

        void redirectInputButton2_MouseLeave(object sender, System.Windows.Input.MouseEventArgs e)
        {
            redirectInputButton2.Foreground = new SolidColorBrush(Colors.White);
            redirectInputButton2.Background = new SolidColorBrush(Color.FromRgb(0, 70, 110));
        }

        void redirectInputButton2_MouseEnter(object sender, System.Windows.Input.MouseEventArgs e)
        {
            redirectInputButton2.Foreground = new SolidColorBrush(Color.FromRgb(0, 70, 110));
            redirectInputButton2.Background = new SolidColorBrush(Color.FromRgb(210, 220, 230));
        }

        void redirectInputButton2_MouseUp(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            CuRT.CuRT.SendInputToHost(Program.CurrentUser.UserName, this.screenName);
        }

        void redirectInputButton_MouseLeave(object sender, System.Windows.Input.MouseEventArgs e)
        {
            redirectInputButton.Foreground = new SolidColorBrush(Colors.White);
            
        }

        void redirectInputButton_MouseEnter(object sender, System.Windows.Input.MouseEventArgs e)
        {
            redirectInputButton.Foreground = new SolidColorBrush(Color.FromRgb(0, 70, 110));
            
        }

        void redirectInputButton_Click(object sender, RoutedEventArgs e)
        {
            CuRT.CuRT.SendInputToHost(Program.CurrentUser.UserName, this.screenName);
        }


    
        void BackgroundRectangleMouseDownEventHandler(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            this.isMouseDown = true;
            this.ptMouseDown = Utilities.MouseUtilities.GetMousePosition(this);
        }

        void BackgroundRectangleMouseMoveEventHandler(object sender, System.Windows.Input.MouseEventArgs e)
        {
            if (isMouseDown && ptMouseDown != null)
            {
                Point ptCursor = Utilities.MouseUtilities.GetMousePosition(this);

                double currentX = Canvas.GetLeft(this);
                Canvas.SetLeft(this, currentX + (ptCursor.X - this.ptMouseDown.X));
            }
        }

        void BackgroundRectangleMouseUpEventHandler(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            this.isMouseDown = false;
        }

        void MouseUpEventHandler(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            isDragInProgress = false;
            Console.WriteLine("MOUSE UP EVENT");
        }

        void MouseDownEventHandler(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            isDragInProgress = true;
            Console.WriteLine("MOUSE Down EVENT");

        }


        private void MouseEnterEventHandler(object sender, System.Windows.Input.MouseEventArgs e)
        {
            if (!isSummaryShown)
                this.ShowSharedScreen();

            if (driftGracePeriodTimer.IsEnabled)
            {
                driftGracePeriodTimer.Stop();
                driftGracePeriodTimer.IsEnabled = false;

                Console.WriteLine("TIMER DELAYED");
            }

                
        }

        private void MouseLeaveEventHandler(object sender, System.Windows.Input.MouseEventArgs e)
        {
            if (!isDragInProgress)
            {

                if (!driftGracePeriodTimer.IsEnabled && isSummaryShown)
                {

                    driftGracePeriodTimer.IsEnabled = true;
                    driftGracePeriodTimer.Start();

                    Console.WriteLine("MOUSE OUT");
                }
            }

            else
            {
                Console.WriteLine("DENY MOUSE OUT");
            }

        }

        void DragLeaveEventHandler(object sender, DragEventArgs e)
        {
            if (isSummaryShown && !driftGracePeriodTimer.IsEnabled)
            {
                if (isDragInProgress)
                {
                    if (!isDropEventSet)
                    {
                        eventParent.Drop += new DragEventHandler(DropEventHandler);
                        isDropEventSet = true;
                        Console.WriteLine("DRAG MOUSE EVENT SET");
                    }

                }
                else
                {
                    driftGracePeriodTimer.Interval = TimeSpan.FromMilliseconds(500);

                    driftGracePeriodTimer.IsEnabled = true;
                    driftGracePeriodTimer.Start();

                    Console.WriteLine("DRAG MOUSE OUT");
                }

            }
        }

        void DragEnterEventHandler(object sender, DragEventArgs e)
        {
            if (!isSummaryShown)
                this.ShowSharedScreen();

            if (driftGracePeriodTimer.IsEnabled)
            {
                driftGracePeriodTimer.Stop();
                driftGracePeriodTimer.IsEnabled = false;

                Console.WriteLine("TIMER DELAYED");
            }
        }


        void DropEventHandler(object sender, DragEventArgs e)
        {
            Rect currentCollaboratorBounds = VisualTreeHelper.GetDescendantBounds(this);
            Point currentMousePosition = Utilities.MouseUtilities.GetMousePosition(this);

            if (isSummaryShown && !currentCollaboratorBounds.Contains(currentMousePosition))
                this.HideSharedScreen();

            eventParent.Drop -= DropEventHandler;
            isDropEventSet = false;
            isDragInProgress = false;

            foreach (string format in e.Data.GetFormats())
            {
                Console.WriteLine(format);
            }
        }

        private void SetupTimers()
        {
            hideTimer = new System.Windows.Threading.DispatcherTimer();
            hideTimer.Interval = TimeSpan.FromMilliseconds(500);
            hideTimer.Tick += new EventHandler(hideTimerEventHandler);

            driftGracePeriodTimer = new System.Windows.Threading.DispatcherTimer();
            driftGracePeriodTimer.Interval = TimeSpan.FromMilliseconds(755);
            driftGracePeriodTimer.Tick += new EventHandler(DriftGracePeriodTimerEventHandler);

            unhideFullViewTimer = new System.Windows.Threading.DispatcherTimer();
            unhideFullViewTimer.Interval = TimeSpan.FromMilliseconds(750);
            unhideFullViewTimer.Tick += new EventHandler(UnhideFullViewTimerEventHandler);

            unhideSummaryTimer = new System.Windows.Threading.DispatcherTimer();
            unhideSummaryTimer.Interval = TimeSpan.FromMilliseconds(250);
            unhideSummaryTimer.Tick += new EventHandler(UnhideSummaryViewTimerEventHandler);

        }


        void hideTimerEventHandler(object sender, EventArgs e)
        {
            hideTimer.Stop();
            hideTimer.IsEnabled = false;
            isSummaryShown = false;
        }

        private void DriftGracePeriodTimerEventHandler(object sender, EventArgs e)
        {
            Console.WriteLine("GRACE OVER");

            this.HideSharedScreen();
        }

        void expandButtonEventHandler(object sender, ExpandButtonEventArgs e)
        {
            if (isExpanded)
            {
                this.CollapseSharedSreen();

            }
            else
            {
                this.ExpandSharedScreen();
            }
        }


        private void InitalizeAnimations()
        {
            //common items
            RotateTransform pullRotateTransform = new RotateTransform();
            this.RegisterName("pullRotateTransform", pullRotateTransform);
            pullRotateTransform.CenterX = (expandButton.Width / 2);
            pullRotateTransform.CenterY = (expandButton.Height / 2);

            TranslateTransform pullTranslateTransform = new TranslateTransform();
            this.RegisterName("pullTranslateTransform", pullTranslateTransform);

            TransformGroup pullTransformGroup = new TransformGroup();
            pullTransformGroup.Children.Add(pullRotateTransform);
            pullTransformGroup.Children.Add(pullTranslateTransform);
            expandButton.RenderTransform = pullTransformGroup;

            redirectInputButton2.RenderTransform = pullTranslateTransform;


            // show Animation
            DoubleAnimationUsingKeyFrames showAnimation = new DoubleAnimationUsingKeyFrames();
            showAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(backgroundRectangle.Height, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(0))));
            showAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(100, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(250))));
            showAnimation.AutoReverse = false;

            DoubleAnimationUsingKeyFrames showExpandButtonFadeInAnimation = new DoubleAnimationUsingKeyFrames();
            showExpandButtonFadeInAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(0, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(250))));
            showExpandButtonFadeInAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(1, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(750))));
            showExpandButtonFadeInAnimation.AutoReverse = false;

            DoubleAnimationUsingKeyFrames showRedirectInputButtonFadeInAnimation = new DoubleAnimationUsingKeyFrames();
            showRedirectInputButtonFadeInAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(0, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(250))));
            showRedirectInputButtonFadeInAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(1, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(750))));
            showRedirectInputButtonFadeInAnimation.AutoReverse = false;

            DoubleAnimationUsingKeyFrames showExpandButtonRotateAnimation = new DoubleAnimationUsingKeyFrames();
            showExpandButtonRotateAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(0, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(0))));
            showExpandButtonRotateAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(90, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(10))));
            showExpandButtonRotateAnimation.AutoReverse = false;

            showStoryboard = new Storyboard();
            showStoryboard.Children.Add(showAnimation);
            showStoryboard.Children.Add(showExpandButtonFadeInAnimation);
            showStoryboard.Children.Add(showRedirectInputButtonFadeInAnimation);
            showStoryboard.Children.Add(showExpandButtonRotateAnimation);

            Storyboard.SetTargetName(showAnimation, backgroundRectangle.Name);
            Storyboard.SetTargetName(showExpandButtonFadeInAnimation, expandButton.Name);
            Storyboard.SetTargetName(showRedirectInputButtonFadeInAnimation, redirectInputButton2.Name);
            Storyboard.SetTargetName(showExpandButtonRotateAnimation, "pullRotateTransform");

            Storyboard.SetTargetProperty(showAnimation, new PropertyPath(System.Windows.Shapes.Rectangle.HeightProperty));
            Storyboard.SetTargetProperty(showExpandButtonFadeInAnimation, new PropertyPath(System.Windows.Shapes.Path.OpacityProperty));
            Storyboard.SetTargetProperty(showRedirectInputButtonFadeInAnimation, new PropertyPath(System.Windows.Shapes.Path.OpacityProperty));
            Storyboard.SetTargetProperty(showExpandButtonRotateAnimation, new PropertyPath(RotateTransform.AngleProperty));


            // hide Animation
            DoubleAnimationUsingKeyFrames hideAnimation = new DoubleAnimationUsingKeyFrames();
            hideAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(100, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(0))));
            hideAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(backgroundRectangle.Height, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(750))));
            hideAnimation.AutoReverse = false;

            DoubleAnimationUsingKeyFrames hideExpandButtonFadeOutAnimation = new DoubleAnimationUsingKeyFrames();
            hideExpandButtonFadeOutAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(1, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(0))));
            hideExpandButtonFadeOutAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(0, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(1))));
            hideExpandButtonFadeOutAnimation.AutoReverse = false;

            DoubleAnimationUsingKeyFrames hideRedirectInputButtonFadeOutAnimation = new DoubleAnimationUsingKeyFrames();
            hideRedirectInputButtonFadeOutAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(1, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(0))));
            hideRedirectInputButtonFadeOutAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(0, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(1))));
            hideRedirectInputButtonFadeOutAnimation.AutoReverse = false;


            hideStoryboard = new Storyboard();
            hideStoryboard.Children.Add(hideAnimation);
            hideStoryboard.Children.Add(hideExpandButtonFadeOutAnimation);
            hideStoryboard.Children.Add(hideRedirectInputButtonFadeOutAnimation);

            Storyboard.SetTargetName(hideAnimation, backgroundRectangle.Name);
            Storyboard.SetTargetName(hideExpandButtonFadeOutAnimation, expandButton.Name);
            Storyboard.SetTargetName(hideRedirectInputButtonFadeOutAnimation, redirectInputButton2.Name);

            Storyboard.SetTargetProperty(hideAnimation, new PropertyPath(System.Windows.Shapes.Rectangle.HeightProperty));
            Storyboard.SetTargetProperty(hideExpandButtonFadeOutAnimation, new PropertyPath(System.Windows.Shapes.Path.OpacityProperty));
            Storyboard.SetTargetProperty(hideRedirectInputButtonFadeOutAnimation, new PropertyPath(System.Windows.Shapes.Path.OpacityProperty));


            // expand Animation
            DoubleAnimationUsingKeyFrames expandAnimation = new DoubleAnimationUsingKeyFrames();
            expandAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(100, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(0))));
            expandAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(300, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(500))));
            expandAnimation.AutoReverse = false;

            DoubleAnimationUsingKeyFrames expandExpandButtonRotateAnimation = new DoubleAnimationUsingKeyFrames();
            expandExpandButtonRotateAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(90, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(0))));
            expandExpandButtonRotateAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(270, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(10))));
            expandExpandButtonRotateAnimation.AutoReverse = false;

            DoubleAnimationUsingKeyFrames expandExpandButtonTranslateAnimation = new DoubleAnimationUsingKeyFrames();
            expandExpandButtonTranslateAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(0, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(0))));
            expandExpandButtonTranslateAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(200, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(500))));
            expandExpandButtonTranslateAnimation.AutoReverse = false;

            DoubleAnimationUsingKeyFrames expandRedirectInputButtonTranslateAnimation = new DoubleAnimationUsingKeyFrames();
            expandRedirectInputButtonTranslateAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(0, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(0))));
            expandRedirectInputButtonTranslateAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(200, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(500))));
            expandRedirectInputButtonTranslateAnimation.AutoReverse = false;

            expandStoryboard = new Storyboard();
            expandStoryboard.Children.Add(expandAnimation);
            expandStoryboard.Children.Add(expandExpandButtonRotateAnimation);
            expandStoryboard.Children.Add(expandExpandButtonTranslateAnimation);
            expandStoryboard.Children.Add(expandRedirectInputButtonTranslateAnimation);

            Storyboard.SetTargetName(expandAnimation, backgroundRectangle.Name);
            Storyboard.SetTargetName(expandExpandButtonRotateAnimation, "pullRotateTransform");
            Storyboard.SetTargetName(expandExpandButtonTranslateAnimation, "pullTranslateTransform");
            Storyboard.SetTargetName(expandRedirectInputButtonTranslateAnimation, "pullTranslateTransform");


            Storyboard.SetTargetProperty(expandAnimation, new PropertyPath(System.Windows.Shapes.Rectangle.HeightProperty));
            Storyboard.SetTargetProperty(expandExpandButtonRotateAnimation, new PropertyPath(RotateTransform.AngleProperty));
            Storyboard.SetTargetProperty(expandExpandButtonTranslateAnimation, new PropertyPath(TranslateTransform.YProperty));
            Storyboard.SetTargetProperty(expandRedirectInputButtonTranslateAnimation, new PropertyPath(TranslateTransform.YProperty));



            // collapse Animation
            DoubleAnimationUsingKeyFrames collapseAnimation = new DoubleAnimationUsingKeyFrames();
            collapseAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(300, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(0))));
            collapseAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(100, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(500))));
            collapseAnimation.AutoReverse = false;

            DoubleAnimationUsingKeyFrames collapseExpandButtonRotateAnimation = new DoubleAnimationUsingKeyFrames();
            collapseExpandButtonRotateAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(270, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(0))));
            collapseExpandButtonRotateAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(90, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(1))));
            collapseExpandButtonRotateAnimation.AutoReverse = false;

            DoubleAnimationUsingKeyFrames collapseExpandButtonTranslateAnimation = new DoubleAnimationUsingKeyFrames();
            collapseExpandButtonTranslateAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(200, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(0))));
            collapseExpandButtonTranslateAnimation.KeyFrames.Add(new LinearDoubleKeyFrame(0, KeyTime.FromTimeSpan(TimeSpan.FromMilliseconds(500))));
            collapseExpandButtonTranslateAnimation.AutoReverse = false;

            collapseStoryboard = new Storyboard();
            collapseStoryboard.Children.Add(collapseAnimation);
            collapseStoryboard.Children.Add(collapseExpandButtonRotateAnimation);
            collapseStoryboard.Children.Add(collapseExpandButtonTranslateAnimation);

            Storyboard.SetTargetName(collapseAnimation, backgroundRectangle.Name);
            Storyboard.SetTargetName(collapseExpandButtonRotateAnimation, "pullRotateTransform");
            Storyboard.SetTargetName(collapseExpandButtonTranslateAnimation, "pullTranslateTransform");


            Storyboard.SetTargetProperty(collapseAnimation, new PropertyPath(System.Windows.Shapes.Rectangle.HeightProperty));
            Storyboard.SetTargetProperty(collapseExpandButtonRotateAnimation, new PropertyPath(RotateTransform.AngleProperty));
            Storyboard.SetTargetProperty(collapseExpandButtonTranslateAnimation, new PropertyPath(TranslateTransform.YProperty)); 


        }

        public void InitializeEvents(UIElement eventParent)
        {
            this.eventParent = eventParent;
        }

        private void HideSharedScreen()
        {
            driftGracePeriodTimer.Stop();
            driftGracePeriodTimer.IsEnabled = false;

            if (sharedScreenItemsSummary != null)
            {
                this.Children.Remove(sharedScreenItemsSummary);
                sharedScreenItemsSummary.Unload();
                sharedScreenItemsSummary = null;
            }

            if (sharedScreenItemsFullView != null)
            {
                this.Children.Remove(sharedScreenItemsFullView);
                sharedScreenItemsFullView.Unload();
                sharedScreenItemsFullView = null;
            }

            if (isExpanded)
            {
                collapseStoryboard.Begin(this, true);
                isExpanded = false;
            }

            hideStoryboard.Begin(this, true);

            hideTimer.IsEnabled = true;
            hideTimer.Start();
        }

        private void ShowSharedScreen()
        {
            showStoryboard.Begin(this, true);

            sharedScreenItemsSummary = new SharedScreenItemsSummary(new Point(0, 0), this.host);
            sharedScreenItemsSummary.Visibility = Visibility.Hidden;

            this.Children.Add(sharedScreenItemsSummary);

            if (eventParent != null)
                sharedScreenItemsSummary.InitializeEvents(eventParent);

            isSummaryShown = true;

            unhideSummaryTimer.IsEnabled = true;
            unhideSummaryTimer.Start();
        }

        private void ExpandSharedScreen()
        {
            expandStoryboard.Begin(this, true);

            if (sharedScreenItemsSummary != null)
            {
                this.Children.Remove(sharedScreenItemsSummary);
                sharedScreenItemsSummary.Unload();
                sharedScreenItemsSummary = null;
            }

            sharedScreenItemsFullView = new SharedScreenItemsFullView(new Size(450, 300), this.host, this.sharedScreenResolution);
            sharedScreenItemsFullView.Visibility = Visibility.Hidden;

            this.Children.Add(sharedScreenItemsFullView);

            if (eventParent != null)
                sharedScreenItemsFullView.InitializeEvents(eventParent);

            isExpanded = true;

            unhideFullViewTimer.IsEnabled = true;
            unhideFullViewTimer.Start();
        }

        private void CollapseSharedSreen()
        {
            collapseStoryboard.Begin(this, true);

            if (sharedScreenItemsFullView != null)
            {
                this.Children.Remove(sharedScreenItemsFullView);
                sharedScreenItemsFullView.Unload();
                sharedScreenItemsFullView = null;
            }

            sharedScreenItemsSummary = new SharedScreenItemsSummary(new Point(0, 0), this.host);
            sharedScreenItemsSummary.Visibility = Visibility.Hidden;

            this.Children.Add(sharedScreenItemsSummary);

            if (eventParent != null)
                sharedScreenItemsSummary.InitializeEvents(eventParent);

            isExpanded = false;

            unhideSummaryTimer.IsEnabled = true;
            unhideSummaryTimer.Start();
        }

        void UnhideSummaryViewTimerEventHandler(object sender, EventArgs e)
        {
            unhideSummaryTimer.Stop();
            unhideSummaryTimer.IsEnabled = false;

            if (sharedScreenItemsSummary != null)
                sharedScreenItemsSummary.Visibility = Visibility.Visible;
        }

        void UnhideFullViewTimerEventHandler(object sender, EventArgs e)
        {
            unhideFullViewTimer.Stop();
            unhideFullViewTimer.IsEnabled = false;

            if (sharedScreenItemsFullView != null)
                sharedScreenItemsFullView.Visibility = Visibility.Visible;

        }

        public void InitiateShare(UIElement sourceParent, string sourceItem)
        {
            if (sourceParent.GetType() == typeof(CollaboratorItemsGrid))
            {
                CollaboratorItemsGrid parent = (CollaboratorItemsGrid)sourceParent;

                ClubDev.Database.ClubDevDataSet clubDevDataSet = new ClubDev.Database.ClubDevDataSet();
                ClubDev.Database.ClubDevDataSetTableAdapters.SharedArtifactsTableAdapter sharedArtifactsTableAdapter = new ClubDev.Database.ClubDevDataSetTableAdapters.SharedArtifactsTableAdapter();
                ClubDev.Database.ClubDevDataSetTableAdapters.ReplicationsTableAdapter replicationsTableAdapter = new ClubDev.Database.ClubDevDataSetTableAdapters.ReplicationsTableAdapter();

                long id = long.Parse(sourceItem.Substring(sourceItem.LastIndexOf("__") + 2));
                sharedArtifactsTableAdapter.FillMETAByID(clubDevDataSet.SharedArtifacts, id);


                replicationsTableAdapter.FillBySourceHostSourceHWNDandClientHost(clubDevDataSet.Replications, parent.CollabHost, clubDevDataSet.SharedArtifacts[0].artifactHWND, this.host);

                if(clubDevDataSet.Replications.Count > 0){
                    //MessageBox.Show("Application is already being replication on the current screen!", "Replication already exists!", MessageBoxButton.OK, MessageBoxImage.Information);
                    return;
                }

                replicationsTableAdapter.Insert(parent.CollabHost, clubDevDataSet.SharedArtifacts[0].artifactHWND, this.host, true, null, null, null);

                Console.WriteLine("MOVING " + parent.CollabHost + " " + clubDevDataSet.SharedArtifacts[0].artifactHWND + " TO " + this.host);

                if (sharedScreenItemsSummary != null)
                    sharedScreenItemsSummary.Refresh();
            }
        }
    }
}
