using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Data.SqlClient;
using System.Text;
using System.IO;
using System.Reflection;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Shapes;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using System.Windows.Threading;


using ClubDev.SupportProviders;
using ClubDev.Database;


namespace ClubDev
{
    class SharedScreenItemsFullView : Canvas
    {
        private delegate void NoArgDelegate();
        private delegate void GatherFinishedDelegate(SharedArtifactSurogate[] sharedArtifacts);


        private CanvasDragDropManager<Canvas> dragMgr;

        private SharedScreenItemsFullViewData data = null;
 
        private Size sharedScreenResolution;
        private String host;
        private Canvas sharedArtifactsCanvas;
        private SharedArtifactSurogate[] sharedArtifacts;

        private static BitmapSource defaultScreenShot;
        private static BitmapSource defaultIcon;

        private bool isDragging;
        private Point mouseLocationDown;



        
        public SharedScreenItemsFullView(Size workingSize, String host, Size sharedScreenResolution)
        {

            // Make sure client has permissions 
            try
            {
                SqlClientPermission perm = new SqlClientPermission(System.Security.Permissions.PermissionState.Unrestricted);
                perm.Demand();
            }
            catch
            {
                throw new ApplicationException("No permission");
            }

            try
            {
                Assembly assem = Assembly.GetExecutingAssembly();

                System.IO.StreamReader stream = new StreamReader(assem.GetManifestResourceStream("ClubDev.Icons.defaultScreenShot.jpg"));
                JpegBitmapDecoder bitmapDecoder = new JpegBitmapDecoder(stream.BaseStream, BitmapCreateOptions.PreservePixelFormat, BitmapCacheOption.None);
                defaultScreenShot = bitmapDecoder.Frames[0];
            }
            catch
            {
                Console.WriteLine("Could not import embeded resource");
            }

            try
            {
                Assembly assem = Assembly.GetExecutingAssembly();

                System.IO.StreamReader stream = new StreamReader(assem.GetManifestResourceStream("ClubDev.Icons.defaultIcon.png"));
                PngBitmapDecoder bitmapDecoder = new PngBitmapDecoder(stream.BaseStream, BitmapCreateOptions.PreservePixelFormat, BitmapCacheOption.None);
                defaultIcon = bitmapDecoder.Frames[0];
            }
            catch
            {
                Console.WriteLine("Could not import embeded resource");
            }


            this.host = host;
            this.sharedScreenResolution = sharedScreenResolution;

            this.Width = workingSize.Width;
            this.Height = workingSize.Height;

            sharedArtifactsCanvas = new Canvas();

            double widthRatio = this.Width / (double)sharedScreenResolution.Width;
            double heightRatio = (this.Height - 20) / (double)sharedScreenResolution.Height;

            double ratio = Math.Min(widthRatio, heightRatio) * 0.90;

            sharedArtifactsCanvas.Width = ratio * sharedScreenResolution.Width;
            sharedArtifactsCanvas.Height = ratio * sharedScreenResolution.Height;

            Canvas.SetTop(sharedArtifactsCanvas, (((this.Height - 20) - sharedArtifactsCanvas.Height) / 2));
            Canvas.SetLeft(sharedArtifactsCanvas, (this.Width - sharedArtifactsCanvas.Width) / 2);

            sharedArtifactsCanvas.ClipToBounds = true;

            Rectangle outline = new Rectangle();
            outline.Width = sharedArtifactsCanvas.Width + 6;
            outline.Height = sharedArtifactsCanvas.Height + 6;
            outline.RadiusX = 6;
            outline.RadiusY = 6;
            Canvas.SetTop(outline, (((this.Height - 20) - sharedArtifactsCanvas.Height) / 2) - 3);
            Canvas.SetLeft(outline, ((this.Width - sharedArtifactsCanvas.Width) / 2) -3);

            outline.Stroke = new SolidColorBrush(Color.FromArgb(255, 80, 80, 80));
            outline.StrokeThickness = 4;

            outline.Fill = new SolidColorBrush(Color.FromArgb(180, 255, 255, 255));
            
            this.Children.Add(outline);
            this.Children.Add(sharedArtifactsCanvas);
            
            
            

            
            this.isDragging = false;

            sharedArtifactsCanvas.MouseDown += new System.Windows.Input.MouseButtonEventHandler(MouseDownEventHandler);
            sharedArtifactsCanvas.MouseUp +=new System.Windows.Input.MouseButtonEventHandler(MouseUpEventHandler);

            data = new SharedScreenItemsFullViewData();
            data.OnNewMessage += new SharedScreenItemsFullViewData.NewMessage(OnNewMessage);

            NameScope.SetNameScope(sharedArtifactsCanvas, new NameScope());


            LoadMessages();


        }




        public void Unload()
        {
            data.OnNewMessage -= OnNewMessage;
            data.Unload();
            data = null;

        }


        public void InitializeEvents(UIElement eventParent)
        {
            this.dragMgr = new CanvasDragDropManager<Canvas>(sharedArtifactsCanvas, eventParent);
        }



        void MouseDownEventHandler(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            this.mouseLocationDown = Utilities.MouseUtilities.GetMousePosition(sharedArtifactsCanvas);
            this.isDragging = true;
        }


        void MouseUpEventHandler(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            
            
        }


        void OnNewMessage()
        {
            

            if (this.CheckAccess())
                LoadMessages();
            else
                this.Dispatcher.BeginInvoke(System.Windows.Threading.DispatcherPriority.Normal, (DispatcherOperationCallback)delegate(object arg)
                {
                    // HANDLED ON UI THREAD
                    LoadMessages();
                    return null;
                }, null);


        }



        /// <summary>
        /// Helper method to populate list box
        /// </summary>
        private void LoadMessages()
        {

            if (data == null)
                return;

            Console.WriteLine("INSIDE LOAD MESSAGE");

            // Get the messages
            DataTable dt = data.SetReplicationsChangesEventQuery(this.host);
            foreach (DataRow row in dt.Rows)
            {
                // FOR DEBUG
                string Msg = string.Format("CHANGE EVENT FIRED {0}", row["replicationID"]);
                Console.WriteLine(Msg);
            }

            // If we are in a drag, avoid updating the layout
            if (isDragging)
                return;

            NoArgDelegate fetcher = new NoArgDelegate(this.GatherData);
            fetcher.BeginInvoke(null, null);
       
        }


        private void GatherData()
        {
            

            DataTable dtData = data.GetArtifactsData(this.host);

            SharedArtifactSurogate[] sharedArtifacts = new SharedArtifactSurogate[dtData.Rows.Count];

            for (int i = 0; i < dtData.Rows.Count; i++)
            {
                /*
                double ratio = 1;
                if (returnedSharedArtifacts[i].ScreenShotSource.Width > returnedSharedArtifacts[i].ScreenShotSource.Height)
                    ratio = 450 / (double)returnedSharedArtifacts[i].ScreenShotSource.PixelWidth;
                else
                    ratio = 72 / (double)returnedSharedArtifacts[i].ScreenShotSource.PixelHeight;
                */

                //BitmapSource screenShot = new TransformedBitmap(dtImagesIcons.Rows[i]["artifactImage"], new System.Windows.Media.ScaleTransform(1, 1));

                //BitmapSource icon = new TransformedBitmap(returnedSharedArtifacts[i].IconSource, new System.Windows.Media.ScaleTransform(1, 1));



                BitmapSource screenShotBitmapSource = null;


                if (dtData.Rows[i]["artifactImage"].Equals(System.DBNull.Value))
                    screenShotBitmapSource = defaultScreenShot;

                else
                {
                    MemoryStream screenShotStream = new MemoryStream((byte[])dtData.Rows[i]["artifactImage"]);
                    JpegBitmapDecoder screenShotJpegDecoder = new JpegBitmapDecoder(screenShotStream, BitmapCreateOptions.PreservePixelFormat, BitmapCacheOption.None);
                    screenShotBitmapSource = screenShotJpegDecoder.Frames[0];
                }

                
                BitmapSource iconBitmapSource = null;


                if (dtData.Rows[i]["artifactIcon"].Equals(System.DBNull.Value))
                    iconBitmapSource = defaultIcon;

                else
                {
                    MemoryStream iconStream = new MemoryStream((byte[])dtData.Rows[i]["artifactIcon"]);
                    PngBitmapDecoder iconPngDecoder = new PngBitmapDecoder(iconStream, BitmapCreateOptions.PreservePixelFormat, BitmapCacheOption.None);
                    iconBitmapSource = iconPngDecoder.Frames[0];
                }
                



                sharedArtifacts[i] = new SharedArtifactSurogate();

                int zOrder = 0;
                if (!dtData.Rows[i]["artifactZOrder"].Equals(System.DBNull.Value))
                    zOrder = (int)dtData.Rows[i]["artifactZOrder"];

                int locationX = 0;
                if(!dtData.Rows[i]["replicationLocationX"].Equals(System.DBNull.Value))
                    locationX = (int)dtData.Rows[i]["replicationLocationX"];

                int locationY = 0;
                if(!dtData.Rows[i]["replicationLocationY"].Equals(System.DBNull.Value))
                    locationY = (int)dtData.Rows[i]["replicationLocationY"];


                sharedArtifacts[i].UpdateArtifact((int)((long)dtData.Rows[i]["artifactID"]), (int)dtData.Rows[i]["artifactType"],
                    dtData.Rows[i]["artifactName"].ToString(), dtData.Rows[i]["artifactGroup"].ToString(), 
                    (int)dtData.Rows[i]["artifactHWND"], screenShotBitmapSource, iconBitmapSource,
                    new Size((int)dtData.Rows[i]["artifactWidth"], (int)dtData.Rows[i]["artifactHeight"]),
                    new Point(locationX, locationY), zOrder);

            }
               

            this.Dispatcher.BeginInvoke(System.Windows.Threading.DispatcherPriority.Normal, new GatherFinishedDelegate(DrawFullView), sharedArtifacts);


        }


        private void DrawFullView(SharedArtifactSurogate[] sharedArtifacts)
        {
            this.sharedArtifacts = sharedArtifacts;
            ResetSharedArtifactCanvas();
                        
            for (int i = 0; i < sharedArtifacts.Length; i++)
            {

                Canvas currentArtifact = new Canvas();

                currentArtifact.Width = ((double)((int)sharedArtifacts[i].Size.Width) / (double)sharedScreenResolution.Width) * sharedArtifactsCanvas.Width;
                currentArtifact.Height = ((double)((int)sharedArtifacts[i].Size.Height) / (double)sharedScreenResolution.Height) * sharedArtifactsCanvas.Height;

                Canvas.SetLeft(currentArtifact, ((double)((int)sharedArtifacts[i].Location.X) / (double)sharedScreenResolution.Width) * sharedArtifactsCanvas.Width);
                Canvas.SetTop(currentArtifact, ((double)((int)sharedArtifacts[i].Location.Y) / (double)sharedScreenResolution.Height) * sharedArtifactsCanvas.Height);

                sharedArtifactsCanvas.Children.Add(currentArtifact);

                Image screenShot = new Image();
                screenShot.Source = sharedArtifacts[i].ScreenShotSource;
                screenShot.Stretch = Stretch.Fill;
                screenShot.Width = currentArtifact.Width;
                screenShot.Height = currentArtifact.Height;

                currentArtifact.Children.Add(screenShot);

                Image icon = new Image();
                icon.Source = sharedArtifacts[i].IconSource;
                Canvas.SetLeft(icon, currentArtifact.Width - icon.Source.Width);
                Canvas.SetTop(icon, currentArtifact.Height - icon.Source.Height);

                currentArtifact.Children.Add(icon);

                currentArtifact.Name = sharedArtifacts[i].Group.Replace('.', '_').Replace('-', '_') + "__" + sharedArtifacts[i].ID;
                sharedArtifactsCanvas.RegisterName(currentArtifact.Name, currentArtifact);
            }
             
        }


        private void ResetSharedArtifactCanvas()
        {
            if (sharedArtifactsCanvas != null)
            {
                sharedArtifactsCanvas.Children.Clear();
                NameScope.SetNameScope(sharedArtifactsCanvas, new NameScope());
            }

        }

        public void RepositionLocalArtifact(UIElement draggedElement)
        {
            if (this.mouseLocationDown == null)
                return;

            if(draggedElement.GetType() != typeof(Canvas))
                return;

            double origX = Canvas.GetLeft(draggedElement);
            double origY = Canvas.GetTop(draggedElement);

            Point mousePoint = Utilities.MouseUtilities.GetMousePosition(this.sharedArtifactsCanvas);

            double xDiff = mousePoint.X - this.mouseLocationDown.X;
            double yDiff = mousePoint.Y - this.mouseLocationDown.Y;

            double xRatio = this.sharedScreenResolution.Width / this.sharedArtifactsCanvas.Width;
            double yRatio = this.sharedScreenResolution.Height / this.sharedArtifactsCanvas.Height;

            Point newLocation = new Point((origX + xDiff) * xRatio, (origY + yDiff) * yRatio);

           
            long id = long.Parse(((Canvas)draggedElement).Name.Substring(((Canvas)draggedElement).Name.LastIndexOf("__") + 2));


            Database.ClubDevDataSet clubDevDataSet = new ClubDevDataSet();
            Database.ClubDevDataSetTableAdapters.SharedArtifactsTableAdapter sharedArtifactsTableAdapter = new ClubDev.Database.ClubDevDataSetTableAdapters.SharedArtifactsTableAdapter();
            Database.ClubDevDataSetTableAdapters.ReplicationsTableAdapter replicationsTableAdapter = new ClubDev.Database.ClubDevDataSetTableAdapters.ReplicationsTableAdapter();

            sharedArtifactsTableAdapter.ClearBeforeFill = true;
            sharedArtifactsTableAdapter.FillMETAByID(clubDevDataSet.SharedArtifacts, id);

            if (clubDevDataSet.SharedArtifacts.Count != 1)
                return;


            replicationsTableAdapter.UpdateLocationByClientHostandHWND((int)newLocation.X, (int)newLocation.Y, clubDevDataSet.SharedArtifacts[0].artifactGroup, clubDevDataSet.SharedArtifacts[0].artifactHWND, this.host);

            this.isDragging = false;

            this.GatherData();
        }
    
    }

}


