using System;
using System.Collections.Generic;
using System.Text;
using System.Data.SqlClient;
using System.Data;

namespace ClubDev
{
    public class SharedScreenItemsFullViewData
    {
        private static string m_ConnectionString = ClubDev.Properties.Settings.Default.ClubDevConnectionString;
        private SqlConnection m_sqlConn = null;

        public delegate void NewMessage();
        public event NewMessage OnNewMessage;

        public bool deadlockCheck;

        /// <summary>
        /// Constructor
        /// </summary>
        public SharedScreenItemsFullViewData()
        {
            // Stop an existing services on this connection string
            // just be sure
            SqlDependency.Stop(m_ConnectionString);
            
            // Start the dependency
            // User must have SUBSCRIBE QUERY NOTIFICATIONS permission
            // Database must also have SSB enabled
            // ALTER DATABASE Chatter SET ENABLE_BROKER
            SqlDependency.Start(m_ConnectionString);

            // Create the connection
            m_sqlConn = new SqlConnection(m_ConnectionString);

            deadlockCheck = false;
        }

        /// <summary>
        /// Destructor
        /// </summary>
        ~SharedScreenItemsFullViewData()
        {
            // Stop the dependency before exiting
            SqlDependency.Stop(m_ConnectionString);
        }

        public void Unload()
        {
            // Stop the dependency before exiting
            SqlDependency.Stop(m_ConnectionString);
        }


        /// <summary>
        /// Retreive messages from database
        /// </summary>
        /// <returns></returns>
        public DataTable SetReplicationsChangesEventQuery(string host)
        {
            while (deadlockCheck)
                System.Threading.Thread.Sleep(100);

            deadlockCheck = true;

            DataTable dt = new DataTable();

            try
            {
                // Create command
                // Command must use two part names for tables
                // SELECT <field> FROM dbo.Table rather than 
                // SELECT <field> FROM Table
                // Query also can not use *, fields must be designated
                SqlCommand cmd = new SqlCommand("SELECT replicationID, replicationLocationX, replicationLocationY FROM dbo.Replications WHERE (replicationClientHost= @host)", m_sqlConn);
                cmd.CommandType = CommandType.Text;
                cmd.Parameters.AddWithValue("@host", host);

                // Clear any existing notifications
                cmd.Notification = null;

                // Create the dependency for this command
                SqlDependency dependency = new SqlDependency(cmd);

                // Add the event handler
                dependency.OnChange += new OnChangeEventHandler(OnChange);

                // Open the connection if necessary
                if(m_sqlConn.State == ConnectionState.Closed)
                    m_sqlConn.Open();

                // Get the messages
                dt.Load(cmd.ExecuteReader(CommandBehavior.CloseConnection));
            }
            catch (Exception ex)
            {
                throw ex;
            }

            deadlockCheck = false;

            return dt;
        }


        public DataTable GetArtifactsData(string host)
        {
            while (deadlockCheck)
                System.Threading.Thread.Sleep(100);

            deadlockCheck = true;

            DataTable dt = new DataTable();

            try
            {
                SqlCommand cmd = new SqlCommand("SELECT dbo.SharedArtifacts.artifactID, dbo.SharedArtifacts.artifactType, dbo.SharedArtifacts.artifactName, dbo.SharedArtifacts.artifactGroup, dbo.SharedArtifacts.artifactHWND, dbo.SharedArtifacts.artifactWidth, dbo.SharedArtifacts.artifactHeight, dbo.SharedArtifacts.artifactLocationX, dbo.SharedArtifacts.artifactLocationY, dbo.SharedArtifacts.artifactZOrder, dbo.SharedArtifacts.artifactImage, dbo.SharedArtifacts.artifactIcon , dbo.Replications.replicationLocationX, dbo.Replications.replicationLocationY FROM dbo.SharedArtifacts INNER JOIN dbo.Replications ON dbo.SharedArtifacts.artifactHWND = dbo.Replications.replicationSourceHWND WHERE (dbo.Replications.replicationClientHost = @host) ORDER BY dbo.SharedArtifacts.artifactZOrder DESC", m_sqlConn);
                cmd.CommandType = CommandType.Text;
                cmd.Parameters.AddWithValue("@host", host);
                cmd.Notification = null;

                if (m_sqlConn.State == ConnectionState.Closed)
                    m_sqlConn.Open();

                // Get the messages
                dt.Load(cmd.ExecuteReader(CommandBehavior.CloseConnection));

            }
            catch (Exception ex)
            {
                throw ex;
            }

            deadlockCheck = false;
            return dt;

        }


        /// <summary>
        /// Handler for the SqlDependency OnChange event
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void OnChange(object sender, SqlNotificationEventArgs e)
        {
            SqlDependency dependency = sender as SqlDependency;
            
            // Notices are only a one shot deal
            // so remove the existing one so a new 
            // one can be added
            dependency.OnChange -= OnChange;

            // Fire the event
            if (OnNewMessage != null)
            {
                OnNewMessage();
            }
        }

    }
}
