using System;
using System.IO;
using System.Net;
using System.Windows;
using System.Windows.Input;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Media.Animation;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Media.Effects;
using System.Windows.Threading;

using ClubDev.SupportProviders;


namespace ClubDev
{
    class SharedScreenItemsSummary : Canvas
    {
        private CanvasDragDropManager<Canvas> dragMgr;

        private String host;

        private int positionOffset;
        private const int itemWidth = 82;

        private WaitStar waitStar;
        private ScrollButton leftScrollButton;
        private ScrollButton rightScrollButton;
        private Canvas sharedArtifactsCanvas;
        private BitmapSource closeIconBitmapSource;

        private SharedArtifactSurogate[] sharedArtifacts = null;


        public SharedScreenItemsSummary(Point location, String host)
        {
            this.host = host;

            Canvas.SetLeft(this, location.X);
            Canvas.SetTop(this, location.Y);

            this.Width = 450;
            this.Height = itemWidth;

            this.positionOffset = 0;

            waitStar = new WaitStar();
            this.Children.Add(waitStar);

            double scaleWidth = 40 / waitStar.Width;
            double scaleHeight = 40 / waitStar.Height;

            waitStar.RenderTransform = new ScaleTransform(scaleWidth, scaleHeight, waitStar.Width / 2, waitStar.Height / 2);

            Canvas.SetLeft(waitStar, (this.Width / 2) - (waitStar.Width / 2));
            Canvas.SetTop(waitStar, ((this.Height - 22) / 2) - (scaleHeight * waitStar.Height) / 2);

            waitStar.BeginAnimation();

            sharedArtifactsCanvas = new Canvas();
            this.Children.Add(sharedArtifactsCanvas);
            Canvas.SetLeft(sharedArtifactsCanvas, 20);
            Canvas.SetTop(sharedArtifactsCanvas, 0);
            sharedArtifactsCanvas.Width = this.Width - 40;
            sharedArtifactsCanvas.Height = this.Height;
            NameScope.SetNameScope(sharedArtifactsCanvas, new NameScope());

            try
            {
                System.IO.StreamReader stream = new StreamReader(this.GetType().Assembly.GetManifestResourceStream("ClubDev.Icons.SharedScreenGrid-close.png"));
                PngBitmapDecoder bitmapDecoder = new PngBitmapDecoder(stream.BaseStream, BitmapCreateOptions.PreservePixelFormat, BitmapCacheOption.Default);
                closeIconBitmapSource = bitmapDecoder.Frames[0];
            }
            catch { }


            ArtifactsGatherEngine.RetrieveArtifacts(host, ArtifactsGatherEngine.RetreiveType.Both);
            ArtifactsGatherEngine.Worker.RunWorkerCompleted += new System.ComponentModel.RunWorkerCompletedEventHandler(GatherWorkerCompleteEventHandler);
        }

        public void Unload()
        {
            // first write order preferences to the prefernece engine
            if (sharedArtifacts != null)
            {
                int[] currentSorting = new int[sharedArtifacts.Length];
                for (int i = 0; i < currentSorting.Length; i++)
                    currentSorting[i] = sharedArtifacts[i].ID;

                PreferencesEngine.SetPreferences(host, currentSorting);
            }


            try
            {

                if (waitStar != null)
                    waitStar.StopAnimation();

                //shared artifact canvas remove

                sharedArtifactsCanvas.Children.Clear();

                this.Children.Clear();
                

                //this.waitStar = null;
            }

            catch {
                Console.WriteLine("could not unload sharedscreen items summary");
            }

            GC.Collect();
        }


        public void Refresh()
        {
            sharedArtifactsCanvas.Children.Clear();

            if (this.Children.Contains(leftScrollButton))
                this.Children.Remove(leftScrollButton);

            if (this.Children.Contains(rightScrollButton))
                this.Children.Remove(rightScrollButton);

            waitStar.Visibility = Visibility.Visible;
            waitStar.BeginAnimation();

            ArtifactsGatherEngine.RetrieveArtifacts(this.host, ArtifactsGatherEngine.RetreiveType.Both);
            ArtifactsGatherEngine.Worker.RunWorkerCompleted += new System.ComponentModel.RunWorkerCompletedEventHandler(GatherWorkerCompleteEventHandler);

        }


        public void InitializeEvents(UIElement eventParent)
        {
            this.dragMgr = new CanvasDragDropManager<Canvas>(sharedArtifactsCanvas, eventParent);
        }

        public void AddItem(Canvas addItem, int position)
        {
            

            //need to rewrite to sort array

            /*
            position = (itemWidth * position);

            for (int i = 0; i < sharedArtifactsCanvas.Children.Count; i++)
            {
                if (sharedArtifactsCanvas.Children[i].GetType().Equals(addItem.GetType()))
                {
                    double currentPosition = Canvas.GetLeft(sharedArtifactsCanvas.Children[i]);

                    if (currentPosition >= (double)position)
                    {
                        Canvas.SetLeft(sharedArtifactsCanvas.Children[i], currentPosition + itemWidth);
                        Canvas.SetTop(addItem, 0);
                    }
                }
            }

            Canvas.SetLeft(addItem, position);
            Canvas.SetTop(addItem, 0);
            sharedArtifactsCanvas.Children.Add(addItem);

            itemsCount++;
            */
        }

        
        public void RepositionItem(Canvas repositionItem, Point mouseLocation)
        {
            if (sharedArtifacts.Length <= 1)
                return;

            int newPosition = (int)Math.Round((double)mouseLocation.X / itemWidth);
            newPosition += positionOffset;

            // Check to make sure item is not being positioned outside the bounds of the containing array
            if (newPosition >= sharedArtifacts.Length)
                newPosition = sharedArtifacts.Length - 1;
            else if (newPosition < 0)
                newPosition = 0;

            SharedArtifactSurogate[] removedSharedArtifacts = new SharedArtifactSurogate[sharedArtifacts.Length - 1];
            int currentLocation = 0;

            SharedArtifactSurogate removedSharedArtifact = null;

            for (int i = 0; i < sharedArtifacts.Length; i++)
            {
                String artifactID = this.host.Replace(".", "_").Replace("-", "_") + "__" + sharedArtifacts[i].ID.ToString();
                if (!artifactID.Equals(repositionItem.Name))
                {
                    removedSharedArtifacts[currentLocation] = sharedArtifacts[i];
                    currentLocation++;
                }
                else
                {
                    removedSharedArtifact = sharedArtifacts[i];
                }
            }

            if (removedSharedArtifact == null)
                sharedArtifacts = removedSharedArtifacts;

            else
            {
                SharedArtifactSurogate[] addedSharedArtifacts = new SharedArtifactSurogate[removedSharedArtifacts.Length + 1];

                for (int i = 0; i < newPosition; i++)
                    addedSharedArtifacts[i] = removedSharedArtifacts[i];

                addedSharedArtifacts[newPosition] = removedSharedArtifact;

                for (int i = newPosition + 1; i < addedSharedArtifacts.Length; i++)
                    addedSharedArtifacts[i] = removedSharedArtifacts[i - 1];

                sharedArtifacts = addedSharedArtifacts;

            }

            DrawSharedArtifactGrid(positionOffset, positionOffset + 4);
        }


        public void RemoveItem(Canvas removeItem)
        {
            SharedArtifactSurogate[] newSharedArtifacts = new SharedArtifactSurogate[sharedArtifacts.Length - 1];
            int currentLocation = 0;

            for (int i = 0; i < sharedArtifacts.Length; i++)
            {
                String artifactID = this.host + "__" + sharedArtifacts[i].ID.ToString();

                Console.WriteLine(artifactID);

                if (!artifactID.Equals(removeItem.Name))
                {
                    newSharedArtifacts[currentLocation] = sharedArtifacts[i];
                    currentLocation++;
                }
            }

            sharedArtifacts = newSharedArtifacts;

            DrawSharedArtifactGrid(positionOffset, positionOffset + 4);


        }

        void GatherWorkerCompleteEventHandler(object sender, System.ComponentModel.RunWorkerCompletedEventArgs e)
        {
            if (waitStar != null)
            {
                waitStar.StopAnimation();
                this.Children.Remove(waitStar);
                waitStar.Visibility = Visibility.Hidden;
            }
             
            ArtifactsGatherEngine.AbortRetrieveArtifacts();
            SharedArtifactSurogate[] returnedSharedArtifacts = ArtifactsGatherEngine.SharedArtifactSurogate;

            if (returnedSharedArtifacts != null)
            {
                sharedArtifacts = new SharedArtifactSurogate[returnedSharedArtifacts.Length];

                for (int i = 0; i < returnedSharedArtifacts.Length; i++)
                {

                    double ratio = 1;
                    if (returnedSharedArtifacts[i].ScreenShotSource.Width > returnedSharedArtifacts[i].ScreenShotSource.Height)
                        ratio = 72 / (double)returnedSharedArtifacts[i].ScreenShotSource.PixelWidth;
                    else
                        ratio = 72 / (double)returnedSharedArtifacts[i].ScreenShotSource.PixelHeight;

                    BitmapSource screenShot = new TransformedBitmap(returnedSharedArtifacts[i].ScreenShotSource, new System.Windows.Media.ScaleTransform(ratio, ratio));

                    BitmapSource icon = new TransformedBitmap(returnedSharedArtifacts[i].IconSource, new System.Windows.Media.ScaleTransform(1, 1));

                    sharedArtifacts[i] = new SharedArtifactSurogate();
                    sharedArtifacts[i].UpdateArtifact(returnedSharedArtifacts[i].ID, returnedSharedArtifacts[i].Type, returnedSharedArtifacts[i].ArtifactName.ToString(), returnedSharedArtifacts[i].Group.ToString(), returnedSharedArtifacts[i].HWND, screenShot, icon, returnedSharedArtifacts[i].Size, returnedSharedArtifacts[i].Location, returnedSharedArtifacts[i].ZOrder); 
                }
            }

            ArtifactsGatherEngine.CleanCache();

            leftScrollButton = new ScrollButton(new Point(10, this.Height / 2));
            rightScrollButton = new ScrollButton(new Point(this.Width - 10, this.Height / 2));
            rightScrollButton.RenderTransform = new RotateTransform(180, rightScrollButton.Width / 2, rightScrollButton.Width / 2);

            leftScrollButton.Click += new ScrollButton.ScrollButtonClickHandler(LeftScrollButtonClickEventHandler);
            rightScrollButton.Click += new ScrollButton.ScrollButtonClickHandler(RightScrollButtonClickEventHandler);

            leftScrollButton.Enabled = false;
            rightScrollButton.Enabled = false;


            this.Children.Add(rightScrollButton);
            this.Children.Add(leftScrollButton);

            if(sharedArtifacts != null)
                if(sharedArtifacts.Length > 5)
                    rightScrollButton.Enabled = true;


            DrawSharedArtifactGrid(0, 4);


        }


        void LeftScrollButtonClickEventHandler(object sender, ScrollButtonEventArgs e)
        {
            if (positionOffset > 0)
            {
                positionOffset -= 5;
                DrawSharedArtifactGrid(positionOffset, positionOffset + 4);

                if (!(positionOffset > 0))
                {
                    leftScrollButton.Enabled = false;
                }

                if (sharedArtifacts.Length > 5)
                    rightScrollButton.Enabled = true;
            }

        }

        void RightScrollButtonClickEventHandler(object sender, ScrollButtonEventArgs e)
        {
            if (sharedArtifacts.Length > (positionOffset + 5))
            {
                positionOffset += 5;
                DrawSharedArtifactGrid(positionOffset, positionOffset + 4);

                if (!(sharedArtifacts.Length > (positionOffset + 4)))
                {
                    rightScrollButton.Enabled = false;
                }
                leftScrollButton.Enabled = true;
            }
        }




        private void DrawSharedArtifactGrid(int from, int to)
        {
            ResetSharedArtifactGrid();

            if (to > (sharedArtifacts.Length - 1))
                to = (sharedArtifacts.Length - 1);

            if (from < 0)
                from = 0;

            int currentPosition = 0;

            for (int i = from; i <= to; i++)
            {
                Canvas currentArtifact = new Canvas();

                currentArtifact.Width = itemWidth;
                currentArtifact.Height = itemWidth;


                Image currentArtifactThumbnail = new Image();
                currentArtifactThumbnail.Source = sharedArtifacts[i].ScreenShotSource;

                currentArtifactThumbnail.Width = currentArtifactThumbnail.Source.Width;
                currentArtifactThumbnail.Height = currentArtifactThumbnail.Source.Height;

                Canvas.SetLeft(currentArtifactThumbnail, (currentArtifact.Width - currentArtifactThumbnail.Width) / 2);
                Canvas.SetTop(currentArtifactThumbnail, ((currentArtifact.Height - 10) - currentArtifactThumbnail.Height) / 2);

                currentArtifact.Children.Add(currentArtifactThumbnail);


                Image currentArtifactIcon = new Image();
                currentArtifactIcon.Source = sharedArtifacts[i].IconSource;

                currentArtifactIcon.Width = sharedArtifacts[i].IconSource.Width;
                currentArtifactIcon.Height = sharedArtifacts[i].IconSource.Height;

                Canvas.SetLeft(currentArtifactIcon, currentArtifactThumbnail.Width - currentArtifactIcon.Width);
                Canvas.SetTop(currentArtifactIcon, currentArtifactThumbnail.Height - currentArtifactIcon.Height);

                currentArtifact.Children.Add(currentArtifactIcon);


                Label currentArtifactLabel = new Label();
                currentArtifactLabel.Content = sharedArtifacts[i].ArtifactName;
                currentArtifactLabel.FontSize = 10;
                currentArtifactLabel.VerticalAlignment = VerticalAlignment.Top;

                OuterGlowBitmapEffect glowEffect = new OuterGlowBitmapEffect();
                glowEffect.GlowSize = 10;
                glowEffect.GlowColor = Color.FromArgb(255, 255, 255, 255);
                glowEffect.Noise = 0;
                glowEffect.Opacity = 1;

                currentArtifactLabel.BitmapEffect = glowEffect;

                ToolTip artifactNameTip = new ToolTip();
                artifactNameTip.Content = sharedArtifacts[i].ArtifactName;
                currentArtifact.ToolTip = artifactNameTip;

                currentArtifact.Children.Add(currentArtifactLabel);


                currentArtifact.Name = host.ToString().Replace('.', '_').Replace('-', '_') + "__" + sharedArtifacts[i].ID.ToString();
                sharedArtifactsCanvas.RegisterName(currentArtifact.Name, currentArtifact);


                Image closeIcon = new Image();
                closeIcon.Source = this.closeIconBitmapSource;
                Canvas.SetLeft(closeIcon, Canvas.GetLeft(currentArtifactThumbnail) + currentArtifactThumbnail.Width - closeIcon.Source.Width);
                Canvas.SetTop(closeIcon, Canvas.GetTop(currentArtifactThumbnail));

                closeIcon.Name = host.ToString().Replace('.', '_').Replace('-', '_') + "closeIcon__" + sharedArtifacts[i].ID.ToString();
                sharedArtifactsCanvas.RegisterName(closeIcon.Name, closeIcon);

                currentArtifact.Children.Add(closeIcon);

                closeIcon.MouseDown += new MouseButtonEventHandler(closeIconMouseDownEventHandler);


                Canvas.SetLeft(currentArtifact, currentPosition * itemWidth);
                Canvas.SetTop(currentArtifact, 0);
                sharedArtifactsCanvas.Children.Add(currentArtifact);

                currentPosition++;

                currentArtifactLabel.UpdateLayout();

                FitLabel(currentArtifactLabel, currentArtifact.Width - 4);

                Canvas.SetLeft(currentArtifactLabel, (currentArtifact.Width - currentArtifactLabel.ActualWidth) / 2);
                Canvas.SetTop(currentArtifactLabel, (currentArtifact.Height - currentArtifactLabel.ActualHeight) + 5);

                currentArtifact.MouseEnter += new MouseEventHandler(currentArtifactMouseEnterEventHandler);
                currentArtifact.MouseLeave += new MouseEventHandler(currentArtifactMouseLeaveEventHandler);
                currentArtifact.MouseUp += new MouseButtonEventHandler(currentArtifactMouseUpEventHandler);


            }
        }


        private void ResetSharedArtifactGrid()
        {
            if (sharedArtifactsCanvas != null)
            {
                sharedArtifactsCanvas.Children.Clear();
                NameScope.SetNameScope(sharedArtifactsCanvas, new NameScope());
            }

        }

        private void FitLabel(Label label, double desiredWidth)
        {
            while (label.ActualWidth > desiredWidth)
            {
                if (label.Content.ToString().Equals("..."))
                    break;
                label.Content = label.Content.ToString().Substring(0, (label.Content.ToString().Length - 4)) + "...";
                label.UpdateLayout();
            }
        }

        void currentArtifactMouseEnterEventHandler(object sender, MouseEventArgs e)
        {
            if (sender.GetType() == typeof(Canvas))
            {
                OuterGlowBitmapEffect glowEffect = new OuterGlowBitmapEffect();
                glowEffect.GlowSize = 10;
                glowEffect.GlowColor = Color.FromArgb(255, 255, 255, 255);
                glowEffect.Noise = 0;
                glowEffect.Opacity = 1;

                ((Canvas)sender).BitmapEffect = glowEffect;
            }
        }

        void currentArtifactMouseLeaveEventHandler(object sender, MouseEventArgs e)
        {
            if (sender.GetType() == typeof(Canvas))
            {
                ((Canvas)sender).BitmapEffect = null;
            }

        }
        void currentArtifactMouseUpEventHandler(object sender, MouseButtonEventArgs e)
        {
            if (sender.GetType() == typeof(Canvas))
            {
                ((Canvas)sender).BitmapEffect = null;
            }
        }


        void closeIconMouseDownEventHandler(object sender, MouseButtonEventArgs e)
        {
            if (sender.GetType() != typeof(Image))
                return;

            Image senderImage = (Image)sender;

            long id = long.Parse(senderImage.Name.Substring(senderImage.Name.LastIndexOf("__") + 2));


            ClubDev.Database.ClubDevDataSet clubDevDataSet = new ClubDev.Database.ClubDevDataSet();
            ClubDev.Database.ClubDevDataSetTableAdapters.SharedArtifactsTableAdapter sharedArtifactsTableAdapter = new ClubDev.Database.ClubDevDataSetTableAdapters.SharedArtifactsTableAdapter();
            ClubDev.Database.ClubDevDataSetTableAdapters.ReplicationsTableAdapter replicationsTableAdapter = new ClubDev.Database.ClubDevDataSetTableAdapters.ReplicationsTableAdapter();

            sharedArtifactsTableAdapter.FillMETAByID(clubDevDataSet.SharedArtifacts, id);

            if (clubDevDataSet.SharedArtifacts.Count > 0)
                replicationsTableAdapter.DeleteBySourceHostSourceHWNDClientHost(clubDevDataSet.SharedArtifacts[0].artifactGroup, clubDevDataSet.SharedArtifacts[0].artifactHWND, this.host);

            this.Refresh();
        }



        public void InitiateRelocationToLocalWorkspace(string artifactName)
        {
            long artifactID = long.Parse(artifactName.Substring(artifactName.LastIndexOf("__") + 2));

            ClubDev.Database.ClubDevDataSet clubDevDataSet = new ClubDev.Database.ClubDevDataSet();
            ClubDev.Database.ClubDevDataSetTableAdapters.SharedArtifactsTableAdapter sharedArtifactsTableAdapter = new ClubDev.Database.ClubDevDataSetTableAdapters.SharedArtifactsTableAdapter();
            ClubDev.Database.ClubDevDataSetTableAdapters.ReplicationsTableAdapter replicationsTableAdapter = new ClubDev.Database.ClubDevDataSetTableAdapters.ReplicationsTableAdapter();

            sharedArtifactsTableAdapter.FillMETAByID(clubDevDataSet.SharedArtifacts, artifactID);

            if(clubDevDataSet.SharedArtifacts.Count > 0)
                replicationsTableAdapter.Insert(clubDevDataSet.SharedArtifacts[0].artifactGroup, clubDevDataSet.SharedArtifacts[0].artifactHWND, Program.CurrentUser.UserHost, false, null, null, null);

            this.Refresh();
        }

    }
}
