using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;
using System.Drawing;

namespace ClubDev.Utilities
{
    public class Win32Api
    {
        
        

        public delegate bool EnumWindowsCallBack(int hwnd, IntPtr param);
        public delegate bool EnumChildCallBack(int hwnd, IntPtr param);

        public struct Rect
        {
            public int left;
            public int top;
            public int right;
            public int bottom;

            public int Width
            {
                get
                {
                    return right - left;
                }

            }

            public int Height
            {
                get
                {
                    return bottom - top;
                }
            }
        }
        public struct WindowInfo
        {
            public UInt32 cbSize;
            public Rect rcWindow;
            public Rect rcClient;
            public UInt32 dwStyle;
            public UInt32 dwExStyle;
            public UInt32 dwWindowStatus;
            public UInt32 cxWindowsBorders;
            public UInt32 cyWindowsBorders;
            public UInt16 atomWindowType;
            public UInt16 wCreatorVersion;
        }



        public struct PointWin32
        {
            public int x;
            public int y;
        }

        public struct CursorInfo
        {
            public UInt32 cbSize;
            public UInt32 flags;
            public UInt32 hCursor;
            public PointWin32 ptScreenPos;

        }

        [DllImport("user32.dll")]
        public static extern int EnumWindows(EnumWindowsCallBack cb, IntPtr param);

        [DllImport("user32.dll")]
        public static extern bool GetWindowRect(IntPtr hwnd, ref Rect rectangle);

        [DllImport("user32.dll")]
        public static extern bool IsWindowVisible(IntPtr hwnd);

        [DllImport("user32.dll")]
        public static extern IntPtr GetParent(IntPtr hwnd);

        [DllImport("user32.dll")]
        public static extern int GetWindowText(IntPtr hwnd, StringBuilder lpString, int nMaxCount);

        public const uint GA_PARENT = 1;
        
        [DllImport("user32.dll")]
        public static extern IntPtr GetAncestor(IntPtr hwnd, uint gaFlags);


         public const uint GW_HWNDNEXT     =    2;
         public const uint GW_HWNDPREV    =     3;

        public IntPtr GetNextWindow(IntPtr hwnd, uint wCmd)
        {
            return GetWindow(hwnd, wCmd);
        }

        [DllImport("user32.dll")]
        public static extern IntPtr GetWindow(IntPtr hwnd, uint wCmd);
        
        [DllImport("user32.dll")]
        public static extern IntPtr GetTopWindow(IntPtr hwnd);


        [DllImport("user32.dll")]
        public static extern IntPtr GetDesktopWindow();

        public const uint SWP_NOSIZE = 0x0001;
        public const uint SWP_NOMOVE = 0x0002;
        public const uint SWP_NOZORDER = 0x0004;
        public const uint SWP_NOREDRAW = 0x0008;
        public const uint SWP_NOACTIVATE = 0x0010;
        public const uint SWP_FRAMECHANGED = 0x0020;  /* The frame changed: send WM_NCCALCSIZE */
        public const uint SWP_SHOWWINDOW = 0x0040;
        public const uint SWP_HIDEWINDOW = 0x0080;
        public const uint SWP_NOCOPYBITS = 0x0100;
        public const uint SWP_NOOWNERZORDER = 0x0200;  /* Don't do owner Z ordering */
        public const uint SWP_NOSENDCHANGING = 0x0400;  /* Don't send WM_WINDOWPOSCHANGING */
        public const uint SWP_DEFERERASE  =    0x2000;
        public const uint SWP_ASYNCWINDOWPOS = 0x4000;

        public const int HWND_TOP = 0;
        public const int HWND_BOTTOM   = 1;
        public const int HWND_TOPMOST  =  -1;
        public const int HWND_NOTOPMOST = -2;

        [DllImport("user32.dll")]
        public static extern bool SetWindowPos(int hwnd, int hWndInsertAfter, int x,
                    int y, int cx, int cz, uint flags);

        [DllImport("user32.dll")]
        public static extern bool SetForegroundWindow(int hwnd);

        public const uint PW_CLIENTONLY = 0x00000001;
        [DllImport("user32.dll")]
        public static extern bool PrintWindow(IntPtr hwnd, IntPtr hdcBlt, uint nFlags);


        [DllImport("user32.dll", SetLastError = true)]
        public static extern IntPtr FindWindow(string lpClassName, string lpWindowName);

        [DllImport("user32.dll", SetLastError = true)]
        public static extern IntPtr FindWindow(StringBuilder lpClassName, StringBuilder lpWindowName);
        public const uint WM_PAINT = 0x000F;
        public const uint WM_ACTIVATEAPP = 0x001C;
        public const uint WM_INPUT = 0x00FF;
        public const uint WM_KEYFIRST = 0x0100;
        public const uint WM_KEYDOWN = 0x0100;
        public const uint WM_KEYUP = 0x0101;
        public const uint WM_CHAR = 0x0102;
        public const uint WM_DEADCHAR = 0x0103;
        public const uint WM_SYSKEYDOWN = 0x0104;
        public const uint WM_SYSKEYUP = 0x0105;
        public const uint WM_SYSCHAR = 0x0106;
        public const uint WM_SYSDEADCHAR = 0x0107;
        public const uint WM_UNICHAR = 0x0109;
        public const uint WM_KEYLAST = 0x0109;
        public const uint UNICODE_NOCHAR = 0xFFFF;
        //public const uint WM_KEYLAST                      = 0x0108;
        public const uint WM_IME_STARTCOMPOSITION = 0x010D;
        public const uint WM_IME_ENDCOMPOSITION = 0x010E;
        public const uint WM_IME_COMPOSITION = 0x010F;
        public const uint WM_IME_KEYLAST = 0x010F;


        public const uint WM_MOUSEFIRST = 0x0200;
        public const uint WM_MOUSEMOVE = 0x0200;
        public const uint WM_LBUTTONDOWN = 0x0201;
        public const uint WM_LBUTTONUP = 0x0202;
        public const uint WM_LBUTTONDBLCLK = 0x0203;
        public const uint WM_RBUTTONDOWN = 0x0204;
        public const uint WM_RBUTTONUP = 0x0205;
        public const uint WM_RBUTTONDBLCLK = 0x0206;
        public const uint WM_MBUTTONDOWN = 0x0207;
        public const uint WM_MBUTTONUP = 0x0208;
        public const uint WM_MBUTTONDBLCLK = 0x0209;


        public const uint WM_MOUSEHOVER = 0x02A1;
        public const uint WM_MOUSELEAVE = 0x02A3;
        public const uint WM_NCMOUSEHOVER = 0x02A0;
        public const uint WM_NCMOUSELEAVE = 0x02A2;

        public const uint WM_CAPTURECHANGED = 0x0215;

        public const uint WM_SETREDRAW = 0x000B;

        public const uint MK_LBUTTON = 0x0001;
        public const uint MK_RBUTTON = 0x0002;
        public const uint MK_SHIFT = 0x0004;
        public const uint MK_CONTROL = 0x0008;
        public const uint MK_MBUTTON = 0x0010;

        public const uint MK_XBUTTON1 = 0x0020;
        public const uint MK_XBUTTON2 = 0x0040;

        [DllImport("user32.dll", CharSet = CharSet.Auto)]
        public static extern IntPtr SendMessage(
            IntPtr hWnd, uint nMsg, IntPtr wParam, IntPtr lParam);

        [DllImport("user32.dll", CharSet = CharSet.Auto)]
        public static extern IntPtr PostMessage(
            IntPtr hWnd, uint nMsg, IntPtr wParam, IntPtr lParam);

        [DllImport("user32.dll", CharSet = CharSet.Auto)]
        public static extern IntPtr SendMessage(
            IntPtr hWnd, uint nMsg, IntPtr wParam, StringBuilder lParam);

        [DllImport("user32.dll")]
        public static extern bool EnumChildWindows(
            IntPtr hWnd, EnumChildCallBack cb, IntPtr lParam);


        public const uint CWP_ALL = 0x0000;
        public const uint CWP_SKIPINVISIBLE = 0x0001;
        public const uint CWP_SKIPDISABLED = 0x0002;
        public const uint CWP_SKIPTRANSPARENT = 0x0004;

        [DllImport("user32.dll")]
        public static extern IntPtr ChildWindowFromPointEx(
            IntPtr hWndParent, PointWin32 pt, uint flags);

        [DllImport("user32.dll")]
        public static extern bool InvalidateRect(
            IntPtr hWnd, IntPtr lpRect, bool erase);

        [DllImport("user32.dll")]
        public static extern bool UpdateWindow(
            IntPtr hWnd);

        [DllImport("user32.dll")]
        public static extern IntPtr SetActiveWindow(
            IntPtr hwnd);


        public const long WS_OVERLAPPED = 0x00000000L;
        public const long WS_POPUP = 0x80000000L;
        public const long WS_CHILD = 0x40000000L;
        public const long WS_MINIMIZE = 0x20000000L;
        public const long WS_VISIBLE = 0x10000000L;
        public const long WS_DISABLED = 0x08000000L;
        public const long WS_CLIPSIBLINGS = 0x04000000L;
        public const long WS_CLIPCHILDREN = 0x02000000L;
        public const long WS_MAXIMIZE = 0x01000000L;
        public const long WS_CAPTION = 0x00C00000L;     /* WS_BORDER | WS_DLGFRAME  */
        public const long WS_BORDER = 0x00800000L;
        public const long WS_DLGFRAME = 0x00400000L;
        public const long WS_VSCROLL = 0x00200000L;
        public const long WS_HSCROLL = 0x00100000L;
        public const long WS_SYSMENU = 0x00080000L;
        public const long WS_THICKFRAME = 0x00040000L;
        public const long WS_GROUP = 0x00020000L;
        public const long WS_TABSTOP = 0x00010000L;

        public const long WS_MINIMIZEBOX = 0x00020000L;
        public const long WS_MAXIMIZEBOX = 0x00010000L;



        public const long WS_EX_DLGMODALFRAME = 0x00000001L;
        public const long WS_EX_NOPARENTNOTIFY = 0x00000004L;
        public const long WS_EX_TOPMOST = 0x00000008L;
        public const long WS_EX_ACCEPTFILES = 0x00000010L;
        public const long WS_EX_TRANSPARENT = 0x00000020L;

        public const long WS_EX_MDICHILD = 0x00000040L;
        public const long WS_EX_TOOLWINDOW = 0x00000080L;
        public const long WS_EX_WINDOWEDGE = 0x00000100L;
        public const long WS_EX_CLIENTEDGE = 0x00000200L;
        public const long WS_EX_CONTEXTHELP = 0x00000400L;



        public const long WS_EX_RIGHT = 0x00001000L;
        public const long WS_EX_LEFT = 0x00000000L;
        public const long WS_EX_RTLREADING = 0x00002000L;
        public const long WS_EX_LTRREADING = 0x00000000L;
        public const long WS_EX_LEFTSCROLLBAR = 0x00004000L;
        public const long WS_EX_RIGHTSCROLLBAR = 0x00000000L;

        public const long WS_EX_CONTROLPARENT = 0x00010000L;
        public const long WS_EX_STATICEDGE = 0x00020000L;
        public const long WS_EX_APPWINDOW = 0x00040000L;


        public const long WS_EX_OVERLAPPEDWINDOW = (WS_EX_WINDOWEDGE | WS_EX_CLIENTEDGE);
        public const long WS_EX_PALETTEWINDOW = (WS_EX_WINDOWEDGE | WS_EX_TOOLWINDOW | WS_EX_TOPMOST);


        public const uint WS_EX_LAYERED = 0x00080000;


        public const long WS_EX_NOINHERITLAYOUT = 0x00100000L; // Disable inheritence of mirroring by children
        public const long WS_EX_LAYOUTRTL = 0x00400000L; // Right to left mirroring

        public const long WS_EX_COMPOSITED = 0x02000000L;

        public const long WS_EX_NOACTIVATE = 0x08000000L;


        public const int GWL_HWNDPARENT = (-8);
        public const int GWL_EXSTYLE = (-20);
        public const int GWL_STYLE = (-16);
        public const int GCL_HICON = (-14);
        public const int GCL_HICONSM = (-34);
        public const int WM_QUERYDRAGICON = 0x37;
        public const int WM_GETICON = 0x7F;
        public const int WM_SETICON = 0x80;
        public const int ICON_SMALL = 0;
        public const int ICON_BIG = 1;
        public const int SMTO_ABORTIFHUNG = 0x2;
        public const int TRUE = 1;
        public const int FALSE = 0;

        [DllImport("user32.dll")]
        public static extern bool GetWindowInfo(
            IntPtr hwnd, out WindowInfo pwi);

        [DllImport("user32.dll")]
        public static extern bool GetCursorInfo(
            out CursorInfo pci);


        [DllImport("user32.dll")]
        public static extern bool IsWindow(
            IntPtr hwnd);

        [DllImport("user32.dll")]
        public static extern int ShowCursor(
            bool bShow);

        // CopyCursor = CopyIcon

        [DllImport("user32.dll")]
        public static extern IntPtr CopyIcon(
            IntPtr pcur);

        [DllImport("user32.dll")]
        public static extern IntPtr SetCursor(
            IntPtr hCursor);

        public const uint NORMAL_PRIORITY_CLASS = 0x00000020;
        public const uint IDLE_PRIORITY_CLASS = 0x00000040;
        public const uint HIGH_PRIORITY_CLASS = 0x00000080;
        public const uint REALTIME_PRIORITY_CLASS = 0x00000100;
        public const uint BELOW_NORMAL_PRIORITY_CLASS = 0x00004000;
        public const uint ABOVE_NORMAL_PRIORITY_CLASS = 0x00008000;

        [DllImport("kernel32.dll", SetLastError = true)]
        public static extern bool SetPriorityClass(
            uint hProcess, uint dwPriorityClass);

        [DllImport("user32.dll")]
        public static extern uint GetWindowThreadProcessId(
            IntPtr hWnd, out uint lpdwProcessId);

        [DllImport("kernel32.dll", SetLastError = true)]
        public static extern int GetLastError();

        [DllImport("user32.dll")]
        public static extern int GetClassName(IntPtr hwnd, StringBuilder lpString, int nMaxCount);

        [DllImport("user32.dll")]
        public static extern IntPtr GetForegroundWindow();

        [DllImport("User32.dll")]
        public static extern int SendMessageTimeout(IntPtr hWnd, int uMsg, int wParam, int lParam, int fuFlags, int uTimeout, out int lpdwResult);

        [DllImport("User32.dll")]
        public static extern int GetClassLong(IntPtr hWnd, int index);

        [DllImport("gdi32.dll")]
        public static extern bool DeleteObject(IntPtr hObject);


    }
}
