//////////////////////////////////////////////////////////////////////////////////
//                                                                              //
//                      Cursor Redirection Toolkit (CuRT)                       //
//               Written by Andy McGovern, Orchid Research Group                //
//                           andymcgovern@gmail.com                             //
//                          http://orchid.cs.uiuc.edu                           //
//  Department of Computer Science, University of Illinois at Urbana-Champaign  //
//                                July 4, 2006                                  //
//      Copyright 2006, The Board of Trustees of the University of Illinois     //
//                                                                              //
//////////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace UIUC.DCS.ORCHID.CuRTClientLibrary
{
    //! ClickGuard
    /*!
     *  The ClickGuard form is activated whenever a machine's cursor is redirected to another machine.  The
     *  form is transparent and is set to the dimensions of the primary screen.  When the form's BringToFront 
     *  method is called, the form effectively disables clicking because any clicks are received by the form itself.
    */
    public partial class ClickGuard : Form
    {
        //! The ClickGuard constructor sets the form to the dimensions of the primary screen.
        protected internal ClickGuard()
        {
            InitializeComponent();
            CheckForIllegalCrossThreadCalls = false;

            Rectangle newBounds = new Rectangle();

            for(int i = 0; i < Screen.AllScreens.Length; i++)
            {
                newBounds.Height += Screen.AllScreens[i].Bounds.Height;
                newBounds.Width += Screen.AllScreens[i].Bounds.Width;
            }

            this.SetDesktopBounds(newBounds.X, newBounds.Y, newBounds.Width, newBounds.Height);
            
        }

        //! This method is called to activate the ClickGuard.    
        /*!
         *   The form is shown and brought to the front, and the machine's local cursor is hidden (but NOT deactivated).
         */
        public void ActivateClickGuard()

        {
            this.Show();
            this.BringToFront();
            this.Activate();
            Cursor.Hide();
           
        }

        //! This method is called to deactivate the ClickGuard.    
        /*!
         *   The form is hidden, and the machine's local cursor is shown.
         */
        public void DeactivateClickGuard()
        {
            this.Hide();
            Cursor.Show();
        }

    }
}