//////////////////////////////////////////////////////////////////////////////////
//                                                                              //
//                      Cursor Redirection Toolkit (CuRT)                       //
//               Written by Andy McGovern, Orchid Research Group                //
//                           andymcgovern@gmail.com                             //
//                          http://orchid.cs.uiuc.edu                           //
//  Department of Computer Science, University of Illinois at Urbana-Champaign  //
//                                July 4, 2006                                  //
//      Copyright 2006, The Board of Trustees of the University of Illinois     //
//                                                                              //
//////////////////////////////////////////////////////////////////////////////////

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

namespace CuRTClient
{

    //!  VirtualCursor
    /*!
     * The VirtualCursor class is the superclass of all of the specific VirtualCursor colors.
     */
	public class VirtualCursor : System.Windows.Forms.Form
	{
        public class ScreenBounds
        {
            protected internal int xMin;
            protected internal int xMax;
            protected internal int yMin;
            protected internal int yMax;
        }
        /*! The two panels are used to ensure that of the repeated background, only one single
         *  cursor is visible when the Windows form is displayed.
         */
		private System.Windows.Forms.Panel panel1;
        
        /*! The two panels are used to ensure that of the repeated background, only one single
         *  cursor is visible when the Windows form is displayed.
         */
        private System.Windows.Forms.Panel panel2;

        //! The drag flag indicates whether this virtual cursor is currently enacting a drag.
        protected internal bool drag;

        //! This keeps count of the current miniJumpsUp that this cursor has had, for use with Speed Detection.
        protected internal int numMiniJumpsUp;

        //! This keeps count of the current miniJumpsDown that this cursor has had, for use with Speed Detection.
        protected internal int numMiniJumpsDown;

        //! This keeps count of the current miniJumpsLeft that this cursor has had, for use with Speed Detection.
        protected internal int numMiniJumpsLeft;

        //! This keeps count of the current miniJumpsRight that this cursor has had, for use with Speed Detection.
        protected internal int numMiniJumpsRight;

        //! This keeps count of the current numLoiterMovementsUp that this cursor has had, for use with Loiter Detection.
        protected internal int numLoiterMovementsUp;

        //! This keeps count of the current numLoiterMovementsDown that this cursor has had, for use with Loiter Detection.
        protected internal int numLoiterMovementsDown;

        //! This keeps count of the current numLoiterMovementsLeft that this cursor has had, for use with Loiter Detection.
        protected internal int numLoiterMovementsLeft;

        //! This keeps count of the current numLoiterMovementsRight that this cursor has had, for use with Loiter Detection.
        protected internal int numLoiterMovementsRight;

        //! This keeps track of the cursor's previous X position, for use in relative coordinate movement.
        protected internal int prevX;

        //! This keeps track of the cursor's previous Y position, for use in relative coordinate movement.
        protected internal int prevY;

        //! This enumerated CursorColors object stores the color of this particular VirtualCursor.
        protected internal CursorColors currColor;

        protected internal ScreenBounds[] screenBounds;

		private System.ComponentModel.Container components = null;

        #region Windows Form Designer generated code
        private void InitializeComponent()
        {
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(VirtualCursor));
            this.panel1 = new System.Windows.Forms.Panel();
            this.panel2 = new System.Windows.Forms.Panel();
            this.SuspendLayout();
            // 
            // panel1
            // 
            this.panel1.BackColor = System.Drawing.Color.Silver;
            this.panel1.ForeColor = System.Drawing.Color.Red;
            this.panel1.Location = new System.Drawing.Point(12, 0);
            this.panel1.Name = "panel1";
            this.panel1.Size = new System.Drawing.Size(133, 39);
            this.panel1.TabIndex = 0;
            // 
            // panel2
            // 
            this.panel2.BackColor = System.Drawing.Color.Silver;
            this.panel2.ForeColor = System.Drawing.Color.Red;
            this.panel2.Location = new System.Drawing.Point(0, 21);
            this.panel2.Name = "panel2";
            this.panel2.Size = new System.Drawing.Size(42, 22);
            this.panel2.TabIndex = 1;
            // 
            // VirtualCursor
            // 
            this.BackColor = System.Drawing.Color.IndianRed;
            this.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("$this.BackgroundImage")));
            this.ClientSize = new System.Drawing.Size(10, 18);
            this.ControlBox = false;
            this.Controls.Add(this.panel2);
            this.Controls.Add(this.panel1);
            this.ForeColor = System.Drawing.Color.White;
            this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.None;
            this.MaximizeBox = false;
            this.MaximumSize = new System.Drawing.Size(100, 180);
            this.MinimizeBox = false;
            this.Name = "VirtualCursor";
            this.ShowInTaskbar = false;
            this.SizeGripStyle = System.Windows.Forms.SizeGripStyle.Hide;
            this.Text = "Cursor";
            this.TransparencyKey = System.Drawing.Color.Silver;
            this.ResumeLayout(false);

        }
        #endregion

        //!  VirtualCursor constructor
        /*!
         * The member variables are set to their default values, and the screenBounds array is filled with the
         * bounds of the screens of the local machine.
         */
        protected internal VirtualCursor()
		{

            //this flag allows prevents a Windows Forms exception that occurs when different threads try to access
            //the same data member at the same time
            CheckForIllegalCrossThreadCalls = false;


            InitializeComponent();

			drag = false;

			numMiniJumpsLeft = 0;
			numMiniJumpsRight = 0;
			numMiniJumpsUp = 0;
			numMiniJumpsDown = 0;

            numLoiterMovementsDown = 0;
            numLoiterMovementsLeft = 0;
            numLoiterMovementsRight = 0;
            numLoiterMovementsUp = 0;

			prevX = 100;
			prevY = 100;

            currColor = CursorColors.None;


		}

		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

	}
}
