//////////////////////////////////////////////////////////////////////////////////
//                                                                              //
//                      Cursor Redirection Toolkit (CuRT)                       //
//               Written by Andy McGovern, Orchid Research Group                //
//                           andymcgovern@gmail.com                             //
//                          http://orchid.cs.uiuc.edu                           //
//  Department of Computer Science, University of Illinois at Urbana-Champaign  //
//                                July 4, 2006                                  //
//      Copyright 2006, The Board of Trustees of the University of Illinois     //
//                                                                              //
//////////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using System.Drawing;



namespace ClientWrapper
{
    //! ClientWrapper
    /*!
     *  This class creates a CuRTClient object and shows some examples of method calls that can be made
     *  Not all of these method calls will necessarily do anything, as they depend on the current state 
     *  of the system, including which cursors are currently being remoted to which machines, as well 
     *  as the CuRTServer's IP address.
     */
    public class ClientWrapper
    {

        public static void Main()
        {
            Thread.CurrentThread.IsBackground = true;
            CuRTClient.CuRTClient.CursorOnEdgeListener onEdgeListener = new CuRTClient.CuRTClient.CursorOnEdgeListener();
            CuRTClient.CursorModule.CursorOnEdge += new CuRTClient.CursorOnEdgeHandler(onEdgeListener.AnnounceCursorOnEdge);
            CuRTClient.CursorColors[] colorScheme;
            CuRTClient.EdgeDetectionParams edgeDetectionParams = new CuRTClient.EdgeDetectionParams(2, 30, 5, 100);

            CuRTClient.CuRTClient client = null;
            if (!CuRTClient.CuRTClient.AnotherInstanceRunning())
            {
                colorScheme = new CuRTClient.CursorColors[10];
                colorScheme[0] = CuRTClient.CursorColors.Green;
                colorScheme[1] = CuRTClient.CursorColors.Blue;
                colorScheme[2] = CuRTClient.CursorColors.Red;
                colorScheme[3] = CuRTClient.CursorColors.Yellow;
                colorScheme[4] = CuRTClient.CursorColors.Magenta;
                colorScheme[5] = CuRTClient.CursorColors.Gray;
                colorScheme[6] = CuRTClient.CursorColors.Purple;
                colorScheme[7] = CuRTClient.CursorColors.Aqua;
                colorScheme[8] = CuRTClient.CursorColors.Brown;
                colorScheme[9] = CuRTClient.CursorColors.White;

                client = new CuRTClient.CuRTClient("testuser", "123.456.789.10", CuRTClient.KeyModifiers.Control, Keys.F1, colorScheme, 3, edgeDetectionParams);

                Thread.Sleep(5000);

                Console.WriteLine("local username: " + client.GetLocalClientUsername());
                Thread.Sleep(1000);
                Console.WriteLine("local index: " + client.GetLocalClientIndex());
                Thread.Sleep(1000);
                Console.WriteLine("local color: " + client.GetLocalCursorColor());
                Thread.Sleep(1000);
                Console.WriteLine("local cursor location: " + client.GetLocalCursorPosition().X + ","
                    + client.GetLocalCursorPosition().Y);
                Thread.Sleep(1000);
                Console.WriteLine("local cursor remote index: " + client.GetLocalCursorRemoteIndexLocation());
                Thread.Sleep(1000);
                String[] usernames;
                usernames = client.GetClientNameList();
                Console.WriteLine("master user list: "); 
                for (int i=0; i<10; i++)
                    if (usernames[i]!=null)
                        Console.WriteLine(i+": " +usernames[i]);
                    else
                        Console.WriteLine(i+": null");

                Console.WriteLine("username at index 3: " + client.GetClientUsername(3));
                Thread.Sleep(1000);
                Console.WriteLine("username at index 7: " + client.GetClientUsername(7));
                Thread.Sleep(1000);
                Console.WriteLine("index of user 'whee': " + client.GetClientIndex("whee"));
                Thread.Sleep(1000);
                Console.WriteLine("username at index 'bye': " + client.GetClientIndex("bye"));

                Thread.Sleep(10000);

                Console.WriteLine("client indices currently remoting to local: ");
                bool[] boolsToLocal = client.GetClientIndicesCurrentlyRemotedToLocalMachine();
                String[] stringsToLocal = client.GetClientNamesCurrentlyRemotedToLocalMachine();

                for (int i=0; i<10; i++)
                {
                    Console.WriteLine(i + ": " +boolsToLocal[i] +" , " + stringsToLocal[i]);
                }

                Console.WriteLine("local cursor remote index: " + client.GetLocalCursorRemoteIndexLocation());
                Console.WriteLine("local cursor remote name: "+client.GetLocalCursorRemoteUsernameLocation());

                Console.WriteLine("is local cursor remote? " + client.IsLocalCursorCurrentlyRemote());

                client.RemoveLocalCursorFromRemote(new Point(500, 500));

                Console.WriteLine("is local cursor remote? " + client.IsLocalCursorCurrentlyRemote());

                Thread.Sleep(10000);

                Point newPoint = client.SendRemoteCursorToOwnMachine(0, new Point(10, 500));
                if (newPoint != Point.Empty)
                    Console.WriteLine("returned coords: " + newPoint.X + " " + newPoint.Y);
                else
                    Console.WriteLine("point was empty");
               
                 
                client.SendLocalCursorToRemoteMachine(0, new Point(100, 100));

                Thread.Sleep(5000);

                client.RemoveLocalCursorFromRemote(Point.Empty);

                Thread.Sleep(5000);


                while (true)
                    ;
            }
        }

    }


}
