using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Data.SqlClient;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Runtime.InteropServices;
using System.Collections;




namespace PARTIES
{
    public partial class DispatchServer : Form
    {

        // Member variables

        Socket listen_socket;       //Socket and port that we listen on for new dispatch server connections
        int listen_port_number;

        int lower_port_range;       //Range of ports that this server (and its threads) can use
        int upper_port_range;
        
        public AsyncCallback newClientCallback;                 //Callbacks for receiving new connections or
        public AsyncCallback dataReceivedCallback;              //data
        public AsyncCallback dataReplicationReceivedCallback;
                
        ArrayList replicator_sockets;      //list of replication server threads we've spawned
        ArrayList dispatcher_sockets;      //list of dispatcher servers we are connected to 



        // SQL related member variables
        private DispatchServerData dispatchServerData = null;
        string hostname;
        DataTable current_hosted_reps;
        DataTable current_permissions;
        NotifyIcon sysTrayIcon;

        //public TelepointerPtoP telepointer_server;

        // Class that stores socket information.
        // Used to store information about other dispatch servers
        public class CSocketPacket
        {
            public System.Net.Sockets.Socket thisSocket;    // socket that we use to communicate with the other server
            public byte[] buffer;
            public int bytes_received;
            public bool header_data;
            public byte header;
            public NetworkStream stream;

            public static CSocketPacket findSocketFromIP(IPAddress ip, ArrayList dispatchers)
            {
                foreach (CSocketPacket sock_info in dispatchers)
                {
                    if (ip.Equals(((IPEndPoint)sock_info.thisSocket.RemoteEndPoint).Address))
                        return sock_info;
                }
                return null;
            }
        }

        // Class that stores information about replication servers.
        // Includes socket and thread information
        class ReplicatorSocket : CSocketPacket
        {
            public int port;
            public long key; //key in the database
            public IntPtr hwnd;
            public Thread my_thread;
            

            public static ReplicatorSocket findSocketFromPortAndHWND(int port, IntPtr hwnd, ArrayList replicators)
            {
                foreach (ReplicatorSocket sock_info in replicators)
                {
                    if (sock_info.port == port && (int)hwnd == (int)sock_info.hwnd)
                        return sock_info;
                }
                return null;
            }
        }

        void sysTrayIconSetup()
        {
            ToolStripMenuItem exitItem = new ToolStripMenuItem();
            exitItem.BackColor = System.Drawing.Color.White;
            exitItem.Text = "Exit";
            exitItem.Click += new EventHandler(exitItem_Click);

            ToolStripMenuItem leaveSessionItem = new ToolStripMenuItem();
            leaveSessionItem.BackColor = System.Drawing.Color.White;
            leaveSessionItem.Text = "Leave Session";
            leaveSessionItem.Click += new EventHandler(leaveSessionItem_Click);

            this.sysTrayIcon = new NotifyIcon();
            this.sysTrayIcon.Icon = new System.Drawing.Icon(this.GetType().Assembly.GetManifestResourceStream("PARTIES.systray.ico"));
            this.sysTrayIcon.Text = "Dispatch Server";
            this.sysTrayIcon.Visible = true;
            this.ShowInTaskbar = false;
            this.sysTrayIcon.ContextMenuStrip = new ContextMenuStrip();
            this.sysTrayIcon.ContextMenuStrip.Items.AddRange(
                new System.Windows.Forms.ToolStripItem[]
                {
        //            leaveSessionItem,
                    exitItem
                });
        }

        void leaveSessionItem_Click(object sender, EventArgs e)
        {
            this.sysTrayIcon.Visible = false;
            this.Close();
            
        }

        void exitItem_Click(object sender, EventArgs e)
        {
            this.sysTrayIcon.Visible = false;
            this.Close();
        }

        //Constructor
        public DispatchServer()
        {
            InitializeComponent();
            sysTrayIconSetup();
            this.Text = "Replicated Window Server";
            

            /************ Begin SQL related code**********/
            try
            {
                SqlClientPermission perm = new SqlClientPermission(System.Security.Permissions.PermissionState.Unrestricted);
                perm.Demand();
            }
            catch
            {
                throw new ApplicationException("No permission");
            }

            hostname = System.Net.Dns.GetHostEntry(System.Net.Dns.GetHostName()).HostName;
            current_hosted_reps = new DataTable();
            current_hosted_reps.Columns.Add("replicationID", typeof(long));
            current_hosted_reps.Columns.Add("replicationSourceHost", typeof(string));
            current_hosted_reps.Columns.Add("replicationSourceHWND", typeof(int));
            current_hosted_reps.Columns.Add("replicationClientHost", typeof(string));
            current_hosted_reps.Columns.Add("replicationIsSharedScreen", typeof(bool));

            current_permissions = new DataTable();
            current_permissions.Columns.Add("artifactID", typeof(long));
            current_permissions.Columns.Add("artifactHWND", typeof(int));
            current_permissions.Columns.Add("artifactFullShare", typeof(bool));
            /************ End SQL related code**********/


            //Initialize member variables
            dispatcher_sockets = new ArrayList();
            replicator_sockets = new ArrayList();
            newClientCallback = null;
            dataReceivedCallback = null;
            dataReplicationReceivedCallback = null;


            listen_port_number = 6286;
            lower_port_range = 6287;
            upper_port_range = 6292;
            
            //More SQL related code
            dispatchServerData = new DispatchServerData(hostname);
            //dispatchServerData.OnNewReplicationMessage += new DispatchServerData.NewMessage(OnNewReplicationMessage);
            //dispatchServerData.OnNewPermissionMessage += new DispatchServerData.NewMessage(OnNewPermissionMessage);
            dispatchServerData.OnNewReplicationEvent += new DataEventHandler(OnNewReplicationEvent);
            dispatchServerData.OnNewPermissionEvent += new DataEventHandler(OnNewPermissionEvent);

            //Ok, let's start up the server!
            serverStart();

        }


        void serverStart()
        {
            //telepointer_server = new TelepointerPtoP(20);
            //telepointer_server.Show();
            //telepointer_server.startTelepointerServer();

            outputBox.AppendText("Starting dispatch server ... \n");
            IPAddress serverIP = IPAddress.Any;
            IPEndPoint ipep = new IPEndPoint(serverIP, 6286);

            //Start waiting for clients to connect
            listen_socket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
            listen_socket.Bind(ipep);
            listen_socket.Listen(10);
            outputBox.AppendText("Listening on port 6286 ...\n");
            waitForNewClients();

            //Clean replication table for this host
            dispatchServerData.InitialReplicationTableClean(hostname);

            //SQL startup calls
            //OnNewReplicationEvent();
            //OnNewPermissionEvent();


            if (System.Diagnostics.Debugger.IsAttached)
                writeToOutput("Debugger attached\n");
            else
            {
                writeToOutput("No Debugger attached\n");
                //this.Hide();
                //this.Visible = false;
                this.ShowInTaskbar = false;
                this.Visible = false;
                this.Opacity = 0;
               // MessageBox.Show("Dispatch Server Started");
                
            }

            

        }

        


        

        //Write to the output box on the server form
        //We must use a delegate because only the main UI thread can write to the output box
        private delegate void writeToOutputDelegate(string text);
        private void writeToOutput(string text)
        {
            if (this.InvokeRequired)  //i.e. if we are trying to set the text and we aren't the main thread
            {
                this.BeginInvoke(new writeToOutputDelegate(writeToOutput),
                                                  new object[] { text });

                return;
            }
            outputBox.AppendText(text);
        }
       




        //Asynchronously wait for new client connections
        void waitForNewClients()
        {
            if (newClientCallback == null)
                newClientCallback = new AsyncCallback(OnClientConnect);
            // now start to listen for any data...
            listen_socket.BeginAccept(newClientCallback, null);

        }

        //Asynchronously wait for data from a replication server
        void waitForReplicationData(ReplicatorSocket socket_packet)
        {
            if (dataReplicationReceivedCallback == null)
                dataReplicationReceivedCallback = new AsyncCallback(onReplicationDataReceived);
            // now start to listen for any data...
            socket_packet.thisSocket.BeginReceive(socket_packet.buffer, socket_packet.bytes_received, socket_packet.buffer.Length - socket_packet.bytes_received, SocketFlags.None, dataReplicationReceivedCallback, socket_packet);

        }


        //Asynchronously wait for data from another dispatch server
        void waitForData(CSocketPacket socket_packet)
        {
            if (dataReceivedCallback == null)
                dataReceivedCallback = new AsyncCallback(onDataReceived);
            // now start to listen for any data...
            socket_packet.thisSocket.BeginReceive(socket_packet.buffer, socket_packet.bytes_received, socket_packet.buffer.Length - socket_packet.bytes_received, SocketFlags.None, dataReceivedCallback, socket_packet);

        }



        //Called when a dispatcher server connects to our server
        public void OnClientConnect(IAsyncResult asyn)
        {
            try
            {
                writeToOutput("Client connected \n");
                Socket client = listen_socket.EndAccept(asyn);
                
                //wait for data from new client
                CSocketPacket sock_pkt = new CSocketPacket();
                sock_pkt.thisSocket = client;
                sock_pkt.bytes_received = 0;
                sock_pkt.buffer = new byte[5];
                sock_pkt.header_data = true;
                sock_pkt.stream = new NetworkStream(sock_pkt.thisSocket);
                dispatcher_sockets.Add(sock_pkt);
                waitForData(sock_pkt);

                //wait for more clients to connect
                waitForNewClients();
            }
            catch (ObjectDisposedException)
            {
                System.Diagnostics.Debugger.Log(0, "1", "\n OnClientConnection: Socket has been closed\n");
            }
            catch (SocketException se)
            {
                MessageBox.Show(se.Message);
            }
        }


        //called when we've received data from one of the replication servers (that we spawned)
        public void onReplicationDataReceived(IAsyncResult asyn)
        {
            try
            {
                ReplicatorSocket socket_info = (ReplicatorSocket)asyn.AsyncState;
                int just_received = 0;
                just_received = socket_info.thisSocket.EndReceive(asyn);

                socket_info.bytes_received += just_received;

                if (socket_info.bytes_received < socket_info.buffer.Length)
                {
                    waitForData(socket_info);
                    return;
                }

                socket_info.bytes_received = 0;

                if (socket_info.header_data)
                {
                    socket_info.header = socket_info.buffer[0];
                    int data_buffer_length = BitConverter.ToInt32(socket_info.buffer, 1);
                    socket_info.buffer = new byte[data_buffer_length];
                    socket_info.header_data = false;
                    waitForReplicationData(socket_info);
                }
                else
                {
                    processReplicationDataBuffer(socket_info);
                }

            }
            catch (ObjectDisposedException)
            {
                // Console.WriteLine("onDataReceieved: Socket has been closed");
            }
            catch (SocketException)
            {
                //MessageBox.Show(se.Message);
                ReplicatorSocket close_me = new ReplicatorSocket();
                close_me.header = (byte)NetworkMessages.MessageType.CloseReplicationServer;
                processReplicationDataBuffer(close_me);
            }


        }

        //Called when we have received data from another dispatch server
        public void onDataReceived(IAsyncResult asyn)
        {
            try
            {
                CSocketPacket socket_info = (CSocketPacket)asyn.AsyncState;
                int just_received = 0;
                just_received = socket_info.thisSocket.EndReceive(asyn);


                socket_info.bytes_received += just_received;

                if (socket_info.bytes_received < socket_info.buffer.Length)
                {
                    waitForData(socket_info);
                    return;
                }

                socket_info.bytes_received = 0;

                if (socket_info.header_data)
                {
                    socket_info.header = socket_info.buffer[0];
                    int data_buffer_length = BitConverter.ToInt32(socket_info.buffer, 1);
                    socket_info.buffer = new byte[data_buffer_length];
                    socket_info.header_data = false;
                    waitForData(socket_info);
                }
                else
                {
                    processDataBuffer(socket_info);
                }

            }
            catch (ObjectDisposedException)
            {
                // Console.WriteLine("onDataReceieved: Socket has been closed");
            }
            catch (SocketException se)
            {
                MessageBox.Show(se.Message);
            }
        }


        //Process the data buffer from another dispatch server
        void processDataBuffer(CSocketPacket socket_info)
        {
            if (socket_info.header == (byte)NetworkMessages.MessageType.StartClient)
            {
                startClientThread(socket_info);
                
            }
        }

        //Process the data buffer from a replication server (that was spawned by this server)
        void processReplicationDataBuffer(ReplicatorSocket socket_info)
        {
            if (socket_info.header == (byte)NetworkMessages.MessageType.CloseReplicationServer)
            {
                writeToOutput("Removed entry in replicator_sockets for the server that just closed \n");

                try { socket_info.thisSocket.Shutdown(SocketShutdown.Both); }
                catch (Exception) { }

                try {socket_info.stream.Close();}
                catch (Exception) { }
                
                try {socket_info.thisSocket.Close();}
                catch (Exception) { }
               /* try
                {
                    socket_info.my_thread.Abort();
                }
                catch (Exception e)
                {
                    Console.WriteLine("Exception when aborting thread");
                }*/
                try {replicator_sockets.Remove(socket_info);}
                catch (Exception) { }

                try { dispatchServerData.RemoveReplication(socket_info.key); }
                catch (Exception) { }
            }
        }



        //Start a client thread
        //Only let the main UI thread start a client
        //
        //Parameters:
        //  socket_info:  Information about the dispatch server that sent the request 
        private delegate void startClientThreadDelegate(CSocketPacket socket_info);
        private void startClientThread(CSocketPacket socket_info)
        {
            if (this.InvokeRequired)  //i.e. if we are trying to set the text and we aren't the main thread
            {
                this.BeginInvoke(new startClientThreadDelegate(startClientThread),
                                                  new object[] { socket_info });

                return;
            }

            //socket_info.buffer data format:
            //[0...3] = port number (int) - This is the port that the client thread will connect to
            int port_number = BitConverter.ToInt32(socket_info.buffer,0);
            writeToOutput("Starting a thread for the client form ...\n");
            long key_number = BitConverter.ToInt64(socket_info.buffer, 4);

            //Set all of the required parameters for a new client thread
            Client.StartClientParams cparams = new Client.StartClientParams();
            cparams.port = port_number;
            cparams.location = new Point(0, 0);
            cparams.serverIP = ((IPEndPoint)(socket_info.thisSocket.RemoteEndPoint)).Address;
            cparams.abortMeCallBack = new CallBack(abortMe);
            cparams.key = key_number;

            //Start up the client thread
            ParameterizedThreadStart threadDelegate = new ParameterizedThreadStart(Client.startClient);
            Thread newthread = new Thread(threadDelegate);
            newthread.SetApartmentState(ApartmentState.STA);
            cparams.my_thread = newthread;
            newthread.Start(cparams);
            writeToOutput("... thread started ...\n");

            //Reset the buffer for the socket
            socket_info.header_data = true;
            socket_info.buffer = new byte[5];
            waitForData(socket_info);
        }


        //Callback that threads can call to have themselves aborted
        void abortMe(Thread my_thread)
        {
            Console.WriteLine("Dispatcher server processesing abort_me from rep server");
            my_thread.Abort();
        }



        //Send a message to a dispatch server requesting that it start up a new client thread on its machine
        //
        //Parameters: 
        //      socket_info:  Information about the server that will start the client thread
        //      port:         The port that the new client thread should try to connect to (on this machine)
        void sendStartClientThreadMessage(CSocketPacket socket_info, int port, long key, ReplicatorSocket rep_socket)
        {
            //NetworkStream stream = new NetworkStream(socket);
            writeToOutput("Sending message to start client thread ... \n");
        
            
            //Message format:
            //buffer[0] = Message type
            //buffer[1...4] = buffer size (int)
            //buffer[5...9] = port number

            byte[] buffer = new byte[1 + 4 + 4 + 8];
            buffer[0] = (byte)NetworkMessages.MessageType.StartClient;
            byte[] size = BitConverter.GetBytes(4+8);  //sizeof(int) = 4
            for (int i = 0; i < 4; i++)
                buffer[i + 1] = size[i];
            byte[] port_buffer = BitConverter.GetBytes(port);
            for (int i = 0; i < 4; i++)
                buffer[i + 5] = port_buffer[i];
            byte[] key_buffer = BitConverter.GetBytes(key);
            for (int i = 0; i < 8; i++)
                buffer[i + 9] = key_buffer[i];
            

            //Send the message
            try
            {
                socket_info.stream.Write(buffer, 0, 1 + 4 + 4 + 8);
            }
            catch (Exception)
            {
                endSingleReplicationINTERNAL(rep_socket);
            }
            writeToOutput(" ... message sent ... \n");

        }


        //Check if we can use a port
        //Parameters:
        //  port: The port to check
        private bool isPortUsable(int port)
        {
            Socket s = new Socket(AddressFamily.InterNetwork,SocketType.Stream, ProtocolType.Tcp);
            try {
                s.Bind(new IPEndPoint(IPAddress.Any, port));
                s.Close();
                return true;
            }
            catch (SocketException ex) 
            {
                // EADDRINUSE?
                if (ex.ErrorCode == 10048)
                    return false;
                return false;
            }

        }

        //Get a usable port from the range provided
        //Parameters:
        //     lower_range: start of the port range (inclusive)
        //     upper_range: end of the port range (inclusive)
        //
        //Returns:
        //      int:  Either a usable port or -1

        private int getUsablePort(int lower_range, int upper_range)
        {
            for(int i=lower_range; i<=upper_range; i++)
            {
                if(isPortUsable(i))
                    return i;
            }
            return -1;
        
        }

        //Gets usable ports from the range provided
        //Parameters:
        //     num_ports : number of ports needed
        //     lower_range: start of the port range (inclusive)
        //     upper_range: end of the port range (inclusive)
        //
        //Returns:
        //      int:  Either a list of usable ports or null

        private ArrayList getUsablePorts(int num_ports, int lower_range, int upper_range)
        {
            ArrayList result = new ArrayList();

            for (int i = lower_range; i <= upper_range; i++)
            {
                if (isPortUsable(i))
                    result.Add(i);
                if (result.Count == num_ports)
                    return result;
            }

            return null;
        }



        //Called to end all of the replications of an application
        private void endAllReplicationsSQL(IntPtr hwnd)
        {
            ArrayList replicator_sockets_clone = new ArrayList();
            ArrayList remove_list = new ArrayList();
            foreach (ReplicatorSocket rep_sock in replicator_sockets)
                replicator_sockets_clone.Add(rep_sock);

            //endSingleReplicationSQL can modify replicator_sockets, so we need to iterate through a copy
            //or replicator sockets or the foreach might have problems
            foreach (ReplicatorSocket rep_sock in replicator_sockets_clone)
            {
                if ((int)rep_sock.hwnd == (int)hwnd)
                {
                    //endSingleReplicationSQL(((IPEndPoint)rep_sock.thisSocket.RemoteEndPoint).Address, rep_sock.key, hwnd);
                    endSingleReplicationINTERNAL(rep_sock);
                    remove_list.Add(rep_sock);
                }
            }

            foreach (ReplicatorSocket removeme in remove_list)
                replicator_sockets.Remove(removeme);
        }


        //Called to end a single replication
        private void endSingleReplicationSQL(IPAddress client_ip, long key, IntPtr hwnd)
        {
            foreach (ReplicatorSocket socket in replicator_sockets)
            {
                if (socket.key == key && socket.hwnd == hwnd)
                {
                    endSingleReplicationINTERNAL(socket);
                    replicator_sockets.Remove(socket);
                    return;
                }

            }
        }

        //called to change the permissions of all replications of an application on this machine
        private void changePermissionsSQL(IntPtr hwnd, ReplicationOptions new_permissions)
        {
            foreach (ReplicatorSocket socket in replicator_sockets)
            {
                if (socket.hwnd == hwnd)
                    changeIndividualPermissionsINTERNAL(socket, new_permissions);
            }
        }


        //DO NOT CALL ... internal worker function called by changePermissionsSQL
        private void changeIndividualPermissionsINTERNAL(ReplicatorSocket socket, ReplicationOptions new_permissions)
        {
            byte[] buffer = new byte[1 + 4 + 1];
            byte[] size_buf = BitConverter.GetBytes(1);
            buffer[0] = (byte)NetworkMessages.MessageType.ChangePermissions;

            for (int i = 0; i < 4; i++)
                buffer[i + 1] = size_buf[i];
            buffer[5] = (byte)new_permissions;
            socket.stream.Write(buffer, 0, 6);
            
        }

        //DO NOT CALL ... internal worker function called by endSingleReplicationSQL
        private void endSingleReplicationINTERNAL(ReplicatorSocket socket)
        {
            byte[] buffer = new byte[1 + 4 + 1];
            byte[] size_buf = BitConverter.GetBytes(1);
            buffer[0] = (byte)NetworkMessages.MessageType.CloseReplication;

            for (int i = 0; i < 4; i++)
                buffer[i + 1] = size_buf[i];
            buffer[5] = (byte)NetworkMessages.MessageType.CloseReplication;
            socket.stream.Write(buffer, 0, 6);
            //socket.stream.Close(500);
            //socket.thisSocket.Close();
            //replicator_sockets.Remove(socket);
        }



        //Call to create a new replication
        //This should be called in response to an SQL update that triggers a new replication
        private void newReplicationSQL(IPAddress client_ip, IntPtr hwnd, ReplicationOptions rep_options, long key)
        {
            ArrayList usable_ports = getUsablePorts(2,lower_port_range, upper_port_range);
            if (usable_ports != null)
            {
                writeToOutput("Starting a new server thread ... \n");
                IntPtr hWnd = hwnd;

                Server.StartServerParams startup_params = new Server.StartServerParams();
                startup_params.hWnd = hWnd;
                startup_params.rep_options = rep_options;
                //startup_params.port = getUsablePort(lower_port_range, upper_port_range);
                startup_params.port = (int)(usable_ports[0]);
                startup_params.abortMeCallback = new CallBack(abortMe);
                startup_params.key = key;
                startup_params.databaseServer = dispatchServerData;
                //startup_params.telepointerServer = telepointer_server;

                ReplicatorSocket rep_socket = new ReplicatorSocket();
                rep_socket.hwnd = hwnd;
                //rep_socket.port = getUsablePort(lower_port_range, upper_port_range);
                rep_socket.port = startup_params.port = (int)usable_ports[1];
                rep_socket.buffer = new byte[5];
                rep_socket.bytes_received = 0;
                rep_socket.header_data = true;
                rep_socket.key = key;

                ParameterizedThreadStart threadDelegate = new ParameterizedThreadStart(Server.startNewServer);
                Thread newthread = new Thread(threadDelegate);
                newthread.SetApartmentState(ApartmentState.STA);
                startup_params.my_thread = newthread;
                rep_socket.my_thread = newthread;
                newthread.Start(startup_params);

                rep_socket.thisSocket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
                try
                {
                    rep_socket.thisSocket.Connect(IPAddress.Loopback, rep_socket.port);  // this is a blocking call
                }
                catch (SocketException)
                {
                    Console.WriteLine("interactButton_Click:  Unablet to connect to replication server ...");
                    return;
                }
                rep_socket.stream = new NetworkStream(rep_socket.thisSocket);
                replicator_sockets.Add(rep_socket);
                waitForReplicationData(rep_socket);

                writeToOutput("Searching for dispatcher socket ... \n");
                CSocketPacket socketinfo = CSocketPacket.findSocketFromIP(client_ip, dispatcher_sockets);
                if (socketinfo == null)
                {
                    writeToOutput("... dispatcher socket not found, creating new socket ... \n");
                    socketinfo = new CSocketPacket();
                    socketinfo.thisSocket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
                    socketinfo.buffer = new byte[5];
                    socketinfo.bytes_received = 0;
                    socketinfo.header = (byte)0;
                    socketinfo.header_data = true;

                    // Make the connection
                    try
                    {
                        socketinfo.thisSocket.Connect(client_ip, listen_port_number);  // this is a blocking call
                    }
                    catch (SocketException)
                    {
                        Console.WriteLine("interactButton_Click:  Unablet to connect to dispatch server ...");
                        return;
                    }
                    socketinfo.stream = new NetworkStream(socketinfo.thisSocket);
                    dispatcher_sockets.Add(socketinfo);
                    waitForData(socketinfo);
                    writeToOutput("... sending start client thread message ... \n");
                }
                else { }
                writeToOutput("... found sending start client thread message ... \n");

                sendStartClientThreadMessage(socketinfo, startup_params.port, key, rep_socket);
            }
            else
                writeToOutput("No ports open ... ignoring request");
        }





        /* Below this point is code for the buttons *****/
        private void interactButton_Click(object sender, EventArgs e)
        {
            writeToOutput("Starting a new server thread ... \n");
            //IntPtr hWnd = Win32Api.FindWindow("IEFrame", "Google - Windows Internet Explorer");
            IntPtr hWnd = new IntPtr(0x00020B42);
            //IPAddress xp_box = new IPAddress(0x6BF0AE80);
            IPAddress xp_box = IPAddress.Loopback;
            newReplicationSQL(xp_box, hWnd, ReplicationOptions.INTERACT,0);
        }




        private void viewButton_Click(object sender, EventArgs e)
        {
            writeToOutput("Starting a new server thread ... \n");
            //IntPtr hWnd = Win32Api.FindWindow("IEFrame", "Google - Windows Internet Explorer");
            IntPtr hWnd = new IntPtr(0x00020B42);
            //IPAddress xp_box = new IPAddress(0x6BF0AE80);
            IPAddress xp_box = IPAddress.Loopback;
            newReplicationSQL(xp_box, hWnd, ReplicationOptions.VIEW,0);
        }

        


        private void xpConnectButton_Click(object sender, EventArgs e)
        {
            writeToOutput("Connecting to client machine ... \n");
            IPAddress xp_box = new IPAddress(0x6BF0AE80);
            Socket xp_dispatch_socket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);

            // Make the connection
            try
            {
                xp_dispatch_socket.Connect(xp_box, 6286);  // this is a blocking call
            }
            catch (SocketException)
            {
                // Console.WriteLine(se.Message);
            }

            CSocketPacket sock_pkt = new CSocketPacket();
            sock_pkt.thisSocket = xp_dispatch_socket;
            sock_pkt.bytes_received = 0;
            sock_pkt.buffer = new byte[5];
            sock_pkt.header_data = true;
            sock_pkt.stream = new NetworkStream(sock_pkt.thisSocket);
            dispatcher_sockets.Add(sock_pkt);
        }

        private void viewPermissionsButton_Click(object sender, EventArgs e)
        {
            IntPtr hWnd = Win32Api.FindWindow("IEFrame", "Google - Windows Internet Explorer");

            changePermissionsSQL(hWnd, ReplicationOptions.VIEW);
        }

        private void interactPermissionsButton_Click(object sender, EventArgs e)
        {
            IntPtr hWnd = Win32Api.FindWindow("IEFrame", "Google - Windows Internet Explorer");

            changePermissionsSQL(hWnd, ReplicationOptions.INTERACT);
        }

        private void stopButton_Click(object sender, EventArgs e)
        {
            IntPtr hWnd = Win32Api.FindWindow("IEFrame", "Google - Windows Internet Explorer");

            endAllReplicationsSQL(hWnd);
        }








        void OnNewReplicationEvent(object sender, DataTable result)
        {
            // Get the messages
            //DataTable dt = dispatchServerData.GetReplicationState();
            DataTable dt = result;

            DataTable additions = SQLOperations.Difference(dt, current_hosted_reps);
            DataTable subtractions = SQLOperations.Difference(current_hosted_reps, dt);

            long key;
            IPAddress client_ip;
            IPHostEntry serverhost;
            IPHostEntry clienthost;

            int hwnd;
            int permissions;

            //0 = not found
            //1 = VIEW
            //2 = INTERACT
            foreach (DataRow row in additions.Rows)
            {
                writeToOutput(row["replicationID"].ToString() + " ");

                key = (long)row["replicationID"];

                serverhost = Dns.GetHostEntry(row["replicationSourceHost"].ToString());
                clienthost = Dns.GetHostEntry(row["replicationClientHost"].ToString());
                //host.HostName = row["replicationClientHost"].ToString();
                client_ip = null;

                for (int i = 0; i < clienthost.AddressList.Length; i++)
                {
                    if (clienthost.AddressList[i].AddressFamily == AddressFamily.InterNetwork)
                    {
                        client_ip = clienthost.AddressList[i];
                        break;
                    }
                }


                hwnd = (int)row["replicationSourceHWND"];

                permissions = dispatchServerData.GetCurrentPermission(serverhost.HostName, hwnd);

                if (permissions == 1)
                    newReplicationSQL(client_ip, new IntPtr(hwnd), ReplicationOptions.VIEW, key);
                else if (permissions == 2)
                    newReplicationSQL(client_ip, new IntPtr(hwnd), ReplicationOptions.INTERACT, key);
            }

            foreach (DataRow row in subtractions.Rows)
            {
                //writeToOutput(row["replicationID"].ToString() + "\n");
                // We actually don't need to do anything here ...
                // If a user closes their client replication, it automatically tells its server process to stop
                // The client just needs to update the replications table (for the sake of keeping it clean)
                //writeToOutput(row["replicationID"].ToString() + " ");

                key = (long)row["replicationID"];

                clienthost = Dns.GetHostEntry(row["replicationClientHost"].ToString());
                //host.HostName = row["replicationClientHost"].ToString();
                client_ip = null;

                for (int i = 0; i < clienthost.AddressList.Length; i++)
                {
                    if (clienthost.AddressList[i].AddressFamily == AddressFamily.InterNetwork)
                    {
                        client_ip = clienthost.AddressList[i];
                        break;
                    }
                }

                hwnd = (int)row["replicationSourceHWND"];

                if((bool)row["replicationIsSharedScreen"]==true)
                    endSingleReplicationSQL(client_ip, key, new IntPtr(hwnd));
                
            }

            current_hosted_reps = dt;
        }


        private void OnNewPermissionEvent(object sender, DataTable result)
        {
            int hwnd;
            bool permissions;
            // Get the messages
            //DataTable dt = dispatchServerData.GetShareState();
            DataTable dt = result;


            DataTable additions = SQLOperations.Difference(dt, current_permissions);
            DataTable subtractions = SQLOperations.Difference(current_permissions, dt);

            foreach (DataRow row in additions.Rows)
            {
                foreach (DataRow subtractionsRow in subtractions.Rows)
                {
                    if ((long)row["artifactID"] == (long)subtractionsRow["artifactID"])
                    {
                        subtractionsRow.Delete();
                        writeToOutput("REMOVING " + row["artifactID"].ToString() + " FROM SUBTRACTIONS TABLE \n");
                    }
                }

                writeToOutput("ADD PERMISSIONS FOR: " + row["artifactID"].ToString() + "\n");

                // if artifactFullShare = 1, the write
                hwnd = (int)row["artifactHWND"];
                permissions = (bool)row["artifactFullShare"];
                if (permissions)
                    changePermissionsSQL(new IntPtr(hwnd), ReplicationOptions.INTERACT);
                else // if(permissions == false)
                    changePermissionsSQL(new IntPtr(hwnd), ReplicationOptions.VIEW);

            }

            foreach (DataRow row in subtractions.Rows)
            {
                if (row.RowState != DataRowState.Deleted)
                {
                    writeToOutput("SHOULD STOP SHARING " + row["artifactID"].ToString() + "\n");
                    hwnd = (int)row["artifactHWND"];
                    endAllReplicationsSQL(new IntPtr(hwnd));
                }
            }




            current_permissions = dt;
        }

    }
}