using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Runtime.InteropServices;
using System.Collections;

namespace PARTIES
{
    

    public partial class Server : Form
    {

        //Networking variables
        Socket listen_socket;           // listen for incoming connections
        Socket client;                  // connection to client thread we are hosting
        Socket dispatcher_socket;       // connection to our dispatch server

        NetworkStream stream;
        NetworkStream dispatcher_stream;

        private byte[] buffer;
        private int bytes_received;
        private bool header_data;
        private byte header;

        private byte[] dispatch_buffer;
        private int dispatch_bytes_received;
        private bool dispatch_header_data;
        private byte dispatch_header;

        int lower_port_range;
        int upper_port_range;
        private int port;

        //Application variables
        private IntPtr hwnd_target;     // window we are replicating
        private IntPtr client_hwnd;
        string old_title;
        private bool menu_window;
        private ArrayList menu_hwnds;


        // Replication variables
        string prev_hash;               // previous screenshot hash value
        System.Security.Cryptography.MD5CryptoServiceProvider md5;
        private ScreenCaptureOptions capture_settings;
        ArrayList last_click_params;
        ReplicationOptions rep_options;

        // Timers
        private System.Windows.Forms.Timer screen_capture_timer; 
        private System.Windows.Forms.Timer find_menus_timer;
        
        


        bool no_write;
        
        // Database
        public AsyncCallback dataReceivedCallback;
        public AsyncCallback dataDispatcherReceivedCallback;
        IAsyncResult dataReceivedResult;
        IAsyncResult dataDispatcherReceivedResult;
        private DispatchServerData databaseServer;
        private long key;

        
        // Multi-threading
        private Thread my_thread;
        CallBack abortMeCallback;


        // Telepointers
        private TelepointerPtoP telepointerServer;


        // Class that stores a handle to a window we will send mouse input to, as well as the location
        // to send the mouse input (relative to the hwnd window)
        class MouseClickParameters
        {
            public IntPtr hwnd;         //Handle of window to send input to
            public Point location;      //Location of mouse click in the coordinate system of hwnd
        }

        class KeyPressParameters
        {
            public bool special_character;
            public int key;
            public char charkey;
        }

        public class StartServerParams
        {
            public IntPtr hWnd;
            public int port;
            public int dispatcher_port;
            public ReplicationOptions rep_options;
            public Thread my_thread;
            public CallBack abortMeCallback;
            public long key;
            public DispatchServerData databaseServer;
            public TelepointerPtoP telepointerServer;
        }

        

        public static void startNewServer(object start_params)
        {

            try
            {
                StartServerParams sp = (StartServerParams)start_params;

                Application.Run(new Server(sp.hWnd, ScreenCaptureOptions.WINDOWSHOWCLIENT,
                                sp.rep_options, false, sp.port, sp.my_thread, sp.abortMeCallback,
                                sp.key, sp.databaseServer, sp.telepointerServer));
            }

            catch
            {
                Console.WriteLine("Could not start server");
            }
        }

        public static void startNewMenuServer(object start_params)
        {
            StartServerParams sp = (StartServerParams)start_params;

            Application.Run(new Server(sp.hWnd, ScreenCaptureOptions.WINDOW,
                            sp.rep_options, true, sp.port, sp.my_thread, sp.abortMeCallback,
                            -1, null, null));
        }

        public Server(IntPtr hWnd, ScreenCaptureOptions capturesettings,
            ReplicationOptions rep_options, bool menu_window,int port, Thread thread,
            CallBack abortMeCallBack, long key, DispatchServerData databaseServer, TelepointerPtoP telepointerServer)
        {
            this.databaseServer = databaseServer;
            this.telepointerServer = telepointerServer;
            this.key = key;
            this.abortMeCallback = abortMeCallBack;
            this.my_thread = thread;
            this.rep_options = rep_options;
            capture_settings = capturesettings;
            hwnd_target = hWnd;
            this.menu_window = menu_window;
            this.port = port;
            no_write = false;
            last_click_params = new ArrayList();

            lower_port_range = 6287;
            upper_port_range = 6292;

            InitializeComponent();
            outputBox.ScrollBars = ScrollBars.Both;
            //this.FormBorderStyle = FormBorderStyle.FixedToolWindow;

            if (!menu_window)
            {
                // Setup and start the menu capture timer
                this.find_menus_timer = new System.Windows.Forms.Timer();
                this.find_menus_timer.Interval = 200;
                this.find_menus_timer.Tick += new EventHandler(find_menus_timer_Tick);
                this.find_menus_timer.Start();
                menu_hwnds = new ArrayList();
                //menu_hwnds_closeme = new ArrayList();
            }
            
            // Setup and start the screen capture timer
            this.screen_capture_timer = new System.Windows.Forms.Timer();
            this.screen_capture_timer.Interval = 300;
            this.screen_capture_timer.Tick +=new EventHandler(screen_capture_timer_Tick);
            
            if(!menu_window)
                this.Show();
            
            serverStart();


        }

        private delegate void spawnMenuDelegate(StartServerParams sp);
        public void spawnMenu(StartServerParams sp)
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(new spawnMenuDelegate(spawnMenu), new object[] { sp });
            }
            Console.WriteLine("Spawing menu thread");

            ParameterizedThreadStart threadDelegate = new ParameterizedThreadStart(Server.startNewMenuServer);
            Thread newthread = new Thread(threadDelegate);
            newthread.SetApartmentState(ApartmentState.STA);
            sp.my_thread = newthread;
            newthread.Start(sp);

        }

        private delegate void abortMeDelegate(Thread thread);
        public void abortMe(Thread thread)
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(new abortMeDelegate(abortMe), new object[] { thread });
            }
            
            //thread.Abort();
            Console.WriteLine("Non-menu rep server processing menu abort");
            abortMeCallback(thread);
        }

        private delegate void shutdownServerDelegate();
        private void shutdownServer()
        {

                if (this.InvokeRequired)  //i.e. if we are trying to set the text and we aren't the main thread
                {
                    this.BeginInvoke(new shutdownServerDelegate(shutdownServer));
                    return;
                }

                if (!menu_window)
                {
                    
                    ProcessTelepointerTerminateRequest();


                    try { databaseServer.RemoveReplication(key); }
                    catch (Exception) { }

                    byte[] tell_dispatcher_buffer = new byte[1 + 4 + 4];
                    tell_dispatcher_buffer[0] = (byte)NetworkMessages.MessageType.CloseReplicationServer;
                    byte[] size_buff = BitConverter.GetBytes(4);
                    byte[] port_buff = BitConverter.GetBytes(port);

                    for (int i = 0; i < 4; i++)
                        tell_dispatcher_buffer[1 + i] = size_buff[i];

                    for (int i = 0; i < 4; i++)
                        tell_dispatcher_buffer[5 + i] = port_buff[i];

                    find_menus_timer.Stop();
                    screen_capture_timer.Stop();
                    //socket.Shutdown(SocketShutdown.Both);
                    //client.Shutdown(SocketShutdown.Both);
                    try {stream.Close();} 
                        catch (Exception) { }
                    try { listen_socket.Close(); } 
                        catch (Exception) { }
                    try { client.Close(); }
                        catch (Exception) { }

                    try
                    {
                        dispatcher_stream.Write(tell_dispatcher_buffer, 0, tell_dispatcher_buffer.Length);
                        dispatcher_stream.Close(10000);
                    }
                    catch (Exception) { }

                    try
                    {
                        dispatcher_socket.Shutdown(SocketShutdown.Both);
                    }
                    catch (Exception) { }

                    try
                    {
                        dispatcher_socket.Close();
                    }
                    catch (Exception) { }

                    try { this.Close(); } 
                        catch (Exception) { }
                    try { this.Dispose(); }
                        catch (Exception) { }
                    //dispatcher_stream.Close(500);
                    //dispatcher_socket.Close();



                }
                else
                {
                    try
                    {
                        listen_socket.Shutdown(SocketShutdown.Both); 
                    }
                    catch (Exception) { }
                    screen_capture_timer.Stop();

                    try { listen_socket.Close(); }
                    catch (Exception) { }

                    try
                    {
                        client.Shutdown(SocketShutdown.Both);
                    }
                    catch (Exception) { }
                    
                    try {stream.Close();}
                    catch (Exception) { }
                    try {client.Close();}
                    catch (Exception) { }
                    try {this.Close();}
                    catch (Exception) { }
                    try { this.Dispose(); }
                    catch (Exception) { }
                    //abortMeCallback(my_thread);
                }
            //this.Close();
            //this.Dispose();
            //my_thread.Abort();
            //my_thread.
        }

        private delegate void writeToOutputDelegate(string text);
        private void writeToOutput(string text)
        {
            if (no_write)
                return;
            
            
            if (this.InvokeRequired)  //i.e. if we are trying to set the text and we aren't the main thread
            {
                // Pass the same function to BeginInvoke,
                // but the call would come on the correct
                // thread and InvokeRequired will be false.
                this.BeginInvoke(new writeToOutputDelegate(writeToOutput),
                                                  new object[] { text });

                return;
            }

            outputBox.AppendText(text);

        }

       
        void  screen_capture_timer_Tick(object sender, EventArgs e)
        {
 	        sendScreenShots();
        }

        public void serverStart()
        {

            //writeToOutput("Server starting ...\n");
            
            IPAddress serverIP = IPAddress.Any;
            IPEndPoint ipep = new IPEndPoint(serverIP, port);

            /* Networking stuff */
            listen_socket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
            listen_socket.Bind(ipep);
            listen_socket.Listen(10);
            if (!menu_window)
            {
                //writeToOutput("Waiting for dispatcher ...\n");
                dispatcher_socket = listen_socket.Accept();
                dispatcher_stream = new NetworkStream(dispatcher_socket);
            }
            //writeToOutput("Waiting for client ...\n");
            client = listen_socket.Accept();
            stream = new NetworkStream(client);
            IPEndPoint clientep = (IPEndPoint)client.RemoteEndPoint;
            //writeToOutput("Connected with "+ clientep.Address.ToString() + " at port " + clientep.Port +"\n");

            StringBuilder test_title = new StringBuilder(256);
            Win32Api.GetWindowText(hwnd_target, test_title, test_title.Capacity);
            old_title = test_title.ToString();

            int num_bytes = Encoding.ASCII.GetByteCount(test_title.ToString());
            byte[] data = new byte[1 + 4 + num_bytes];
            data[0] = (byte)NetworkMessages.MessageType.TitleBarText;
            byte[] num_bytes_buf = BitConverter.GetBytes(num_bytes);
            byte[] title_buf = Encoding.ASCII.GetBytes(test_title.ToString());

            for (int i = 0; i < 4; i++)
                data[i + 1] = num_bytes_buf[i];

            for (int i = 0; i < num_bytes; i++)
                data[i + 5] = title_buf[i];

            //writeToOutput("Sending a Title buffer with " + num_bytes.ToString() + " bytes ...\n");
            stream.Write(data, 0, data.Length);

            //writeToOutput("Getting RECT for window\n");



            // --- Grab screenshot of the target app
            Win32Api.Rect rc = new Win32Api.Rect();

            if (capture_settings == ScreenCaptureOptions.CLIENT)
            {
                if (!Win32Api.GetClientRect(hwnd_target, ref rc))
                    return;
            }
            else
            {
                if (!Win32Api.GetWindowRect(hwnd_target, ref rc))
                    return;
            }

            Bitmap screenshot = new Bitmap(rc.Width, rc.Height);
            Graphics gfxBitmap = Graphics.FromImage(screenshot);
            IntPtr hdcBitmap = gfxBitmap.GetHdc();

            // Should we grab the whole window, or just the client area?
            if (capture_settings == ScreenCaptureOptions.CLIENT)
                Win32Api.PrintWindow(hwnd_target, hdcBitmap, Win32Api.PW_CLIENTONLY);
            else //if(capturesettings == ScreenCaptureOptions.WINDOW || capturesettings == ScreenCaptureOptions.WINDOWSHOWCLIENT)
                Win32Api.PrintWindow(hwnd_target, hdcBitmap, 0);

            gfxBitmap.ReleaseHdc(hdcBitmap);
            gfxBitmap.Dispose();

            // --- Finished Grabbing screenshot



            System.IO.MemoryStream ms = new System.IO.MemoryStream();
            screenshot.Save(ms, System.Drawing.Imaging.ImageFormat.Png);

            data = new byte[1 + 4];
            num_bytes_buf = BitConverter.GetBytes(ms.Length);
            data[0] = (byte)NetworkMessages.MessageType.ScreenShot;
            for (int i = 0; i < 4; i++)
                data[i + 1] = num_bytes_buf[i];
            //client.Send(data);
            //writeToOutput("Sending screenshot, size = " + ms.Length.ToString() + " ...\n");

            stream.Write(data, 0, data.Length);
            ms.WriteTo(stream);
            //Bitmap ss = new Bitmap(ms);
            //ss.Save(@"C:\screenshotClubDev.png");
            //numsent = 1;

            //send icon
            if (!menu_window)
            {
                Bitmap icon = GetWindowSmallIconAsBitmap(hwnd_target);
                if (icon == null)
                    icon = new Bitmap(16, 16);

                System.IO.MemoryStream ms_icon = new System.IO.MemoryStream();
                icon.Save(ms_icon, System.Drawing.Imaging.ImageFormat.Png);

                data = new byte[1 + 4];
                num_bytes_buf = BitConverter.GetBytes(ms_icon.Length);
                data[0] = (byte)NetworkMessages.MessageType.Icon;
                for (int i = 0; i < 4; i++)
                    data[i + 1] = num_bytes_buf[i];

                NetworkMessages.safeWrite(stream, data, 0, data.Length, 
                    new NetworkMessages.exceptionCallback(this.shutdownServer));
                //stream.Write(data, 0, data.Length);
                ms_icon.WriteTo(stream);

            }

            //writeToOutput("Sending permissions ...\n");

            data = new byte[1 + 4];
            data[0] = (byte)NetworkMessages.MessageType.Permissions;
            num_bytes_buf = BitConverter.GetBytes(1);
            for (int i = 0; i < 4; i++)
                data[i + 1] = num_bytes_buf[i];

            NetworkMessages.safeWrite(stream, data, 0, data.Length,
                    new NetworkMessages.exceptionCallback(this.shutdownServer));
            //stream.Write(data, 0, data.Length);
            data = new byte[1];
            data[0] = (byte)rep_options;
            NetworkMessages.safeWrite(stream, data, 0, data.Length,
                    new NetworkMessages.exceptionCallback(this.shutdownServer));
            //stream.Write(data, 0, data.Length);
            
            //writeToOutput("Forever sending and receiving ...\n");

            md5 = new System.Security.Cryptography.MD5CryptoServiceProvider();
            byte[] bytehash = md5.ComputeHash(ms.ToArray());
            prev_hash = Convert.ToBase64String(bytehash);

            /*
            writeToOutput("Sent data shutting down ...\n");
            client.Close();
            socket.Close();
            writeToOutput("Goodbye\n");
            */

            header_data = true;
            buffer = new byte[5];
            bytes_received = 0;

            dispatch_buffer = new byte[5];
            dispatch_bytes_received=0;
            dispatch_header_data=true;
            dispatch_header=0;

            waitForData();
            if(!menu_window)
                waitForDispatcherData();
            this.screen_capture_timer.Start();
            
        }

        void find_menus_timer_Tick(object sender, EventArgs e)
        {
            
            // Only the replicated parent app should check for menus
            ////if (!menu_window)
            //{
                // for each child, check if it has the styles WS_EX_TOPMOST WS_EX_TOOLWINDOW and WS_POPUP
                // if it does, assume for now that it's a memu, and create a target for it
                ChildWindowArray warray = new ChildWindowArray(IntPtr.Zero);
                Win32Api.WindowInfo wininfo = new Win32Api.WindowInfo();

                Win32Api.Rect rc = new Win32Api.Rect();
                if (!Win32Api.GetWindowRect(hwnd_target, ref rc))
                    return;

                /*
                wininfo.cbSize = (uint)Marshal.SizeOf(wininfo);
                wininfo = new Win32Api.WindowInfo();
                wininfo.cbSize = (uint)Marshal.SizeOf(wininfo);
                Win32Api.GetWindowInfo(hwnd_target, out wininfo);

                IntPtr parent = new IntPtr();
                parent = Win32Api.GetParent(hwnd_target);

                IntPtr owner = new IntPtr();
                owner = Win32Api.GetAncestor(hwnd_target, Win32Api.GA_ROOTOWNER);
                */
                foreach (int hwnd in warray)
                {
                    StringBuilder class_name = new StringBuilder(256);
                    Win32Api.GetClassName(new IntPtr(hwnd), class_name, class_name.Capacity);

                    wininfo = new Win32Api.WindowInfo();
                    wininfo.cbSize = (uint)Marshal.SizeOf(wininfo);
                    Win32Api.GetWindowInfo(new IntPtr(hwnd), out wininfo);
                    if (class_name.ToString() != "MsoCommandBarShadow" && class_name.ToString() != "OfficeTooltip")
                    {
                        if ((wininfo.dwStyle & Win32Api.WS_POPUP) == Win32Api.WS_POPUP)
                        {
                            if ((wininfo.dwExStyle & Win32Api.WS_EX_TOPMOST) == Win32Api.WS_EX_TOPMOST)
                            {
                                //if ((wininfo.dwExStyle & Win32Api.WS_EX_TOOLWINDOW) == Win32Api.WS_EX_TOOLWINDOW)
                                //{
                                if ((wininfo.dwStyle & Win32Api.WS_VISIBLE) == Win32Api.WS_VISIBLE)
                                {
                                    if (Win32Api.GetParent(new IntPtr(hwnd)) == hwnd_target ||
                                        Win32Api.GetAncestor(new IntPtr(hwnd), Win32Api.GA_ROOTOWNER) == hwnd_target)
                                    {


                                        if (!menu_hwnds.Contains(hwnd))
                                        {
                                            int menu_server_port = getUsablePort(lower_port_range, upper_port_range);
                                            if (menu_server_port > -1)
                                            {
                                                //writeToOutput("New menu found! \n");
                                                menu_hwnds.Add(hwnd);
                                                //menu_hwnds_closeme.Add(false);
                                                /* //*R Point start_location = new Point();
                                                start_location.X = wininfo.rcWindow.left - rc.left;
                                                start_location.X += this.Location.X;
                                                start_location.Y = wininfo.rcWindow.top - rc.top;
                                                start_location.Y += this.Location.Y;
                                                */

                                                //writeToOutput("... starting a new server thread for the menu ... \n");
                                                //now we need to spawn a new server process for the menu
                                                //int menu_server_port = getUsablePort(lower_port_range, upper_port_range);
                                                StartServerParams sp = new StartServerParams();
                                                sp.dispatcher_port = 0;
                                                sp.hWnd = new IntPtr(hwnd);
                                                sp.port = menu_server_port;
                                                sp.rep_options = rep_options;
                                                sp.abortMeCallback = new CallBack(abortMe);

                                                spawnMenu(sp);
                                                /*
                                                ParameterizedThreadStart threadDelegate = new ParameterizedThreadStart(AppRepServer.Server.startNewMenuServer);
                                                Thread newthread = new Thread(threadDelegate);
                                                sp.my_thread = newthread;
                                                newthread.Start(sp);
                                                */

                                                //Server menu = new Server(sp.hWnd, ScreenCaptureOptions.WINDOW,
                                                //    sp.rep_options, true, sp.port);
                                                //menu.Show();
                                                //startNewMenuServer(sp);

                                                //Instead we need to send a message to the client with this info
                                                // We need to send:
                                                //    - Header indicating that we need a new menu
                                                //    - Port
                                                //    - Information to calculate menu position
                                                byte[] to_client_buffer = new byte[1 + 4 + 4 + 4 + 4];
                                                to_client_buffer[0] = (byte)NetworkMessages.MessageType.NewMenu;
                                                byte[] buff_size = BitConverter.GetBytes(4 + 4 + 4);
                                                for (int i = 0; i < 4; i++)
                                                    to_client_buffer[i + 1] = buff_size[i];
                                                byte[] port_buffer = BitConverter.GetBytes(menu_server_port);
                                                byte[] x_offset_buffer = BitConverter.GetBytes(wininfo.rcWindow.left - rc.left);
                                                byte[] y_offset_buffer = BitConverter.GetBytes(wininfo.rcWindow.top - rc.top);

                                                for (int i = 0; i < 4; i++)
                                                    to_client_buffer[5 + i] = port_buffer[i];

                                                for (int i = 0; i < 4; i++)
                                                    to_client_buffer[9 + i] = x_offset_buffer[i];

                                                for (int i = 0; i < 4; i++)
                                                    to_client_buffer[13 + i] = y_offset_buffer[i];


                                                //writeToOutput("... sending a message to the client to start a menu client ...");
                                                stream.Write(to_client_buffer, 0, to_client_buffer.Length);




                                                //R//Target menu = new Target(new IntPtr(hwnd), ScreenCaptureOptions.WINDOW, ReplicationOptions.INTERACT, true, start_location);

                                                //R//menu.Show();
                                                //menu.Location = start_location;
                                                IntPtr foreground_app = Win32Api.GetForegroundWindow();
                                                //foreground_app = Win32Api.GetWindow(foreground_app, Win32Api.GW_HWNDNEXT);
                                                if (hwnd_target != foreground_app && foreground_app != this.Handle)
                                                {
                                                    // menu.location_before_move = new Win32Api.Rect();
                                                    // if (!Win32Api.GetWindowRect(new IntPtr(hwnd), ref menu.location_before_move))
                                                    //     continue;

                                                    //The following line works for word ... but not VS
                                                    //Win32Api.SetWindowPos(hwnd, 0, 1600, 0, 0, 0, Win32Api.SWP_NOSIZE | Win32Api.SWP_NOZORDER);
                                                    //Win32Api.SetWindowPos(hwnd, 0, 1600, 0, 0, 0, Win32Api.SWP_NOSIZE | Win32Api.SWP_NOZORDER);
                                                    //Win32Api.SetWindowPos(hwnd, 0, 1600, 0, 0, 0, Win32Api.SWP_NOSIZE | Win32Api.SWP_ASYNCWINDOWPOS);
                                                }
                                                //R//menu_forms.Add(menu);
                                            }
                                        }

                                    }
                                }
                                //}
                            }
                        }
                    }
                    
                }

                // check for menus 
                // Get a list of all children
                ArrayList remove_list = new ArrayList();


                //foreach (Target t in menu_forms)
                //foreach(int hwnd in menu_hwnds)
                for (int i=0; i<menu_hwnds.Count; i++)
                {
                    wininfo = new Win32Api.WindowInfo();
                    wininfo.cbSize = (uint)Marshal.SizeOf(wininfo);
                    Win32Api.GetWindowInfo(new IntPtr((int)menu_hwnds[i]), out wininfo);
                    if ((!Win32Api.IsWindow(new IntPtr((int)menu_hwnds[i]))) ||
                        !((wininfo.dwExStyle & Win32Api.WS_EX_TOPMOST) == Win32Api.WS_EX_TOPMOST) ||
                        !((wininfo.dwStyle & Win32Api.WS_VISIBLE) == Win32Api.WS_VISIBLE))
                        //|| menu_hwnds_closeme[i] == true)
                    {
                        //t.Hide();
                        //menu_hwnds.RemoveAt(menu_hwnds.IndexOf((int)t.hwnd_target));
                        //t.Close();
                        //writeToOutput("Menu is gone ... \n");
                        remove_list.Add(menu_hwnds[i]);
                    }
                }

                foreach (int menu in remove_list)
                {
                    //menu_hwnds.Remove(hwnd);
                    //menu_forms.Remove(t);
                    //t.Dispose();
                    //writeToOutput("... removed menu hwnd from list ... \n");
                    menu_hwnds.Remove(menu);
                    //menu_hwnds_closeme.RemoveAt(index);
                }
                remove_list.Clear();
            //}
        }

        void sendScreenShots()
        {
            //Win32Api.CursorInfo cinfo;
            //cinfo = new Win32Api.CursorInfo();
            //cinfo.cbSize = (uint)Marshal.SizeOf(cinfo);
            //Win32Api.GetCursorInfo(out cinfo);
            //IntPtr cursor_handle = new IntPtr(cinfo.hCursor);
            //IntPtr copy_cursor = Win32Api.CopyIcon(cursor_handle);
            IntPtr foreground_app = Win32Api.GetForegroundWindow();
            //bool copied_cursor = false;

            if (!Win32Api.IsWindow(hwnd_target))
            {
                byte[] to_client_buffer = new byte[1 + 4 + 1];
                to_client_buffer[0] = (byte)NetworkMessages.MessageType.CloseReplication;
                byte[] buff_size = BitConverter.GetBytes(1);
                for (int i = 0; i < 4; i++)
                    to_client_buffer[i + 1] = buff_size[i];

                to_client_buffer[5] = (byte)NetworkMessages.MessageType.CloseReplication;

                NetworkMessages.safeWrite(stream, to_client_buffer, 0, 6,
                    new NetworkMessages.exceptionCallback(this.shutdownServer));
                //stream.Write(to_client_buffer, 0, 6);
                //writeToOutput("... tell client to close replication ...");
                //stream.Close(500);
                //socket.Close();
                
                shutdownServer();
                return;
            }

            if (menu_window)
            {
                Win32Api.WindowInfo wininfo = new Win32Api.WindowInfo();
                wininfo = new Win32Api.WindowInfo();
                wininfo.cbSize = (uint)Marshal.SizeOf(wininfo);
                Win32Api.GetWindowInfo(hwnd_target, out wininfo);
                if ((!Win32Api.IsWindow(hwnd_target)) ||
                        !((wininfo.dwExStyle & Win32Api.WS_EX_TOPMOST) == Win32Api.WS_EX_TOPMOST) ||
                        !((wininfo.dwStyle & Win32Api.WS_VISIBLE) == Win32Api.WS_VISIBLE))
                {
                    byte[] to_client_buffer = new byte[1 + 4 + 1];
                    to_client_buffer[0] = (byte)NetworkMessages.MessageType.CloseReplication;
                    byte[] buff_size = BitConverter.GetBytes(1);
                    for (int i = 0; i < 4; i++)
                        to_client_buffer[i + 1] = buff_size[i];

                    to_client_buffer[5] = (byte)NetworkMessages.MessageType.CloseReplication;

                    NetworkMessages.safeWrite(stream, to_client_buffer, 0, 6,
                        new NetworkMessages.exceptionCallback(this.shutdownServer));
                    //stream.Write(to_client_buffer, 0, 6);
                    //writeToOutput("... tell client to close menu ...");
                    //stream.Close(500);
                    //socket.Close();
                    shutdownServer();
                    return;
                }

            }

            // This call will prevent flicker from the PrintWindow call
            //if (!menu_window)
            //{
                //Win32Api.SendMessage(hwnd_target, Win32Api.WM_SETREDRAW, IntPtr.Zero, IntPtr.Zero);
                /*if (hwnd_target == foreground_app)
                {
                    Win32Api.SetCursor(copy_cursor);
                    copied_cursor = true;
                }*/
            //}
            // --- Grab screenshot of the target app
            Win32Api.Rect rc = new Win32Api.Rect();
            if (capture_settings == ScreenCaptureOptions.CLIENT)
            {
                if (!Win32Api.GetClientRect(hwnd_target, ref rc))
                    return;
            }
            else
            {
                if (!Win32Api.GetWindowRect(hwnd_target, ref rc))
                    return;
            }

            Bitmap screenshot = new Bitmap(rc.Width, rc.Height);

            Graphics gfxBitmap = Graphics.FromImage(screenshot);
            IntPtr hdcBitmap = gfxBitmap.GetHdc();

            //if window is on top, use bitblit
            if (Win32Api.GetForegroundWindow() == hwnd_target)
            {
                //Console.WriteLine("Using BitBlt");
                IntPtr targetDC = Win32Api.GetDCEx(
                    hwnd_target, IntPtr.Zero, Win32Api.DCX_WINDOW);
                Win32Api.BitBlt(hdcBitmap, 0, 0, screenshot.Width, screenshot.Height,
                    targetDC, 0, 0, Win32Api.SRCCOPY);
                Win32Api.ReleaseDC(hwnd_target, targetDC);
            }
            else  //if on bottom ... use printwindow
            {
                //Console.WriteLine("Using PrintWindow");
                if (capture_settings == ScreenCaptureOptions.CLIENT)
                    Win32Api.PrintWindow(hwnd_target, hdcBitmap, Win32Api.PW_CLIENTONLY);
                else
                    Win32Api.PrintWindow(hwnd_target, hdcBitmap, 0);
            }
            //Win32Api.InvalidateRect(hwnd_target, IntPtr.Zero, true);
            gfxBitmap.ReleaseHdc(hdcBitmap);
            gfxBitmap.Dispose();
            // --- Finished Grabbing


            int True = 1;
            IntPtr TRUE = new IntPtr(True);
            //if(!menu_window)
            //    Win32Api.SendMessage(hwnd_target, Win32Api.WM_SETREDRAW, TRUE, IntPtr.Zero);
         

            ChildWindowArray warray = new ChildWindowArray(hwnd_target);
            foreach (int win in warray)
            {
                Win32Api.InvalidateRect(new IntPtr(win), IntPtr.Zero, true);
            }
            //if (copied_cursor)
            //    Win32Api.SetCursor(cursor_handle);

    
            
            
            
            // The title may have changed
            StringBuilder title = new StringBuilder(256);
            Win32Api.GetWindowText(hwnd_target, title, title.Capacity);
            if(!old_title.Equals(title.ToString()))
            {
                old_title = title.ToString();
                //writeToOutput("... title change detected ... sending new title ...\n");

                int num_bytes = Encoding.ASCII.GetByteCount(title.ToString());
                byte[] title_data = new byte[1 + 4 + num_bytes];
                title_data[0] = (byte)NetworkMessages.MessageType.TitleBarText;
                byte[] title_num_bytes_buf = BitConverter.GetBytes(num_bytes);
                byte[] title_buf = Encoding.ASCII.GetBytes(title.ToString());

                for (int i = 0; i < 4; i++)
                    title_data[i + 1] = title_num_bytes_buf[i];

                for (int i = 0; i < num_bytes; i++)
                    title_data[i + 5] = title_buf[i];
                try
                {
                    NetworkMessages.safeWrite(stream, title_data, 0, title_data.Length,
                        new NetworkMessages.exceptionCallback(this.shutdownServer));
                }
                catch (ObjectDisposedException)
                {
                    shutdownServer();
                }
 
            }



            /* NEW NETWORKING STUFF */

            System.IO.MemoryStream ms = new System.IO.MemoryStream();
            screenshot.Save(ms, System.Drawing.Imaging.ImageFormat.Png);

            md5 = new System.Security.Cryptography.MD5CryptoServiceProvider();
            byte[] bytehash = md5.ComputeHash(ms.ToArray());
            string hash = Convert.ToBase64String(bytehash);
            if (hash.Equals(prev_hash))
                return;

            prev_hash = hash;
            byte[] data = new byte[1 + 4];
            int data_size = (int)ms.Length;
            byte[] num_bytes_buf = BitConverter.GetBytes(data_size);
            data[0] = (byte)NetworkMessages.MessageType.ScreenShot;
            for (int i = 0; i < 4; i++)
                data[i + 1] = num_bytes_buf[i];
            try
            {
                stream.Write(data, 0, data.Length);
                ms.WriteTo(stream);
            }
            catch (Exception)
            {
                shutdownServer();
            }   
            
            //writeToOutput("... new screenshot sent, size = "+ms.Length.ToString()+"\n");
        }

        void waitForData()
        {

            if (dataReceivedCallback == null)
                dataReceivedCallback = new AsyncCallback(onDataReceived);
            // now start to listen for any data...

            dataReceivedResult = client.BeginReceive(buffer, bytes_received, buffer.Length - bytes_received, SocketFlags.None, dataReceivedCallback, null);
        }

        void waitForDispatcherData()
        {
            if (dataDispatcherReceivedCallback == null)
                dataDispatcherReceivedCallback = new AsyncCallback(onDispatcherDataReceived);
            // now start to listen for any data...
            dataDispatcherReceivedResult = dispatcher_socket.BeginReceive(dispatch_buffer, dispatch_bytes_received, dispatch_buffer.Length - dispatch_bytes_received, SocketFlags.None, dataDispatcherReceivedCallback, null);

        }


        public void onDispatcherDataReceived(IAsyncResult asyn)
        {
            try
            {
                int just_received = 0;
                just_received = dispatcher_socket.EndReceive(asyn);
                dispatch_bytes_received += just_received;
                // Console.WriteLine("Total bytes received = " + bytes_received.ToString());
                if (dispatch_bytes_received < dispatch_buffer.Length)
                {
                    waitForDispatcherData();
                    return;
                }
                dispatch_bytes_received = 0;
                //do whatever we need to with what we just received
                //we will either be waiting for a header or for data ...
                if (dispatch_header_data)
                {
                    dispatch_header = dispatch_buffer[0];
                    int data_buffer_length = BitConverter.ToInt32(dispatch_buffer, 1);
                    dispatch_buffer = new byte[data_buffer_length];
                    dispatch_header_data = false;
                }
                else
                {
                    processDispatcherDataBuffer();
                    dispatch_header_data = true;
                    dispatch_buffer = new byte[5];
                }


                //set up on data received
                waitForDispatcherData();
            }
            catch (ObjectDisposedException)
            {
                // Console.WriteLine("onDataReceieved: Socket has been closed");
            }
            catch (SocketException)
            {
                //MessageBox.Show(se.Message);
                //shutdownServer();
            }


        }

        /*
         * KeyBoardBuffer,           
            MouseDown,              
            MouseDblClick,
         */

        // We finished receieving data
        //We can receive:
        //
        //   - Screenshots
        //   - Window title updates
        //   - New menu messages (not yet implemented)
        //
        //   - This function will be called twice for each of these
        //      The first time will be to pull off a header type and data buffer size
        //      The second time wil be the actual data
        public void onDataReceived(IAsyncResult asyn)
        {
            try
            {

                int just_received = 0;
                just_received = client.EndReceive(asyn);
                bytes_received += just_received;
                // Console.WriteLine("Total bytes received = " + bytes_received.ToString());
                if (bytes_received < buffer.Length)
                {
                    waitForData();
                    return;
                }
                bytes_received = 0;
                //do whatever we need to with what we just received
                //we will either be waiting for a header or for data ...
                if (header_data)
                {
                    header = buffer[0];
                    if (header == (byte)NetworkMessages.MessageType.ClientClosing)
                        processClientClosing();
                    int data_buffer_length = BitConverter.ToInt32(buffer, 1);
                    buffer = new byte[data_buffer_length];
                    header_data = false;
                }
                else
                {
                    processDataBuffer();
                    header_data = true;
                    buffer = new byte[5];
                }

                //set up on data received
                waitForData();

            }
            catch (ObjectDisposedException)
            {
                // Console.WriteLine("onDataReceieved: Socket has been closed");
            }
            catch (SocketException)
            {
                //MessageBox.Show(se.Message);
                shutdownServer();
            }

        }

        void processClientClosing()
        {
            //writeToOutput("Client closed ... shutting down server");
           /* stream.Close();
            socket.Close();
            if (!menu_window)
            {
                byte[] tell_dispatcher_buffer = new byte[1 + 4 + 4];
                tell_dispatcher_buffer[0] = (byte)NetworkMessages.MessageType.CloseReplicationServer;
                byte[] size_buff = BitConverter.GetBytes(4);
                byte[] port_buff = BitConverter.GetBytes(port);

                for (int i = 0; i < 4; i++)
                    tell_dispatcher_buffer[1 + i] = size_buff[i];

                for (int i = 0; i < 4; i++)
                    tell_dispatcher_buffer[5 + i] = port_buff[i];

                dispatcher_stream.Write(tell_dispatcher_buffer, 0, tell_dispatcher_buffer.Length);
                dispatcher_stream.Close(500);
                dispatcher_socket.Close();
            }*/
            shutdownServer();
            
        }

        void processDispatcherDataBuffer()
        {
            //writeToOutput("Received message from dispatcher ...");
            if (dispatch_header == (byte)NetworkMessages.MessageType.ChangePermissions)
            {
                byte[] to_client_buffer = new byte[1 + 4 + 1];
                to_client_buffer[0] = (byte)NetworkMessages.MessageType.ChangePermissions;
                byte[] buff_size = BitConverter.GetBytes(1);
                for (int i = 0; i < 4; i++)
                    to_client_buffer[i + 1] = buff_size[i];

                to_client_buffer[5] = dispatch_buffer[0];
                NetworkMessages.safeWrite(stream, to_client_buffer, 0, 6,
                        new NetworkMessages.exceptionCallback(this.shutdownServer));
                //writeToOutput("... forwarded message to client ...");
            }
            else if (dispatch_header == (byte)NetworkMessages.MessageType.CloseReplication)
            {
                byte[] to_client_buffer = new byte[1 + 4 + 1];
                to_client_buffer[0] = (byte)NetworkMessages.MessageType.CloseReplication;
                byte[] buff_size = BitConverter.GetBytes(1);
                for (int i = 0; i < 4; i++)
                    to_client_buffer[i + 1] = buff_size[i];

                to_client_buffer[5] = (byte)NetworkMessages.MessageType.CloseReplication;

                NetworkMessages.safeWrite(stream, to_client_buffer, 0, 6,
                        new NetworkMessages.exceptionCallback(this.shutdownServer));
                //writeToOutput("... forwarded message to client ...");
                shutdownServer();
            }
        }

        void processDataBuffer()
        {
            if (header == (byte)NetworkMessages.MessageType.MouseDown)
            {
                // Console.WriteLine("Mouse down even received");

                byte button = buffer[0];
                Point click_location = new Point();
                click_location.X = BitConverter.ToInt32(buffer, 1);
                click_location.Y = BitConverter.ToInt32(buffer, 5);



                ArrayList paramlist = getClickParams(click_location);
                last_click_params = paramlist;
                if (paramlist != null)
                {
                    foreach (MouseClickParameters param in paramlist)
                    {
                        //Win32Api.SetActiveWindow(param.hwnd);
                        if (button == (byte)NetworkMessages.MouseButton.Left)
                        {
                            StringBuilder class_name = new StringBuilder(256);
                            Win32Api.GetClassName(param.hwnd, class_name, class_name.Capacity);
                            if (class_name.ToString() == "MsoCommandBar")
                            {
                                //uint wParam = Win32Api.WA_CLICKACTIVE;
                                //IntPtr wparam = new IntPtr(wParam);
                                Win32Api.PostMessage(param.hwnd, Win32Api.WM_ACTIVATE, Win32Api.WA_CLICKACTIVE, 0);
                                //postLocalMouseMessage(param, Win32Api.WM_NCHITTEST, 0);
                                //Win32Api.SetWindowPos((int)hwnd_target, 0, 0, 0, 0, 0, Win32Api.SWP_NOZORDER |
                                //        Win32Api.SWP_NOSIZE);

                            }
                            
                            //Console.WriteLine("send left click at " + param.location.ToString() + " to hwnd:" + param.hwnd.ToString());
                            if (class_name.ToString() == "EzMdiTabs")
                            {
                                //postLocalMouseMessage(param, Win32Api.WM_LBUTTONDBLCLK, Win32Api.MK_LBUTTON);
                                postLocalMouseMessage(param, Win32Api.WM_RBUTTONDOWN, Win32Api.MK_RBUTTON);
                                postLocalMouseMessage(param, Win32Api.WM_RBUTTONUP, 0);
                                param.location.X -= 1;
                                param.location.Y -= 20;
                                postLocalMouseMessage(param, Win32Api.WM_LBUTTONDOWN, Win32Api.MK_LBUTTON);
                                postLocalMouseMessage(param, Win32Api.WM_LBUTTONUP, 0);
                            }
                            else
                            {
                                postLocalMouseMessage(param, Win32Api.WM_LBUTTONDOWN, Win32Api.MK_LBUTTON);
                                postLocalMouseMessage(param, Win32Api.WM_LBUTTONUP, 0);
                            }
                            

                            //if (class_name.ToString() == "EzMdiTabs")
                            //    Win32Api.PostMessage(param.hwnd, Win32Api.WM_CAPTURECHANGED, 0, (uint)param.hwnd);
                            /*
                            if (class_name.ToString() == "MsoCommandBar")
                                Win32Api.SetWindowPos((int)hwnd_target, 0, 1000, 0, 0, 0, Win32Api.SWP_NOZORDER |
                                    Win32Api.SWP_NOSIZE);
                            */
                        }
                        else if (button == (byte)NetworkMessages.MouseButton.Middle)
                        {

                            postLocalMouseMessage(param, Win32Api.WM_MBUTTONDOWN, Win32Api.MK_MBUTTON);
                            postLocalMouseMessage(param, Win32Api.WM_MBUTTONUP, 0);
                        }
                        else if (button == (byte)NetworkMessages.MouseButton.Right)
                        {
                            //Console.WriteLine("send right click at "+param.location.ToString()+" to hwnd:" + param.hwnd.ToString());
                            postLocalMouseMessage(param, Win32Api.WM_RBUTTONDOWN, Win32Api.MK_RBUTTON);
                            postLocalMouseMessage(param, Win32Api.WM_RBUTTONUP, 0);
                        }
                    }
                }
                // if this is a menu, make sure we invalidate it before we close our copy
                if (menu_window)
                    Win32Api.InvalidateRect(hwnd_target, IntPtr.Zero, false);

                /*if (!menu_window)
                {
                    foreach (Target t in menu_forms)
                        t.close_me = true;
                }*/
            }
            else if (header == (byte)NetworkMessages.MessageType.KeyBoardBuffer)
            {
                ArrayList keypress_buffer = new ArrayList();

                for (int i = 0; i < buffer.Length; i += 6)
                {
                    KeyPressParameters press = new KeyPressParameters();
                    if (buffer[i] == 0)
                        press.special_character = false;
                    else
                        press.special_character = true;
                    press.key = BitConverter.ToInt32(buffer, i + 1);
                    press.charkey = (char)buffer[i + 5];
                    keypress_buffer.Add(press);

                }


                //Process keystroke buffer
                foreach (MouseClickParameters windows in last_click_params)
                {
                    StringBuilder class_name = new StringBuilder(256);
                    Win32Api.GetClassName(windows.hwnd, class_name, class_name.Capacity);
                    if (class_name.ToString() != "MDIClient" &&
                        class_name.ToString() != "EzMdiContainer" &&
                        class_name.ToString() != "DockingView" &&
                        class_name.ToString() != "GenericPane" &&
                        class_name.ToString() != "VsSplitterRoot" &&
                        class_name.ToString() != "VsEditPane" &&
                        class_name.ToString() != "IEFrame" &&
                        class_name.ToString() != "TabWindowClass" &&
                        class_name.ToString() != "Shell DocObject View")
                    {
                        //foreach (char c in keypress_buffer)
                        foreach (KeyPressParameters kpress in keypress_buffer)
                        {
                            

                            if (!kpress.special_character)
                            {
                                postKeystrokeMessages(windows.hwnd, Win32Api.WM_CHAR, (uint)kpress.charkey, 0);
                            }
                            else
                            {
                                postKeystrokeMessages(windows.hwnd, Win32Api.WM_KEYDOWN, (uint)kpress.key, 0x00000001);
                                postKeystrokeMessages(windows.hwnd, Win32Api.WM_KEYUP, (uint)kpress.key, 0xC0000001);
                               
                            
                            }

                        }
                    }

                }
                keypress_buffer.Clear();

             
            }
            else if (header == (byte)NetworkMessages.MessageType.MouseDblClick)
            {
                // Console.WriteLine("Mouse down even received");

                byte button = buffer[0];
                Point click_location = new Point();
                click_location.X = BitConverter.ToInt32(buffer, 1);
                click_location.Y = BitConverter.ToInt32(buffer, 5);



                ArrayList paramlist = getClickParams(click_location);
                last_click_params = paramlist;
                if (paramlist != null)
                {
                    foreach (MouseClickParameters param in paramlist)
                    {
                        //Win32Api.SetActiveWindow(param.hwnd);
                        if (button == (byte)NetworkMessages.MouseButton.Left)
                        {
                            StringBuilder class_name = new StringBuilder(256);
                            Win32Api.GetClassName(param.hwnd, class_name, class_name.Capacity);
                            if (class_name.ToString() == "MsoCommandBar")
                            {
                                //uint wParam = Win32Api.WA_CLICKACTIVE;
                                //IntPtr wparam = new IntPtr(wParam);
                                Win32Api.PostMessage(param.hwnd, Win32Api.WM_ACTIVATE, Win32Api.WA_CLICKACTIVE, 0);
                                //postLocalMouseMessage(param, Win32Api.WM_NCHITTEST, 0);
                                //Win32Api.SetWindowPos((int)hwnd_target, 0, 0, 0, 0, 0, Win32Api.SWP_NOZORDER |
                                //        Win32Api.SWP_NOSIZE);

                            }

                            postLocalMouseMessage(param, Win32Api.WM_LBUTTONDBLCLK, Win32Api.MK_LBUTTON);
                            //postLocalMouseMessage(param, Win32Api.WM_LBUTTONUP, 0);

                            /*
                            if (class_name.ToString() == "MsoCommandBar")
                                Win32Api.SetWindowPos((int)hwnd_target, 0, 1000, 0, 0, 0, Win32Api.SWP_NOZORDER |
                                    Win32Api.SWP_NOSIZE);
                            */
                        }
                        else if (button == (byte)NetworkMessages.MouseButton.Middle)
                        {
                            postLocalMouseMessage(param, Win32Api.WM_MBUTTONDBLCLK, Win32Api.MK_MBUTTON);
                            //postLocalMouseMessage(param, Win32Api.WM_MBUTTONUP, 0);
                        }
                        else if (button == (byte)NetworkMessages.MouseButton.Right)
                        {
                            postLocalMouseMessage(param, Win32Api.WM_RBUTTONDBLCLK, Win32Api.MK_RBUTTON);
                            //postLocalMouseMessage(param, Win32Api.WM_RBUTTONUP, 0);
                        }
                    }
                }
                // if this is a menu, make sure we invalidate it before we close our copy
                if (menu_window)
                    Win32Api.InvalidateRect(hwnd_target, IntPtr.Zero, false);

                /*if (!menu_window)
                {
                    foreach (Target t in menu_forms)
                        t.close_me = true;
                }*/
            }

            else if (header == (byte)NetworkMessages.MessageType.ClipboardPaste)
            {
                Console.WriteLine("paste command received " + buffer.Length);
                
                DataObject dataObject = new DataObject();

                int currentPosition = 0;

                while(currentPosition < buffer.Length)
                {
                    int sizeOfFormat = BitConverter.ToInt32(buffer, currentPosition);
                    int sizeOfObjectType = BitConverter.ToInt32(buffer, currentPosition + 4);
                    long sizeOfMemoryStream = BitConverter.ToInt64(buffer, currentPosition + 8);


                    char[] formatInBytes = new char[sizeOfFormat];
                    char[] objectTypeInBytes = new char[sizeOfObjectType];
                    byte[] memoryStreamInBytes = new byte[sizeOfMemoryStream];

                    for (int i = 0; i < formatInBytes.Length; i++)
                        formatInBytes[i] = (char)buffer[currentPosition + 16 + i];

                    for (int i = 0; i < objectTypeInBytes.Length; i++)
                        objectTypeInBytes[i] = (char)buffer[currentPosition + 16 + formatInBytes.Length + i];

                    for (int i = 0; i < memoryStreamInBytes.Length; i++)
                        memoryStreamInBytes[i] = buffer[currentPosition + 16 + formatInBytes.Length + objectTypeInBytes.Length + i];


                    string format = new String(formatInBytes);
                    string objectType = new String(objectTypeInBytes);

                    System.IO.MemoryStream memoryStream = new System.IO.MemoryStream(memoryStreamInBytes);


                    ProcessClipboardDataObject(objectType, format, memoryStream, ref dataObject);

                    
                    currentPosition += 1 + 16 + sizeOfFormat + sizeOfObjectType + (int)sizeOfMemoryStream;

                }



                SetClipboardData(dataObject);


            }

            else if (header == (byte)NetworkMessages.MessageType.ClipboardCopy)
            {
                Console.WriteLine("copy command received");

                
                GetAndSendClipboardData(NetworkMessages.MessageType.ClipboardCopy);
            }

            else if (header == (byte)NetworkMessages.MessageType.ClipboardCut)
            {
                Console.WriteLine("cut command received");
                GetAndSendClipboardData(NetworkMessages.MessageType.ClipboardCut);
            }

            else if (header == (byte)NetworkMessages.MessageType.ClientHWND)
            {
                this.client_hwnd = (IntPtr)BitConverter.ToInt32(buffer, 0);
                ProcessTelepointerSupportRequest();
            }

        }


        private delegate void GetAndSendClipboardDataDelegate(NetworkMessages.MessageType messageType);
        public void GetAndSendClipboardData(NetworkMessages.MessageType messageType)
        {

            if (this.InvokeRequired)
            {
                this.BeginInvoke(new GetAndSendClipboardDataDelegate(GetAndSendClipboardData), new object[] {messageType});
                return;
            }


            IDataObject existingClipboardData = Clipboard.GetDataObject();

            IntPtr currentWindow = Win32Api.GetForegroundWindow();

            Win32Api.SetForegroundWindow(hwnd_target.ToInt32());

            System.Threading.Thread.Sleep(10);

            if (messageType == NetworkMessages.MessageType.ClipboardCopy)
                SendInputHelper.press(3);
            else
                SendInputHelper.press(24);

            System.Threading.Thread.Sleep(5);

            Win32Api.SetForegroundWindow(currentWindow.ToInt32());


            IDataObject clipboardData = Clipboard.GetDataObject();

            string[] formats = clipboardData.GetFormats();
            byte[][] formatsInByteArrays = new byte[formats.Length][];
            int i = 0;

            foreach (string format in formats)
            {
                object clipboardDataObject = null;

                if (format.ToLower() != "enhancedmetafile")
                {
                    clipboardDataObject = clipboardData.GetData(format);
                    if (clipboardDataObject != null)
                    {
                        formatsInByteArrays[i] = ClipboardOperations.ConvertObjectToByteArray(clipboardDataObject, format);
                    }
                }

                i++;
            }

            int totalSize = 0;
            foreach (byte[] formatInByteArray in formatsInByteArrays)
                totalSize += formatInByteArray.Length;


            byte[] bufferedData = new byte[1 + 4 + totalSize];

            bufferedData[0] = (byte)messageType;

            byte[] totalPayloadSize = BitConverter.GetBytes(totalSize);

            for (int j = 0; j < 4; j++)
                bufferedData[1 + j] = totalPayloadSize[j];

            int currentOffset = 5;

            foreach (byte[] formatInByteArray in formatsInByteArrays)
            {
                for (int j = 0; j < formatInByteArray.Length; j++)
                    bufferedData[currentOffset + j] = formatInByteArray[j];

                currentOffset += formatInByteArray.Length;
            }

            Clipboard.SetDataObject(existingClipboardData, true);

            NetworkMessages.safeWrite(stream, bufferedData, 0, bufferedData.Length,
                new NetworkMessages.exceptionCallback(this.shutdownServer));

        }

        private delegate void SetClipboardDataDelegate(DataObject dataObject);
        public void SetClipboardData(DataObject dataObject)
        {
            if (this.InvokeRequired)
            {
                this.BeginInvoke(new SetClipboardDataDelegate(SetClipboardData), new object[] { dataObject });

                return;
            }

            // Get the contents currently in the clipboard so they can be preserved
            IDataObject clipboardData = Clipboard.GetDataObject();

            
            /*
            string[] formats = dataObject.GetFormats();

            foreach(string format in formats)
                if (format.ToLower() == "system.string")
                {
                    //currentObjects.SetData(format, clipboardData.GetData(format));

                    String clipboardContent = (String)dataObject.GetData(format);

                    char[] clipboardContentAsCharArray = clipboardContent.ToCharArray();


                    //foreach (char character in clipboardContentAsCharArray)
                    //{
                    //    postKeystrokeMessages(hwnd_target, Win32Api.WM_CHAR, (uint)character, 0);
                    //}

                    foreach (MouseClickParameters windows in last_click_params)
                    {
                        StringBuilder class_name = new StringBuilder(256);
                        Win32Api.GetClassName(windows.hwnd, class_name, class_name.Capacity);
                        if (class_name.ToString() != "MDIClient" &&
                            class_name.ToString() != "EzMdiContainer" &&
                            class_name.ToString() != "DockingView" &&
                            class_name.ToString() != "GenericPane" &&
                            class_name.ToString() != "VsSplitterRoot" &&
                            class_name.ToString() != "VsEditPane" &&
                            class_name.ToString() != "IEFrame" &&
                            class_name.ToString() != "TabWindowClass" &&
                            class_name.ToString() != "Shell DocObject View")
                        {
                            foreach (char character in clipboardContentAsCharArray)
                            {
                                postKeystrokeMessages(windows.hwnd, Win32Api.WM_CHAR, (uint)character, 0);

                            }
                        }
                    }
                }
                    
            */

            
                Clipboard.SetDataObject(dataObject, true);

                IntPtr currentWindow = Win32Api.GetForegroundWindow();

                Win32Api.SetForegroundWindow(hwnd_target.ToInt32());

                System.Threading.Thread.Sleep(15);
                SendInputHelper.press(22);
                System.Threading.Thread.Sleep(15);
                

                Win32Api.SetForegroundWindow(currentWindow.ToInt32());
            



            Clipboard.SetDataObject(clipboardData, true);
            
        }


        private DataObject ProcessClipboardDataObject(string objectType, string format, System.IO.MemoryStream memStream, ref DataObject dataObject)
        {

            //here we handle the different object types before adding them to the clipboard data object.
            if (objectType == "Bitmap" || objectType == "System.Drawing.Bitmap")
            {
                System.Drawing.Bitmap bitmap = new Bitmap(memStream);
                dataObject.SetData(format, bitmap);
            }
            else if (objectType == "MemoryStream" || objectType == "System.IO.MemoryStream")
            {
                dataObject.SetData(format, memStream);
            }
            else if (objectType == "String" || objectType == "System.String")
            {
                byte[] memoryStreamInBytes = new byte[memStream.Length];
                memStream.Read(memoryStreamInBytes, 0, (int)memStream.Length);

                UnicodeEncoding uniEncoding = new UnicodeEncoding();
                string ourText = uniEncoding.GetString(memoryStreamInBytes);
                dataObject.SetData(format, ourText);
            }

            /*
            else if (objectType == "File")
            {


                //parse out the format and filename
                int indexOfFileName = clipBoardStream.Format.IndexOf("FILENAME");
                string format = clipBoardStream.Format.Substring(indexOfFileName + 8);
                string fileName = clipBoardStream.Format.Substring(0, indexOfFileName);

                string filePath = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments) + "\\ClipboardAPP\\" + clipBoardStream.ClipBoardID;

                if (!Directory.Exists(filePath))
                {
                    try
                    {
                        //delete any existing files from previous pastes before creating the new directory
                        Directory.Delete(Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments) + "\\ClipboardAPP", true);
                    }
                    catch
                    {
                        // do nothing.
                    }
                    Directory.CreateDirectory(filePath);
                }

                filePath += "\\" + fileName;

                if (System.IO.File.Exists(filePath))
                {
                    System.IO.File.Delete(filePath);
                }

                //determind the length of the file itself (not including the embedded name) and write it to disk.

                FileStream fileStream = new FileStream(filePath, System.IO.FileMode.CreateNew);
                fileStream.Write(clipBoardStream.Buffer, 0, clipBoardStream.Buffer.Length);
                fileStream.Close();


                string[] fileNames = (string[])dataObject.GetData(format);
                if (fileNames == null)
                {
                    Array.Resize(ref fileNames, 1);
                }
                else
                {
                    Array.Resize(ref fileNames, fileNames.Length + 1);
                }

                fileNames[fileNames.Length - 1] = filePath;

                dataObject.SetData(format, fileNames);

            }
            */ 
            else
            {
                dataObject.SetData(format, memStream);
            }

            return dataObject;

        }



        private void ProcessTelepointerSupportRequest()
        {

            if (client_hwnd.Equals(null))
                return;

            IPEndPoint clientEndPoint = (IPEndPoint)client.RemoteEndPoint;
            IPAddress ip = clientEndPoint.Address;


            KnownColor pointerColor = KnownColor.Indigo;

            telepointerServer.addLocalClient(this.hwnd_target, this.client_hwnd, ip, pointerColor, (float)Convert.ToDouble(1));
            telepointerServer.addRemoteClient(this.client_hwnd, this.hwnd_target, ip, pointerColor, (float)Convert.ToDouble(1));
        }

        private void ProcessTelepointerTerminateRequest()
        {
            if (client_hwnd.Equals(null))
                return;

            IPEndPoint clientEndPoint = (IPEndPoint)client.RemoteEndPoint;
            IPAddress ip = clientEndPoint.Address;

            telepointerServer.stopLocalClient(this.hwnd_target, this.client_hwnd, ip);
            telepointerServer.stopRemoteClient(this.client_hwnd, this.hwnd_target, ip);
        }




        private void postKeystrokeMessages(IntPtr hwnd, uint message, uint wparam, uint lparam)
        {
            Win32Api.PostMessage(hwnd, message, wparam, lparam);
        }


        //returns the hwnd of the window to send input to, and the properly transformed
        //location - click location within the parent window
        private ArrayList getClickParams(Point location)
        {
            MouseClickParameters mparams = new MouseClickParameters();
            // Get an array of child windows

            ArrayList mparamsarray = new ArrayList();

            //IntPtr child_hwnd;
            Win32Api.PointWin32 point;
            point.x = location.X;
            point.y = location.Y;



            ChildWindowArray cwarray = new ChildWindowArray(hwnd_target);


            Win32Api.Rect parentrec = new Win32Api.Rect();
            if (!Win32Api.GetWindowRect(hwnd_target, ref parentrec))
                return null;




            if (menu_window)
            {
                Win32Api.Rect rec = new Win32Api.Rect();
                Win32Api.GetWindowRect(hwnd_target, ref rec);

                rec.left -= parentrec.left;
                rec.right -= parentrec.left;
                rec.top -= parentrec.top;
                rec.bottom -= parentrec.top;


                MouseClickParameters addme = new MouseClickParameters();
                addme.hwnd = hwnd_target;
                addme.location.X = point.x - (rec.left);
                addme.location.Y = point.y - (rec.top);
                mparamsarray.Add(addme);
            }




            bool is_mozilla_ui = false;
            StringBuilder class_name = new StringBuilder(256);
            Win32Api.GetClassName(hwnd_target, class_name, class_name.Capacity);

            if (class_name.ToString() == "MozillaUIWindowClass")
                is_mozilla_ui = true;

            // Find the child window to send input to
            foreach (int hwnd in cwarray)
            {

                Win32Api.Rect rec = new Win32Api.Rect();
                if (!Win32Api.GetWindowRect(new IntPtr(hwnd), ref rec))
                    continue;


                /*Win32Api.Rect rec;
                if(menu_window)
                    rec = location_before_move;
                else
                {
                    rec = new Win32Api.Rect();
                    if (!Win32Api.GetWindowRect(new IntPtr(hwnd), ref rec))
                        continue;
                }*/

                
                rec.left -= parentrec.left;
                rec.right -= parentrec.left;
                rec.top -= parentrec.top;
                rec.bottom -= parentrec.top;
                // point is inside child window

                if ((is_mozilla_ui) || (point.x > rec.left && point.x < rec.right && point.y > rec.top && point.y < rec.bottom))
                //if(hwnd == 4459162)
                {
                    //StringBuilder class_name = new StringBuilder(256);
                    Win32Api.GetClassName(new IntPtr(hwnd), class_name, class_name.Capacity);

                    if (class_name.ToString() != "MsoDockTop" && class_name.ToString() != "MsoDockTop" && class_name.ToString() != "MsoCommandBarDock")
                    {
                        MouseClickParameters addme = new MouseClickParameters();
                        addme.hwnd = new IntPtr(hwnd);
                        if (!is_mozilla_ui)
                        {
                            addme.location.X = point.x - (rec.left);
                            addme.location.Y = point.y - (rec.top);
                        }
                        else
                        {
                            addme.location.X = (point.x - (rec.left)) + (parentrec.left + 5);
                            addme.location.Y = (point.y - (rec.top)) + (parentrec.top + 139);
                        }
                        mparamsarray.Add(addme);

                     
                    }


                    if (class_name.ToString() == "VsTextEditPane" ||
                        //class_name.ToString() == "MsoCommandBar" )//||  //MsoCommandBar seems to only recognize                                                                        //commands if on top
                        class_name.ToString() == "EzMdiTabs")
                        return mparamsarray;


                    //last_click_pos = point;
                }


            }
            return mparamsarray;

          
        }

        private void postLocalMouseMessage(MouseClickParameters mparams, uint message, uint wparam)
        {
            if (mparams == null)
                return;

            int location = (mparams.location.Y * 0x10000) | (mparams.location.X & 0xFFFF);
            Win32Api.PostMessage(mparams.hwnd, message, wparam, (uint)location);
        }

        private bool isPortUsable(int port)
        {
            Socket s = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
            try
            {
                s.Bind(new IPEndPoint(IPAddress.Any, port));
                s.Close();
                return true;
            }
            catch (SocketException ex)
            {
                // EADDRINUSE?
                if (ex.ErrorCode == 10048)
                    return false;
                return false;
            }

        }


        private int getUsablePort(int lower_range, int upper_range)
        {
            for (int i = lower_range; i <= upper_range; i++)
            {
                if (isPortUsable(i))
                    return i;
            }
            return -1;

        }

        /// <summary>
        /// Gets a small icon from the window as a Bitmap
        /// </summary>
        public static Bitmap GetWindowSmallIconAsBitmap(IntPtr hWnd)
        {
            try
            {
                int result;
                Win32Api.SendMessageTimeout(hWnd, Win32Api.WM_GETICON, Win32Api.ICON_SMALL, 0, Win32Api.SMTO_ABORTIFHUNG, 1000, out result);

                IntPtr hIcon = new IntPtr(result);

                if (hIcon == IntPtr.Zero)
                {
                    result = Win32Api.GetClassLong(hWnd, Win32Api.GCL_HICONSM);
                    hIcon = new IntPtr(result);
                }

                if (hIcon == IntPtr.Zero)
                {
                    Win32Api.SendMessageTimeout(hWnd, Win32Api.WM_QUERYDRAGICON, 0, 0, Win32Api.SMTO_ABORTIFHUNG, 1000, out result);
                    hIcon = new IntPtr(result);
                }

                if (hIcon == IntPtr.Zero)
                    return null;
                else
                    return Bitmap.FromHicon(hIcon);
            }
            catch (System.Exception)
            {
            }
            return null;
        }





    }
}