using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Drawing;
using System.IO;
using System.Windows.Forms;

namespace PARTIES
{

    public class ClipboardOperations
    {
        public static byte[] ConvertObjectToByteArray(object clipboardDataObject, string format)
        {
            //if the object type is a MemoryStream already all we have to do is cast it as a stream and then send to
            //our helper function.
            if (clipboardDataObject.GetType().ToString() == "System.IO.MemoryStream")
            {
                MemoryStream memoryStream = (MemoryStream)clipboardDataObject;
                return GetObjectInBytes(format, clipboardDataObject.GetType().ToString(), memoryStream);
            }

            //if the object is a BitMap then we'll first need to convert to a MemoryStream bofore sending. 
            else if (clipboardDataObject.GetType().ToString() == "System.Drawing.Bitmap")
            {
                System.Drawing.Bitmap bitmap = (System.Drawing.Bitmap)clipboardDataObject;
                MemoryStream bitmapStream = new MemoryStream();

                bitmap.Save(bitmapStream, System.Drawing.Imaging.ImageFormat.Bmp);
                return GetObjectInBytes(format, clipboardDataObject.GetType().ToString(), bitmapStream);

            }
            //if the object is a string then we should encode to a MemoryStream again so we can use our same helper function
            else if (clipboardDataObject.GetType().ToString() == "System.String")
            {

                UnicodeEncoding uniEncoding = new UnicodeEncoding();
                byte[] buffer = uniEncoding.GetBytes((string)clipboardDataObject);
                MemoryStream memStream = new MemoryStream(buffer);
                return GetObjectInBytes(format, clipboardDataObject.GetType().ToString(), memStream);
            }
            //a copy from the file system is different because we must grab each file from disk, append the file name into the stream
            //so we can get at it later when we rebuild the file on the target machine, and then copy to a MemooryStream so we can use
            //our helper function again.
            /*
        else if (format == "FileDrop")
        {
            foreach (string fileName in (Array)clipboardDataObject)
            {
                if (File.Exists(fileName))
                {

                    FileStream fileStream = File.Open(fileName, FileMode.Open, FileAccess.Read);
                    FileInfo fi = new FileInfo(fileName);

                    byte[] buffer = new byte[(int)fileStream.Length];
                    fileStream.Read(buffer, 0, (int)fileStream.Length);

                    MemoryStream memoryStream = new MemoryStream();
                    memoryStream.Write(buffer, 0, buffer.Length);

                    CopyPaste_SendData(fi.Name.Replace("_", "") + "FILENAME" + format, "File", memoryStream);
                }
            }
        }
             */

            else
            {
                return null;
            }

        }

        private static byte[] GetObjectInBytes(string format, string objectType, MemoryStream memoryStream)
        {
            memoryStream.Position = 0;


            char[] formatInCharArray = format.ToCharArray();
            char[] objectTypeInCharArray = objectType.ToCharArray();

            byte[] formatInByteArray = new byte[formatInCharArray.Length];
            byte[] objectTypeInByteArray = new byte[objectTypeInCharArray.Length];

            int i = 0;
            foreach (char ch in formatInCharArray)
            {
                formatInByteArray[i] = (byte)ch;
                i++;
            }

            i = 0;
            foreach (char ch in objectTypeInCharArray)
            {
                objectTypeInByteArray[i] = (byte)ch;
                i++;
            }



            byte[] bufferedData = new byte[1 + 4 + 4 + 8 + formatInByteArray.Length + objectTypeInByteArray.Length + memoryStream.Length];

            //Data format [a -> byte] = Type a was converted to a byte for transfer:
            //
            // data[0...3] = size of format
            // data[4...7] = size of objectType
            // data[8...15] = size of memoryStream
            // data[16...n] = payload




            byte[] formatLengthInBytes = BitConverter.GetBytes(formatInByteArray.Length);
            byte[] objectTypeLengthInBytes = BitConverter.GetBytes(objectTypeInByteArray.Length);
            byte[] memoryStreamLengthInBytes = BitConverter.GetBytes(memoryStream.Length);


            for (int k = 0; k < 4; k++)
            {
                bufferedData[k] = formatLengthInBytes[k];
                bufferedData[k + 4] = objectTypeLengthInBytes[k];
            }

            for (int k = 0; k < 8; k++)
                bufferedData[k + 8] = memoryStreamLengthInBytes[k];



            for (int j = 0; j < formatInByteArray.Length; j++)
                bufferedData[j + 16] = formatInByteArray[j];

            for (int j = 0; j < objectTypeInByteArray.Length; j++)
                bufferedData[formatInByteArray.Length + j + 16] = objectTypeInByteArray[j];

            byte[] memoryStreamInBytes = new byte[memoryStream.Length];

            memoryStream.Read(memoryStreamInBytes, 0, (int)memoryStream.Length);

            for (long j = 0; j < memoryStreamInBytes.Length; j++)
                bufferedData[formatInByteArray.Length + objectTypeInByteArray.Length + (int)j + 16] = memoryStreamInBytes[j];


            return bufferedData;
        }
    }
}