using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.Net.Sockets;
using System.Collections;
using System.Runtime.InteropServices;

namespace PARTIES
{
    public partial class TelepointerPtoP : Form
    {
        private int port = 5130; // Port number used for TCP and UDP communications
        
        public AsyncCallback newClientCallback;
        public AsyncCallback dataReceivedCallback;

        private IntPtr last_update_hwnd;


        // Class that stores socket information.
        // Used to store information about other telepointer servers
        protected class CSocketPacket
        {
            public System.Net.Sockets.Socket thisSocket;    // socket that we use to communicate with the other server
            public byte[] buffer;
            public int bytes_received;
            public bool header_data;
            public byte header;
            public NetworkStream stream;

            public static CSocketPacket findSocketFromIP(IPAddress ip, ArrayList socket_array)
            {
                foreach (CSocketPacket sock_info in socket_array)
                {
                    if (ip.Equals(((IPEndPoint)sock_info.thisSocket.RemoteEndPoint).Address))
                        return sock_info;
                }
                return null;
            }
        }


        //Data for a single window we are hosting
        protected class TPointerHostSessions
        {
            public IntPtr server_hwnd;              // Handle to window we will broadcast
            public ArrayList client_hwnds;          // Windows on other machines that will display telepointer
            public ArrayList client_sockets;        // List of CSocketPackets that are connected to machines
            public bool mouse_in_window;            // Currently not used, toggle variable that indicates if the
                                                    // mouse is over the hosted window
            public TPointerHostSessions()
            {
                client_hwnds = new ArrayList();
                client_sockets = new ArrayList();
            }
        }


        //Data for a client window we are receiving updates for
        protected class TPointerClientSession
        {
            public IntPtr server_hwnd;          // The source handle (on the source machine)
            public IPEndPoint server_endpoint;  // The remote machine we are getting updates from
            public IntPtr client_hwnd;          // Client window the telepointer is displayed on
            public Telepointer pointer;         // The Telepointer itself
        }

        private ArrayList t_pointer_host_sessions;      // Array of TPointerHostSessions
        private ArrayList t_pointer_client_sessions;    // Array of TPointerClientSessions
        private ArrayList telepointer_server_sockets;   // Array of CSocketPackets of other telepointer servers


        private System.Windows.Forms.Timer mouse_position_capture_timer;    //Each tick send mouse position updates

        // Socket for sending/receiving telepointer requests
        private Socket tcp_socket;
        //private Socket udp_socket;  TODO:  Use UDP

        ///<summary>
        /// Telepointer peer-to-peer client/server constructor
        /// </summary>
        ///
        ///<param name="capture_interval">
        /// Timer tick interval.  Controls how often mouse position updates are sent (milliseconds)
        /// </param>
        ///
        public TelepointerPtoP(int capture_interval)
        {
            InitializeComponent();
            mouse_position_capture_timer = new System.Windows.Forms.Timer();
            mouse_position_capture_timer.Interval = capture_interval;
            mouse_position_capture_timer.Tick += new EventHandler(mouse_position_capture_timer_Tick);
            t_pointer_host_sessions = new ArrayList();
            t_pointer_client_sessions = new ArrayList();
            telepointer_server_sockets = new ArrayList();
            last_update_hwnd = IntPtr.Zero;
        }


        // Mouse timer tick function
        // Every timer tick this function sends mouse position updates
        void mouse_position_capture_timer_Tick(object sender, EventArgs e)
        {
            //1 - Find the window the mouse is currently over
            Win32Api.CursorInfo cinfo;
            cinfo = new Win32Api.CursorInfo();
            cinfo.cbSize = (uint)Marshal.SizeOf(cinfo);
            Win32Api.GetCursorInfo(out cinfo);
            IntPtr hwnd = Win32Api.WindowFromPoint(cinfo.ptScreenPos);
    
            //2 - Check if we are monitoring that window
            TPointerHostSessions sessions = null;
            IntPtr parent = Win32Api.GetAncestor(hwnd,Win32Api.GA_PARENT);
            IntPtr root = Win32Api.GetAncestor(hwnd, Win32Api.GA_ROOT);
            
            foreach (TPointerHostSessions temp in t_pointer_host_sessions)
            {
                if (temp.server_hwnd == hwnd || temp.server_hwnd == parent || temp.server_hwnd == root)
                {
                    sessions = temp;
                    break;
                }
            }


            //3 - If yes, send position updates
            if (sessions != null)
            {
                // 1 - get the position of the window
                Win32Api.Rect rc = new Win32Api.Rect();
                if (!Win32Api.GetWindowRect(sessions.server_hwnd, ref rc))
                    return;
                

                // 2 - compute the relative position of the mouse
                int relative_coordinates_x = cinfo.ptScreenPos.x - rc.left;
                int relative_coordinates_y = cinfo.ptScreenPos.y - rc.top;

                // 3 - for each client, send the mouse position
                for (int i = 0; i < sessions.client_hwnds.Count; i++)
                {
                    //Message format:
                    //buffer[0] = Message type
                    //buffer[1...4] = buffer size (int)
                    //buffer[5...8] = server hwnd (our hwnd)
                    //buffer[9...12] = client hwnd
                    //buffer[13...16] = x coordinate
                    //buffer[17...20] = y coordinate

                    byte[] buffer = new byte[1 + 4 + 4 + 4 + 4 + 4];
                    buffer[0] = (byte)NetworkMessages.TelepointerMessages.MousePosition;
                    byte[] size = BitConverter.GetBytes(4 + 4 + 4 + 4);  //sizeof(int) = 4
                    for (int j = 0; j < 4; j++)
                        buffer[j + 1] = size[j];

                    byte[] server_hwnd_buffer = BitConverter.GetBytes((uint)sessions.server_hwnd);
                    for (int j = 0; j < 4; j++)
                        buffer[j + 5] = server_hwnd_buffer[j];

                    byte[] client_hwnd_buffer = BitConverter.GetBytes((uint)(IntPtr)(sessions.client_hwnds[i]));
                    for (int j = 0; j < 4; j++)
                        buffer[j + 9] = client_hwnd_buffer[j];

                    byte[] xcoord_buffer = BitConverter.GetBytes(relative_coordinates_x);
                    for (int j = 0; j < 4; j++)
                        buffer[j + 13] = xcoord_buffer[j];

                    byte[] ycoord_buffer = BitConverter.GetBytes(relative_coordinates_y);
                    for (int j = 0; j < 4; j++)
                        buffer[j + 17] = ycoord_buffer[j];

                    NetworkMessages.safeWrite(((CSocketPacket)(sessions.client_sockets[i])).stream,
                        buffer, 0, buffer.Length, NetworkMessages.noOp);
                    //((CSocketPacket)(sessions.client_sockets[i])).stream.Write(buffer, 0, buffer.Length);

                }
            }

            if (sessions == null || (last_update_hwnd != sessions.server_hwnd))
            {
                TPointerHostSessions old_session = null;
                foreach (TPointerHostSessions temp in t_pointer_host_sessions)
                {
                    if (temp.server_hwnd == last_update_hwnd)
                    {
                        old_session = temp;
                        break;
                    }
                }
                if (old_session != null)
                {
                    for (int i = 0; i < old_session.client_hwnds.Count; i++)
                    {
                        //Message format:
                        //buffer[0] = Message type
                        //buffer[1...4] = buffer size (int)
                        //buffer[5...8] = server hwnd (our hwnd)
                        //buffer[9...12] = client hwnd
                        

                        byte[] buffer = new byte[1 + 4 + 4 + 4];
                        buffer[0] = (byte)NetworkMessages.TelepointerMessages.MouseOut;
                        byte[] size = BitConverter.GetBytes(4 + 4);  //sizeof(int) = 4
                        for (int j = 0; j < 4; j++)
                            buffer[j + 1] = size[j];

                        byte[] server_hwnd_buffer = BitConverter.GetBytes((uint)old_session.server_hwnd);
                        for (int j = 0; j < 4; j++)
                            buffer[j + 5] = server_hwnd_buffer[j];

                        byte[] client_hwnd_buffer = BitConverter.GetBytes((uint)(IntPtr)(old_session.client_hwnds[i]));
                        for (int j = 0; j < 4; j++)
                            buffer[j + 9] = client_hwnd_buffer[j];


                        NetworkMessages.safeWrite(((CSocketPacket)(old_session.client_sockets[i])).stream,
                            buffer, 0, buffer.Length, NetworkMessages.noOp);
                        //((CSocketPacket)(sessions.client_sockets[i])).stream.Write(buffer, 0, buffer.Length);

                    }
                }
                if (sessions == null)
                    last_update_hwnd = IntPtr.Zero;
                else
                    last_update_hwnd = sessions.server_hwnd;
            }

            //writeToOutput("mouse over hwnd " + hwnd.ToString() + "parent = "+ Win32Api.GetAncestor(hwnd,Win32Api.GA_PARENT).ToString() +"\n");

            
        }


        ///<summary>
        /// Changes the timer tick interval
        /// </summary>
        ///
        ///<param name="new_interval">
        /// New imer tick interval.  Controls how often mouse position updates are sent (milliseconds)
        /// </param>
        ///
        public void changeCaptureInterval(int new_interval)
        {
            mouse_position_capture_timer.Interval = new_interval;
        }


        
        public delegate void addLocalClientDelegate(IntPtr client_hwnd, IntPtr server_hwnd,
            IPAddress remote_machine, KnownColor color, float brightness);

        ///<summary>
        /// Adds a telepointer to a window running on the local machine.
        /// </summary>
        ///
        ///<param name="client_hwnd">
        /// Handle to the window that will display the telepointer
        /// </param>
        /// ///<param name="server_hwnd">
        /// Handle to the window that will track the mouse cursor location.  This is usually the handle of a window
        /// on a remote machine.
        /// </param>
        /// ///<param name="remote_machine">
        /// Address of the remote machine.  This is the machine with the server_hwnd window.
        /// </param>
        ///<param name="color">
        /// Color of the telepointer.
        /// </param>
        ///<param name="brightness">
        /// Brightness of the telepointer color.  1.0 is the standard recommended value.
        /// </param>
        public void addLocalClient(IntPtr client_hwnd, IntPtr server_hwnd,
            IPAddress remote_machine, KnownColor color, float brightness)
        {
            if (this.InvokeRequired)  //i.e. if we are trying to set the text and we aren't the main thread
            {
                this.BeginInvoke(new addLocalClientDelegate(addLocalClient),
                                                  new object[] { client_hwnd,server_hwnd,remote_machine, color, brightness });

                return;
            }
            
            IPAddress server_machine = remote_machine;
            // 1 - Check if we already have a connection to this machine

            writeToOutput("Searching for active socket ... \n");
                
            CSocketPacket socketinfo = CSocketPacket.findSocketFromIP(server_machine, telepointer_server_sockets);

            if (socketinfo == null)
            {
                writeToOutput("... socket to requested machine not found, creating new socket ... \n");
                socketinfo = new CSocketPacket();
                socketinfo.thisSocket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
                socketinfo.buffer = new byte[5];
                socketinfo.bytes_received = 0;
                socketinfo.header = (byte)0;
                socketinfo.header_data = true;

                // Make the connection
                try
                {
                    socketinfo.thisSocket.Connect(server_machine, port);  // this is a blocking call
                }
                catch (SocketException)
                {
                    Console.WriteLine("addLocalClient:  Unablet to connect to machine ...");
                    return;
                }
                socketinfo.stream = new NetworkStream(socketinfo.thisSocket);
                telepointer_server_sockets.Add(socketinfo);
                waitForData(socketinfo);
            }

            Win32Api.Rect rc = new Win32Api.Rect();
            TPointerClientSession new_client_session = new TPointerClientSession();
            new_client_session.client_hwnd = client_hwnd;
            new_client_session.server_hwnd = server_hwnd;
            new_client_session.server_endpoint = (IPEndPoint)(socketinfo.thisSocket.RemoteEndPoint);
            if (!Win32Api.GetWindowRect(client_hwnd, ref rc))
                new_client_session.pointer = new Telepointer(color, brightness, new Point(5, 5));
            else
                new_client_session.pointer = new Telepointer(color, brightness, new Point(rc.left + 5, rc.top + 5));
            new_client_session.pointer.showMe();
            t_pointer_client_sessions.Add(new_client_session);


            writeToOutput("... Success! Sending telepointer server request ... \n");

            //Message format:
            //buffer[0] = Message type
            //buffer[1...4] = buffer size (int)
            //buffer[5...8] = server hwnd (our hwnd)
            //buffer[9...12] = client hwnd
           
            byte[] buffer = new byte[1 + 4 + 4 + 4 ];
            buffer[0] = (byte)NetworkMessages.TelepointerMessages.ServerRequest;
            byte[] size = BitConverter.GetBytes(4 + 4);  //sizeof(int) = 4
            for (int j = 0; j < 4; j++)
                buffer[j + 1] = size[j];

            byte[] server_hwnd_buffer = BitConverter.GetBytes((int)server_hwnd);
            for (int j = 0; j < 4; j++)
                buffer[j + 5] = server_hwnd_buffer[j];

            byte[] client_hwnd_buffer = BitConverter.GetBytes((int)client_hwnd);
            for (int j = 0; j < 4; j++)
                buffer[j + 9] = client_hwnd_buffer[j];

            NetworkMessages.safeWrite(socketinfo.stream, buffer, 0, buffer.Length, NetworkMessages.noOp);
            //socketinfo.stream.Write(buffer, 0, buffer.Length);

        }

        
        // Add a client on a remote machine
        ///<summary>
        /// Adds a telepointer to a window running on a remote machine.
        /// </summary>
        ///
        ///<param name="client_hwnd">
        /// Handle to the window that will display the telepointer.  This is the handle on the remote machine.
        /// </param>
        /// ///<param name="server_hwnd">
        /// Handle to the window that will track the mouse cursor location.  (window is running on the local machine)
        /// </param>
        /// ///<param name="remote_machine">
        /// Address of the remote machine.  This is the machine with the client_hwnd window.
        /// </param>
        ///<param name="color">
        /// Color of the telepointer.
        /// </param>
        ///<param name="brightness">
        /// Brightness of the telepointer color.  1.0 is the standard recommended value.
        /// </param>
        public void addRemoteClient(IntPtr client_hwnd, IntPtr server_hwnd,
            IPAddress remote_machine, KnownColor color, float brightness)
        {
            IPAddress client_machine = remote_machine;

            //Check if we already have a connection to this machine
            writeToOutput("Searching for active socket ... \n");

            CSocketPacket socketinfo = CSocketPacket.findSocketFromIP(client_machine, telepointer_server_sockets);

            if (socketinfo == null)
            {
                writeToOutput("... socket to requested machine not found, creating new socket ... \n");
                socketinfo = new CSocketPacket();
                socketinfo.thisSocket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
                socketinfo.buffer = new byte[5];
                socketinfo.bytes_received = 0;
                socketinfo.header = (byte)0;
                socketinfo.header_data = true;

                // Make the connection
                try
                {
                    socketinfo.thisSocket.Connect(client_machine, port);  // this is a blocking call
                }
                catch (SocketException)
                {
                    Console.WriteLine("addLocalClient:  Unablet to connect to machine ...");
                    return;
                }
                socketinfo.stream = new NetworkStream(socketinfo.thisSocket);
                telepointer_server_sockets.Add(socketinfo);
                waitForData(socketinfo);
            }
            
            
           
            
            writeToOutput("... Success! Sending telepointer client request ... \n");

            //Message format
            // [0] = header
            // [1 ... 4] = size
            // [5 ... 8] = server_hwnd
            // [6 ... 12] = client_hwnd
            // [13 ... 14] = color (KnownColor)
            // [15 ...18] = brightness (float)
            byte[] buffer = new byte[1 + 4 + 4 + 4 + 2 + 4];
            buffer[0] = (byte)NetworkMessages.TelepointerMessages.ClientRequest;
            byte[] size = BitConverter.GetBytes(4 + 4 + 2 + 4);  //sizeof(int) = 4
            for (int j = 0; j < 4; j++)
                buffer[j + 1] = size[j];

            byte[] server_hwnd_buffer = BitConverter.GetBytes((int)server_hwnd);
            for (int j = 0; j < 4; j++)
                buffer[j + 5] = server_hwnd_buffer[j];

            byte[] client_hwnd_buffer = BitConverter.GetBytes((int)client_hwnd);
            for (int j = 0; j < 4; j++)
                buffer[j + 9] = client_hwnd_buffer[j];

            ushort color_short = (ushort)color;
            byte[] color_buffer = BitConverter.GetBytes(color_short);
            for (int j = 0; j < 2; j++)
                buffer[j + 13] = color_buffer[j];

            byte[] brightness_buffer = BitConverter.GetBytes(brightness);
            for (int j = 0; j < 4; j++)
                buffer[j + 15] = brightness_buffer[j];

            NetworkMessages.safeWrite(socketinfo.stream, buffer, 0, buffer.Length, NetworkMessages.noOp);
            //socketinfo.stream.Write(buffer, 0, buffer.Length);

        }
        


        // Stop a client telepointer running on our machine
        public delegate void stopLocalClientDelegate(IntPtr client_hwnd, IntPtr server_hwnd,
            IPAddress remote_machine);
        ///<summary>
        /// Stops a telepointer that is being displayed on a local window.
        /// </summary>
        ///
        ///<param name="client_hwnd">
        /// Handle to the window currently displaying the telepointer.
        /// </param>
        /// ///<param name="server_hwnd">
        /// Handle to the window (usually on a remote machine) that is providing position updates for this telepointer.
        /// </param>
        /// ///<param name="remote_machine">
        /// Address of the remote machine.  This is the machine with the server_hwnd window.
        /// </param>
        public void stopLocalClient(IntPtr client_hwnd, IntPtr server_hwnd,
            IPAddress remote_machine)
        {
            if (this.InvokeRequired)  //i.e. if we are trying to set the text and we aren't the main thread
            {
                this.BeginInvoke(new stopLocalClientDelegate(stopLocalClient),
                                                  new object[] { client_hwnd, server_hwnd, remote_machine });

                return;
            }
            //Find the connection to remote machine
            writeToOutput("Searching for active socket ... \n");

            CSocketPacket socketinfo = CSocketPacket.findSocketFromIP(remote_machine, telepointer_server_sockets);

            if (socketinfo == null)
            {
                writeToOutput("... socket to requested machine not found, creating new socket ... \n");
                socketinfo = new CSocketPacket();
                socketinfo.thisSocket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
                socketinfo.buffer = new byte[5];
                socketinfo.bytes_received = 0;
                socketinfo.header = (byte)0;
                socketinfo.header_data = true;

                // Make the connection
                try
                {
                    socketinfo.thisSocket.Connect(remote_machine, port);  // this is a blocking call
                }
                catch (SocketException)
                {
                    Console.WriteLine("addLocalClient:  Unablet to connect to machine ...");
                    return;
                }
                socketinfo.stream = new NetworkStream(socketinfo.thisSocket);
                telepointer_server_sockets.Add(socketinfo);
                waitForData(socketinfo);
            }

            writeToOutput("Success! Sending ending client notification ... \n");

            byte[] buffer = new byte[1 + 4 + 4 + 4];
            buffer[0] = (byte)NetworkMessages.TelepointerMessages.EndingClientTelepointer;
            byte[] size = BitConverter.GetBytes(4 + 4);  //sizeof(int) = 4
            for (int j = 0; j < 4; j++)
                buffer[j + 1] = size[j];

            byte[] server_hwnd_buffer = BitConverter.GetBytes((int)server_hwnd);
            for (int j = 0; j < 4; j++)
                buffer[j + 5] = server_hwnd_buffer[j];

            byte[] client_hwnd_buffer = BitConverter.GetBytes((int)client_hwnd);
            for (int j = 0; j < 4; j++)
                buffer[j + 9] = client_hwnd_buffer[j];

            NetworkMessages.safeWrite(socketinfo.stream, buffer, 0, buffer.Length, NetworkMessages.noOp);
            //socketinfo.stream.Write(buffer, 0, buffer.Length);


            writeToOutput("Finding local telepointer to remove ... \n");
            // Find all ongoing sessions and add them to a list of telelpointers to remove
            ArrayList client_sessions = new ArrayList();
            foreach (TPointerClientSession session in t_pointer_client_sessions)
            {
                if (session.server_endpoint.Address.Equals(remote_machine)
                    && session.server_hwnd == server_hwnd
                    && session.client_hwnd == client_hwnd)
                {
                    client_sessions.Add(session);
                }
            }

            
            // Close the telepointer and remove it from the list of active sessions
            foreach (TPointerClientSession session in client_sessions)
            {
                writeToOutput("Removing local telepointer ... \n");
                session.pointer.Close();
                t_pointer_client_sessions.Remove(session);
            }


        }

        // Stop a client running on a remote machine
        ///<summary>
        /// Stops a telepointer that is being displayed on a remote machine window.
        /// </summary>
        ///
        ///<param name="client_hwnd">
        /// Handle to the window currently displaying the telepointer (on the remote machine).
        /// </param>
        /// ///<param name="server_hwnd">
        /// Handle to the window (on the local machine) that is providing position updates for this telepointer.
        /// </param>
        /// ///<param name="remote_machine">
        /// Address of the remote machine.  This is the machine with the client_hwnd window.
        /// </param>
        public void stopRemoteClient(IntPtr client_hwnd, IntPtr server_hwnd,
            IPAddress remote_machine)
        {
             IPAddress client_machine = remote_machine;

            //Check if we already have a connection to this machine
            writeToOutput("Searching for active socket ... \n");

            CSocketPacket socketinfo = CSocketPacket.findSocketFromIP(client_machine, telepointer_server_sockets);

            if (socketinfo == null)
            {
                writeToOutput("... socket to requested machine not found, creating new socket ... \n");
                socketinfo = new CSocketPacket();
                socketinfo.thisSocket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
                socketinfo.buffer = new byte[5];
                socketinfo.bytes_received = 0;
                socketinfo.header = (byte)0;
                socketinfo.header_data = true;

                // Make the connection
                try
                {
                    socketinfo.thisSocket.Connect(client_machine, port);  // this is a blocking call
                }
                catch (SocketException)
                {
                    Console.WriteLine("addLocalClient:  Unablet to connect to machine ...");
                    return;
                }
                socketinfo.stream = new NetworkStream(socketinfo.thisSocket);
                telepointer_server_sockets.Add(socketinfo);
                waitForData(socketinfo);
            }
            
            
           
            
            writeToOutput("... Success! Sending end telepointer client request ... \n");

            byte[] buffer = new byte[1 + 4 + 4 + 4];
            buffer[0] = (byte)NetworkMessages.TelepointerMessages.EndClientRequest;
            byte[] size = BitConverter.GetBytes(4 + 4);  //sizeof(int) = 4
            for (int j = 0; j < 4; j++)
                buffer[j + 1] = size[j];

            byte[] server_hwnd_buffer = BitConverter.GetBytes((int)server_hwnd);
            for (int j = 0; j < 4; j++)
                buffer[j + 5] = server_hwnd_buffer[j];

            byte[] client_hwnd_buffer = BitConverter.GetBytes((int)client_hwnd);
            for (int j = 0; j < 4; j++)
                buffer[j + 9] = client_hwnd_buffer[j];

            NetworkMessages.safeWrite(socketinfo.stream, buffer, 0, buffer.Length, NetworkMessages.noOp);
            //socketinfo.stream.Write(buffer, 0, buffer.Length);
        }

        // Stop a server running on our machine
        ///<summary>
        /// Stops all telepointer clients corresponding to the specified window handle.
        /// 
        /// This can only be called on a window handle on the local machine.
        /// </summary>
        ///
        /// ///<param name="server_hwnd">
        /// Handle to the window (on the local machine) that is providing position updates.  All local and remote telepointer clients
        /// listening for updates from this window will be stopped.
        /// </param>
        public void stopServer(IntPtr server_hwnd)
        {
            writeToOutput("Stopping telepointer server for window " + server_hwnd.ToString() + " ... \n");
            //just call stop remote client for all sessions with this server hwnd
            TPointerHostSessions session = null;

            foreach (TPointerHostSessions current_sessions in t_pointer_host_sessions)
            {
                if (current_sessions.server_hwnd == server_hwnd)
                    session = current_sessions;
            }

            if (session != null)
            {
                for (int i = 0; i < session.client_sockets.Count; i++)
                {
                    stopRemoteClient((IntPtr)session.client_hwnds[i], server_hwnd,
                        ((IPEndPoint)((CSocketPacket)(session.client_sockets[i])).thisSocket.RemoteEndPoint).Address);

                }
            }
        }
            
           


        
        //Write to the output box on the server form
        //We must use a delegate because only the main UI thread can write to the output box
        private delegate void writeToOutputDelegate(string text);
        private void writeToOutput(string text)
        {
            if (this.InvokeRequired)  //i.e. if we are trying to set the text and we aren't the main thread
            {
                this.BeginInvoke(new writeToOutputDelegate(writeToOutput),
                                                  new object[] { text });

                return;
            }
            textBox1.AppendText(text);
        }

        
        //Asynchronously wait for new client connections
        void waitForNewClients()
        {
            if (newClientCallback == null)
                newClientCallback = new AsyncCallback(OnClientConnect);
            // now start to listen for any data...
            tcp_socket.BeginAccept(newClientCallback, null);

        }


        // DONE !!!
        //Asynchronously wait for data from another dispatch server
        void waitForData(CSocketPacket socket_packet)
        {
            if (dataReceivedCallback == null)
                dataReceivedCallback = new AsyncCallback(onDataReceived);
            // now start to listen for any data...
            socket_packet.thisSocket.BeginReceive(socket_packet.buffer, socket_packet.bytes_received, socket_packet.buffer.Length - socket_packet.bytes_received, SocketFlags.None, dataReceivedCallback, socket_packet);

        }


        // DONE!!!
        //Called when another telepointer p2p connects with us
        public void OnClientConnect(IAsyncResult asyn)
        {
            try
            {
                writeToOutput("Client connected \n");
                Socket client = tcp_socket.EndAccept(asyn);
                
                //wait for data from new client
                CSocketPacket sock_pkt = new CSocketPacket();
                sock_pkt.thisSocket = client;
                sock_pkt.bytes_received = 0;
                sock_pkt.buffer = new byte[5];
                sock_pkt.header_data = true;
                sock_pkt.stream = new NetworkStream(sock_pkt.thisSocket);
                telepointer_server_sockets.Add(sock_pkt);
                waitForData(sock_pkt);
                
                //wait for more clients to connect
                waitForNewClients();
            }
            catch (ObjectDisposedException)
            {
                System.Diagnostics.Debugger.Log(0, "1", "\n OnClientConnection: Socket has been closed\n");
            }
            catch (SocketException se)
            {
                MessageBox.Show(se.Message);
            }
        }


        
        //Called when we have received data from another telepointer 
        public void onDataReceived(IAsyncResult asyn)
        {
            try
            {
                CSocketPacket socket_info = (CSocketPacket)asyn.AsyncState;
                int just_received = 0;
                just_received = socket_info.thisSocket.EndReceive(asyn);


                socket_info.bytes_received += just_received;

                if (socket_info.bytes_received < socket_info.buffer.Length)
                {
                    waitForData(socket_info);
                    return;
                }

                socket_info.bytes_received = 0;

                if (socket_info.header_data)
                {
                    socket_info.header = socket_info.buffer[0];
                    int data_buffer_length = BitConverter.ToInt32(socket_info.buffer, 1);
                    socket_info.buffer = new byte[data_buffer_length];
                    socket_info.header_data = false;
                    waitForData(socket_info);
                }
                else
                {
                    processDataBuffer(socket_info);
                }

            }
            catch (ObjectDisposedException)
            {
                // Console.WriteLine("onDataReceieved: Socket has been closed");
            }
            catch (SocketException se)
            {
                MessageBox.Show(se.Message);
            }
        }


        // TODO:  Add color and location startup code for ClientRequest
        //Process the data buffer from another dispatch server
        void processDataBuffer(CSocketPacket socket_info)
        {
            // A machine has contacted us because they want us to run a server for their client
            if (socket_info.header == (byte)NetworkMessages.TelepointerMessages.ServerRequest)
            {
                writeToOutput("Server request received ...");
                //Message data includes:
                //  - the hwnd of the window on our machine
                //  - the client window hwnd (on the remote machine)

                //First, read off the data
                //[0...3] = server hwnd (our hwnd)
                IntPtr server_hwnd = (IntPtr)BitConverter.ToInt32(socket_info.buffer, 0);
                //[4...7] = client hwnd
                IntPtr client_hwnd = (IntPtr)BitConverter.ToInt32(socket_info.buffer, 4);

                //Check if we already are hosting this window
                TPointerHostSessions session = null;

                foreach (TPointerHostSessions current_sessions in t_pointer_host_sessions)
                {
                    if (current_sessions.server_hwnd == server_hwnd)
                        session = current_sessions;
                }

                if (session == null)
                {
                    writeToOutput(" ... creating new hosting session ...\n");
                    session = new TPointerHostSessions();
                    session.server_hwnd = server_hwnd;
                    session.client_hwnds.Add(client_hwnd);
                    session.client_sockets.Add(socket_info);
                    session.mouse_in_window = false;
                    t_pointer_host_sessions.Add(session);
                }
                else
                {
                    writeToOutput(" ... existing hosting session found ...\n");
                    session.client_hwnds.Add(client_hwnd);
                    session.client_sockets.Add(socket_info);
                }
                socket_info.header_data = true;
                socket_info.buffer = new byte[5];
                waitForData(socket_info);

            }

            // A machine actiing as a server is asking us to start a new telepointer client
            else if (socket_info.header == (byte)NetworkMessages.TelepointerMessages.ClientRequest)
            {
                writeToOutput("Client request received ...");
                //First, read off the data
                //[0...3] = server hwnd (our hwnd)
                IntPtr server_hwnd = (IntPtr)BitConverter.ToInt32(socket_info.buffer, 0);
                //[4...7] = client hwnd
                IntPtr client_hwnd = (IntPtr)BitConverter.ToInt32(socket_info.buffer, 4);
                //[8 ..9] = color (ushort KnownColor)
                ushort color_code = BitConverter.ToUInt16(socket_info.buffer, 8);
                KnownColor color = (KnownColor)color_code;

                float brightness = BitConverter.ToSingle(socket_info.buffer,10);

                socket_info.header_data = true;
                socket_info.buffer = new byte[5];
                waitForData(socket_info);

                addLocalClient(client_hwnd, server_hwnd, ((IPEndPoint)socket_info.thisSocket.RemoteEndPoint).Address, color, brightness);
            }

            //We are being asked to end one of clients
            else if (socket_info.header == (byte)NetworkMessages.TelepointerMessages.EndClientRequest)
            {
                //First, read off the data
                //[0...3] = server hwnd 
                IntPtr server_hwnd = (IntPtr)BitConverter.ToInt32(socket_info.buffer, 0);
                //[4...7] = client hwnd (our hwnd)
                IntPtr client_hwnd = (IntPtr)BitConverter.ToInt32(socket_info.buffer, 4);

                stopLocalClient(client_hwnd,server_hwnd,((IPEndPoint)socket_info.thisSocket.RemoteEndPoint).Address);

                socket_info.header_data = true;
                socket_info.buffer = new byte[5];
                waitForData(socket_info);
            }
            // A client that we were broadcasting to is now ending
            else if (socket_info.header == (byte)NetworkMessages.TelepointerMessages.EndingClientTelepointer)
            {
                //Message data includes:
                //  - the hwnd of the window on our machine
                //  - the client window hwnd (on the remote machine)

                //First, read off the data
                //[0...3] = server hwnd (our hwnd)
                IntPtr server_hwnd = (IntPtr)BitConverter.ToInt32(socket_info.buffer, 0);
                //[4...7] = client hwnd
                IntPtr client_hwnd = (IntPtr)BitConverter.ToInt32(socket_info.buffer, 4);

                TPointerHostSessions host_session = null;
                foreach (TPointerHostSessions sessions in t_pointer_host_sessions)
                {
                    if (server_hwnd == sessions.server_hwnd)
                    {
                        host_session = sessions;
                        break;
                    }
                }

                if (host_session != null)
                {

                    int index = -1;

                    for (int i = 0; i < host_session.client_hwnds.Count; i++)
                    {
                        if (((IntPtr)host_session.client_hwnds[i]) == client_hwnd)
                        {
                            index = i;
                            break;
                        }
                    }

                    if (index >= 0)
                    {

                        //we now have the index of the telepointer to remove

                        host_session.client_hwnds.RemoveAt(index);
                        host_session.client_sockets.RemoveAt(index);
                    }
                }
                socket_info.header_data = true;
                socket_info.buffer = new byte[5];
                waitForData(socket_info);

            }
            else if (socket_info.header == (byte)NetworkMessages.TelepointerMessages.MousePosition)
            {
                //First, read off the data
                //[0...3] = server hwnd 
                IntPtr server_hwnd = (IntPtr)BitConverter.ToInt32(socket_info.buffer, 0);
                //[4...7] = client hwnd (our hwnd)
                IntPtr client_hwnd = (IntPtr)BitConverter.ToInt32(socket_info.buffer, 4);
                //[8...11] = mouse x position (relative to left corner
                int mouse_x = BitConverter.ToInt32(socket_info.buffer, 8);
                //[12...15] = mouse x position (relative to left corner
                int mouse_y = BitConverter.ToInt32(socket_info.buffer, 12);

                // Find all ongoing sessions being served and add them to a list of telelpointers to update
                ArrayList client_sessions = new ArrayList();
                foreach (TPointerClientSession session in t_pointer_client_sessions)
                {
                    if (session.server_endpoint.Address.Equals(((IPEndPoint)socket_info.thisSocket.RemoteEndPoint).Address)
                        && session.server_hwnd == server_hwnd)
                    {
                        client_sessions.Add(session);
                    }
                }

                // Move the telepointer
                foreach (TPointerClientSession session in client_sessions)
                {
                    // Move the telepointer                   
                    moveTelepointer(session, mouse_x, mouse_y);               
                }
                socket_info.header_data = true;
                socket_info.buffer = new byte[5];
                waitForData(socket_info);


            }
            //Mouse has left the window and we should hide the telepointer
            else if (socket_info.header == (byte)NetworkMessages.TelepointerMessages.MouseOut)
            {
                writeToOutput("Server request received ...");
                //Message data includes:
                //  - the hwnd of the window on our machine
                //  - the client window hwnd (on the remote machine)

                //First, read off the data
                //[0...3] = server hwnd (our hwnd)
                IntPtr server_hwnd = (IntPtr)BitConverter.ToInt32(socket_info.buffer, 0);
                //[4...7] = client hwnd
                IntPtr client_hwnd = (IntPtr)BitConverter.ToInt32(socket_info.buffer, 4);

                // Find all ongoing sessions and add them to a list of telelpointers to hide
                foreach (TPointerClientSession session in t_pointer_client_sessions)
                {
                    if (session.server_endpoint.Address.Equals(((IPEndPoint)(socket_info.thisSocket.RemoteEndPoint)).Address)
                        && session.server_hwnd == server_hwnd
                        && session.client_hwnd == client_hwnd)
                    {
                        session.pointer.hideMe();
                    }
                }

                socket_info.header_data = true;
                socket_info.buffer = new byte[5];
                waitForData(socket_info);
              

            }
            //Reset the buffer for the socket
            
        }

        //Telepointer will flicker if this code is not run by the main thread.
        //The reason is that the occlusion check and moving the cursor won't be synchronized, and so sometimes the
        //occlusion check would hit the cursor itself
        private delegate void moveTelepointerDelegate(TPointerClientSession session, int mouse_x, int mouse_y);
        private void moveTelepointer(TPointerClientSession session, int mouse_x, int mouse_y)
        {
            if (this.InvokeRequired)  //i.e. if we are trying to set the text and we aren't the main thread
            {
                this.BeginInvoke(new moveTelepointerDelegate(moveTelepointer),
                                                  new object[] { session, mouse_x, mouse_y });

                return;
            }
        
            Win32Api.Rect rc = new Win32Api.Rect();
            if (!Win32Api.GetWindowRect(session.client_hwnd, ref rc))
                return;
            session.pointer.moveTo(new Point(rc.left + mouse_x, rc.top + mouse_y));
            Win32Api.PointWin32 location = new Win32Api.PointWin32();
            location.x = rc.left + mouse_x+5;  //we don't want to hit ourselves so offet to
            location.y = rc.top + mouse_y-2;   //the top right of the mouse
            IntPtr hwnd = Win32Api.WindowFromPoint(location);
            IntPtr parent = Win32Api.GetAncestor(hwnd, Win32Api.GA_PARENT);
            IntPtr root = Win32Api.GetAncestor(hwnd, Win32Api.GA_ROOT);
            if (session.client_hwnd == hwnd || session.client_hwnd == parent || session.client_hwnd == root)
            {
                session.pointer.Show();
            }
            else
            {
                session.pointer.Hide();
            }

        }

        // TODO: UDP socket setup
        public void startTelepointerServer()
        {
            //Open TCP and UDP port
            //
            //  -- TCP = new telepointer client or server support
            //  -- UDP = mouse position updates

            writeToOutput("Starting telepointers ... \n");

            IPAddress serverIP = IPAddress.Any;
            IPEndPoint ipep = new IPEndPoint(serverIP, port);

            //Start waiting for clients to connect
            tcp_socket = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
            tcp_socket.Bind(ipep);
            tcp_socket.Listen(10);
            writeToOutput("Listening on port "+port.ToString()+" ...\n");
            waitForNewClients();

         
            if (System.Diagnostics.Debugger.IsAttached)
                writeToOutput("Debugger attached\n");
            else
            {
                writeToOutput("No Debugger attached\n");
                this.ShowInTaskbar = false;
                this.Visible = false;
                this.Opacity = 0;
            }

            mouse_position_capture_timer.Start();

        }


        // TODO: Look over everything
        // NOT CURRENTLY USED
        // TCP IS CURRENTLY USED, SEE THE TIMER FUNCTION
        private static void sendMousePosition(Point position, Socket udp_sock, IPEndPoint ip_end_point)
        {
            byte[] mouse_location_X_buf = BitConverter.GetBytes(position.X);
            byte[] mouse_location_Y_buf = BitConverter.GetBytes(position.Y);
            byte[] bufferedData = new byte[1 + 4 + 4 + 4];
            byte[] buffer_size = BitConverter.GetBytes(4 + 4);

            bufferedData[0] = (byte)NetworkMessages.MessageType.MousePosition;

            for (int i = 0; i < 4; i++)
            {
                bufferedData[1 + i] = buffer_size[i];
            }

            //Copy mouse click location
            for (int i = 0; i < 4; i++)
            {
                bufferedData[5 + i] = mouse_location_X_buf[i];
            }

            for (int i = 0; i < 4; i++)
            {
                bufferedData[9 + i] = mouse_location_Y_buf[i];
            }

            udp_sock.SendTo(bufferedData, ip_end_point);
        }
    }
}